#include <qwidget.h>
#include <qpainter.h>
#include <qkeycode.h>
#include <qpainter.h>
#include <qdrawutil.h>
#include "bildtabelle.h"
//.............................................. Bildtabelle (Konstruktor)
Bildtabelle::Bildtabelle( int zeilZahl, int spaltZahl,
                          QWidget *p, const char *nam ) : QGridView(p,nam) {
    setFocusPolicy( StrongFocus );  // Tastaturfokus ist erlaubt
    setNumCols( spaltZahl );  // Festlegen der Spalten- und
    setNumRows( zeilZahl );   // Zeilenzahl in der Tabelle
    setCellWidth( 100 );    // Voreingest. Festlegen der Breite und
    setCellHeight( 100 );   //  Hoehe der Zellen (in Pixel)
    frameWidth = border = 0;
    resize( 600, 600 );
    bilder = new QPixmap [zeilZahl * spaltZahl]; // Bilder
    marks  = new bool [zeilZahl * spaltZahl]; // Markierungen
    resetAllMarks();
}
//................................................................ setCell
void Bildtabelle::setCell( int  i, int j, QString bildName ) {
   if ( i < 0 || i >= numRows() || j < 0 || j >= numCols() )
      return;
   bilder[ indexOf( i, j) ].load( bildName );
   repaintCell( i, j, false );
}
//................................................................ setCell
void Bildtabelle::setCell( int  i, int j, QPixmap bild ) {
   if ( i < 0 || i >= numRows() || j < 0 || j >= numCols() )
      return;
   bilder[ indexOf( i, j) ] = bild;
   repaintCell( i, j, false );
}
//............................................................ setCellSize
void Bildtabelle::setCellSize( int  w, int h, int border, int frameWidth ) {
   setCellWidth( w );
   setCellHeight( h );
   this->border = border;
   this->frameWidth = frameWidth;
}
//............................................................ setCellMark
void Bildtabelle::setCellMark( int  i, int j, bool mark ) {
   if ( i < 0 || i >= numRows() || j < 0 || j >= numCols() ||
        marks[ indexOf( i, j) ] == mark )
      return;
   marks[ indexOf( i, j) ] = mark;
   repaintCell( i, j, false );
}
//.......................................................... resetAllMarks
void Bildtabelle::resetAllMarks( void ) {
   for ( int i=0; i<numRows(); i++ )
      for ( int j=0; j<numCols(); j++ )
         setCellMark( i, j, false );
}
//.............................................................. paintCell
void Bildtabelle::paintCell( QPainter* p, int zeile, int spalte ) {
    int     width = cellWidth(),
            height = cellHeight();
    double  w = width - border*2 - frameWidth*2,
            h = height - border*2 - frameWidth*2;
    QBrush b( Qt::lightGray );
    if ( marks[ indexOf( zeile, spalte ) ] ) {
       b.setColor( Qt::blue );
       qDrawShadeRect (p, 0, 0, width, height, colorGroup(), true,
                       frameWidth, 0, &b );
    } else
       qDrawShadeRect (p, 0, 0, width, height, colorGroup(), false,
                       frameWidth, 0, &b );
      //... Bild ausgeben
    QPixmap bild = bilder[ indexOf( zeile, spalte ) ] ;
    double  xFactor = w / bild.width(),
            yFactor = h / bild.height();
    p->scale( xFactor, yFactor );
    p->drawPixmap( int((frameWidth+border)/xFactor),
                   int((frameWidth+border)/yFactor), bild );
    p->scale( 1/xFactor, 1/yFactor );
}
