#include  "bruch.h"

//.................................................... Konstruktoren
CBruch::CBruch() {
   m_Zaehler = 0; // Vorsicht: Ein Bruch ist nur definiert, wenn Nenner != 0 ist.
   m_Nenner  = 1;
}

CBruch::CBruch(long z, long n) {
   if (m_Nenner != 0) {
      m_Zaehler = z;
      m_Nenner  = n;
   }
}

//...................................................public-Methoden
long CBruch::GetNenner() {
   return m_Nenner;
}

long CBruch::GetZaehler() {
   return m_Zaehler;
}

void CBruch::SetBruch(long z, long n) {
   if (n > 0) {
      m_Zaehler = z;
      m_Nenner  = n;
   }
}

/* Obwohl zulaessig, auf die Membervariablen m_Nenner und m_Zaehler des
   uebergebenen Objektes direkt zuzugreifen, gebietet es die
   "Hoeflichkeit", nicht auf die privaten Daten eines anderen Objektes
   direkt zuzugreifen, auch wenn dieses Objekt aus der gleichen Klasse ist.
*/

void CBruch::Add(CBruch& b) {
   m_Zaehler = m_Zaehler * b.GetNenner() + b.GetZaehler() * m_Nenner;
   m_Nenner *= b.GetNenner();
   Kuerzen();
}

void CBruch::Sub(CBruch& b) {
      // Subtraktion = Addition mit negativen Bruch
   b.SetBruch(- b.GetZaehler(), b.GetNenner());
   Add(b);
}

void CBruch::Mul(CBruch& b) {
   m_Nenner  *= b.GetNenner();
   m_Zaehler *= b.GetZaehler();
   Kuerzen();
}

void CBruch::Div(CBruch& b) {
      // Division = Multiplikation mit Kehrbruch
   m_Nenner*= b.GetZaehler();
   m_Zaehler*= b.GetNenner();
   Kuerzen();
}

void CBruch::Kuerzen(void) {
   long  ggTeiler = ggT(m_Zaehler, m_Nenner);

   m_Zaehler /= ggTeiler;
   m_Nenner  /= ggTeiler;
}

//..................................................private-Methoden
long CBruch::ggT(long n, long m) {
   return (m==0) ? n : ggT(m, n%m);
}
