#include <qapplication.h>
#include <qwidget.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qlabel.h>
#include <qstring.h>
#include <qframe.h>

const char *farbName[] = { "Weiss", "Schwarz", "Rot", "Blau" };
const char *elemName[] = { "Wasser", "Luft", "Erde" };
const int farbZahl = sizeof(farbName) / sizeof(*farbName);
const int elemZahl = sizeof(elemName) / sizeof(*elemName);
//-------------------------------------------------------- Buttons Klasse -----
class Buttons : public QWidget {
   Q_OBJECT
public:
   Buttons();
private slots:
   void ok_slot() {
      zeigeWahl( "Ok gedrueckt", Qt::green );
      sichereAlteWerte();
   }
   void cancel_slot() { 
      setzeAlteWerte();
      zeigeWahl( "Cancel gedrueckt", Qt::white );
   }
private:
   QCheckBox     *farbe[farbZahl];
   QRadioButton  *element[elemZahl];
   QButtonGroup  *buttonBox1, *buttonBox2;
   QLabel        *textausgabe;
   bool           altFarbe[farbZahl], altElement[elemZahl];

   void sichereAlteWerte( void ) {
      for (int i=0; i<farbZahl; i++) altFarbe[i] = farbe[i]->isChecked();
      for (int i=0; i<elemZahl; i++) altElement[i] = element[i]->isChecked();
   }
   void setzeAlteWerte( void ) {
      for (int i=0; i<farbZahl; i++) farbe[i]->setChecked( altFarbe[i] );
      for (int i=0; i<elemZahl; i++) element[i]->setChecked( altElement[i] );
   }
   void zeigeWahl( QString titel, QColor c ) {
      textausgabe->setPalette( QPalette( c, c ) ); 
      QString neuer_text( titel ), str;
      neuer_text += "\n\nMomentan gewaehltes Grundelement:\n";
      for (int i=0; i<elemZahl; i++)
         if ( element[i]->isChecked() )
            neuer_text += str.sprintf("  - %s\n", elemName[i]);
      neuer_text += "\nMomentan gewaehlte Farben:\n";
      for (int i=0; i<farbZahl; i++)
         if ( farbe[i]->isChecked() )
            neuer_text += str.sprintf("  - %s\n", farbName[i]);
      textausgabe->setText( neuer_text );
   }
};
#include "buttons.moc"
//---------------------------------------------------- Konstruktor Buttons -----
Buttons::Buttons() {
   int i;
   setFont( QFont("Times", 12) );
   buttonBox1 = new QButtonGroup( 1, Horizontal,
                                  "Grundelemente (RadioButtons)", this);
   buttonBox1->setGeometry( 20, 20, 220, 150 );
   buttonBox1->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );
   for (i=0; i<elemZahl; i++)
      element[i] = new QRadioButton( elemName[i], buttonBox1 );
   element[1]->setChecked( true );

   buttonBox2 = new QButtonGroup( 1, Horizontal, "Farben (CheckBoxen)", this );
   buttonBox2->setGeometry( 260, 20, 220, 150 );
   buttonBox2->setFrameStyle( QFrame::Panel | QFrame::Raised );
   for (i=0; i<farbZahl; i++) {
      farbe[i] = new QCheckBox( farbName[i], buttonBox2 );
      buttonBox2->insert( farbe[i] );
   }
   farbe[0]->setChecked( true );
   farbe[3]->setChecked( true );
   sichereAlteWerte();
   QPushButton* ok     = new QPushButton( "Ok", this );
    ok->setGeometry( 20, 200, 100, 40 );
   QPushButton* cancel = new QPushButton( "Cancel", this );
    cancel->setGeometry( 150, 200, 100, 40 );
   QPushButton* quit = new QPushButton( "Quit", this );
    quit->setGeometry( 380, 200, 100, 40 );
   QObject::connect( ok,     SIGNAL( clicked() ), SLOT( ok_slot() ) );
   QObject::connect( cancel, SIGNAL( clicked() ), SLOT( cancel_slot() ) );
   QObject::connect( quit,   SIGNAL( clicked() ), qApp, SLOT( quit() ) );
   textausgabe = new QLabel( "Noch keine Wahl", this );
   textausgabe->setGeometry( 10, 250, 480, 240 );
   textausgabe->setFrameStyle( QFrame::Box | QFrame::Sunken );
}
//------------------------------------------------------------------- main ------
int main( int argc, char* argv[] ) {
   QApplication myapp( argc, argv );
   Buttons* mywidget = new Buttons();
   mywidget->setGeometry( 10, 10, 500, 500 );
   mywidget->show();
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
