#include <qapplication.h>
#include <qcanvas.h>
#include <stdlib.h>
#include <time.h>

class Node;

//............................................................. Klasse Edge
class Edge: public QCanvasLine
{
public:
    Edge( Node *from, Node *to, QCanvas *canvas );

    void setFromPoint( int x, int y ) {
       setPoints( x, y, endPoint().x(), endPoint().y() );
    }

    void setToPoint( int x, int y ) {
       setPoints( startPoint().x(), startPoint().y(), x, y );
    }
    void moveBy(double, double ) { }
};

//............................................................. Klasse Node
class Node: public QCanvasEllipse {
public:
    Node( QCanvas *canvas ) : QCanvasEllipse( 20, 20, canvas ) {
       setBrush( QBrush( QColor( ::rand()&255, ::rand()&255, ::rand()&255 ) ) );
    }

    void addInEdge( Edge *edge ) { inList.append( edge ); }
    void addOutEdge( Edge *edge ) { outList.append( edge ); }

    void moveBy(double dx, double dy) {
       QCanvasEllipse::moveBy( dx, dy );

       QPtrListIterator<Edge> it1( inList );
       Edge *edge;
       while (( edge = it1.current() )) {
          ++it1;
          edge->setToPoint( int(x()), int(y()) );
       }

       QPtrListIterator<Edge> it2( outList );
       while (( edge = it2.current() )) {
          ++it2;
          edge->setFromPoint( int(x()), int(y()) );
       }
    }

private:
    QPtrList<Edge> inList;
    QPtrList<Edge> outList;
};

//...................................................... Edge (Konstruktor)
Edge::Edge( Node *from, Node *to, QCanvas *canvas )
                                             : QCanvasLine( canvas ) {
   setPen( Qt::blue );
   from->addOutEdge( this );
   to->addInEdge( this );
   setPoints( int(from->x()), int(from->y()), int(to->x()), int(to->y()) );
}

//......................................................... Klasse MyWidget
class MyWidget : public QCanvasView {

public:
    MyWidget( QCanvas& c, QWidget* parent=0, const char* name=0,
              WFlags f=0) : QCanvasView( &c, parent, name, f) {
       int w = c.width();
       int h = c.height();
       int i, j;

       QArray<Node*> points(20);
       for ( i = 0; i < 20; i++ ) {
           Node *el = new Node( &c );
           el->move( ::rand()%(w-40)+20, ::rand()%(h-40)+20 );
           el->show();
           el->setZ( 100 );
           points[i] = el;
       }
       for ( i=0; i<20; i++ ) {
          for ( j=i+1; j<20; j++ ) {
             Edge *e = new Edge( points[i], points[j], &c );
             e->show();
             e->setZ( 1 );
          }
       }
       resize( sizeHint() );
    }

protected:
    void contentsMousePressEvent(QMouseEvent* e) {
       QCanvasItemList l=canvas()->collisions(e->pos());
       for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {
          moving = *it;
          moving_start = e->pos();
          return;
       }
       moving = 0;
    }

    void contentsMouseMoveEvent(QMouseEvent* e) {
       if ( moving ) {
          moving->moveBy(e->pos().x() - moving_start.x(),
                         e->pos().y() - moving_start.y());
          moving_start = e->pos();
          canvas()->update();
       }
    }

private:
    QCanvasItem* moving;
    QPoint moving_start;
};

int main( int argc, char *argv[] )
{
    QApplication app(argc,argv);

    QCanvas canvas( 800, 600 );

    MyWidget m( canvas );

    m.show();
    qApp->setMainWidget( &m );

    return app.exec();
}

