#include <qapplication.h>
#include <qcolor.h>
#include <qpalette.h>
#include <qlabel.h>
#include <qslider.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlcdnumber.h>

//........................................................ Klasse schiebLCD
class SchiebLCD : public QWidget {
   Q_OBJECT   // notwendig, da eigene Slots enthalten sind

public:
   SchiebLCD( int was, int nr, int wert, QString text, QWidget *parent )
                                                 : QWidget( parent ) {
      typ = was;
      nummer = nr;
         //................ Erzeugen eines Text-Labels
      QLabel* name = new QLabel( this );
      name->setText( text );
      name->setGeometry( 1, 10, 10, 30 );
         //................ Erzeugen eines Schiebebalken
      schiebbalk = new QSlider( 0, 255, // min-/max-Wert
                                5, wert,   // Schrittweite u. Startwert
                       Horizontal, this ); // Ausrichtung u. Elternwidget
      schiebbalk->setGeometry( 20, 10, 180, 30 );
         //................ Erzeugen einer LCD-Anzeige
      lcdzahl = new QLCDNumber( 3, this ); // maximal 3 Ziffern
      lcdzahl->setGeometry( 200, 10, 60, 30 );
      lcdzahl->display( wert );
         //................ Verbinde Schiebebalken und LCD-Anzeige
      QObject::connect( schiebbalk, SIGNAL( valueChanged( int ) ), 
                        this, SLOT( aktualisiereSchiebLCD( int ) ) );
   }
signals:
   void farbeGeaendert( int typ, int nr, int wert );
public slots:
   void aktualisiereSchiebLCD( int wert ) {
      schiebbalk->setValue( wert );
      lcdzahl->display( wert );
      emit farbeGeaendert( typ, nummer, wert ); // Sig. farbeGeaendert schicken
   }
private:
   int         typ, nummer;
   QSlider*    schiebbalk;
   QLCDNumber* lcdzahl;
};
//...................................................... Klasse farbeGruppe
class farbeGruppe : public QWidget {
   Q_OBJECT   // wegen Slots
public:
   farbeGruppe( QWidget *parent ) : QWidget( parent ) { //... Konstruktor
     int  i;
        //................ Initialisierungen
      for ( i=0; i<2; i++) {
         w[i] = new QWidget( this );
         w[i]->setGeometry( 10, i*180, 120, 150 );
      }
      w[1]->setDisabled( true );
      buttonFarbe[0] = palette().active().button().red();
      buttonFarbe[1] = palette().disabled().button().green();
      buttonFarbe[2] = palette().inactive().button().blue();
      backgroundFarbe[0] = palette().active().background().red();
      backgroundFarbe[1] = palette().disabled().background().green();
      backgroundFarbe[2] = palette().inactive().background().blue();
       //............. Erzeugen von zweimal drei Schiebebalken fuer RGB
      QString rgbText[] = { "R", "G", "B" };
      QLabel *l = new QLabel( "ButtonColor", this );
      l->setGeometry( 150, 0, 300, 40 );
      for ( i=0; i<3; i++ ) {
         rgbButton[i] = new SchiebLCD( 1, i, buttonFarbe[i], rgbText[i], this );
         rgbButton[i]->setGeometry( 150, 30+i*50, 300, 60 );
         QObject::connect( rgbButton[i],
                           SIGNAL( farbeGeaendert( int, int, int ) ), 
                           this, SLOT( rgbNeueFarbe( int, int, int ) ) );
      }
      l = new QLabel( "BackgroundColor", this );
      l->setGeometry( 150, 180, 150, 30 );
      for ( i=0; i<3; i++ ) {
         rgbBackground[i] = new SchiebLCD( 2, i, backgroundFarbe[i],
                                           rgbText[i], this );
         rgbBackground[i]->setGeometry( 150, 210+i*50, 300, 60 );
         QObject::connect( rgbBackground[i],
                           SIGNAL( farbeGeaendert( int, int, int ) ), 
                           this, SLOT( rgbNeueFarbe( int, int, int ) ) );
      }
      for (int i=0; i<2; i++) {
         QLabel *l1 = new QLabel( w[i] );
         l1->setGeometry( 10, 0, 100, 30 );
         l1->setText( i==0 ? "active" : "disabled" );
         QLineEdit *l2 = new QLineEdit( w[i] );
         l2->setText( "Highlighted" );
         l2->selectAll();
         l2->setGeometry( 10, 35, 100, 30 );
         QLineEdit *l3 = new QLineEdit( w[i] );
         l3->setText( "Normal" );
         l3->setGeometry( 10, 70, 100, 30 );
         QPushButton *b = new QPushButton( w[i] );
         b->setText( "Button" );
         b->setGeometry( 10, 110, 100, 30 );
      }
      zeigeFarben();  //.................. Anzeigen der Farbgruppen
   }
private slots:
   void rgbNeueFarbe( int typ, int nr, int wert ) {
      if ( typ == 1 )
         buttonFarbe[ nr ] = wert; 
      else
         backgroundFarbe[ nr ] = wert; 
      zeigeFarben();
   }
private:
   QWidget    *w[2];
   int        buttonFarbe[3], backgroundFarbe[3];
   SchiebLCD  *rgbButton[3], *rgbBackground[3];

   void zeigeFarben() {
      QColor neueButtonFarbe( buttonFarbe[0], buttonFarbe[1], buttonFarbe[2] );
      QColor neueBackgroundFarbe( backgroundFarbe[0],
                                  backgroundFarbe[1], backgroundFarbe[2] );
      QPalette neuPalette( neueButtonFarbe, neueBackgroundFarbe );
      setPalette( neuPalette );
   }
};
//................................................... Klasse inactiveWidget
class inactiveWidget : public QWidget {
   Q_OBJECT   // wegen Slots
public:
   inactiveWidget(farbeGruppe *otherWindow, QWidget *parent ) :
                                    QWidget( parent ) { //... Konstruktor
      QLabel *l1 = new QLabel( this );
      l1->setGeometry( 10, 0, 100, 30 );
      l1->setText( "inactive" );
      QLineEdit *l2 = new QLineEdit( this );
      l2->setText( "Highlighted" );
      l2->selectAll();
      l2->setGeometry( 10, 35, 100, 30 );
      QLineEdit *l3 = new QLineEdit( this );
      l3->setText( "Normal" );
      l3->setGeometry( 10, 70, 100, 30 );
      QPushButton *b = new QPushButton( this );
      b->setText( "Button" );
      b->setGeometry( 10, 110, 100, 30 );
      zeigeFarben();  //.................. Anzeigen der Farbgruppen
   }
private:
   int        buttonFarbe[3], backgroundFarbe[3];

   void zeigeFarben() {
      QColor neueButtonFarbe( buttonFarbe[0], buttonFarbe[1], buttonFarbe[2] );
      QColor neueBackgroundFarbe( backgroundFarbe[0],
                                  backgroundFarbe[1], backgroundFarbe[2] );
      QPalette neuPalette( neueButtonFarbe, neueBackgroundFarbe );
      setPalette( neuPalette );
   }
};
#include "colorgroups.moc"
//.................................................................... main
int main( int argc, char* argv[] ) {
   QApplication myapp( argc, argv );
   
   farbeGruppe *mywidget1 = new farbeGruppe( 0 );
   QWidget     *mywidget2 = new QWidget( 0 );
   mywidget1->setGeometry( 100, 100, 450, 400 );
   mywidget2->setGeometry( 700, 100, 300, 300 );
  
   mywidget1->show();
   mywidget2->show();
   myapp.setMainWidget( mywidget1 );
   
   return myapp.exec();
}
