#include <qapplication.h>
#include <qwidget.h>
#include <qvbox.h>
#include <qtranslator.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <qdatetime.h>
#include <qmainwindow.h>
#include <qstatusbar.h>
#include <qstring.h>
 
//......................................................... class MyWidget
class MyWidget : public QMainWindow
{
   Q_OBJECT
public:
    MyWidget( QWidget* parent=0, const char* name=0 )
                                          : QMainWindow( parent, name ) {
        QVBox* central = new QVBox(this);
        setCentralWidget(central);

        setCaption( tr( "Country dependent date" ) ); 
        statusBar()->setPalette( QPalette( Qt::lightGray, Qt::yellow ) );
        statusBar()->message( tr("Country: America") );

        QDate   d;
        d = QDate::currentDate();
        
        QString text( tr( "The date of today is ") );
        QString t;
        if ( tr("german") == "german" )
           t.sprintf( "%d/%d/%d", d.month(), d.day(), d.year() );
        else
           t.sprintf( "%d.%d.%d", d.day(), d.month(), d.year() );
        text += t;
        QLabel *l = new QLabel( central );
        l->setAlignment( QFrame::AlignCenter );
        l->setFont( QFont( "Helvetica", 16, QFont::Bold ) );
        l->setText( text );
    }
};

//............................................................... showLang
MyWidget* showLang( QString lang ) {
    static QTranslator *translator = 0;
    
    if ( lang.isEmpty() ) {
       QMessageBox::warning( 0, "Missing Arguments",
                            "usage: countrydate de  or  countrydate us" );
       return 0;
    }

    lang = "countrydate_" + lang + ".qm";
    QFileInfo fi( lang );
    if ( !fi.exists() ) {
       QString msg( "Cannot find translation for language: " );
       msg += lang + "\n" + "(try 'de' or 'us')";
       QMessageBox::warning( 0, "File error", msg );
       return 0;
    }
    if ( translator ) {
       qApp->removeTranslator( translator );
       delete translator;
    }   
    translator = new QTranslator( 0 );
    translator->load( lang, "." );
    qApp->installTranslator( translator );

    MyWidget *m = new MyWidget;
    return m;
}

#include "countrydate.moc"

//................................................................... main
int main( int argc, char *argv[] )
{
    QApplication app( argc, argv );

    QString lang = "";

    if ( argc >= 2 )
       lang = argv[1];

    MyWidget* m = showLang(lang);
    if ( m ) {
       app.setMainWidget( m );
       m->show();
       m->setPalette( QPalette( Qt::lightGray, Qt::white ) );
       m->setGeometry( 50, 50, 350, 100);
    } else
       return 1;
    app.exec();
}
