#include <qapplication.h>
#include <qlabel.h>
#include <qbitmap.h>
#include <qcursor.h>
#include <qlayout.h>
  // cb_bits und cm_bits wurden mit dem X-Programm bitmap erzeugt
#define cb_width  32
#define cb_height 32
static unsigned char cb_bits[] = {      // cursor bitmap
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x0f, 0x00,
   0x00, 0x06, 0x30, 0x00, 0x80, 0x01, 0xc0, 0x00, 0x40, 0x00, 0x00, 0x01,
   0x20, 0x00, 0x00, 0x02, 0x10, 0x00, 0x00, 0x04, 0x08, 0x3e, 0x3e, 0x08,
   0x08, 0x03, 0xe0, 0x08, 0xc4, 0x00, 0x00, 0x11, 0x04, 0x1e, 0x78, 0x10,
   0x02, 0x0c, 0x30, 0x20, 0x02, 0x40, 0x00, 0x20, 0x02, 0x40, 0x00, 0x20,
   0x02, 0x40, 0x00, 0x20, 0x02, 0x20, 0x04, 0x20, 0x02, 0x20, 0x04, 0x20,
   0x02, 0x10, 0x08, 0x20, 0x02, 0x08, 0x08, 0x20, 0x02, 0xf0, 0x07, 0x20,
   0x04, 0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x10, 0x08, 0x00, 0xc0, 0x08,
   0x08, 0x3c, 0x30, 0x08, 0x10, 0xe6, 0x19, 0x04, 0x20, 0x00, 0x0f, 0x02,
   0x40, 0x00, 0x00, 0x01, 0x80, 0x01, 0xc0, 0x00, 0x00, 0x06, 0x30, 0x00,
   0x00, 0xf8, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00};
#define cm_width  32
#define cm_height 32
static unsigned char cm_bits[] = {      // cursor bitmap mask
   0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x1f, 0x00, 0x00, 0xfe, 0x3f, 0x00,
   0x80, 0x07, 0xf0, 0x00, 0xc0, 0x01, 0xc0, 0x01, 0x60, 0x00, 0x00, 0x03,
   0x30, 0x00, 0x00, 0x06, 0x18, 0x00, 0x00, 0x0c, 0x0c, 0x3e, 0x3e, 0x18,
   0x0e, 0x03, 0xe0, 0x18, 0xc6, 0x00, 0x00, 0x31, 0x07, 0x1e, 0x78, 0x30,
   0x03, 0x0c, 0x30, 0x60, 0x03, 0x40, 0x00, 0x60, 0x03, 0x40, 0x00, 0x60,
   0x03, 0x40, 0x00, 0x60, 0x03, 0x20, 0x04, 0x60, 0x03, 0x20, 0x04, 0x60,
   0x03, 0x10, 0x08, 0x60, 0x03, 0x08, 0x08, 0x60, 0x03, 0xf0, 0x07, 0x60,
   0x06, 0x00, 0x00, 0x30, 0x06, 0x00, 0x00, 0x30, 0x0c, 0x00, 0xc0, 0x18,
   0x0c, 0x3c, 0x30, 0x18, 0x18, 0xe6, 0x19, 0x0c, 0x30, 0x00, 0x0f, 0x06,
   0x60, 0x00, 0x00, 0x03, 0xc0, 0x01, 0xc0, 0x01, 0x80, 0x07, 0xf0, 0x00,
   0x00, 0xfe, 0x3f, 0x00, 0x00, 0xf8, 0x0f, 0x00};
//.............................................................. CursorView
class CursorView : public QWidget {
 public:
    CursorView() {
       static struct {
          CursorShape shape;               
          const char  *name;
       } list[] = {   { Qt::ArrowCursor,        "arrowCursor"        },
                      { Qt::UpArrowCursor,      "upArrowCursor"      },
                      { Qt::CrossCursor,        "crossCursor"        },
                      { Qt::WaitCursor,         "waitCursor"         },
                      { Qt::IbeamCursor,        "ibeamCursor"        },
                      { Qt::SizeVerCursor,      "sizeVerCursor"      },
                      { Qt::SizeHorCursor,      "sizeHorCursor"      },
                      { Qt::SizeBDiagCursor,    "sizeBDiagCursor"    },
                      { Qt::SizeFDiagCursor,    "sizeFDiagCursor"    },
                      { Qt::SizeAllCursor,      "sizeAllCursor"      },
                      { Qt::BlankCursor,        "blankCursor"        },
                      { Qt::SplitVCursor,       "splitVCursor"       },
                      { Qt::SplitHCursor,       "splitHCursor"       },
                      { Qt::PointingHandCursor, "pointingHandCursor" },
                      { Qt::ForbiddenCursor,    "ForbiddenCursor"    },
                      { Qt::WhatsThisCursor,    "WhatsThisCursor"    } };
       int listSize = sizeof( list ) / sizeof( *list );

       static struct {
          const char *fileName;
          const char *text;
       } list2[] = {  { "hi.xpm",       "Hi-Custom cursor"       },
                      { "chess.xpm"  ,  "Chess-Custom cursor"    },
                      { "penguin.xpm",  "Pinguin-Custom cursor"  },
                      { "snake.xpm",    "Schlange-Custom cursor" },
                      { "suse.xpm",     "SuSE-Custom cursor"     },
                      { "gv.xpm",       "Ghost-Custom cursor"    },
                      { "monalisa.xpm", "Monalisa-Custom cursor" },
                      { "lausbub.xpm",  "Lausbub-Custom cursor"  } };
       setCaption( "CursorFormen" );
       QGridLayout* grid = new QGridLayout( this, 6, 5, 20 );
       QLabel *label;
       int i=0;
       for ( int y=0; y<4; y++ ) {
          for ( int x=0; x<6; x++ ) {
             label = new QLabel( this );
             label->setAlignment( AlignCenter );
             label->setFrameStyle( QFrame::Box | QFrame::Raised );
             label->setFont( QFont( "Times", 12 ) );
             if ( i < listSize ) {
                label->setCursor( QCursor( list[i].shape ) );
                label->setText( list[i].name );
             } else {
                QPixmap  cb( list2[i-listSize].fileName );
                QCursor custom( cb );
                label->setCursor( custom );
                label->setPaletteBackgroundColor( Qt::blue.light( 180 ) );
                label->setText( list2[i-listSize].text );
             }
             grid->addWidget( label, x, y );
             i++;
          }
       }
       QBitmap cb( cb_width, cb_height, cb_bits, true );
        QBitmap cm( cm_width, cm_height, cm_bits, true );
        QCursor custom( cb, cm );
       label = new QLabel( this );
        label->setCursor( custom );
        label->setPaletteBackgroundColor( Qt::blue.light( 180 ) );
       label->setText( "Transparent Big Smiley (custom)" );
        label->setAlignment( AlignCenter );
        label->setFrameStyle( QFrame::Box | QFrame::Sunken );
       grid->addMultiCellWidget( label, 6, 6, 0, 3 );
    }
};
//.................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    CursorView   v;
    a.setMainWidget( &v );
    v.show();
    return a.exec();
}
