#include <qapplication.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qcolor.h>

QColor colors[] = {
          Qt::black,      Qt::white,    Qt::darkGray, Qt::gray,
          Qt::lightGray,  Qt::red,      Qt::green,    Qt::blue,
          Qt::cyan,       Qt::magenta,  Qt::yellow,   Qt::darkRed,
          Qt::darkGreen,  Qt::darkBlue, Qt::darkCyan, Qt::darkMagenta  };
const int colorNo = sizeof( colors ) / sizeof( colors[0] );

//................................................ Klasse ChangeEvent
class ChangeEvent : public QCustomEvent
{
public:
    ChangeEvent( int nr, QColor color ) : QCustomEvent(nr), farbe(color) { }
    QColor getColor() const { return farbe; }

private:
    QColor farbe;
};

//.................................................. Klasse ColorLabel
class ColorLabel : public QLabel
{
    Q_OBJECT
public:
    ColorLabel( int id, QWidget* p=0 ) : QLabel(p), nr(id) { }

protected:
    virtual void mousePressEvent( QMouseEvent * ) {
       ChangeEvent *event = new ChangeEvent( 20000+nr, colors[nr] );
       QApplication::postEvent( parentWidget(), event );
    }

private:
    int  nr;
};

//.................................................... Klasse MyWidget
class MyWidget : public QWidget
{
    Q_OBJECT
public:
    MyWidget( QWidget* p=0, const char* name=0 ): QWidget(p, name) {
       for ( int c=0; c<colorNo; c++ ) {
          l[c] = new ColorLabel( c, this );
          l[c]->setPaletteBackgroundColor( colors[c] );
          l[c]->setFrameStyle( QFrame::Box | QFrame::Raised );
          l[c]->setGeometry( width()-50, c*20, 50, 20 );
       }
    }

protected:
    virtual void resizeEvent( QResizeEvent * ) {
       for ( int c=0; c<colorNo; c++ )
          l[c]->setGeometry( width()-50, c*20, 50, 20 );
    }

protected:
    virtual void customEvent( QCustomEvent *ev ) {
       setPaletteBackgroundColor( ((ChangeEvent *)ev)->getColor() ); 
    }

private:
    ColorLabel *l[colorNo];
};

#include "customevent3.moc"

//............................................................... main
int main( int argc, char *argv[] )
{
    QApplication a(argc,argv);

    MyWidget w;
    a.setMainWidget( &w );
    w.resize( 450, 350 );
    w.show();
    return a.exec();
}
