#include <qapplication.h>
#include <qpainter.h>
#include <qmenubar.h>
#include <qlabel.h>
#include <qcursor.h>
#include <qpopupmenu.h>
//................................................................ EigenEintrag
class EigenEintrag : public QCustomMenuItem
{
public:
   EigenEintrag(const QString& s, const QFont& f) : string(s), font(f) {}

   void paint( QPainter* p, const QColorGroup& /*cg*/, bool /*act*/,
               bool /*enabled*/, int x, int y, int w, int h ) {
      p->setFont ( font );
      p->drawText( x, y, w, h,
                   AlignLeft | AlignVCenter | DontClip | ShowPrefix, string );
   }
   QSize sizeHint() {
      return QFontMetrics( font ).size( AlignLeft | AlignVCenter |
                                        ShowPrefix | DontClip,  string );
   }
private:
    QString string;
    QFont   font;
};
//.................................................................. CustomMenu
class CustomMenu : public QWidget {
    Q_OBJECT
public:
    CustomMenu( QWidget *parent=0, const char *name=0 );
public slots:
    void normal() { anzeige(false, false, QString("Normal"));                   }
    void fett()   { anzeige(!f.bold(), f.underline(), QString("Fett"));         }
    void unter()  { anzeige(f.bold(), !f.underline(), QString("Unterstrichen"));}
signals:
    void zeigeWahl( const QString& );
private:
    void anzeige(bool fett, bool unterstrich, QString text) {
       f.setBold( fett );
       f.setUnderline( unterstrich );
       label->setFont( f );
       popup->setItemChecked( 200, fett );
       popup->setItemChecked( 201, unterstrich );
       emit zeigeWahl( "Format / " + text );
    }
    virtual void contextMenuEvent( QContextMenuEvent * ) {
       QPopupMenu* contextMenu = new QPopupMenu( this );
       QLabel *caption = new QLabel( "<font color=darkblue><u><b>"
                                     "Kontextmenu</b></u></font>", this );
       caption->setAlignment( Qt::AlignCenter );
       contextMenu->insertItem( caption );
       contextMenu->insertItem( "Normal",  this, SLOT(normal()) );
       contextMenu->insertSeparator();
       contextMenu->insertItem( new EigenEintrag( "Fett", fBold ), 200 );
       contextMenu->connectItem( 200, this, SLOT(fett()) );
       contextMenu->insertItem( new EigenEintrag("Unterstrichen", fUnder ), 201);
       contextMenu->connectItem( 201, this, SLOT(unter()) );
       contextMenu->setItemChecked( 200, f.bold() );
       contextMenu->setItemChecked( 201, f.underline() );
       contextMenu->exec( QCursor::pos() );
       delete contextMenu;
    }
    QPopupMenu *popup;
    QLabel     *label;
    QFont      f, fBold, fUnder;
};
//...................................................... CustomMenu-Konstruktor
CustomMenu::CustomMenu( QWidget *parent, const char *name )
                                          : QWidget( parent, name ) {
    fBold = font();  fBold.setBold( true );
    fUnder = font(); fUnder.setUnderline( true );
    popup = new QPopupMenu( this );
     popup->insertItem( "Normal", this, SLOT(normal()) );
     popup->insertSeparator();
     popup->insertItem( new EigenEintrag( "Fett", fBold ), 200 );
     popup->connectItem( 200, this, SLOT( fett()) );
     popup->insertItem( new EigenEintrag("Unterstrichen", fUnder ), 201);
     popup->connectItem( 201, this, SLOT(unter()) );
    QMenuBar *menu = new QMenuBar( this );
     menu->insertItem( "Format", popup );
    label = new QLabel( this );
     label->setGeometry( 20, rect().center().y()-20, width()-40, 40 );
     label->setPaletteBackgroundColor( Qt::white );
     label->setLineWidth( 1 );
     label->setAlignment( AlignCenter );
    connect( this,  SIGNAL(zeigeWahl(const QString&)),
             label, SLOT(setText(const QString&)) );
}
#include "custommenu.moc"
//........................................................................ main
int main( int argc, char **argv ) {
    QApplication a( argc, argv );
    CustomMenu m;
    m.setCaption("Beispiel zu QCustomMenuItem und contextMenuEvent()");
    a.setMainWidget( &m );
    m.show();
    return a.exec();
}
