#include <qapplication.h>
#include <qlabel.h>
#include <qdatetimeedit.h>
#include <qlcdnumber.h>
//.......................................................... zeitDatumEingabe
class zeitDatumEingabe : public QWidget {
    Q_OBJECT
public:
    zeitDatumEingabe( QWidget* parent = 0, const char* name = 0 );
    ~zeitDatumEingabe() {}

    QDateEdit     *datumEdit;
    QTimeEdit     *zeitEdit;
    QDateTimeEdit *datumZeitEdit;
    QLCDNumber    *anzeige[2]; // [0] = Datum, [1] = Zeit

private slots:
    void datumGeaendert(const QDate &datum) { 
       anzeige[0]->display( datum.toString( "dd.MM.yyyy" ) );
       if (datumZeitEdit->dateEdit()->date() != datum)
          datumZeitEdit->dateEdit()->setDate( datum);
    }
    void zeitGeaendert(const QTime &zeit) { 
       anzeige[1]->display( zeit.toString( QString("hh:mm:ss") ) );
       if (datumZeitEdit->timeEdit()->time() != zeit)
          datumZeitEdit->timeEdit()->setTime( zeit);
    }
    void datumZeitGeaendert(const QDateTime &datumZeit) { 
       anzeige[0]->display( datumZeit.date().toString( "dd.MM.yyyy" ) );
       anzeige[1]->display( datumZeit.time().toString( "hh:mm:ss" ) );
       if (zeitEdit->time() != datumZeit.time() )
           zeitEdit->setTime( datumZeit.time() );
       if (datumEdit->date() != datumZeit.date() )
           datumEdit->setDate( datumZeit.date() );
    }
};
//.............................................. zeitDatumEingabe-Konstruktor
zeitDatumEingabe::zeitDatumEingabe( QWidget* parent, const char* name )
                                                      : QWidget( parent, name ) {
    datumEdit = new QDateEdit( this );
    datumEdit->setDate( QDate::currentDate() );
    datumEdit->setAutoAdvance( true );
    datumEdit->setOrder( QDateEdit::DMY );
    datumEdit->setSeparator( "/" );
    datumEdit->setGeometry( 10, 10, 150, 40 );

    zeitEdit = new QTimeEdit( this );
    zeitEdit->setTime( QTime::currentTime() );
    zeitEdit->setAutoAdvance( true );
    zeitEdit->setDisplay( int( QTimeEdit::Seconds |
                               QTimeEdit::Minutes | QTimeEdit::Hours ) );
    zeitEdit->setSeparator( "-" );
    zeitEdit->setGeometry( 10, 60, 150, 40 );

    datumZeitEdit = new QDateTimeEdit( this );
    datumZeitEdit->setDateTime( QDateTime::currentDateTime() );
    datumZeitEdit->setGeometry( 10, 140, 300, 40 );

    for (int i=0; i<2; i++) {
       anzeige[i] = new QLCDNumber( this );
       anzeige[i]->setGeometry( 240, 10+i*50, 190, 40 );
       anzeige[i]->setPaletteBackgroundColor( Qt::yellow );
       anzeige[i]->setFont( QFont( "Times", 20)  );
       anzeige[i]->setSegmentStyle( QLCDNumber::Filled );
       anzeige[i]->setNumDigits( 10 );
    }
    connect( zeitEdit, SIGNAL( valueChanged(const QTime&) ),
             this,     SLOT( zeitGeaendert(const QTime&) ) );
    connect( datumEdit, SIGNAL( valueChanged(const QDate&) ),
             this,      SLOT( datumGeaendert(const QDate&) ) );
    connect( datumZeitEdit, SIGNAL( valueChanged(const QDateTime&) ),
             this,          SLOT( datumZeitGeaendert(const QDateTime&) ) );
    datumZeitGeaendert( QDateTime::currentDateTime() );
}
#include "datetimeedit.moc"
//...................................................................... main
int main( int argc, char *argv[] ) {
   QApplication  myapp( argc, argv );
   zeitDatumEingabe* mywidget = new zeitDatumEingabe();
   mywidget->setGeometry( 20, 20, 480, 200 );
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
