#include <qapplication.h>
#include <qpushbutton.h>
#include <qdial.h>
#include <qlcdnumber.h>
#include <qlabel.h>
#include <qspinbox.h>
//................................................................. DialSpinLCD
class DialSpinLCD : public QWidget {
   Q_OBJECT
public:
   DialSpinLCD( QWidget *parent, QString text ) : QWidget( parent, text ) {
        //... Text-Label
      name = new QLabel( this );
      name->setText( farbName = text );
      name->setGeometry( 1, 10, 15, 30 );
        //... Drehknopf
      drehKnopf = new QDial( 0, 255, 5, 120, this );
      drehKnopf->setWrapping( true );
      drehKnopf->setNotchesVisible( true );
      drehKnopf->setGeometry( 20, 0, 100, 100 );
        //... LCD-Anzeige
      lcdzahl = new QLCDNumber( 3, this );
      lcdzahl->setSegmentStyle( QLCDNumber::Filled );
      lcdzahl->setPaletteBackgroundColor( Qt::darkGray );
      lcdzahl->setGeometry( 130, 40, 60, 30 );
      lcdzahl->display( 120 );
        //... Spinboxen
      spinbox = new QSpinBox( -1, 255, 1, this );
      spinbox->setValue( 120 );  // Startwert
      spinbox->setSpecialValueText( "Default" );  // ist: 120
      spinbox->setWrapping( true );
      spinbox->setGeometry( 200, 40, 80, 30 );
        //...Verbinde Drehknopf und Spinbox mit LCD-Anzeige
      QObject::connect( drehKnopf, SIGNAL( valueChanged( int ) ), 
                        this,      SLOT( aktualisiereDialSpinLCD( int ) ) );
      QObject::connect( spinbox, SIGNAL( valueChanged( int ) ), 
                        this,    SLOT( aktualisiereDialSpinLCD( int ) ) );
      aktualisiereDialSpinLCD( 120 );
   }
signals:
   void farbeGeaendert( int farbwert );
private slots:
   void aktualisiereDialSpinLCD( int wert ) {
      int v = wert==-1 ? 120 : wert;
      QColor f;
      if ( farbName == "R" )      f = QColor( v, 0, 0 );
      else if ( farbName == "G" ) f = QColor( 0, v, 0 );
      else if ( farbName == "B" ) f = QColor( 0, 0, v );
      drehKnopf->setPalette( QPalette( Qt::lightGray, f ) );
      spinbox->setValue( wert );
      drehKnopf->setValue( v );
      lcdzahl->display( v );
      emit farbeGeaendert( v );
   }
private:
   QLabel*     name;
   QDial*      drehKnopf;
   QLCDNumber* lcdzahl;
   QSpinBox*   spinbox;
   QString     farbName;
};
//................................................................... FarbLabel
class FarbLabel : public QLabel {
   Q_OBJECT
public:
   FarbLabel( QWidget *parent ) : QLabel( parent ) {
      setPaletteBackgroundColor( QColor( rot=120, gruen=120, blau=120 ) );
   }
public slots:
   void rotNeueFarbe( int neuefarbe ) {
      setPaletteBackgroundColor( QColor( rot=neuefarbe, gruen, blau ) );
   }
   void gruenNeueFarbe( int neuefarbe ) {
      setPaletteBackgroundColor( QColor( rot, gruen=neuefarbe, blau ) );
   }
   void blauNeueFarbe( int neuefarbe ) {
      setPaletteBackgroundColor( QColor( rot, gruen, blau=neuefarbe ) );
   }
private:
   int rot, gruen, blau;
};
#include "dial.moc"
//........................................................................ main
int main( int argc, char* argv[] ) {
   QApplication myapp( argc, argv );
   QWidget* mywidget = new QWidget();
   mywidget->setGeometry( 100, 100, 440, 400 );
     //................ Erzeugen von drei DialSpinLCD-Balken
   DialSpinLCD* rotBalken = new DialSpinLCD( mywidget, "R" );
   rotBalken->setGeometry( 10, 30, 300, 100 );
   DialSpinLCD* gruenBalken = new DialSpinLCD( mywidget, "G" );
   gruenBalken->setGeometry( 10, 150, 300, 100 );
   DialSpinLCD* blauBalken = new DialSpinLCD( mywidget, "B" );
   blauBalken->setGeometry( 10, 270, 300, 100 );
     //................ Erzeugen einer Farbanzeige
   FarbLabel* farbAnzeige = new FarbLabel( mywidget );
   farbAnzeige->setAlignment( FarbLabel::AlignCenter );
   farbAnzeige->setText( "Farbe" );
   farbAnzeige->setGeometry( 320, 30, 100, 350 );
      //................ Verbinde RGB-Balken mit Farbanzeige
   QObject::connect( rotBalken, SIGNAL( farbeGeaendert( int ) ), 
                     farbAnzeige, SLOT( rotNeueFarbe( int ) ) );
   QObject::connect( gruenBalken, SIGNAL( farbeGeaendert( int ) ), 
                     farbAnzeige, SLOT( gruenNeueFarbe( int ) ) );
   QObject::connect( blauBalken, SIGNAL( farbeGeaendert( int ) ), 
                     farbAnzeige, SLOT( blauNeueFarbe( int ) ) );
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
