#include <qapplication.h>
#include <qmessagebox.h>
#include <qcheckbox.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <stdlib.h>

//................................................................. MyDialog
class MyDialog : public QDialog
{
public:
    MyDialog( QWidget *parent=0, const char *name=0, bool modal=false )
                                       : QDialog( parent, name ) {
       setModal( modal );
       QVBoxLayout* vb = new QVBoxLayout( this, 30 );
       vb->setAutoAdd( true );
       hb = 0;
    }

    void addButtons( const QString& cancel=QString::null,
                     const QString& ok=QString::null,
                     const QString& mid1=QString::null,
                     const QString& mid2=QString::null,
                     const QString& mid3=QString::null) {
       addButton( ok.isNull() ? (QString)"OK" : ok, 1 );
       if ( !mid1.isNull() ) addButton( mid1, 2 );
       if ( !mid2.isNull() ) addButton( mid2, 3 );
       if ( !mid3.isNull() ) addButton( mid3, 4 );
       addButton( cancel.isNull() ? (QString)"Cancel" : cancel, 0 );
    }

private:
    void addButton( const QString& text, int result ) {
       if ( hb == 0 ) {
          hb = new QButtonGroup( 5, Qt::Horizontal, this  );
          QObject::connect( hb, SIGNAL( clicked(int) ),
                            this, SLOT( done(int) ) );
       }
       QPushButton *c = new QPushButton( text, hb );
       hb->remove( c );
       hb->insert( c, result );
    }

    QButtonGroup  *hb;
};

//........................................................... Klasse ShowFlag
class ShowFlag : public QLabel
{
    Q_OBJECT
public:
    ShowFlag( QString landName, QWidget *p = 0 ) : QLabel( p ) {
       QPixmap bild;
       QString fileName( landName );
       fileName += ".xpm";
 
       if ( !bild.load( fileName ) ) {
          QString text;
          text.sprintf( "Kann Datei %s nicht ffnen", (const char *)fileName );
          QMessageBox::warning( 0, "Ladefehler", text );
          emit closed();
       }
       setAlignment( Qt::AlignCenter );
       setPixmap( bild );
       setCaption( landName );
       resize( 200, 100 );

       QObject::connect( this, SIGNAL( closed() ), qApp, SLOT( quit() ) );
    }

signals:
    void closed();
 
protected:
    virtual void closeEvent( QCloseEvent* e ) {
       QWidget::closeEvent(e);
       emit closed();
    } 
};

#include "dialog2.moc"

//..................................................................... main
int main( int argc, char *argv[] ) {
    QApplication app( argc, argv );

    const char* name[]= { "france", "britain", "germany",
                          "italy",  "spain",   "USA",     0 };

    MyDialog dlg( 0, 0, true );
    QCheckBox* qmb[sizeof(name)/sizeof(name[0])];

    QButtonGroup *bg = new QButtonGroup( 2, Qt::Vertical,
                                         "Whle Flaggen aus", &dlg);
    for ( int i=0; name[i]; i++ )
       qmb[i] = new QCheckBox((const char*)name[i], bg);

    dlg.addButtons( "Cancel", "OK", "Europische", "Alle" );
    int  z=0,
         r = dlg.exec();

    if ( r ) {
        for ( int i=0; name[i]; i++ ) {
           if ( ( r == 2 && name[i] != "USA" ) || r == 3 ||
                qmb[i]->isChecked() ) {
               ShowFlag* w = new ShowFlag((const char*)name[i]);
               w->show();
               z++;
           }
        }
        if ( z == 0 )
           exit(0);
    } else
        exit(0);
    return app.exec();
}
