#include  <qdir.h>
#include  <qfile.h>
#include  <qfileinfo.h>
#include  <qapplication.h> 
#include  "dirbrows.h"

//................................................ Konstruktoren .......
Directory::Directory(QListView *par) : QListViewItem(par), f("/"), p(0) { }

Directory::Directory(Directory *par, const char *filename)
                              : QListViewItem(par), f(filename), p(par) { }
//......................................................... text .......
       //  Diese virtuelle Methode liefert immer den Text, der in
       //  Spalte 'c' angezeigt wird. Durch die Reimplementierung
       //  hier erreicht man die entspr. Anzeige fuer Directories
QString Directory::text( int c ) const {
   switch ( c ) {
      case 0:  return f.name();
      case 1:  return readable ? "Dir." : "Unreadable Dir.";
      default: return "";
   }
}              
//..................................................... fullName .......
QString Directory::fullName() {
    QString s;
    if ( p ) {
        s = p->fullName();
        s.append( f.name() );
        s.append( "/" );
    } else
        s = "/";
    return s;
}
//...................................................... setOpen .......
void Directory::setOpen( bool o ) {
    if ( o && !childCount() ) {   // nur ausfuehren, wenn Eintrag das erste
                                  // Mal geoeffnet wird (childCount()==0)
        QString s( fullName() ); //... vollstaendiger Pfadname 
        QDir thisDir( s );
        setText( 0, f.name() );
        if ( !readable )
            return;  //.... wenn Directory nicht lesbar, sofort zurueckkehren
           //.... Alle Dateien des Directories durchlaufen und dafuer
           //.... entsprechende Listeneintraege erzeugen
        const QFileInfoList * files = thisDir.entryInfoList();
        if ( files ) {
            QFileInfoListIterator it( *files );
            QFileInfo * f;
            while( ( f=it.current()) != 0 ) {
                ++it;
                sprintf(size, "%d", f->size());
                strcpy(time, f->lastModified().toString());
                sprintf(perm, "%s%s%s", f->isReadable() ? "r" : "-",
                                        f->isWritable() ? "w" : "-",
                                        f->isExecutable() ? "x" : "-");
                if ( f->fileName() == "." || f->fileName() == ".." )
                   ; // nichts zu tun
                else if ( f->isSymLink() )
                   new QListViewItem( this, (const char *)f->fileName(),
                                 "Symbolic Link", (const char *)f->readLink(),
                                 size, time, perm, 0 );
                else if ( f->isDir() )
                   new Directory( this, f->fileName() );
                else
                   new QListViewItem( this, (const char *)f->fileName(),
                                 f->isFile() ? "File" : "Special", "",
                                 size, time, perm, 0 );
            }
        }
    }
    QListViewItem::setOpen( o );
}
//........................................................ setup .......
void Directory::setup()   //... wird aufgerufen, bevor Listeneintrag
{                         //... das erste Mal angezeigt wird.
   QString s( fullName() ); //... vollstaendiger Pfadname 
   QFileInfo fi( s );
   setExpandable( readable = fi.isReadable() );
   QListViewItem::setup();
}
