#include <qapplication.h>
#include <qpushbutton.h>
#include <qtextview.h>
#include <qmessagebox.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qfile.h>
#include <qdom.h>
#include <qstringlist.h>
#include <iostream>
using namespace std;

//.............................................................. MeinRichText
class MeinRichText : public QVBox {
    Q_OBJECT
public:
    MeinRichText( const QStringList &s, QWidget *p=0, const char *n=0 )
                                                         : QVBox( p, n ) {
       texte = s;
       setMargin( 5 );

       QBrush paper;
       paper.setPixmap( QPixmap( "marble.xpm" ) );

       anzeige = new QTextView( this );
       anzeige->setPaper( paper );
       anzeige->setFont( QFont( "Times", 12 ) );

       anzeige->setText( texte[0] );
       anzeige->setMinimumSize( 450, 250 );

       QHBox *buttons = new QHBox( this );
       buttons->setMargin( 5 );

       bBeenden = new QPushButton( "&Beenden", this );
       QWidget *w = new QWidget( this );
       w->setMaximumHeight( bBeenden->sizeHint().height() );

       bZurueck = new QPushButton( "<< &Zurueck", buttons );
       bWeiter  = new QPushButton( "&Weiter >>", buttons );

       buttons->setMaximumHeight( bWeiter->sizeHint().height() + 10 );
       bZurueck->setEnabled( false );

       connect( bBeenden, SIGNAL( clicked() ), qApp, SLOT( quit() ) );
       connect( bZurueck, SIGNAL( clicked() ), this, SLOT( zurueck() ) );
       connect( bWeiter, SIGNAL( clicked() ), this, SLOT( weiter() ) );

       nr = 0;
   }

protected:
    QTextView   *anzeige;
    QPushButton *bBeenden, *bWeiter, *bZurueck;
    QStringList texte;
    int nr;

protected slots:
    void zurueck() {
       if ( nr <= 0 )
          return;
       nr--;
       anzeige->setText( texte[nr] );
       if ( nr == 0 )
          bZurueck->setEnabled( false );
       bWeiter->setEnabled( true );
    }

    void weiter() {
       if ( nr >= (int)texte.count()-1 )
          return;
       nr++;
       anzeige->setText( texte[nr] );
       if ( nr >= (int)texte.count()-1 )
          bWeiter->setEnabled( false );
       bZurueck->setEnabled( true );
    }
};

//............................................................. QuoteHandler
class QuoteHandler : public QDomDocument
{
public:
    QuoteHandler() { }
    ~QuoteHandler() { }

    QStringList quotes()    { return quoteList; }

    void parse( QDomElement docElem ) {
       quoteList.clear();
       state = StateInit;
       parseTree( docElem );
    }

    void parseTree( QDomElement docElem ) {
       QDomNode n = docElem;
       while( !n.isNull() ) {
          QDomElement e = n.toElement(); // konvertiere Knoten in ein Element
          if( !e.isNull() ) {
              startElement( e );
              if ( !e.firstChild().isNull() )
                 parseTree( e.firstChild().toElement() );
              endElement();
          }
          n = n.nextSibling();
       }
    }

    void startElement( QDomElement e ) {
       QString nodeName = e.nodeName();
       if ( nodeName == "quotations" && state == StateInit ) {
          state = StateDocument;
       } else if ( nodeName == "quote" && state == StateDocument ) {
          state = StateQuote;
          QString tmp;
          quoteList.append( tmp );
          author    = e.attributeNode( "author" ).value();
          reference = e.attributeNode( "reference" ).value();
          mytitle   = e.attributeNode( "mytitle" ).value();
          quoteList.last() += "<b>";
          quoteList.last() += QString::number( quoteList.count() ) + ". ";
          quoteList.last() += (mytitle.isEmpty()) ? QString("") : mytitle;
          quoteList.last() += "</b><br><hr><br><br><big>";
       } else if ( nodeName == "line" && state == StateQuote ) {
          state = StateLine;
          quoteList.last() += e.text();
       } else if ( nodeName == "heading" && state == StateQuote ) {
          state = StateHeading;
          quoteList.last() += "<b>";
          quoteList.last() += e.text();
       } else if ( nodeName == "p" && state == StateQuote ) {
          state = StateP;
          quoteList.last() += "<p>";
          quoteList.last() += e.text();
       }
    }

    void endElement( void ) {
       switch ( state ) {
          case StateQuote:
             state = StateDocument;
             quoteList.last() += "</big><br><br><center><i>";
             if ( author.isEmpty() )
                quoteList.last() += QString("");
             else
                quoteList.last() += "-- " + author;
             if ( !reference.isEmpty() )
                quoteList.last() += ", \"" + reference + "\"";
             quoteList.last() += "</i></center><i>";
             break;
          case StateLine:
             state = StateQuote; quoteList.last() += "<br>";     break;
          case StateHeading:
             state = StateQuote; quoteList.last() += "</b><br>"; break;
          case StateP:
             state = StateQuote; quoteList.last() += "</p>";     break;
          default: break;
       }
    }

private:
    QStringList quoteList;
    QString     author, reference, mytitle;

    enum State { StateInit, StateDocument, StateQuote,
                 StateLine, StateHeading,  StateP };
    State state;
};

#include "domrichtext.moc"

//...................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );

    QuoteHandler handler;

    QFile file( "sayings.xml" );
    if ( !file.open( IO_ReadOnly ) )
       return -1;
    if ( !handler.setContent( &file ) ) {
        file.close();
        return -1;
    }
    file.close();

    handler.parse( handler.documentElement() );

    MeinRichText richtext( handler.quotes() );
    richtext.resize( 450, 350 );
    richtext.setCaption( "Kluge Sprueche und Anekdoten" );
    a.setMainWidget( &richtext );
    richtext.show();
    return a.exec();
}
