#include <qapplication.h>
#include <qnetwork.h>
#include <qvbox.h>
#include <qstring.h>
#include <qpushbutton.h>
#include <qtextedit.h>
#include <qfiledialog.h>
#include <qurloperator.h>
#include <qprogressdialog.h>                                                                                       
//............................................................... class View
class View : public QVBox {
    Q_OBJECT
public:
    View() : QVBox() {
       setSpacing( 5 );
       setMargin( 5 );

       QPushButton *b = new QPushButton( "Download a file...", this );
       connect( b,    SIGNAL( clicked() ),
                this, SLOT( downloadFile() ) );
       b->setFont( QFont( "Times", 24 ) );

       logInfo = new QTextEdit( this );
       logInfo->setReadOnly( true );
       first = true;

       connect( &op,
           SIGNAL( dataTransferProgress( int, int, QNetworkOperation * ) ),
           this, SLOT( downloadProgress( int, int, QNetworkOperation * ) ) );

       connect( &op,  SIGNAL( finished( QNetworkOperation * ) ),
                this, SLOT( done( QNetworkOperation * ) ) );
    }
    
private slots:
    void downloadFile() {
       from = getOpenFileName();
       if ( !from.isEmpty() ) {
           to = QFileDialog::getExistingDirectory( "." );
           if ( !to.isEmpty() )
              op.copy( from, to );
       }
    }
    void downloadProgress( int bytesDone, int bytesTotal,
                           QNetworkOperation *o ) {
       if ( o->operation() == QNetworkProtocol::OpGet ) {
          if ( first ) {
             first = false;
             QString t;
             t.sprintf( "Download of file `%s", (const char *)from );
             pd = new QProgressDialog( t, "Cancel", bytesTotal, 0, 0, true );
             pd->setCaption( t );
             logInfo->append( t );
          }
          if ( pd->wasCancelled() ) {
             op.stop();
             delete pd;
          } else
             pd->setProgress( bytesDone );
       }
    }

    void done( QNetworkOperation *op ) {
       first = true;
       if ( op->state() != QNetworkProtocol::StInProgress )
          if ( op->state() == QNetworkProtocol::StDone )
             logInfo->append( " ....successful\n" );
          else
             logInfo->append( " ....not successful\n" );
    }

private:
    QTextEdit      *logInfo;
    QUrlOperator    op;
    QString         from, to;
    bool            first;
    QProgressDialog *pd;
 
    QString getOpenFileName() {
       static QString workingDirectory = "ftp://ftp.suse.com/";
       QFileDialog *dlg = new QFileDialog( workingDirectory,
                                           QString::null, 0, 0, true );
       dlg->setCaption( "Open" );
       dlg->setMode( QFileDialog::ExistingFile );
       QString result;
       if ( dlg->exec() == QDialog::Accepted ) {
          result = dlg->selectedFile();
          workingDirectory = dlg->url();
       }
       delete dlg;
       return result;
    }
};

#include "download.moc"

//..................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    qInitNetworkProtocols(); //... Initialisieren der implementierten
                             //... Netzwerkprotokolle
    View v;
    v.resize( 600, 400 );
    v.show();
    a.setMainWidget( &v );
    
    return a.exec();
}
