#include <qapplication.h>
#include <qevent.h>
#include <qpixmap.h>
#include <qdragobject.h>
#include <qimage.h>
#include <qlabel.h>
#include <qlayout.h>
#include <stdlib.h>

//---------------------------- Eigener MIME-Typ geheim/mein --------------
//------------------------------------------------------------------------

//...................................................... Klasse GeheimDrag
class GeheimDrag: public QStoredDrag
{
public:
    GeheimDrag( int geheim, QWidget * parent=0, const char * name=0 )
               : QStoredDrag( "geheim/mein", parent, name ) {
       QByteArray data(1);
       data[0]= geheim;
       setEncodedData( data );
    }

    ~GeheimDrag() {}

    static bool canDecode( QDragMoveEvent* e ) {
       return e->provides( "geheim/mein" );
    }

    static bool decode( QDropEvent* e, QString& str ) {
       QByteArray dekodiertInString = e->encodedData( "geheim/mein" );
       if ( dekodiertInString.size() ) {
          e->accept();
          QString msg;
          msg.sprintf("Die Geheimnummer ist: %d", dekodiertInString[0] );
          str = msg;
          return true;
       }
       return false;
    }
};

//...................................................... Klasse GeheimQuelle
class GeheimQuelle: public QLabel
{
public:
    GeheimQuelle( QWidget *parent=0, const char *name=0 )
                                    : QLabel( "Geheim", parent, name ) {
       if ( !smiley.load( "smiley.xpm" ) ) {
          smiley = QPixmap( 20, 20 );
          smiley.fill( Qt::green );
       }
       setPaletteBackgroundColor( Qt::green );
       setFrameStyle( Box | Sunken );
       setMinimumHeight( sizeHint().height()*2 );
       setAlignment( AlignCenter );
       geheimZahl = ::rand()%100;
    }
    ~GeheimQuelle() { }

protected:
    void mousePressEvent( QMouseEvent * ) {
       GeheimDrag *gd = new GeheimDrag( geheimZahl++, this );
       gd->setPixmap( smiley, QPoint(8,8) );
       gd->dragCopy();
    }

private:
    QPixmap  smiley;
    int      geheimZahl;
};

//---------------------------- Klasse DropStelle -------------------------
//------------------------------------------------------------------------

class DropStelle: public QLabel
{
    Q_OBJECT
public:
    DropStelle( QWidget *parent = 0, const char *name=0 )
                                  : QLabel( parent, name ) {
       setAcceptDrops( true );
    }
    ~DropStelle() { }

signals:
    void mimeTypen( const QString& );

protected:
    void dragEnterEvent( QDragEnterEvent *e ) {
       if ( e->source() == this )
          return;
       if ( GeheimDrag::canDecode( e ) ||
            QTextDrag::canDecode( e )  || QImageDrag::canDecode( e ) )
           e->accept();
         //... Ausgabe aller akzeptierten MIME-Typen
       QString t;
       for( int i=0; e->format( i ); i++ ) {
          if ( *(e->format( i )) ) {
             t += e->format( i );
             t += "\n";
          }
       }
       emit mimeTypen( t );
       setPaletteBackgroundColor( Qt::white );
    }

    void dragLeaveEvent( QDragLeaveEvent * ) {
       emit mimeTypen("");
       setPaletteBackgroundColor( Qt::lightGray );
    }

    void dropEvent( QDropEvent * e ) {
       QString str;
       if ( QTextDrag::decode( e, str ) ) { //... Ist es Text ?
          setText( str );
          setMinimumSize( minimumSize().expandedTo( sizeHint() ) );
          return;
       }
       QPixmap pm;
       if ( QImageDrag::decode( e, pm ) ) { //... Ist es ein Bild ?
          setPixmap( pm );
          setMinimumSize( minimumSize().expandedTo( sizeHint() ) );
          return;
       }
       if ( GeheimDrag::decode( e, str ) ) { //... Ist es Typ geheim/mein ?
          setText( str );
          setMinimumSize( minimumSize().expandedTo( sizeHint() ) );
       }
    }

    void mousePressEvent( QMouseEvent * ) {
       QDragObject *d;
       QPixmap     pm;
       if ( pixmap() ) {
          d = new QImageDrag( pixmap()->convertToImage(), this );
             //... Drag-Objekt bei Drag-and-Drop eines Bildes
             //... in Kleinformat (Drittel des Originals) anzeigen
          pm.convertFromImage(pixmap()->convertToImage().smoothScale(
                     pixmap()->width()/3,pixmap()->height()/3));
       } else {
          d = new QTextDrag( text(), this );
          if ( !pm.load( "text.xpm" ) ) {
             pm = QPixmap( 20, 20 );
             pm.fill( Qt::white );
          }
       }
       d->setPixmap( pm );
       d->dragCopy();
    }
};

//---------------------------- Restlicher Code ---------------------------
//------------------------------------------------------------------------
static void neuesWidget( QWidget * parent, QString datei, bool secret = false )
{
    QVBoxLayout *vbox = new QVBoxLayout( parent, 10 );
    DropStelle * d = new DropStelle( parent );
    d->setFrameStyle( QFrame::Sunken | QFrame::Panel );
    vbox->addWidget( d );
    if ( !datei.isNull() ) {
       QPixmap bild;
       if ( !bild.load( datei ) ) {
           bild = QPixmap(20,20);
           bild.fill(Qt::blue);
       }
       d->setPixmap( bild );
    } else
       d->setText("Drag-and-Drop Text");
    d->setFont(QFont("Helvetica",18));
    d->setMinimumSize( d->sizeHint() );

    if ( secret ) {
       GeheimQuelle *s = new GeheimQuelle( parent );
       vbox->addWidget( s );
    }

    QLabel * format = new QLabel( "\nNoch nichts geschehen\n", parent );
    format->setMinimumSize( 200, 150 );
    vbox->addWidget( format );
    parent->resize( parent->sizeHint() );

    QObject::connect( d, SIGNAL(mimeTypen(const QString&)),
                      format, SLOT(setText(const QString&)) );
}

int main( int argc, char *argv[] ) 
{
    QApplication a( argc, argv );

    QWidget mw1;
    neuesWidget( &mw1, "lausbub.xpm" );
    mw1.setCaption( "Lausbub" ); mw1.show();

    QWidget mw2;
    neuesWidget( &mw2, QString::null );
    mw2.setCaption( "Text" ); mw2.show();

    QWidget mw3;
    neuesWidget( &mw3, "penguin.xpm", true );
    mw3.setCaption( "Pinguin" ); mw3.show();

    QObject::connect(qApp,SIGNAL(lastWindowClosed()),qApp,SLOT(quit()));
    return a.exec();
}

#include "dragdropmime.moc"
