#include <qapplication.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qlabel.h>
#include <qlayout.h>
#include <float.h>

//.................................................... Klasse DynValidator
class DynValidator : public QDoubleValidator
{
public:
    DynValidator( QWidget *parent=0, const char *name=0 )
                                : QDoubleValidator( parent, name ) { }
};

//...................................................... Klasse InputLines
class InputLines : public QWidget
{
    Q_OBJECT
public:
    InputLines( QWidget *parent=0, const char *name=0 )
                                            : QWidget( parent, name ) {
       dValidator = new QDoubleValidator( this );
       iValidator = new QIntValidator( this );
       dynValidator = new DynValidator( this );

       QGridLayout *g = new QGridLayout( this, 6, 2, 10, 20 );

       QLabel *lb = new QLabel( "Bottom", this );
       g->addWidget( lb, 0, 0 );
       bottom = new QLineEdit( this );
       bottom->setValidator( dValidator );
       g->addWidget( bottom, 0, 1 );

       QLabel *lt = new QLabel( "Top", this );
       g->addWidget( lt, 1, 0 );
       top = new QLineEdit( this );
       top->setValidator( dValidator );
       g->addWidget( top, 1, 1 );

       QLabel *ld = new QLabel( "Decimals", this );
       g->addWidget( ld, 2, 0 );
       decimals = new QLineEdit( this );
       decimals->setValidator( iValidator );
       g->addWidget( decimals, 2, 1 );

       QString t;
       t.sprintf( "Double-Input (Bottom=%g, Top=%g, Decimals=%d)",
                  -DBL_MAX, DBL_MAX, DBL_DIG );
       configLabel = new QLabel( t, this );
       g->addMultiCellWidget( configLabel, 3, 3, 0, 1 );

       input = new QLineEdit( this );
       input->setValidator( dynValidator );
       g->addMultiCellWidget( input, 4, 4, 0, 1 );

       logMeldung = new QLabel( "Noch nicht mit Eingabe begonnen", this );
       logMeldung->setPaletteBackgroundColor( Qt::blue.light( 180 ) );
       g->addMultiCellWidget( logMeldung, 5, 5, 0, 1 );
  
       QObject::connect( bottom, SIGNAL( textChanged( const QString & ) ),
                         this,   SLOT( configChanged( const QString & ) ) );
       QObject::connect( top,   SIGNAL( textChanged( const QString & ) ),
                         this,  SLOT( configChanged( const QString & ) ) );
       QObject::connect( decimals, SIGNAL( textChanged( const QString & ) ),
                         this,     SLOT( configChanged( const QString & ) ) );

       QObject::connect( input, SIGNAL( textChanged( const QString & ) ),
                         this,  SLOT( slotTextChanged( const QString & ) ) );
       QObject::connect( input, SIGNAL( returnPressed() ),
                         this,  SLOT( slotReturnPressed() ) );
   }

private slots:

   void configChanged( const QString &text ) {
      double  b = bottom->text().isEmpty() ? -DBL_MAX 
                                           : bottom->text().toDouble(),
              t = top->text().isEmpty() ? DBL_MAX
                                        : top->text().toDouble();
      int     d = decimals->text().isEmpty() ? DBL_DIG
                                             : decimals->text().toInt();
      QString s;
      s.sprintf( "Double-Input (Bottom=%g, Top=%g, Decimals=%d)", b, t, d );
      configLabel->setText( s );

      dynValidator->setRange( b, t, d );
      slotTextChanged( input->text() );
   }

   void slotTextChanged( const QString &text ) {
       QString  t = text;
       int      pos;
       QValidator::State zustand = dynValidator->validate( t, pos );
       if ( zustand == QValidator::Acceptable ) {
          logMeldung->setPaletteBackgroundColor( Qt::yellow.light( 160 ) );
          t.sprintf("gltig, aber noch nicht fertig; bisher: %s",
                    (const char *)text );
       } else {
          logMeldung->setPaletteBackgroundColor( Qt::red.light( 160 ) );
          t.sprintf("ungltig; bisher: %s", (const char *)text );
       }
       logMeldung->setText( t ); 
   }

   void slotReturnPressed() {
       QString  endeText;
       endeText.sprintf("Fertig und akzeptiert: %s",
                        (const char *)input->text() ); 
       logMeldung->setText( endeText ); 
       logMeldung->setPaletteBackgroundColor( Qt::green.light( 160 ) );
   }

private:
   QLineEdit        *bottom,
                    *top,
                    *decimals,
                    *input;
   QDoubleValidator *dValidator;
   QIntValidator    *iValidator;
   DynValidator     *dynValidator;
   QLabel           *configLabel,
                    *logMeldung;
};

#include "dynvalidator.moc"

//................................................................... main
int main( int argc, char *argv[] )
{
    QApplication a( argc, argv );

    InputLines w;
    w.resize( 400, w.sizeHint().height() );

    a.setMainWidget( &w );
    w.show();
    return a.exec();
}
