#include <qapplication.h>
#include <qmainwindow.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qtextedit.h>
#include <qlineedit.h>
#include <qfiledialog.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qkeycode.h>

enum { LetzteOeffnenID = 200, SchliessenID,     SpeichernID, 
       SpeichernUnterID,      RueckgaengigID,   
       AusschneidenID,        KopierenID,       EinfuegenID,
       pAusschneidenID,       pKopierenID,      pEinfuegenID,
       AuswahlAufhebenID,     BearbeitenMenuID                 };


//==========================================================================
//              E i n g a b e W i d g e t  -  K l a s s e
//==========================================================================
//          erzeugt ein eigenes Widget zum Suchen von Text
//          oder zur Eingabe einer Zeile, in die zu springen ist.
//--------------------------------------------------------------------------
class EingabeWidget : public QWidget
{
    Q_OBJECT

    public:
       EingabeWidget( const char* text, QWidget* parent = 0,
                      const char* name = 0 ) : QWidget( parent, name ) {
          this->setCaption( name );
          zuSuchen = new QLabel( text, this ); 
           zuSuchen->setGeometry(0, 0, 220, 30);
          suchText = new QLineEdit( this ); 
           suchText->setGeometry(220, 0, 200, 30);
           suchText->setPaletteBackgroundColor( QColor( 0, 255, 255 ) );
          cancelButton = new QPushButton( "Beenden", this ); 
           cancelButton->setGeometry(420, 0, 80, 30);
          connect( cancelButton, SIGNAL( clicked() ),
                   this, SLOT( versteckeEingabeWidget() ) );
          connect( suchText, SIGNAL( returnPressed() ),
                   this, SLOT( enterGedruecktSlot() ) );
       }
       QString  text() { return suchText->text(); }
    signals:
       void enterGedrueckt();
    private slots:
       void versteckeEingabeWidget() { this->hide();          }
       void enterGedruecktSlot()     { emit enterGedrueckt(); }
    private:
       QLabel*      zuSuchen;
       QLineEdit*   suchText;
       QPushButton* cancelButton;
};

//==========================================================================
//                  T e x t E d i t  -  K l a s s e
//==========================================================================
//          das eigentliche Textfenster, das den zu editierenden
//          Text enthaelt.
//--------------------------------------------------------------------------
class TextEdit : public QTextEdit
{
   Q_OBJECT  // notwendig, da Slots enthalten sind

   public:
      TextEdit( QWidget* parent = 0, const char* name = 0 ) :
                                       QTextEdit( parent, name ) {
         gesichert = echtGeaendert = true;
         geaendert = clipboard = false;
         popupMenu = new QPopupMenu();  //... erzeugt Kontext-Menue
         popupMenu->insertItem( "Ausschneiden", this,
                        SLOT( ausschneiden() ), 0, pAusschneidenID );
         popupMenu->insertItem( "Kopieren", this,
                        SLOT( kopieren() ), 0, pKopierenID );
         popupMenu->insertItem( "Einfuegen", this,
                        SLOT( einfuegen() ), 0, pEinfuegenID );
         QObject::connect( this, SIGNAL( textChanged() ), 
                           SLOT( textGeaendert() ) );
      }

      bool  istGesichert()   { return gesichert;                            }
      bool  istGeaendert()   { return geaendert;                            }
      void  setzeGesichert() { gesichert = true;                            }
      bool  textMarkiert()   { return hasSelectedText();                    }
      bool  einfuegenMoegl() { return clipboard;                            }
      void  cursorRechts()   { moveCursor( QTextEdit::MoveForward, true );  }
      void  setzeText()      { altText = vorherText = text();               }
      void  setzeAltText()   { 
         echtGeaendert = false;
         setText( vorherText );
         vorherText = altText;
         altText = text();
         echtGeaendert = true;
      }
   public slots:
      void  kopieren()       { clipboard = true; copy();                 }
      void  einfuegen()      { textGeaendert(); paste();                 }
      void  ausschneiden()   { textGeaendert(); clipboard = true; cut(); }
      void  loeschen()       { textGeaendert(); del();                   }
      virtual void passeanTextEdit( int x, int y, int w, int h) {
         setGeometry( x, y, w, h);
         show();
      }
   private slots:
      void textGeaendert() {
         if ( echtGeaendert ) {
            vorherText = altText;
            altText = text();
            geaendert = true;
            gesichert = false;
         }
      }
   protected:
      void mouseReleaseEvent( QMouseEvent* event ) {
         if ( event->button() == RightButton ) {
            popupMenu->setItemEnabled( pAusschneidenID, textMarkiert() );
            popupMenu->setItemEnabled( pKopierenID, textMarkiert() );
            popupMenu->setItemEnabled( pEinfuegenID, einfuegenMoegl() );
            popupMenu->popup( mapToGlobal( event->pos() ) );
         }
      }
   private:
      QPopupMenu*  popupMenu;
      bool         gesichert;     // Ist Text bereits gesichert ?
      bool         geaendert;     // Wurde Text bereits geaendert ?
      bool         clipboard;     // Ist Text im Clipboard vorhanden ?
      bool         echtGeaendert; // fuer Rueckgaengig-Operation
      QString      altText;       // fuer Rueckgaengig-Operation
      QString      vorherText;    // fuer Rueckgaengig-Operation
};

//==========================================================================
//                  E d i t o r  -  K l a s s e
//==========================================================================
//          das Hauptfenster mit seinen Menues, in dem ein Textfenster
//          mit dem zu editierenden Text eingefuegt wird.
//--------------------------------------------------------------------------
class Editor : public QMainWindow
{
   Q_OBJECT  // notwendig, da Slots enthalten sind

   public:
      Editor();
   private slots:
      void schliessenSlot();
      void neuSlot();
      void beendenSlot();
      void suchenSlot();
      void ausschneidenSlot()    { editFenster->ausschneiden();   }
      void kopierenSlot()        { editFenster->kopieren();       }
      void einfuegenSlot()       { editFenster->einfuegen();      }
      void loeschenSlot()        { editFenster->loeschen();       }
      void allesAuswaehlenSlot() { editFenster->selectAll(true);  }
      void auswahlAufhebenSlot() { editFenster->selectAll(false); }

      void oeffnenSlot() {
         neuSlot();
         dateiName = QFileDialog::getOpenFileName( ".", "*.txt", this );
         leseDatei();
      }
      void oeffneDateiSlot( int id ) {
         neuSlot();
         dateiName = letzteOffenMenu->text(id);
         leseDatei();
      }
      void speichernSlot()       { if (editFenster != NULL) 
                                      speichereText( false );   }
      void speichernUnterSlot()  { if (editFenster != NULL) 
                                      speichereText( true );    }
      void aboutSlot() { QMessageBox::information( this, 
                               "Editor", "Einfacher Editor\n");      }
      void aboutQtSlot() { QMessageBox::aboutQt( this, "About Qt" ); }
      void rueckgaengigSlot() { editFenster->setzeAltText();         } 
      void findenSlot() { suchFenster->show();  suchFenster->setFocus();    } 
      void gehezuSlot() { gehezuFenster->show(); gehezuFenster->setFocus(); } 
      void springezuSlot() {
         QString s = gehezuFenster->text();
         int zeilenNummer = s.toInt();
         if ( zeilenNummer-1 < 0 )
            QMessageBox::information( this, "Unerlaubte Eingabe",
                                  s+" ist keine erlaubte Zeilennummer");
         else if ( zeilenNummer-1 > editFenster->lines() )
            QMessageBox::information( this, "Unerlaubte Eingabe",
                                  "Zeilennummer "+s+" ist zu gross");
         else {
            editFenster->setCursorPosition( zeilenNummer-1, 0 );
            gehezuFenster->hide();
         }
      }

      void menuGeaendert() {
         bearbeitenMenu->setItemEnabled( RueckgaengigID,
                                         editFenster->istGeaendert() );
         bearbeitenMenu->setItemEnabled( AusschneidenID,
                                         editFenster->textMarkiert() );
         bearbeitenMenu->setItemEnabled( KopierenID,
                                         editFenster->textMarkiert() );
         bearbeitenMenu->setItemEnabled( EinfuegenID,
                                         editFenster->einfuegenMoegl() );
         bearbeitenMenu->setItemEnabled( AuswahlAufhebenID,
                                         editFenster->textMarkiert() );
      }
   protected:
      virtual void resizeEvent( QResizeEvent * ) {
         if (editFenster != NULL)
            editFenster->passeanTextEdit( 0, menuBalken->height(),
                         width(), height() - menuBalken->height() );
      }
   private:
      QMenuBar*       menuBalken;
      QPopupMenu*     dateiMenu;
      QPopupMenu*     letzteOffenMenu;
      QPopupMenu*     bearbeitenMenu;
      QPopupMenu*     hilfeMenu;
      TextEdit*       editFenster;
      EingabeWidget*  suchFenster;
      EingabeWidget*  gehezuFenster;
      QString         dateiName;
      QString         altDateiname[10];
      QString         puffer;
     
      void  erzeugeDateiMenu();
      void  erzeugeBearbeitenMenu();
      void  erzeugeMenuBalken();
      void  leseDatei();
      void  speichereText( bool neuerName );

      void  setzeTitel( QString dateiname ) {
         QString s = "Editor - Datei: ";
         s += dateiname;
         setCaption( s );
      }
};

#include "editor.moc"

//==========================================================================
//                  E d i t o r  -  K o n s t r u k t o r
//==========================================================================
Editor::Editor() : QMainWindow ( 0, "Einfacher Editor" ) {
   editFenster = NULL;
   suchFenster = NULL;
   gehezuFenster = NULL;
   dateiName   = "";
   setzeTitel( "keine");
   
   erzeugeDateiMenu();
   erzeugeBearbeitenMenu();

   hilfeMenu = new QPopupMenu; //... erzeugt ein Hilfe-Menue
   hilfeMenu->insertItem( "&About Editor", this, SLOT( aboutSlot() ) );
   hilfeMenu->insertItem( "About &Qt", this, SLOT( aboutQtSlot() ) );

   erzeugeMenuBalken();
}

//==========================================================================
//           E d i t o r  -  E r z e u g e n   d e r   M e n u s
//==========================================================================

//---------------------------------- Editor::erzeugeDateiMenu --------------
void Editor::erzeugeDateiMenu() {
   int  i = 0, j = 0;

   letzteOffenMenu =  new QPopupMenu; //... erzeugt ein Sub-Menue
   QFile  datei( ".editconfig" ); // .editconfig enthaelt Namen von
                                  // frueher geoeffneten Dateien
   if ( datei.open( IO_ReadOnly ) ) {
       QTextStream text( &datei );  // verwende einen text stream
       QString s;
       while ( !text.eof() ) {
          s = text.readLine();
          letzteOffenMenu->insertItem( altDateiname[i] = s, i+150  );
          i++;
       }
       datei.close();
   }
   j = i;
   while (i < 10)
      altDateiname[i++] = "";

   QPixmap neuBild( "filenew.xpm" ),
           oeffneBild( "fileopen.xpm" ),
           sichereBild( "filesave.xpm" );
  
   dateiMenu = new QPopupMenu; //... erzeugt ein Datei-Menue
   dateiMenu->insertItem( neuBild, "&Neu", this,
                          SLOT( neuSlot() ), CTRL + Key_N );
   dateiMenu->insertItem( oeffneBild, "Oe&ffnen...", this,
                          SLOT( oeffnenSlot() ), CTRL + Key_O );
   dateiMenu->insertItem( "&Letzte oeffnen",
                         letzteOffenMenu, LetzteOeffnenID);
   dateiMenu->setItemEnabled( LetzteOeffnenID, j > 0 );

   dateiMenu->insertItem( "S&chliessen", this,
                          SLOT( schliessenSlot() ),
                          CTRL + Key_W, SchliessenID );
   dateiMenu->setItemEnabled( SchliessenID, false );

   dateiMenu->insertSeparator();
   dateiMenu->insertItem( sichereBild, "&Speichern", this,
                          SLOT( speichernSlot() ),
                          CTRL + Key_S, SpeichernID );
   dateiMenu->setItemEnabled( SpeichernID, false );
   dateiMenu->insertItem( "Speichern &unter...",
                          this, SLOT( speichernUnterSlot() ),
                          CTRL + Key_U, SpeichernUnterID );
   dateiMenu->setItemEnabled( SpeichernUnterID, false );

   dateiMenu->insertSeparator();

   dateiMenu->insertItem( "&Beenden", this,
                          SLOT( beendenSlot() ), CTRL + Key_Q );

   QObject::connect( letzteOffenMenu, SIGNAL( activated ( int ) ),
                     SLOT( oeffneDateiSlot( int ) ) );
}

//----------------------------- Editor::erzeugeBearbeitenMenu --------------
void Editor::erzeugeBearbeitenMenu() {
   bearbeitenMenu = new QPopupMenu; //... erzeugt ein Bearbeiten-Menue
   bearbeitenMenu->insertItem( "&Rueckgaengig", this,
                               SLOT( rueckgaengigSlot() ),
                               CTRL + Key_Z, RueckgaengigID );
   bearbeitenMenu->insertSeparator();
   bearbeitenMenu->insertItem( "&Ausschneiden", this,
                               SLOT( ausschneidenSlot() ),
                               CTRL + Key_X, AusschneidenID );
   bearbeitenMenu->insertItem( "&Kopieren", this,
                               SLOT( kopierenSlot() ),
                               CTRL + Key_C, KopierenID );
   bearbeitenMenu->insertItem( "&Einfuegen", this,
                               SLOT( einfuegenSlot() ),
                               CTRL + Key_V, EinfuegenID );
   bearbeitenMenu->insertItem( "&Loeschen", this,
                               SLOT( loeschenSlot() ),
                               Key_Delete );
   bearbeitenMenu->insertSeparator();
   bearbeitenMenu->insertItem( "Alles a&uswaehlen", this,
                               SLOT( allesAuswaehlenSlot() ),
                               CTRL + Key_A );
   bearbeitenMenu->insertItem( "Auswahl auf&heben", this,
                               SLOT( auswahlAufhebenSlot() ),
                               CTRL + Key_H, AuswahlAufhebenID );
   bearbeitenMenu->insertSeparator();
   bearbeitenMenu->insertItem( "&Finden", this,
                               SLOT( findenSlot() ), CTRL + Key_F );
   bearbeitenMenu->insertSeparator();
   bearbeitenMenu->insertItem( "&Gehezu", this,
                               SLOT( gehezuSlot() ), CTRL + Key_G );
   QObject::connect( bearbeitenMenu, SIGNAL( aboutToShow() ), 
                     SLOT( menuGeaendert() ) );
}

//--------------------------------- Editor::erzeugeMenuBalken --------------
void Editor::erzeugeMenuBalken() {
   menuBalken = new QMenuBar( this ); //... erzeugt einen Menue-Balken
   menuBalken->insertItem( "&Datei", dateiMenu );
   menuBalken->insertItem( "&Bearbeiten", bearbeitenMenu,
                           BearbeitenMenuID );
   menuBalken->setItemEnabled( BearbeitenMenuID, false );
   menuBalken->insertSeparator();
   menuBalken->insertItem( "&Hilfe", hilfeMenu );
}

//==========================================================================
//                             S l o t s
//==========================================================================

//-------------------------- Editor::schliessenSlot (private) --------------
void Editor::schliessenSlot() {
   if ( editFenster != NULL ) {
      if ( !editFenster->istGesichert() &&
           !QMessageBox::information( this, "Speichern?",
                "Text ist noch nicht gespeichert",
                "Sichern", "Nicht sichern" ) )
         speichereText( false );
      delete editFenster;
      editFenster = NULL;
      dateiName = "";
      setzeTitel( "keine" );
   }
   if (suchFenster != NULL) {
       delete suchFenster;
       suchFenster = NULL;
   }
   if (gehezuFenster != NULL) {
       delete gehezuFenster;
       gehezuFenster = NULL;
   }
}

//--------------------------------- Editor::neuSlot (private) --------------
void Editor::neuSlot() {
   schliessenSlot();
   menuBalken->setItemEnabled( BearbeitenMenuID, true );
   dateiMenu->setItemEnabled( LetzteOeffnenID, true );
   dateiMenu->setItemEnabled( SchliessenID, true );
   dateiMenu->setItemEnabled( SpeichernID, true );
   dateiMenu->setItemEnabled( SpeichernUnterID, true );
   
   editFenster = new TextEdit( this );
   editFenster->setGeometry( 0, menuBalken->height(),
                    width(), height() - menuBalken->height() );
   editFenster->show();
   suchFenster = new EingabeWidget( " Zu suchender Text:", 0, "Suchfenster" );
   suchFenster->setGeometry( 280, 100, 500, 30 );
   connect( suchFenster, SIGNAL( enterGedrueckt() ),
            this, SLOT( suchenSlot() ) );
   gehezuFenster = new EingabeWidget( " Zeilennummer:", 0, "Gehezu Zeile" );
   gehezuFenster->setGeometry( 280, 200, 300, 30 );
   connect( gehezuFenster, SIGNAL( enterGedrueckt() ),
            this, SLOT( springezuSlot() ) );
}

//----------------------------- Editor::beendenSlot (private) --------------
void Editor::beendenSlot() {
   schliessenSlot();
   QFile  datei( ".editconfig" );
   if ( datei.open(IO_WriteOnly) ) {
       uint  i = 0;
       QTextStream text( &datei );  // verwende einen text stream
       while ( !altDateiname[i].isEmpty() ) {
          datei.writeBlock( altDateiname[i], strlen(altDateiname[i]) );
          datei.putch('\n');
          i++;
       }
       datei.close();
   }
   qApp->quit();
}

//------------------------------ Editor::suchenSlot (private) --------------
void Editor::suchenSlot() {
   int  z, s=0, i, j, start_z;
   bool  gefunden = false, neuVonVorne = false;
   QString suchText = suchFenster->text();
   QRegExp r(suchText); // erlaubt die Eingabe von regulren Ausdrcken
   r.setMinimal( true );

   editFenster->getCursorPosition( &z, &s );
   start_z = z;
   editFenster->selectAll( false );
   while ( 1 ) {
      i = r.search( editFenster->text(z), s);
      if ( i >= 0 ) {
         editFenster->setCursorPosition( z, i );
         for (j=1; j<=r.matchedLength(); j++)
            editFenster->cursorRechts(); // Markieren des Textes
         gefunden = true;
         break;
      }
      s = 0;
      if ( ++z > start_z &&
           neuVonVorne == true && gefunden == false) {
         QMessageBox::information( this, "Suchen",
                                  suchText+" nicht gefunden");
         break;
      }
      if ( z >= editFenster->lines() ) {
         neuVonVorne = true;
         z = 0;
      }
   }
}

//==========================================================================
//                    p r i v a t e  - M e t h o d e n
//==========================================================================

//------------------------------- Editor::leseDatei (private) --------------
void Editor::leseDatei() {
   QFile  datei;
   if ( !dateiName.isEmpty() ) {
      datei.setName( dateiName );
      setzeTitel( dateiName );
      if ( datei.open(IO_ReadOnly) ) {
         int i, j;
         QTextStream text( &datei );     // verwende einen text stream
         editFenster->setText( text.read() );
         datei.close();
         editFenster->setzeGesichert();
         editFenster->setzeText();
             // nachfolgend wird sichergestellt, dass keine
             // doppelten Dateinamen im "Letzte oeffnen"-Menue
             // vorhanden sind.
         for (i = 9; i > 0; i--)
            altDateiname[i] = altDateiname[i-1];
         altDateiname[0] = dateiName;

         for (i = 1; i < 9; i++)
            if ( altDateiname[0] == altDateiname[i] ) 
               for ( j=i; j<9; j++)
                  altDateiname[j] = altDateiname[j+1];

         for (i = 0; i < 9; i++)
            if ( letzteOffenMenu->findItem(i+150) != NULL )
               letzteOffenMenu->changeItem( altDateiname[i], i+150  );
            else if ( !altDateiname[i].isEmpty() )
               letzteOffenMenu->insertItem( altDateiname[i], i+150  );
      }
   }
}

//--------------------------- Editor::speichereText (private) --------------
void Editor::speichereText( bool neuerName ) {
   QFile  datei;
   if ( dateiName.isEmpty() || neuerName == true )
      dateiName = QFileDialog::getSaveFileName( ".", "*.txt", this );
   if ( !dateiName.isEmpty() ) {
      datei.setName( dateiName );
      setzeTitel( dateiName );
      if ( datei.open(IO_WriteOnly) ) {
         QString  text = editFenster->text();
         datei.writeBlock( text, strlen(text) );
         datei.close();
         editFenster->setzeGesichert();
      }
   }
}

//==========================================================================
//                              m a i n
//==========================================================================
int main( int argc, char* argv[] )
{
   QApplication myapp( argc, argv );

   Editor* mywidget = new Editor();
   mywidget->setGeometry( 10, 10, 600, 600 );
 
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
