#include <qapplication.h>
#include <qwidget.h>
#include <qlabel.h>

//............................................................ Klasse MyLabel
class MyLabel : public QLabel
{
public:
   MyLabel( QWidget *parent=0 ) : QLabel( parent ) {
      setFont( QFont( "Times", 12 ) );
      mousePos = mouseGlobalPos = QPoint( 0, 0 );
      keyPressed = false;
      oldSize = newSize = size();
      oldPos = newPos = pos();

      setAlignment( AlignCenter );
      setMouseTracking( true );
   }

   void setKeyPressed( bool value ) {
      keyPressed = value;
      printText();
   }

   void parentMoved( QPoint oPos, QPoint nPos ) {
      oldPos = oPos;
      newPos = nPos;
      printText();
   }

protected:
   virtual void mouseMoveEvent( QMouseEvent* ev ) {
      mousePos = ev->pos();
      mouseGlobalPos = ev->globalPos();
      printText();
   }

   virtual void resizeEvent( QResizeEvent* ev ) {
      oldSize = ev->oldSize();
      newSize = ev->size();
      printText();
   }


private:
   QPoint mousePos, mouseGlobalPos;
   bool   keyPressed;
   QSize  oldSize, newSize;
   QPoint oldPos, newPos;

   void printText( void ) {
      QString t;
      t.sprintf(
           "rel. mouse pos: %d, %d; global mouse pos: %d,%d\n"
           "%sKey currently pressed\n"
           "old size: %d, %d; new size: %d,%d\n"
           "old position: %d, %d; new position: %d,%d",
                 mousePos.x(), mousePos.y(),
                 mouseGlobalPos.x(), mouseGlobalPos.y(),
                 keyPressed ? "" : "No ",
                 oldSize.width(), oldSize.height(),
                 newSize.width(), newSize.height(),
                 oldPos.x(), oldPos.y(), newPos.x(), newPos.y() );
      setText( t );
   }
};

//........................................................... Klasse MyWidget
class MyWidget : public QWidget
{
public:
   MyWidget() : QWidget( 0 ) {
      label = new MyLabel( this );
   }

protected:
   virtual void resizeEvent( QResizeEvent* ev ) {
      label->resize( ev->size() );
   }

   virtual void keyPressEvent( QKeyEvent* ) {
      label->setKeyPressed( true );
   }

   virtual void keyReleaseEvent( QKeyEvent* ) {
      label->setKeyPressed( false );
   }

   virtual void moveEvent( QMoveEvent* ev ) {
      label->parentMoved( ev->oldPos(), ev->pos() );
   }

private:
   MyLabel *label;
};

//.................................................................. main 
int main( int argc, char* argv[] )
{
   QApplication myapp( argc, argv );

   MyWidget* mywidget = new MyWidget();
   mywidget->setGeometry( 50, 50, 350, 200 );
   mywidget->setMinimumSize( 350, 200 );
 
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
