#include <qapplication.h>
#include <qcolor.h>
#include <qpalette.h>
#include <qlabel.h>
#include <qslider.h>
#include <qlcdnumber.h>

struct {
     const char             *name;
     QColorGroup::ColorRole typ;
} farbGruppe[] = { { "Foreground",      QColorGroup::Foreground      },
                   { "Background",      QColorGroup::Background      },
                   { "Base",            QColorGroup::Base            },
                   { "Text",            QColorGroup::Text            },
                   { "Button",          QColorGroup::Button          },
                   { "ButtonText",      QColorGroup::ButtonText      },
                   { "Light",           QColorGroup::Light           },
                   { "Midlight",        QColorGroup::Midlight        },
                   { "Dark",            QColorGroup::Dark            },
                   { "Mid",             QColorGroup::Mid             },
                   { "Shadow",          QColorGroup::Shadow          },
                   { "Highlight",       QColorGroup::Highlight       },
                   { "HighlightedText", QColorGroup::HighlightedText },
                   { "BrightText",      QColorGroup::BrightText      },
                   { "Link",            QColorGroup::Link            },
                   { "LinkVisited",     QColorGroup::LinkVisited     } };
const int farbGruppeZahl = sizeof(farbGruppe) / sizeof(*farbGruppe);
//........................................................ Klasse schiebLCD
class SchiebLCD : public QWidget {
   Q_OBJECT   // notwendig, da eigene Slots enthalten sind

public:
   SchiebLCD( int was, int nr, int wert, QString text, QWidget *parent )
                                                 : QWidget( parent ) {
      typ = was;
      nummer = nr;
         //................ Erzeugen eines Text-Labels
      QLabel* name = new QLabel( this );
      name->setText( text );
      name->setGeometry( 1, 10, 10, 30 );
         //................ Erzeugen eines Schiebebalken
      schiebbalk = new QSlider( 0, 255, // min-/max-Wert
                                5, wert,   // Schrittweite u. Startwert
                       Horizontal, this ); // Ausrichtung u. Elternwidget
      schiebbalk->setGeometry( 20, 10, 180, 30 );
         //................ Erzeugen einer LCD-Anzeige
      lcdzahl = new QLCDNumber( 3, this ); // maximal 3 Ziffern
      lcdzahl->setGeometry( 200, 10, 60, 30 );
      lcdzahl->display( wert );
         //................ Verbinde Schiebebalken und LCD-Anzeige
      QObject::connect( schiebbalk, SIGNAL( valueChanged( int ) ), 
                        this, SLOT( aktualisiereSchiebLCD( int ) ) );
   }
signals:
   void farbeGeaendert( int typ, int nr, int wert );
public slots:
   void aktualisiereSchiebLCD( int wert ) {
      schiebbalk->setValue( wert );
      lcdzahl->display( wert );
      emit farbeGeaendert( typ, nummer, wert ); // Sig. farbeGeaendert schicken
   }
private:
   int         typ, nummer;
   QSlider*    schiebbalk;
   QLCDNumber* lcdzahl;
};
//...................................................... Klasse farbeGruppe
class farbeGruppe : public QWidget {
   Q_OBJECT   // wegen Slots
public:
   farbeGruppe( QWidget *parent ) : QWidget( parent ) { //... Konstruktor
     int  i;
        //................ Initialisierungen
      for ( i=0; i<3; i++)
          w[i] = 0;
      buttonFarbe[0] = palette().active().button().red();
      buttonFarbe[1] = palette().disabled().button().green();
      buttonFarbe[2] = palette().inactive().button().blue();
      backgroundFarbe[0] = palette().active().background().red();
      backgroundFarbe[1] = palette().disabled().background().green();
      backgroundFarbe[2] = palette().inactive().background().blue();
       //............. Erzeugen von zweimal drei Schiebebalken fuer RGB
      QString rgbText[] = { "R", "G", "B" };
      QLabel *l = new QLabel( "ButtonColor", this );
      l->setGeometry( 400, 80, 300, 60 );
      for ( i=0; i<3; i++ ) {
         rgbButton[i] = new SchiebLCD( 1, i, buttonFarbe[i], rgbText[i], this );
         rgbButton[i]->setGeometry( 400, 120+i*50, 300, 60 );
         QObject::connect( rgbButton[i],
                           SIGNAL( farbeGeaendert( int, int, int ) ), 
                           this, SLOT( rgbNeueFarbe( int, int, int ) ) );
      }
      l = new QLabel( "BackgroundColor", this );
      l->setGeometry( 400, 300, 300, 30 );
      for ( i=0; i<3; i++ ) {
         rgbBackground[i] = new SchiebLCD( 2, i, backgroundFarbe[i],
                                           rgbText[i], this );
         rgbBackground[i]->setGeometry( 400, 330+i*50, 300, 60 );
         QObject::connect( rgbBackground[i],
                           SIGNAL( farbeGeaendert( int, int, int ) ), 
                           this, SLOT( rgbNeueFarbe( int, int, int ) ) );
      }
        //.................. Anzeigen der Farbgruppen
      zeigeFarben( 0, "Active",    10, QPalette::Active   );
      zeigeFarben( 1, "Disabled", 140, QPalette::Disabled );
      zeigeFarben( 2, "Inactive", 270, QPalette::Inactive   );
   }
private slots:
   void rgbNeueFarbe( int typ, int nr, int wert ) {
      if ( typ == 1 )
         buttonFarbe[ nr ] = wert; 
      else
         backgroundFarbe[ nr ] = wert; 
      zeigeFarben( 0, "Active",    10, QPalette::Active   );
      zeigeFarben( 1, "Disabled", 140, QPalette::Disabled );
      zeigeFarben( 2, "Inactive", 270, QPalette::Inactive   );
   }
private:
   QWidget    *w[3];
   int        buttonFarbe[3], backgroundFarbe[3];
   SchiebLCD  *rgbButton[3], *rgbBackground[3];

   void zeigeFarben( int nr, QString titel, int offset,
                     enum QPalette::ColorGroup fGruppe ) {
      if ( w[nr] != 0 )
         delete w[nr];
      w[nr] = new QWidget( this );
      w[nr]->setGeometry( offset, 30, 120, 550 );

      QColor neueButtonFarbe( buttonFarbe[0], buttonFarbe[1], buttonFarbe[2] );
      QColor neueBackgroundFarbe( backgroundFarbe[0],
                                  backgroundFarbe[1], backgroundFarbe[2] );
      QPalette neuPalette( neueButtonFarbe, neueBackgroundFarbe );
      w[nr]->setPalette( neuPalette );

      QLabel *l = new QLabel( w[nr] );
      l->setGeometry( 10, 0, 100, 30 );
      l->setText( titel );
      for (int i=0; i<farbGruppeZahl; i++) {
         QLabel *l = new QLabel( w[nr] );
         l->setFont( QFont("Times", 10) );
         l->setPalette( w[nr]->palette().color( fGruppe, farbGruppe[i].typ ) );
         l->setAlignment( AlignCenter );
         l->setText( farbGruppe[i].name );
         l->setGeometry( 10, 30+30*i, 100, 30 );
      }
      w[nr]->show();
   }
};
#include "farbgruppe.moc"
//.................................................................... main
int main( int argc, char* argv[] ) {
   QApplication myapp( argc, argv );
   
   farbeGruppe* mywidget = new farbeGruppe( 0 );
   mywidget->setGeometry( 100, 100, 670, 600 );
  
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
