#include <qapplication.h>
#include <qgrid.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qfileinfo.h>

class DateiInfoBox : public QGrid {
    Q_OBJECT
public:
    DateiInfoBox( QWidget *parent, const char *name, const QString &dateiname )
                             : QGrid( 2, parent, name ) {
       setMargin( 10 );
       setSpacing( 5 );
       setupDateiInfo( dateiname );
    }
private slots:
    void updateDateiInfo();
private:
    QLineEdit *dname;
    QLabel    *pfad, *groesse, *lread, *lmodif, *lrechte, *besitzer, *gruppe;
    void setupDateiInfo( QString dateiname );
};
void DateiInfoBox::setupDateiInfo( QString dateiname ) {
    (void)new QLabel( "Dateiname:", this );
    dname = new QLineEdit( dateiname, this );
     dname->setFocus();
     connect( dname, SIGNAL( returnPressed() ),
              this,  SLOT( updateDateiInfo() ) );
    (void)new QLabel( "Pfad:", this );
     pfad = new QLabel( this );
     pfad->setFrameStyle( QFrame::Panel | QFrame::Sunken );
     pfad->setPaletteBackgroundColor( Qt::yellow );
    (void)new QLabel( "Gre:", this );
     groesse = new QLabel( this );
     groesse->setFrameStyle( QFrame::Panel | QFrame::Sunken );
     groesse->setPaletteBackgroundColor( Qt::yellow );
    (void)new QLabel( "Zuletzt Gelesen:", this );
     lread = new QLabel( this );
     lread->setFrameStyle( QFrame::Panel | QFrame::Sunken );
     lread->setPaletteBackgroundColor( Qt::yellow );
    (void)new QLabel( "Zuletzt Modifiziert:", this );
     lmodif = new QLabel( this );
     lmodif->setFrameStyle( QFrame::Panel | QFrame::Sunken );
     lmodif->setPaletteBackgroundColor( Qt::yellow );
    (void)new QLabel( "Zugriffsrechte:", this );
     lrechte = new QLabel( this );
     lrechte->setFrameStyle( QFrame::Panel | QFrame::Sunken );
     lrechte->setPaletteBackgroundColor( Qt::yellow );
    (void)new QLabel( "Eigentmer", this );
     besitzer = new QLabel( this );
     besitzer->setFrameStyle( QFrame::Panel | QFrame::Sunken );
     besitzer->setPaletteBackgroundColor( Qt::yellow );
    (void)new QLabel( "Gruppe", this );
     gruppe = new QLabel( this );
     gruppe->setFrameStyle( QFrame::Panel | QFrame::Sunken );
     gruppe->setPaletteBackgroundColor( Qt::yellow );
    updateDateiInfo();
}
void DateiInfoBox::updateDateiInfo() {
    QString    r, dateiname = dname->text();
    QFileInfo  fileinfo( dateiname );

    pfad->setText( fileinfo.dirPath( true ) );
    groesse->setText( QString( "%1 Bytes" ).arg( fileinfo.size() ) );
    lread->setText( fileinfo.lastRead().toString() );
    lmodif->setText( fileinfo.lastModified().toString() );
    r.sprintf( "%s lesbar\n%s beschreibbar\n%s ausfhrbar\n", 
               fileinfo.isReadable() ? "" : " nicht",
               fileinfo.isWritable() ? "" : " nicht",
               fileinfo.isExecutable() ? "" : " nicht" );
    lrechte->setText( r );
    besitzer->setText( fileinfo.owner() );
    gruppe->setText( fileinfo.group() );
}
#include "fileinfo.moc"
int main( int argc, char *argv[] ) {
   QApplication a( argc, argv );
   QWidget w;
   DateiInfoBox dateiInfo(&w, "Datei-Infos", QString( argc<2 ? "." : argv[1] ));
   dateiInfo.setCaption( "Beispiel zu QFileInfo" );
   dateiInfo.resize( 450, 550 );
   a.setMainWidget( &w );
   w.setGeometry( 20, 20, 450, 550 );
   w.show();
   return a.exec();
}
