#include <qapplication.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qframe.h>

enum { spId=0x1000, shId=0x2000, lId=0x4000, mlId=0x8000 };

QString shapeName[] = { // angeordnet in Reihenfolge von enum Shape (qframe.h)
       "NoFrame", "Box", "Panel", "WinPanel", "HLine", "VLine", "StyledPanel",
       "PopupPanel", "MenuBarPanel", "ToolBarPanel", "LineEditPanel",
       "TabWidgetPanel", "GroupBoxPanel" };
enum QFrame::Shadow sArray[] = { QFrame::Plain, QFrame::Raised, QFrame::Sunken };
//......................................................... Klasse MyWidget
class MyWidget : public QWidget {
   Q_OBJECT
public:
   MyWidget() : lWidth(0), midlWidth(0),
                shape(QFrame::NoFrame), shadow(QFrame::Plain) {
      setFont( QFont( "Helvetica", 12 ) );
         //............... Frame initialisieren
      anzeige = new QFrame( this );
      anzeige->setGeometry( 50, 70, 300, 280 );
         //............... Shape-Menu
      shapeMenu = new QPopupMenu;
      for (int i = int(QFrame::NoFrame); i<=QFrame::GroupBoxPanel; i++)
         shapeMenu->insertItem( shapeName[i-QFrame::NoFrame], i+spId);
      QObject::connect( shapeMenu, SIGNAL( activated( int ) ),
                        this,      SLOT( shapeSlot( int ) ) );
         //............... Shadow-Menu
      shadowMenu = new QPopupMenu;
      shadowMenu->insertItem( "Plain", shId );
      shadowMenu->insertItem( "Raised", shId+1 );
      shadowMenu->insertItem( "Sunken", shId+2 );
      QObject::connect( shadowMenu, SIGNAL( activated( int ) ),
                        this,       SLOT( shadowSlot( int ) ) );
         //............... lineWidth / midLineWidth-Menus
      lineWidthMenu = new QPopupMenu;
      midlineWidthMenu = new QPopupMenu;
      for (int i=0; i<= 20; i++) {
         QString s;
         s.sprintf("%d Pixel", i);
         lineWidthMenu->insertItem( s, i | lId );
         midlineWidthMenu->insertItem( s, i | mlId );
      }
      QObject::connect( lineWidthMenu, SIGNAL( activated( int ) ),
                        this,          SLOT( lineSlot( int ) ) );
      QObject::connect( midlineWidthMenu, SIGNAL( activated( int ) ),
                        this,             SLOT( midLineSlot( int ) ) );
         //............... Menu-Balken
      menuBalken = new QMenuBar( this );
      menuBalken->insertItem( "Frame Shapes",  shapeMenu );
      menuBalken->insertItem( "Shadow Style",  shadowMenu );
      menuBalken->insertItem( "Line Width",    lineWidthMenu );
      menuBalken->insertItem( "Midline Width", midlineWidthMenu );
      aktualisiere();
   }
private slots:
   void shapeSlot( int s )    { shape = s-spId;             aktualisiere(); }
   void shadowSlot( int s )   { shadow = sArray[s-shId];    aktualisiere(); }
   void lineSlot( int l )     { lWidth = ( l & ~lId );      aktualisiere(); }
   void midLineSlot( int ml ) { midlWidth = ( ml & ~mlId ); aktualisiere(); }
private:
   QMenuBar   *menuBalken;
   QPopupMenu *shapeMenu, *shadowMenu, *lineWidthMenu, *midlineWidthMenu;
   QFrame*     anzeige;
   int         lWidth, midlWidth, shape, shadow;

   void aktualisiere( ) {
      anzeige->setFrameStyle( shape | shadow );
      anzeige->setLineWidth( lWidth );
      anzeige->setMidLineWidth( midlWidth );
      anzeige->update();
      for (int i = int(QFrame::NoFrame); i<=QFrame::GroupBoxPanel; i++)
         shapeMenu->setItemChecked( i | spId, shape == i );
      for (int i = 0; i<3; i++)
         shadowMenu->setItemChecked( i | shId, shadow == sArray[i] );
      for (int i=0; i <= 20; i++)
         lineWidthMenu->setItemChecked( i | lId, lWidth == i );
      for (int i=0; i <= 20; i++)
         midlineWidthMenu->setItemChecked( i | mlId, midlWidth== i );
   }
};
#include "frame.moc"
//.................................................................... main
int main( int argc, char* argv[] ) {
   QApplication myapp( argc, argv );
   MyWidget*mywidget = new MyWidget();
   mywidget->setGeometry( 10, 10, 400, 400 );
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
