#include <qapplication.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include "functionplot.h"
#include "parser.h"

//....................................................... Klasse MyWidget
class MyWidget: public QWidget
{
    Q_OBJECT
public:
    MyWidget( QWidget *p=0, const char *n=0 ) : QWidget( p, n ) {
       QLabel *l;
       f = 0; 

       QGridLayout *grid = new QGridLayout( this, 5, 2, 10 );

       funcEingabe = new QLineEdit( this );
       l = new QLabel( "Funktionsterm", this );
       grid->addWidget( l, 0, 0 );   
       grid->addWidget( funcEingabe, 0, 1 );   

       fromX = new QLineEdit( this );
       fromX->setText( "-1" );
       l = new QLabel( "minimaler X-Wert", this );
       grid->addWidget( l, 1, 0 );   
       grid->addWidget( fromX, 1, 1 );   

       toX = new QLineEdit( this );
       toX->setText( "1" );
       l = new QLabel( "maximaler X-Wert", this );
       grid->addWidget( l, 2, 0 );   
       grid->addWidget( toX, 2, 1 );   

       widthX = new QLineEdit( this );
       widthX->setText( "0.1" );
       l = new QLabel( "X-Schrittweite", this );
       grid->addWidget( l, 3, 0 );   
       grid->addWidget( widthX, 3, 1 );   

       b = new QPushButton( "ok", this );
       grid->addWidget( b, 4, 1 );   

       QObject::connect( b,    SIGNAL( clicked() ),
                         this, SLOT( paintFunction() ) );
    }
 
private slots:
    void paintFunction( void ) {
       bool      ok = true, error = false;
       valuePair v;
       int       i, z=0;

       if ( funcEingabe->text().isEmpty() )
          return;

       if ( fromX->text().isEmpty() )
          fromX->setText( "-1" );
       lowX = fromX->text().toDouble(&ok);
       if ( !ok ) return;

       if ( toX->text().isEmpty() )
          toX->setText( "1" );
       highX = toX->text().toDouble();
       if ( !ok ) return;

       if ( widthX->text().isEmpty() )
          widthX->setText( "0.1" );
       stepX = widthX->text().toDouble();
       if ( !ok ) return;

       for ( double x=lowX; x<=highX; x+=stepX )  // Zaehlen der Werte
           z++;
       f = new FunctionPlot( z );
       i = 0;
       Parser p( funcEingabe->text() );
       for ( v.x=lowX; v.x<=highX; v.x+=stepX ) {
          v.y = p.parseExpression( v.x, &error );
          if ( error ) {
             funcEingabe->setSelection( p.getPos()-1, 1 );
             QMessageBox::information( 0, "Syntax error",
                             p.getErrorMessage(), QMessageBox::Ok );
             return;
          }
          f->setValue( i, v );
          ++i;
       }
       f->plotIt();
       f->resize( 600, 600 );
       f->setMinimumSize( 200, 200 );
       f->setCaption( funcEingabe->text() );
       f->show();
    }

private:
    QLineEdit    *funcEingabe,
                 *fromX, *toX, *widthX;
    QPushButton  *b;
    FunctionPlot *f;
    double       lowX, highX, stepX;
};
                                                                        
#include "functionparser.moc"

//.................................................................. main
int main( int argc, char* argv[] )
{
   QApplication myapp( argc, argv );

   MyWidget* myWidget = new MyWidget;

   myapp.setMainWidget( myWidget );
   myWidget->resize( myWidget->sizeHint() );
   myWidget->show();
   return myapp.exec();
}
