#ifndef  FUNCTIONPLOT_H 
#define  FUNCTIONPLOT_H 

#include <qapplication.h>
#include <qwidget.h>

typedef struct {
     double  x, y; 
} valuePair;

//................................................... Klasse FunctionPlot
class FunctionPlot : public QWidget
{
public:
       //... Konstruktor, dem die Anzahl der Wertepaare fuer
       //... die zu plottende Funktion uebergeben wird
   FunctionPlot( int n, QWidget *p=0, const char *name=0 );

       //... Destruktor; gibt den fuer die Wertepaare
       //... reservierten Speicherplatz wieder frei
   ~FunctionPlot()  { delete [] values; }

       //... setzt das i.te Wertepaar auf die Werte v.x und v.y
   void setValue( int i, valuePair v );

       //... setzt die ersten n Wertepaare auf die Werte aus dem Array v
   void setValues( int n, valuePair v[] );

       //... legt die minimalen und maximalen x- und y-Werte
       //... des Bereichs fest, der von der Funktion anzuzeigen ist 
   void setPlotView( double minx, double miny, double maxx, double maxy );

       //... bewirkt, dass die Funktion gezeichnet wird
   void plotIt( void );

protected:

       //... bewirkt ein Vergroessern des Funktionsbereichs
       //...  "Cursor rechts": negative x-Achse wird verlaengert
       //...  "Cursor links" : positive x-Achse wird verlaengert
       //...  "Cursor hoch"  : negative y-Achse wird verlaengert
       //...  "Cursor tief"  : positive y-Achse wird verlaengert
   virtual void keyPressEvent( QKeyEvent *ev );

       //... leitet ein Zoomen (Verkleinern des Funktionsbereichs) ein
   virtual void mousePressEvent( QMouseEvent *ev );

       //... Zoomen (Verkleinern des Funktionsbereichs) findet gerade statt
   virtual void mouseMoveEvent( QMouseEvent *ev );

       //... Zoomen (Verkleinern des Funktionsbereichs) wird abgeschlossen
   virtual void mouseReleaseEvent( QMouseEvent *ev );

       //... bewirkt ein Neumalen der Funktion in dem aktuell
       //... festgelegten Funktionsbereich
   virtual void paintEvent( QPaintEvent *ev );

private:
      
       //... zeichnen die x- bzw. y-Achse
   void drawXScale( QPainter *p, double i, int yp );
   void drawYScale( QPainter *p, double i, int xp );

   int        valueNo;  // Anzahl der Wertepaare der Funktion

   valuePair  *values;  // enthaelt die einzelnen Wertepaare

   bool       plotViewSet; // zeigt an, ob explizit ein eigener
                           // anzuzeigender Funktionsbereich festgelegt
                           // wurde, also setPlotView() aufgerufen wurde. 
                           // plotViewSet=false, werden die minimalen
                           // und maximalen x- und y-Werte des
                           // anzuzeigenden Bereichs implizit bestimmt

   double     minX, minY, maxX, maxY, // minimale und maximale x- und y-Werte
                                      // des anzuzeigenden Bereichs
              xFactor, yFactor; // interne Projektionsfaktoren

   QPoint     startPos, letztePos, neuePos; // Mauspositionen, die fuer
                                            // das Zoomen mit der Maus
                                            // benoetigt werden.
   bool       dragging, ersteMal; // fuer Ziehen der Maus benoetigt

   QColor     farbe; // zeigt an, ob Zoomen aktiviert ist (gruen)
                     // oder nicht (rot)
};

#endif
