#include <qpainter.h>
#include <stdio.h>
#include "functionplot.h"

//.............................................. Konstruktor FunctionPlot
FunctionPlot::FunctionPlot( int n, QWidget *p, const char *name )
                                         : QWidget( p, name ) {
   valueNo = n;
   values = new valuePair [n];
   for ( int i=0; i<n; i++ )
      values[i].x = values[i].y = 0.0;
   plotViewSet = false;
}

//.............................................................. setValue
void FunctionPlot::setValue( int i, valuePair v ) {
   if ( i >= valueNo || i < 0 )
      return; 
   values[i] = v;
}

//............................................................. setValues
void FunctionPlot::setValues( int n, valuePair v[] ) {
   for ( int i=0; i<n; i++ )
      setValue( i, v[i] );
}

//........................................................... setPlotView
void FunctionPlot::setPlotView( double minx, double miny,
                                double maxx, double maxy ) {
   plotViewSet = true;
   minX = minx;
   minY = miny;
   maxX = maxx;
   maxY = maxy;
}

//.............................................................. plotIt
void FunctionPlot::plotIt( void ) {
   if ( !plotViewSet ) {
      minX = maxX = values[0].x;
      minY = maxY = values[0].y;
      for ( int i=1; i<valueNo; i++ ) {
         if ( values[i].x < minX ) minX = values[i].x;
         if ( values[i].x > maxX ) maxX = values[i].x;
         if ( values[i].y < minY ) minY = values[i].y;
         if ( values[i].y > maxY ) maxY = values[i].y;
      }
        //.... Sicherstellen, dass Achsen sichtbar
      if ( minX > 0 ) minX = 0;
      if ( maxX < 0 ) maxX = 0;
      if ( minY > 0 ) minY = 0;
      if ( maxY < 0 ) maxY = 0;
   }
   repaint();
}

//....................................................... keyPressEvent
void FunctionPlot::keyPressEvent( QKeyEvent *ev ) {
   double       diffX = (maxX-minX) / 10,
                diffY = (maxY-minY) / 10;
   switch (ev->key() ) {
      case Key_Right:  minX -= diffX; repaint(); break;
      case Key_Left:   maxX += diffX; repaint(); break;
      case Key_Up:     minY -= diffY; repaint(); break;
      case Key_Down:   maxY += diffY; repaint(); break;
   }
}

//..................................................... mousePressEvent
void FunctionPlot::mousePressEvent( QMouseEvent *ev ) {
   ersteMal = true;
   dragging = false;
   startPos = ev->pos();
   repaint( false );
}

//...................................................... mouseMoveEvent
void FunctionPlot::mouseMoveEvent( QMouseEvent *ev ) {
   if ( !dragging )
         //.... Dragging (Ziehen) einschalten, wenn
         //.... neue Position mind. 20 Pixel entfernt ist
      if ( QABS( startPos.x() - ev->pos().x() ) >= 20 ||
           QABS( startPos.y() - ev->pos().y() ) >= 20 )
         dragging = true;
   if ( dragging ) {
      neuePos = ev->pos();
        //.... Farbe zeigt an, ob Zoom aktiviert ist
      if ( QABS( startPos.x() - ev->pos().x() ) >= 50 &&
           QABS( startPos.y() - ev->pos().y() ) >= 50 )
         farbe = Qt::green.light( 60 );
      else
         farbe = Qt::red.light( 150 );
      repaint( false );
   }
}

//................................................... mouseReleaseEvent
void FunctionPlot::mouseReleaseEvent( QMouseEvent *ev ) {
      //.... Bedingung soll versehentliches Zoomen verhindern
   if ( QABS( startPos.x() - ev->pos().x() ) >= 50 &&
        QABS( startPos.y() - ev->pos().y() ) >= 50 ) {
      if ( ev->pos().x() > startPos.x() ) { 
         maxX = ev->pos().x() / xFactor + minX;
         minX = startPos.x() / xFactor + minX;
      } else {
         maxX = startPos.x() / xFactor + minX;
         minX = ev->pos().x() / xFactor + minX;
      }

      if ( ev->pos().y() > startPos.y() ) { 
         minY = maxY - ev->pos().y() / yFactor;
         maxY = maxY - startPos.y() / yFactor;
      } else {
         minY = maxY - startPos.y() / yFactor;
         maxY = maxY - ev->pos().y() / yFactor;
      }
   }
   dragging = false;
   repaint();
}

//.......................................................... paintEvent
void FunctionPlot::paintEvent( QPaintEvent * ) {
   QPainter p( this );

   double  xp, yp, xpAlt, ypAlt, diffX, diffY, i;

   xFactor = width() / (maxX -minX),
   yFactor = height() / (maxY -minY);

   if ( dragging ) {
      if ( !ersteMal )
         p.eraseRect( QRect( startPos, letztePos ) );
      p.fillRect( QRect( startPos, neuePos ), farbe );
      letztePos = neuePos;
      ersteMal = false;
   }

   p.setPen( Qt::yellow );           //.... x- und y-Achsen zeichnen
   xp = -minX * xFactor;
   yp = maxY * yFactor;
   p.drawLine( 0, int(yp), width(), int(yp) );  // x-Achse
   p.drawLine( int(xp), 0, int(xp), height() ); // y-Achse

   diffX = (maxX-minX) / 10;        //.... x-Skalen zeichnen/beschriften
   for ( i = -diffX; i>=minX; i-=diffX )
      drawXScale( &p, i, int(yp) );
   for ( i = diffX; i<=maxX; i+=diffX )
      drawXScale( &p, i, int(yp) );

   diffY = (maxY-minY) / 10;        //.... y-Skalen zeichnen/beschriften
   for ( i = -diffY; i>=minY; i-=diffY )
      drawYScale( &p, i, int(xp) );
   for ( i = diffY; i<=maxY; i+=diffY )
      drawYScale( &p, i, int(xp) );
                                   //.... Funktion zeichnen
   xpAlt = xp = ( values[0].x - minX ) * xFactor;
   ypAlt = yp = ( maxY - values[0].y ) * yFactor;
   p.setPen( Qt::black );
   p.setBrush( Qt::black );
   p.drawEllipse( int(xp-1), int(yp-1), 2, 2 );

   for ( i=1; i<valueNo; i++ ) {
      xp = ( values[(int)i].x - minX ) * xFactor;
      yp = ( maxY - values[(int)i].y ) * yFactor;
      p.setPen( Qt::blue.light( 140 ) );
      p.drawLine( int(xpAlt), int(ypAlt), int(xp), int(yp) );
      p.setPen( Qt::blue.light( 120 ) );
      p.drawEllipse( int(xp), int(yp), 2, 2 );
      xpAlt = xp;
      ypAlt = yp;
   }
}

//............................................................ drawXScale
void FunctionPlot::drawXScale( QPainter *p, double i, int yp ) {
   QString text;
   double  xs = (i-minX) * xFactor;

   text.sprintf( "%.3g", i );
   p->drawLine( int(xs), int(yp-2), int(xs), int(yp+2) );
   p->drawText( int(xs+1), int(yp-2), text );
   p->setPen( QPen( Qt::yellow, 0, Qt::DotLine ) ); // Raster
   p->drawLine( int(xs), 0, int(xs), height() );
   p->setPen( QPen( Qt::yellow, 0, Qt::SolidLine ) );
}

//............................................................ drawYScale
void FunctionPlot::drawYScale( QPainter *p, double i, int xp ) {
   QString text;
   double ys = (maxY-i) * yFactor;

   text.sprintf( "%.3g", i );
   p->drawLine( int(xp-2), int(ys), int(xp+2), int(ys) );
   p->drawText( int(xp+4), int(ys), text );
   p->setPen( QPen( Qt::yellow, 0, Qt::DotLine ) ); // Raster
   p->drawLine( 0, int(ys), width(), int(ys) );
   p->setPen( QPen( Qt::yellow, 0, Qt::SolidLine ) );
}
