#include <qapplication.h>
#include <qwidget.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlayout.h>
#include <qframe.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qkeycode.h>
#include <qgl.h>

//............................................................... class GLBox
class GLBox : public QGLWidget {
    Q_OBJECT
public:
    GLBox( QWidget* parent, const char* name ) : QGLWidget( parent, name ) {
       xRot = yRot = zRot = 0.0;   // voreingest. Objekt-Rotation
       scale = 1.25;               // voreingest. Objekt-Skalierung
       object = 0;  // OpenGl-Objekt hier noch 0
    }

    ~GLBox() {  // lscht die Displayliste
       glDeleteLists( object, 1 ); // 2. Arg. gibt Anzahl der Displaylisten
    }

public slots:
    void setXRotation( int degrees ) {  // Festlegen des Rotationswinkels um
       xRot = (GLfloat)(degrees % 360); // die X-Achse;
       updateGL();                      // bedingt durch oberen Slider
    }

    void setYRotation( int degrees ) {  // Festlegen des Rotationswinkels um
       yRot = (GLfloat)(degrees % 360); // die Y-Achse;
       updateGL();                      // bedingt durch mittleren Slider
    }

    void setZRotation( int degrees ) {  // Festlegen des Rotationswinkels um
       zRot = (GLfloat)(degrees % 360); // die Z-Achse;
       updateGL();                      // bedingt durch unteren Slider
    }

protected:
    void initializeGL() { //... initialisiert das OpenGL-Subsystem
       qglClearColor( green );   // OpenGL soll alles auf grn lschen
       object = makeObject();   // Erzeuge eine OpenGL-Displayliste
                                // (Gruppe von OpenGL-Kommandos)
       glShadeModel( GL_FLAT ); // flache Schattierung; wegen Performance
    }

    void paintGL() {  //........ malt die Box; aktuellen OpenGL-Kommandos
                      //         zum Malen der Box werden hier ausgefhrt
       glClear( GL_COLOR_BUFFER_BIT ); // Puffer lschen u. farbiges
                                       // Zeichnen zulassen
       glLoadIdentity(); // ersetze akt. Matrix durch Identittsmatrix
       glTranslatef( 0.0, 0.0, -10.0 ); // Transformiere akt. Matrix mit den
                                        // folg. x-, y- und z-Werten
       glScalef( scale, scale, scale ); // Transformiere akt. Matrix mit den
                                        // folg. x-, y- und z-Werten (1.25)
       glRotatef( xRot, 1.0, 0.0, 0.0 ); // Rotiere Matrix um x-Achse um xRot
       glRotatef( yRot, 0.0, 1.0, 0.0 ); // Rotiere Matrix um y-Achse um yRot
       glRotatef( zRot, 0.0, 0.0, 1.0 ); // Rotiere Matrix um z-Achse um zRot
       glCallList( object ); // Fhre Displayliste aus (neue Box zeichnen)
    }

    void resizeGL( int w, int h ) { //... wird aufgerufen, wenn Widgetgre
                                    //    verndert wird
       glViewport( 0, 0, (GLint)w, (GLint)h );  // OpenGL-Viewport
       glMatrixMode( GL_PROJECTION ); // akt. Matrix auf Projektionsmatrix
                                      // setzen
       glLoadIdentity(); // ersetze akt. Matrix durch Identittsmatrix
         // setze Perspektiven der Matrix; siehe auch OpenGL-Dokumentation
         // auf www.opengl.org
       glFrustum( -1.0, 1.0, -1.0, 1.0, 5.0, 15.0 );
       glMatrixMode( GL_MODELVIEW ); // akt. Matrix auf Modelview-Matrix setz.
    }

    virtual GLuint makeObject() { //.... Erzeugt OpenGL-Displayliste fuer
       GLuint list;               //     das zu zeigende Objekt

       list = glGenLists( 1 ); // erzeugt leere Displayliste

       glNewList( list, GL_COMPILE ); // startet Definition der neuen
                                      // Displayliste
       qglColor( blue );   // setzt Zeichenfarbe auf blau
       glLineWidth( 2.0 );  // setzt Breite des Zeichenstiftes auf 2

       glBegin( GL_LINE_LOOP ); // nachfolg. Scheitelpunkte verbinden
       glVertex3f(  0.0,  0.5,  0.0 ); // Zeichne drei Scheitelpunkte
       glVertex3f(  1.0, -0.5, -1.0 ); //  1.Arg=x, 2.Arg=y, 3.Arg=z
       glVertex3f( -1.0, -0.5, -1.0 );
       glEnd();

       glBegin( GL_LINE_LOOP ); // wie zuvor
       glVertex3f(  0.0,  0.5, 0.0 );
       glVertex3f(  1.0, -0.5, 1.0 );
       glVertex3f( -1.0, -0.5, 1.0 );
       glEnd();

       glBegin( GL_LINES ); // Folg. Scheitelpaar als unabh. Liniensegment
                            // behandeln
       glVertex3f(  1.0, -0.5, -1.0 ); glVertex3f(  1.0, -0.5, 1.0 );
       glVertex3f( -1.0, -0.5, -1.0 ); glVertex3f( -1.0, -0.5, 1.0 );
       glEnd();

       glEndList(); // Ende der Definition der Displayliste

       return list; // Rckgabe der Displayliste
    }

private:
    GLuint   object;
    GLfloat  xRot, yRot, zRot, scale;
};

//...................................................... class GLObjectWindow
class GLObjectWindow : public QWidget {
    Q_OBJECT
public:
    GLObjectWindow( QWidget* parent = 0, const char* name = 0 )
                                                  : QWidget( parent, name ) {
       QFrame* f = new QFrame( this, "frame" );
       f->setFrameStyle( QFrame::Sunken | QFrame::Panel );
       f->setLineWidth( 2 );

       GLBox* c = new GLBox( f, 0 ); // Erzeuge das OpenGL-Widget
            //... Slider fr x-Achse
       QSlider* x = new QSlider( 0, 360, 60, 0, QSlider::Vertical, this );
       x->setTickmarks( QSlider::Left );
       QObject::connect(x, SIGNAL(valueChanged(int)),c,SLOT(setXRotation(int)));
            //... Slider fr y-Achse
       QSlider* y = new QSlider( 0, 360, 60, 0, QSlider::Vertical, this );
       y->setTickmarks( QSlider::Left );
       QObject::connect(y, SIGNAL(valueChanged(int)),c,SLOT(setYRotation(int)));
            //... Slider fr z-Achse
       QSlider* z = new QSlider( 0, 360, 60, 0, QSlider::Vertical, this );
       z->setTickmarks( QSlider::Left );
       QObject::connect(z, SIGNAL(valueChanged(int)),c,SLOT(setZRotation(int)));
            //... Layout fr die Slider
       QVBoxLayout* vlayout = new QVBoxLayout( 20 );
       vlayout->addWidget( x );
       vlayout->addWidget( y );
       vlayout->addWidget( z );
            //... Layout fr das OpenGL-Window
       QHBoxLayout* flayout = new QHBoxLayout( f, 2, 2 );
       flayout->addWidget( c, 1 );
       QHBoxLayout* hlayout = new QHBoxLayout( this, 20, 20 );
       hlayout->addLayout( vlayout );
       hlayout->addWidget( f, 1 );
    }
};

#include "glpyramide.moc"

//...................................................................... main
int main( int argc, char *argv[] ) {
    QApplication::setColorSpec( QApplication::CustomColor );
    QApplication a(argc,argv);

    if ( !QGLFormat::hasOpenGL() ) {
       qWarning( "This system has no OpenGL support. Exiting." );
       return -1;
    }

    GLObjectWindow w;
    w.resize( 400, 350 );
    a.setMainWidget( &w );
    w.show();
    return a.exec();
}
