#include <qgl.h>
#include <qapplication.h>
#include <math.h>

//.............................................................. Funktion gear
static void gear( GLfloat inner_radius, GLfloat outer_radius, GLfloat width,
                  GLint teeth, GLfloat tooth_depth ) {
    const double pi = 4*atan(1);
    GLint   i;
    GLfloat r0 = inner_radius,
            r1 = outer_radius - tooth_depth/2.0,
            r2 = outer_radius + tooth_depth/2.0,
            da = 2.0*pi / teeth / 4.0,
            angle, u, v, len;

    glShadeModel( GL_FLAT );

    glNormal3f( 0.0, 0.0, 1.0 );

    glBegin( GL_QUAD_STRIP ); // zeichne Vorderansicht
    for (i=0;i<=teeth;i++) {
       angle = i * 2.0*pi / teeth;
       glVertex3f( r0*cos(angle), r0*sin(angle), width*0.5 );
       glVertex3f( r1*cos(angle), r1*sin(angle), width*0.5 );
       glVertex3f( r0*cos(angle), r0*sin(angle), width*0.5 );
       glVertex3f( r1*cos(angle+3*da), r1*sin(angle+3*da), width*0.5 );
    }
    glEnd();

    glBegin( GL_QUADS ); // zeichne Vorderansicht der Zhne
    da = 2.0*pi / teeth / 4.0;
    for (i=0;i<teeth;i++) {
       angle = i * 2.0*pi / teeth;
       glVertex3f( r1*cos(angle),      r1*sin(angle), width*0.5 );
       glVertex3f( r2*cos(angle+da),   r2*sin(angle+da), width*0.5 );
       glVertex3f( r2*cos(angle+2*da), r2*sin(angle+2*da), width*0.5 );
       glVertex3f( r1*cos(angle+3*da), r1*sin(angle+3*da), width*0.5 );
    }
    glEnd();

    glNormal3f( 0.0, 0.0, -1.0 );

    glBegin( GL_QUAD_STRIP ); // zeichne Hinteransicht
    for (i=0;i<=teeth;i++) {
       angle = i * 2.0*pi / teeth;
       glVertex3f( r1*cos(angle), r1*sin(angle), -width*0.5 );
       glVertex3f( r0*cos(angle), r0*sin(angle), -width*0.5 );
       glVertex3f( r1*cos(angle+3*da), r1*sin(angle+3*da), -width*0.5 );
       glVertex3f( r0*cos(angle), r0*sin(angle), -width*0.5 );
    }
    glEnd();

    glBegin( GL_QUADS ); // zeichne Hinteransicht der Zhne
    da = 2.0*pi / teeth / 4.0;
    for (i=0;i<teeth;i++) {
       angle = i * 2.0*pi / teeth;
       glVertex3f( r1*cos(angle+3*da), r1*sin(angle+3*da), -width*0.5 );
       glVertex3f( r2*cos(angle+2*da), r2*sin(angle+2*da), -width*0.5 );
       glVertex3f( r2*cos(angle+da),   r2*sin(angle+da), -width*0.5 );
       glVertex3f( r1*cos(angle),      r1*sin(angle), -width*0.5 );
    }
    glEnd();

    glBegin( GL_QUAD_STRIP );  // zeichne Aussenansicht der Zhne
    for (i=0;i<teeth;i++) {
       angle = i * 2.0*pi / teeth;
       glVertex3f( r1*cos(angle),      r1*sin(angle), width*0.5 );
       glVertex3f( r1*cos(angle),      r1*sin(angle), -width*0.5 );
       u = r2*cos(angle+da) - r1*cos(angle);
       v = r2*sin(angle+da) - r1*sin(angle);
       len = sqrt( u*u + v*v );
       u /= len;
       v /= len;
       glNormal3f( v, -u, 0.0 );
       glVertex3f( r2*cos(angle+da),   r2*sin(angle+da), width*0.5 );
       glVertex3f( r2*cos(angle+da),   r2*sin(angle+da), -width*0.5 );
       glNormal3f( cos(angle), sin(angle), 0.0 );
       glVertex3f( r2*cos(angle+2*da), r2*sin(angle+2*da),  width*0.5 );
       glVertex3f( r2*cos(angle+2*da), r2*sin(angle+2*da), -width*0.5 );
       u = r1*cos(angle+3*da) - r2*cos(angle+2*da);
       v = r1*sin(angle+3*da) - r2*sin(angle+2*da);
       glNormal3f( v, -u, 0.0 );
       glVertex3f( r1*cos(angle+3*da), r1*sin(angle+3*da),  width*0.5 );
       glVertex3f( r1*cos(angle+3*da), r1*sin(angle+3*da), -width*0.5 );
       glNormal3f( cos(angle), sin(angle), 0.0 );
    }
    glVertex3f( r1*cos(0.0), r1*sin(0.0), width*0.5 );
    glVertex3f( r1*cos(0.0), r1*sin(0.0), -width*0.5 );

    glEnd();
    glShadeModel( GL_SMOOTH );

    glBegin( GL_QUAD_STRIP ); // zeichne Innenradius-Zylinder
    for (i=0;i<=teeth;i++) {
       angle = i * 2.0*pi / teeth;
       glNormal3f( -cos(angle), -sin(angle), 0.0 );
       glVertex3f( r0*cos(angle), r0*sin(angle), -width*0.5 );
       glVertex3f( r0*cos(angle), r0*sin(angle), width*0.5 );
    }
    glEnd();
}

static GLfloat view_rotx=20.0, view_roty=30.0, view_rotz=0.0;
static GLint gear1, gear2, gear3;
static GLfloat angle = 0.0;

static int timer_interval = 10; // Timer-Intervall (Millisek.)

//........................................................ class ZahnradWidget
class ZahnradWidget : public QGLWidget
{
public:
    ZahnradWidget( QWidget *parent=0, const char *name=0 )
                                   : QGLWidget( parent, name ) {
       startTimer( timer_interval );
    }
protected:
    void initializeGL() {
       static GLfloat pos[4]    = {5.0, 5.0, 10.0, 1.0 };
       static GLfloat ared[4]   = {0.8, 0.1,  0.0, 1.0 };
       static GLfloat agreen[4] = {0.0, 0.8,  0.2, 1.0 };
       static GLfloat ablue[4]  = {0.2, 0.2,  1.0, 1.0 };

       glLightfv( GL_LIGHT0, GL_POSITION, pos );
       glEnable( GL_CULL_FACE );
       glEnable( GL_LIGHTING );
       glEnable( GL_LIGHT0 );
       glEnable( GL_DEPTH_TEST );
       gear1 = glGenLists(1);  // erzeuge die Zahnrder
        glNewList(gear1, GL_COMPILE);
        glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, ared );
        gear( 1.0, 4.0, 1.0, 20, 0.7 );
        glEndList();
       gear2 = glGenLists(1);
        glNewList(gear2, GL_COMPILE);
        glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, agreen );
        gear( 0.5, 2.0, 2.0, 10, 0.7 );
        glEndList();
       gear3 = glGenLists(1);
        glNewList(gear3, GL_COMPILE);
        glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, ablue );
        gear( 1.3, 2.0, 0.5, 10, 0.7 );
        glEndList();
       glEnable( GL_NORMALIZE );
    }

    void resizeGL( int width, int height ) {
       GLfloat w = (float) width / (float) height;
       GLfloat h = 1.0;

       glViewport( 0, 0, width, height );
       glMatrixMode(GL_PROJECTION);
       glLoadIdentity();
       glFrustum( -w, w, -h, h, 5.0, 60.0 );
       glMatrixMode(GL_MODELVIEW);
       glLoadIdentity();
       glTranslatef( 0.0, 0.0, -40.0 );
    }

    void paintGL() {
       angle += 2.0;
       view_roty += 1.0;
       glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
       glPushMatrix();
        glRotatef( view_rotx, 1.0, 0.0, 0.0 );
        glRotatef( view_roty, 0.0, 1.0, 0.0 );
        glRotatef( view_rotz, 0.0, 0.0, 1.0 );
       glPushMatrix();
        glTranslatef( -3.0, -2.0, 0.0 );
        glRotatef( angle, 0.0, 0.0, 1.0 );
        glCallList(gear1);
        glPopMatrix();
       glPushMatrix();
        glTranslatef( 3.1, -2.0, 0.0 );
        glRotatef( -2.0*angle-9.0, 0.0, 0.0, 1.0 );
        glCallList(gear2);
        glPopMatrix();
       glPushMatrix();
        glTranslatef( -3.1, 2.2, -1.8 );
        glRotatef( 90.0, 1.0, 0.0, 0.0 );
        glRotatef( 2.0*angle-2.0, 0.0, 0.0, 1.0 );
        glCallList(gear3);
        glPopMatrix();
       glPopMatrix();
    }

    void timerEvent(QTimerEvent*) { updateGL(); }
};

//....................................................................... main
int main( int argc, char *argv[] ) {
   QApplication a( argc, argv );

   if ( !QGLFormat::hasOpenGL() ) {
      qWarning( "This system has no OpenGL support. Exiting." );
      return -1;
   }
   ZahnradWidget w;
   a.setMainWidget( &w );
   w.show();
   return a.exec();
}
