#include <qapplication.h>
#include <qsocket.h>
#include <qserversocket.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qtextbrowser.h>
#include <qtextstream.h>
#include <qregexp.h>
#include <qpushbutton.h>
#include <qtimer.h>
#include <stdlib.h>
//......................................................... class HttpDaemon
class HttpDaemon : public QServerSocket {
    Q_OBJECT
public:
    HttpDaemon( QObject* parent=0 ) : QServerSocket( 8080, 0, parent ) {
       if ( !ok() ) {
          qWarning("Failed to bind to port 8080");
          exit(1);
       }
    }
    void newConnection( int sockNr ) {
       socket = new QSocket(this);
       connect( socket, SIGNAL( readyRead() ), this, SLOT( readClient() ) );
       connect( socket, SIGNAL( delayedCloseFinished() ),
                this,   SLOT( discardClient() ) );
       socket->setSocket(sockNr);
       emit newConnect();
    }
signals:
    void newConnect();
    void endConnect();
    void wroteToClient();
private slots:
    void readClient() {
       if ( socket->canReadLine() ) {
          QStringList tokens = QStringList::split( QRegExp("[ \n\r][ \n\r]*"),
                                                   socket->readLine() );
          if ( tokens[0] == "GET" ) {
             nr = 1;
             ts.setDevice( socket );
             ts << "<h1>I'll count now up to ten</h1>" << endl;
             timer = new QTimer;
             connect( timer, SIGNAL( timeout() ), this,  SLOT( sendNumber() ) );
             timer->start( 500 );
          }
       }
    }
    void discardClient() {
       delete socket;
       emit endConnect();
    }
    void sendNumber() {
       ts << nr++ << "<br>";
       if ( nr > 10 ) {
          ts << "<br><br><a href=\"http://www.trolltech.com.\"><b>Trolltech-Homepage</b></a>";
          ts << "<br><br><a href=\"http://www.suse.de.\"><b>SuSE-Homepage</b></a>";
          delete timer;
          emit wroteToClient();
          socket->close();
       }
       socket->flush();
    }
private:
    QSocket     *socket;
    QTextStream ts;
    int         nr;
    QTimer      *timer;
};

//........................................................... class HttpInfo
class HttpInfo : public QVBox {
    Q_OBJECT
public:
    HttpInfo() {
       HttpDaemon *httpd = new HttpDaemon( this );
       QString itext = QString(
                 "This is a small httpd example.\n"
                 "You can connect with your\n"
                 "web browser to port %1").arg( httpd->port() );
       QLabel *lb = new QLabel( itext, this );
       lb->setAlignment( AlignHCenter );
       infoText = new QTextBrowser( this );
       QPushButton *quit = new QPushButton( "quit" , this );
       connect( httpd, SIGNAL(newConnect()), SLOT(newConnect()) );
       connect( httpd, SIGNAL(endConnect()), SLOT(endConnect()) );
       connect( httpd, SIGNAL(wroteToClient()), SLOT(wroteToClient()) );
       connect( quit, SIGNAL(pressed()), qApp, SLOT(quit()) );
    }
    ~HttpInfo() { } 
private slots:
    void newConnect()    { infoText->append( "New connection" );    }
    void endConnect()    { infoText->append( "Connection closed" ); }
    void wroteToClient() { infoText->append( "Wrote to client" );   }
private:
    QTextBrowser *infoText;
};
#include "httpd.moc"
//..................................................................... main
int main( int argc, char *argv[] ) {
    QApplication app( argc, argv );
    HttpInfo info;
    app.setMainWidget( &info );
    info.show();
    return app.exec();
}
