#include <qapplication.h>
#include <qfile.h>
#include <qintdict.h>
#include <qsocketnotifier.h>
#include <qstring.h>
#include <qtextstream.h>
#include <qtimer.h>
/* Platform-specific includes. */
#ifdef _WS_X11_
  #include <netinet/in.h>
  #include <sys/socket.h>
  #include <sys/types.h>
  #include <unistd.h>
#else
  #error "Nur fuer Linux/Unix implementiert"
#endif
#define PORT 8080
//........................................................... class Socket
class Socket {
 public:
   Socket()  { _file = 0; _tstream = 0; _fd = -1;        }
   ~Socket() { close(); delete _file; delete _tstream;   }
   void close()                   { ::close( _fd ); _file->close(); }
   int  handle()                  { return _fd;                     }
   void read( QString& text )     { text = _tstream->readLine();    }
   void write( const char* text ) { *_tstream << text;              }
   void setFd( int newfd ) {
      _fd = newfd;
      delete _file;
      delete _tstream;
      _file = new QFile();
      _file->open( IO_ReadWrite | IO_Raw, newfd );
      _tstream = new QTextStream( _file );
   }
 private:
   int         _fd;
   QFile       *_file;
   QTextStream *_tstream;
};
//..................................................... class ServerSocket
class ServerSocket : public Socket {
 public:
   ServerSocket( int port ) {
      int newfd = socket( AF_INET, SOCK_STREAM, 0 );
      struct sockaddr_in serverAddress;
      memset( &serverAddress, 0, sizeof( serverAddress ) );
      serverAddress.sin_family = AF_INET;
      serverAddress.sin_addr.s_addr = htonl( INADDR_ANY );
      serverAddress.sin_port = htons( port );
      bind( newfd, (sockaddr*)&serverAddress, sizeof( serverAddress ) );
      listen( newfd, 5 );
      setFd( newfd );
   }
   Socket* accept() {
      struct sockaddr client_addr;
      socklen_t len;
      int newfd = ::accept( handle(), &client_addr, &len );
      Socket* newSocket = new Socket();
      newSocket->setFd( newfd );
      return newSocket;
   }
};
//................................................. class HTTPMirrorServer
class HTTPMirrorServer : QObject {
  Q_OBJECT
public:
   HTTPMirrorServer() {
      _serversocket = new ServerSocket( PORT );
      _notifier = new QSocketNotifier( _serversocket->handle(),
                                       QSocketNotifier::Read );
      connect( _notifier, SIGNAL( activated( int ) ),
               this, SLOT( connectionWanted( int ) ) );
   }
   ~HTTPMirrorServer() { delete _notifier; delete _serversocket; }
private slots:
   void connectionWanted( int ) {
      Socket* socket = _serversocket->accept();
      _socketlist.insert( socket->handle(), socket );
      QSocketNotifier* datanotifier =
               new QSocketNotifier( socket->handle(),
                                    QSocketNotifier::Read );
      connect( datanotifier, SIGNAL( activated( int ) ),
               this, SLOT( dataArrived( int ) ) );
      _notifierlist.insert( socket->handle(), datanotifier );
   }
   void dataArrived( int handle ) {
      socket = _socketlist.take( fd = handle );
      if ( !socket )
         return;
      QString line;
      socket->read( line );
      while ( !line.isEmpty() ) {
         line += "<br>";
         socket->write( line );
         socket->read( line );
      }
      nr = 1;
      socket->write( "<h1>I'll count now up to ten</h1><br>");
      timer = new QTimer;
      connect( timer, SIGNAL( timeout() ), this,  SLOT( sendNumber() ) );
      timer->start( 500 );
   }
   void sendNumber() {
      QString t;
      t.sprintf( "%d<br>", nr++);
      socket->write( t );
      if ( nr > 10 ) {
          socket->write("<br><br><a href=\"http://www.trolltech.com.\">");
          socket->write("<b>Trolltech-Homepage</b></a>");
          socket->write("<br><br><a href=\"http://www.suse.de.\">");
          socket->write("<b>SuSE-Homepage</b></a>");
          delete timer;
          delete socket;
          delete _notifierlist.take( fd );
       }
    }
private:
    Socket                   *socket;
    int                      fd, nr;
    ServerSocket             *_serversocket;
    QSocketNotifier          *_notifier;
    QIntDict<Socket>          _socketlist;
    QIntDict<QSocketNotifier> _notifierlist;
    QTimer                   *timer;
};
#include "httpd2.moc"
//................................................................... main
int main( int argc, char* argv[] ) {
  QApplication a( argc, argv );
  HTTPMirrorServer server;
  return a.exec();
}
