#include <qapplication.h>
#include <qiconview.h>
#include <qdragobject.h>
#include <qpixmap.h>
#include <qdir.h>
#include <qfileinfo.h>
#include <qsplitter.h>
#include <qlistbox.h>
//............................................................. MyListBox
class MyListBox : public QListBox {
    Q_OBJECT
  public:
    MyListBox( QWidget *p=0, const char *nam=0) : QListBox(p, nam) {
       setAcceptDrops( true );
       dragging = false;
    }
      // Low-level drag and drop
    void dragEnterEvent( QDragEnterEvent *e ) {
       if ( QTextDrag::canDecode( e ) ) 
          e->accept();
    }
    void dropEvent( QDropEvent *e ) {
       QString text;
       if ( QTextDrag::decode( e, text ) ) 
          insertItem( text );
    }
    void mousePressEvent( QMouseEvent *e ) {
       QListBox::mousePressEvent( e );
       dragging = true;
    }
    void mouseMoveEvent( QMouseEvent * ) {
       if ( dragging ) {
          QDragObject *d = new QTextDrag( currentText(), this );
          d->dragCopy(); // d nicht lschen
          dragging = false;
       }
    }
  private:
    bool dragging;
};
//.............................................................. MyIconViewItem
class MyIconViewItem : public QIconViewItem {
  public:
    MyIconViewItem(QIconView *p, const QString &text, const QPixmap &icon)
                                           : QIconViewItem (p, text, icon ) { }
    MyIconViewItem(QIconView *p, const QString &text) : QIconViewItem (p, text) {}

      // High-level drag and drop
    bool acceptDrop( const QMimeSource *mime ) const {
        if ( mime->provides( "text/plain" ) )
            return true;
        return false;
    }
    void dropped( QDropEvent *e, const QValueList<QIconDragItem>& ) {
        QString label;
        if ( QTextDrag::decode( e, label ) )
            setText( label );
    }
};
//.................................................................. MyIconView
class MyIconView : public QIconView {
    Q_OBJECT
  public:
    MyIconView( QWidget *p=0, const char *nam=0 ) : QIconView( p, nam ) {
         //.... Mehrfachauswahl zulassen
       MyIconViewItem *item;
       setSelectionMode( QIconView::Extended );
       QDir dir( ".", "*.xpm" );
       for (uint i=0; i < dir.count(); i++) {
          item = new MyIconViewItem( this, dir[i], QPixmap( dir[i] ) );
          item->setRenameEnabled( true );
       }
       setCaption( "Icondrag" );
    }
      // High-level drag and drop
    QDragObject *dragObject() {
       return new QTextDrag(currentItem()->text(), this);
    }
  public slots:
    void slotNewItem( QDropEvent *e, const QValueList<QIconDragItem>& list ) {
       QString label;
       if ( QTextDrag::decode( e, label ) ) {
          MyIconViewItem *item = new MyIconViewItem( this, label );
          item->setRenameEnabled( true );
       }
    }
};
#include "icondrag.moc"
//........................................................................ main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    QSplitter *split = new QSplitter(); // Fr zwei Subwidgets in einem Widget
    MyIconView *iv   = new MyIconView( split );
              (void)   new MyListBox( split );
    a.setMainWidget( split );
    split->resize( 600, 600 );
    iv->resize(400, 600);
    split->show();
      // Signal-Slot-Verbindung fr Drag-and-Drop
    QObject::connect(
       iv, SIGNAL(dropped(QDropEvent*, const QValueList<QIconDragItem>&)),
       iv, SLOT(slotNewItem(QDropEvent*, const QValueList<QIconDragItem>&)));
    return a.exec();
}
