#include <qapplication.h>
#include <qtranslator.h>
#include <qfileinfo.h>
#include <qmessagebox.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qsignalmapper.h>
#include <qmainwindow.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qstatusbar.h>
#include <qstring.h>
#include <qlistbox.h>
#include <stdlib.h>
#include <time.h>

static const char* choices[] = {
    QT_TRANSLATE_NOOP( "MyWidget", "First" ), 
    QT_TRANSLATE_NOOP( "MyWidget", "Second" ), 
    QT_TRANSLATE_NOOP( "MyWidget", "Third" ), 
    0
}; 
//......................................................... class MyWidget
class MyWidget : public QMainWindow {
    Q_OBJECT
public:
    MyWidget( QWidget *p=0, const char *nam=0 ) : QMainWindow( p, nam ) {
        QVBox* central = new QVBox(this);
         central->setMargin( 5 ); 
         central->setSpacing( 5 ); 
         setCentralWidget(central);
        QPopupMenu* file = new QPopupMenu(this);
        file->insertItem( tr("E&xit"), qApp, SLOT(quit()),
                          QKeySequence(tr("Ctrl+Q", "File|Exit")) );
        menuBar()->insertItem( tr("&File"), file );
        setCaption( tr( "Internationalization Example" ) ); 
        statusBar()->message( tr("Language: English") );
        (void)new QLabel( tr( "The Main Window" ), central ); 
        QButtonGroup* gbox = new QButtonGroup( 1, QGroupBox::Horizontal, 
                                               tr( "View" ), central ); 
        (void)new QRadioButton( tr( "Perspective" ), gbox ); 
        (void)new QRadioButton( tr( "Isometric" ), gbox ); 
        (void)new QRadioButton( tr( "Oblique" ), gbox ); 
        initChoices(central); 
    }
signals:
    void closed();
protected:
    void closeEvent( QCloseEvent* e ) {
       QWidget::closeEvent( e );
       emit closed();
    }
private:
    void initChoices( QWidget* parent ) {
       QListBox* lb = new QListBox( parent ); 
       for ( int i = 0; choices[i]; i++ )
          lb->insertItem( tr( choices[i] ) ); 
    }
};
//......................................................... class QVDialog
class QVDialog : public QDialog {
    Q_OBJECT
public:
    QVDialog( QWidget *parent=0, const char *name=0, bool modal=false,
              WFlags f=0)           : QDialog( parent, name, modal, f ) {
        QVBoxLayout* vb = new QVBoxLayout(this,8);
        vb->setAutoAdd( true );
        hb = 0;
        sm = new QSignalMapper(this);
        connect( sm, SIGNAL(mapped(int)), this, SLOT(done(int)) );
    }
    void addButtons( const QString& cancel=QString::null,
                     const QString& ok=QString::null,
                     const QString& mid1=QString::null,
                     const QString& mid2=QString::null,
                     const QString& mid3=QString::null) {
       addButton(ok.isNull() ? tr("OK") : ok, 1);
       if ( !mid1.isNull() ) addButton(mid1,2);
       if ( !mid2.isNull() ) addButton(mid2,3);
       if ( !mid3.isNull() ) addButton(mid3,4);
       addButton( cancel.isNull() ? tr("Cancel") : cancel, 0 );
    }
    void addButton( const QString& text, int result ) {
       if ( !hb )
          hb = new QHBox(this);
       QPushButton *c = new QPushButton( text, hb );
       sm->setMapping( c, result );
       connect( c, SIGNAL(clicked()), sm, SLOT(map()) );
    }
private:
    QSignalMapper *sm;
    QHBox         *hb;
};
//............................................................... showLang
MyWidget* showLang(QString lang) {
    static QTranslator *translator = 0;
    qApp->setPalette( QPalette( Qt::lightGray,
                      QColor(rand()&255, rand()&255, rand()&255)));
    lang = "international_" + lang + ".qm";
    QFileInfo fi( lang );
    if ( !fi.exists() ) {
       QMessageBox::warning( 0, "File error",
               QString("Cannot find translation for language: "+lang+
                   "\n(try eg. 'de', 'ko' or 'no')") );
       return 0;
    }
    if ( translator ) {
       qApp->removeTranslator( translator );
       delete translator;
    }   
    translator = new QTranslator( 0 );
    translator->load( lang, "." );
    qApp->installTranslator( translator );
    MyWidget *m = new MyWidget;
    return m;
}
#include "international.moc" 
//................................................................... main
int main( int argc, char *argv[] ) {
    QApplication app( argc, argv );

    const char* qm[]= { "de", "el", "en", "eo", "fr",
                        "it", "jp", "ko", "no", "ru", 0 };
    srand( time(NULL) );

    QString lang;
    if ( argc >= 2 )
        lang = argv[1];
    if ( argc < 2 || lang == "all" ) {
       QVDialog dlg( 0, 0, true );
       QCheckBox* qmb[sizeof(qm)/sizeof(qm[0])];
       int r;
       if ( lang == "all" ) {
          r = 2;
       } else {
          QButtonGroup *bg =
               new QButtonGroup( 4, Qt::Vertical, "Choose Locales", &dlg);
          for ( int i=0; qm[i]; i++ )
             qmb[i] = new QCheckBox( (const char*) qm[i], bg );
          dlg.addButtons( "Cancel", "OK", "All");
          r = dlg.exec();
       }
       if ( r ) {
          bool tight = qApp->desktop()->width() < 1024;
          int x=5;
          int y=50;
          for ( int i=0; qm[i]; i++ ) {
             if ( r == 2 || qmb[i]->isChecked() ) {
                 MyWidget* w = showLang((const char*)qm[i]);
                 QObject::connect(w, SIGNAL(closed()), qApp, SLOT(quit()));
                 w->setGeometry( x, y, 192, 306);
                 w->show();
                 if ( tight ) {
                    x += 8;
                    y += 8;
                 } else {
                    x += 200;
                    if ( x > 1000 ) {
                       x = 5;
                       y += 334;
                    }
                 }
              }
          }
       } else
          exit( 0 );
    } else {
       QString lang = argv[1];
       QWidget* m = showLang(lang);
       app.setMainWidget( m );
       m->show();
       m->setGeometry( 5, 50, 192, 306);
    }
    return app.exec();
}
