#include <qapplication.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qlabel.h>
#include <qvbox.h>
//...................................................... Klasse WochenZeit
class WochenZeit : public QVBox {
    Q_OBJECT
public:
    WochenZeit( QWidget *parent=0, const char *name=0 ) : QVBox( parent, name ) {
       setMargin( 10 );
       QVBox *box = new QVBox( this ); // Widget fuer das Layout
       box->setMargin( 5 );
       box->setSpacing( 20 );
       (void)new QLabel(
           "Geben Sie Ihre Wochen-Arbeitszeit (in Stunden) ein: ", box );
       eingabe = new QLineEdit( box );
       iValidator = new QIntValidator( 0, 168, this );
       eingabe->setValidator( iValidator );
       logMeldung = new QLabel( "Noch nicht mit Eingabe begonnen", box );
       logMeldung->setPaletteBackgroundColor( Qt::blue.light( 180 ) );
       QObject::connect( eingabe, SIGNAL( textChanged( const QString & ) ),
                         this,    SLOT( slotTextChanged( const QString & ) ) );
       QObject::connect( eingabe, SIGNAL( returnPressed() ),
                         this,    SLOT( slotReturnPressed() ) );
   }
private slots:
   void slotTextChanged( const QString &text ) {
       QString  t = text;
       int      pos;
       QValidator::State zustand = iValidator->validate( t, pos );
       if ( zustand == QValidator::Acceptable ) {
          logMeldung->setPaletteBackgroundColor( Qt::yellow.light( 160 ) );
          t.sprintf("gltig, aber noch nicht fertig; bisher: %s",
                    (const char *)text );
       } else {
          logMeldung->setPaletteBackgroundColor( Qt::red.light( 160 ) );
          t.sprintf("ungltig; bisher: %s", (const char *)text );
       }
       logMeldung->setText( t ); 
   }
   void slotReturnPressed() {
       QString  endeText;
       endeText.sprintf("Fertig und akzeptiert: %s",
                        (const char *)eingabe->text() ); 
       logMeldung->setText( endeText ); 
       logMeldung->setPaletteBackgroundColor( Qt::green.light( 160 ) );
   }
private:
   QLineEdit     *eingabe;
   QIntValidator *iValidator;
   QLabel        *logMeldung;
};
#include "intvalidator.moc"
//................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    WochenZeit w;
    w.resize( 400, w.sizeHint().height() );
    w.setCaption( "QIntValidator-Beispiel" );
    a.setMainWidget( &w );
    w.show();
    return a.exec();
}
