#include <qapplication.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qvalidator.h>
#include <qtimer.h>
#include <math.h>

#define MAXX  600
#define MAXY  600

//.......................................................... Klasse CatMouse
class CatMouse : public QWidget
{
   Q_OBJECT

public:
   CatMouse( QWidget *p=0 ) : QWidget( p ) {
       catSpeed = mouseSpeed = 0;
       changeMs = delayMs = 0;
       distance = 0;
       first = true;
       PI = 4 * atan(1);

       QLabel *l = new QLabel( " Speed of the cat (m/s)", this );
       l->setFont( QFont( "Times", 14 ) );
       l->setGeometry( MAXX, 0, 250, 30 );
       speedCat = new QLineEdit( this );
       speedCat->setValidator(
                      new QDoubleValidator( 0.0, 1000.0, 4, speedCat ) );
       speedCat->setGeometry( MAXX+250, 0, 50, 30 );
       QObject::connect( speedCat, SIGNAL( textChanged( const QString & ) ),
                         this,     SLOT( readSpeedCat( const QString & ) ) );
       l = new QLabel( " Speed of the mouse (m/s)", this );
       l->setFont( QFont( "Times", 14 ) );
       l->setGeometry( MAXX, 30, 250, 30 );
       speedMouse = new QLineEdit( this );
       speedMouse->setValidator(
                      new QDoubleValidator( 0.0, 1000.0, 4, speedMouse ) );
       speedMouse->setGeometry( MAXX+250, 30, 50, 30 );
       QObject::connect( speedMouse, SIGNAL( textChanged( const QString & ) ),
                         this,       SLOT( readSpeedMouse( const QString & ) ) );
       l = new QLabel( " Direction change of cat (Millisek)", this );
       l->setFont( QFont( "Times", 14 ) );
       l->setGeometry( MAXX, 60, 250, 30 );
       changeDir = new QLineEdit( this );
       changeDir->setValidator( new QIntValidator( changeDir ) );
       changeDir->setGeometry( MAXX+250, 60, 50, 30 );
       QObject::connect( changeDir, SIGNAL( textChanged( const QString & ) ),
                         this,      SLOT( readChangeDir( const QString & ) ) );
       l = new QLabel( " Delay (Millisek)", this );
       l->setFont( QFont( "Times", 14 ) );
       l->setGeometry( MAXX, 90, 250, 30 );
       delayStep = new QLineEdit( this );
       delayStep->setValidator( new QIntValidator( delayStep ) );
       delayStep->setGeometry( MAXX+250, 90, 50, 30 );
       QObject::connect( delayStep, SIGNAL( textChanged( const QString & ) ),
                         this,      SLOT( readDelay( const QString & ) ) );

       timer = new QTimer( this );
       QObject::connect( timer, SIGNAL( timeout() ),
                         this,  SLOT( nextStep() ) );

       distanceLabel = new QLabel( this );
       distanceLabel->setPaletteBackgroundColor( Qt::yellow );
       distanceLabel->setGeometry( MAXX, 120, 300, 100 );
       distanceLabel->setAlignment( Qt::AlignCenter );
       distanceLabel->setFont( QFont( "Times", 30 ) );
   }

protected:
   virtual void mousePressEvent( QMouseEvent *e ) {
      if ( catSpeed!=0 && mouseSpeed!=0 && changeMs!=0 && delayMs!=0 ) {
         mousePosX = MAXX/2;
         mousePosY = 0;
         catPosX = e->pos().x();
         catPosY = MAXY-e->pos().y();

         distanceLabel->setPaletteBackgroundColor( Qt::yellow );
         first = true;
         repaint( true );
         timer->start( delayMs, false );
      }
   }

   virtual void paintEvent( QPaintEvent* ) {
      if ( catSpeed!=0 && mouseSpeed!=0 && changeMs!=0 && delayMs!=0 ) {
         QPainter p( this );
         QString d;

         if ( first ) {
            p.setBrush( QBrush( Qt::white, SolidPattern ) );
            p.drawRect( 0, 0, MAXX, MAXY );
            first = false;
         }
         distance = sqrt( (mousePosX-catPosX)*(mousePosX-catPosX) +
                          (mousePosY-catPosY)*(mousePosY-catPosY) );
         p.setPen( Qt::red );
         p.drawPoint( (int)catPosX, (int)(MAXY-catPosY) );
         p.setPen( Qt::blue );
         p.drawPoint( (int)mousePosX, (int)(MAXY-mousePosY) );
         d.sprintf( "%.2f Meter ", distance );
         distanceLabel->setText( d );

         double factor = catSpeed * (changeMs/1000.0) / distance;
         catPosX += (mousePosX-catPosX) * factor;
         catPosY += (mousePosY-catPosY) * factor;

         mousePosX += mouseSpeed * sin(PI/4)*(changeMs/1000.0);
         mousePosY += mouseSpeed * cos(PI/4)*(changeMs/1000.0);

         if ( distance < 0.5 ) {
            timer->stop();
            distanceLabel->setPaletteBackgroundColor( Qt::red );
            distanceLabel->setText( "Guten Appetit,\n bse Katze" );
            p.setBrush( QBrush( Qt::red, SolidPattern ) );
            p.drawEllipse( int(mousePosX-4), int(MAXY-mousePosY-4), 8, 8 );
         } else if ( mousePosX > MAXX ) {
            timer->stop();
            distanceLabel->setPaletteBackgroundColor( Qt::blue );
            distanceLabel->setText( "Guten Appetit,\n liebe Maus" );
         }
      }
   }

private slots:
   void readSpeedCat( const QString & text ) {
      catSpeed = text.toDouble();
   }
   void readSpeedMouse( const QString & text ) {
      mouseSpeed = text.toDouble();
   }
   void readChangeDir( const QString & text ) {
      changeMs = text.toInt();
   }
   void readDelay( const QString & text ) {
      delayMs = text.toInt();
   }
   void nextStep( void ) {
      repaint( false );
   }
 
private:
   QLineEdit *speedCat,
             *speedMouse,
             *changeDir,
             *delayStep;
   QLabel    *distanceLabel;
   double    catSpeed, mouseSpeed;
   int       changeMs, delayMs;
   QTimer    *timer;
   double    catPosX, catPosY,
             mousePosX, mousePosY;
   double    distance;
   double    PI;
   bool      first;
};

#include "katzmaus.moc"

//..................................................................... main
int main( int argc, char *argv[] )
{
   QApplication myapp( argc, argv );

   CatMouse* mywidget = new CatMouse();
   mywidget->setGeometry( 50, 50, MAXX+300, MAXY );

   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
