#include <qapplication.h>
#include <qlabel.h>
#include <qcolor.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qscrollview.h>
#include <qtextedit.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
//................................................... Klasse MeinLayout
class MeinLayout : public QWidget {
  public:
    MeinLayout( QWidget *p=0, const char *nam=0 ) : QWidget( p, nam ) {
         //... Toplevel-Layout ist eine vertikale Box,
         //... die alle Widgets und Sub-Layouts enthaelt
         //.....................................................
       QBoxLayout *topLayout = new QVBoxLayout( this, 5 );
         //... Erzeugen einer Menueleiste
       QPopupMenu* popup = new QPopupMenu;
        popup->insertItem( "&Beenden", qApp, SLOT(quit()) );
       QMenuBar *menubar = new QMenuBar( this );
        menubar->insertItem( "&Datei", popup );
        menubar->setPaletteBackgroundColor( Qt::blue );
         //... Menueleiste dem Toplevel-Layout-Manager uebergeben
         //.....................................................
       topLayout->setMenuBar( menubar );
         //... Anlegen einer HBox, die ein Label und vier Buttons
         //... enthaelt, die mit Stretch-Faktoren versehen sind.
         //.....................................................
       QLabel* label1 = new QLabel( this );
        label1->setText( "mit Stretch: ");
        label1->setPaletteBackgroundColor( Qt::yellow );
       QBoxLayout *buttons1 = new QHBoxLayout( topLayout );
        buttons1->addWidget( label1, 2);
       for ( int i = 1; i <= 4; i++ ) {
          QPushButton* but = new QPushButton( this );
          QString s;
          s.sprintf( "Button %d", i );
          but->setText( s );
            // Horizontaler Stretch-Faktor ist i (1, 2, 3, 4).
            // In vertikaler Richtung findet fuer die Labels keine
            // Dehnung (Stretch) statt, da das restWidget unten
            // die vollstaendige vertikale Dehnung beinhaltet.
          buttons1->addWidget( but, i );
          if ( i==2 )
             buttons1->addStretch( 2 );
       }
         //... Anlegen einer weiteren HBox, die eine links-justierte
         //... Zeile mit einem Label und zwei Buttons enthaelt.
         //.....................................................
       QLabel* label2 = new QLabel( this );
        label2->setText( "ohne Stretch: ");
        label2->setPaletteBackgroundColor( Qt::green );
       QBoxLayout *buttons2 = new QHBoxLayout( topLayout );
        buttons2->addWidget( label2 );
       QPushButton* but = new QPushButton( "Button fuenf", this );
        buttons2->addWidget( but );
        but = new QPushButton( "Button sechs", this );
        buttons2->addWidget( but );
         // Rest der Zeile mit einem Stretch-Faktor versehen, so dass
         // die Buttons ihre statische minimale Breite erhalten und
         // links in der Zeile angeordnet werden.
       buttons2->addStretch( 3 );
         //... Anlegen eines Editorfensters, das den uebrigen
         //... vollstaendigen Platz in der Mitte belegt.
         //.....................................................
       QTextEdit *restWidget = new QTextEdit( this );
       restWidget->setText( 
              "Editor; nimmt den restlichen Platz ein.\n"
              "wird in beide Richtungen gedehnt:\n"
              " - horizontal, da neben ihm kein weiteres Widget\n"
              " - vertikal, da direkt im Toplevel-Layout-Manager" );
       restWidget->setFrameStyle( QFrame::Box | QFrame::Sunken );
       restWidget->setPaletteBackgroundColor( Qt::white );
       topLayout->addWidget( restWidget );
         //... Anlegen eines Grid-Layouts, das eine vertikale
         //... Tabelle von QLabel/QLineEdit-Paaren enthaelt, neben
         //... der ein Bild mit Laufbalken eingeblendet wird.
         //.....................................................
       const int zeilenZahl     = 5;
       const int labelSpalte    = 0;
       const int lineeditSpalte = 1;
       const int bildSpalte     = 2;
          // Zwischenraum zwischen den Widgets im Grid-Layout = 10 Pixel
       QGridLayout *grid = new QGridLayout( topLayout, 0, 0, 10 );
       for ( int z=0; z < zeilenZahl; z++ ) {
          QLineEdit *ed = new QLineEdit( this );
          grid->addWidget( ed, z, lineeditSpalte );   
          QLabel *label = new QLabel( this );
          QString s;
          if      ( z==0 ) s.sprintf( "&Name" );
          else if ( z==1 ) s.sprintf( "&Vorname" );
          else if ( z==2 ) s.sprintf( "&Strasse" );
          else if ( z==3 ) s.sprintf( "PLZ/&Wohnort" );
          else if ( z==4 ) s.sprintf( "&Beruf" );
          label->setText( s );
          grid->addWidget( label, z, labelSpalte );
          label->setBuddy( ed ); // Beschleuniger fr QLineEdit-Widgets
                                 // einschalten, um Tastatur-Fokus mittels
                                 // Alt-N, ALt-V, Alt-S, Alt-W oder Alt-B
                                 // auf entspr. LineEdit-Widget zu legen.
       }
         // Das eingeblendete Bild mit Laufbalken nimmt den ganzen
         // vertikalen Platz des Grids  (Zeilen 0 bis zeilenZahl)
         // ein und ist in Spalte 2 untergebracht
       QPixmap bild;
       bild.load( "lausbub.xpm" );
       QScrollView *sv = new QScrollView( this );
       QLabel *bildLabel = new QLabel( sv->viewport() );
        bildLabel->setPixmap( bild );
        bildLabel->adjustSize();
        sv->addChild( bildLabel );
       grid->addMultiCellWidget( sv, 0, -1, bildSpalte, bildSpalte );
         // Die Labels nehmen statisch den fuer sie erforderlichen Platz ein
         // und werden nicht gedehnt.
         // Den restl. horizontalen Platz teilen sich die QLineEdit-Widgets
         // mit dem Bild im Verhltnis 1 zu 2.
       grid->setColStretch( lineeditSpalte, 1 );
       grid->setColStretch( bildSpalte, 2 );
         // Hinzufuegen einer Statuszeile, die eine feste Hoehe hat,
         // aber immer den ganzen verfuegbaren horizontalen Platz nutzt
         //.....................................................
       QLabel* sb = new QLabel( this );
        sb->setText("Koennte eine Statuszeile sein....");
        sb->setFrameStyle( QFrame::Panel | QFrame::Sunken );
        sb->setFixedHeight( sb->sizeHint().height() );
        sb->setAlignment( AlignVCenter | AlignLeft );
        sb->setPaletteBackgroundColor( Qt::yellow );
        topLayout->addWidget( sb );
   }
   ~MeinLayout() { }
};
//................................................................ main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    QWidget *l = new MeinLayout;
    l->setGeometry( 0, 0, 800, 600 );
    a.setMainWidget(l);
    l->show();
    return a.exec();
}
