#include <qapplication.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlcdnumber.h>
#include <qslider.h>
#include <qcolor.h>
#include <qfont.h>
#include <qlineedit.h>

class LCDZahl : public QWidget
{
   Q_OBJECT   // notwendig, da eigene Slots enthalten sind

   public:
      LCDZahl( const QString&           titel,
               int                      ziffernZahl,
               QLCDNumber::Mode         zsystem,
               int                      base,
               QColor                   hintergrundFarbe,
               QLCDNumber::SegmentStyle stil,
               QWidget*                 parent ) : QWidget( parent ) {
         basis = base;
         QLabel*    text = new QLabel( this );
          text->setGeometry( 0, 0, 150, 20 );
          text->setText( titel );
          text->setAlignment( QLabel::AlignCenter );
  
         QLCDNumber *lcd  = new QLCDNumber( this );
          lcd->setGeometry( 0, 20, 150, 150 );
          lcd->setNumDigits( ziffernZahl );
          lcd->setMode( zsystem );
          lcd->setPaletteBackgroundColor( hintergrundFarbe );
          lcd->setSegmentStyle( stil );
          lcd->display( 0 );
  
         slider = new QSlider( 0, 65535, 1, 100, Horizontal, this );
          slider->setGeometry( 0, 180, 150, 20 );
          slider->setValue( 0 );
          QObject::connect( slider, SIGNAL( valueChanged(int) ),
                            lcd,    SLOT( display(int) ) );
          QObject::connect( slider, SIGNAL( valueChanged(int) ),
                                    SIGNAL( valueChanged(int) ) );
  
         eingabe = new QLineEdit( this );
          eingabe->setGeometry( 0, 210, 150, 30 );
          eingabe->setText( "0" );
          QObject::connect( eingabe, SIGNAL( textChanged(const QString &) ),
                            this,    SLOT( neuEingabe(const QString &) ) );
      }

      int value() const { return slider->value(); }

   public slots:
      void setValue( int value ) { 
         QString t;
         t.setNum( value, basis );
         if ( altText != t ) {
            eingabe->setText( t );
            altText = t;
         }
         slider->setValue( value );
      }
      void neuEingabe( const QString &text ) {
         int  value;
         bool ok = true;
         QString t = text;
         if ( t.isEmpty() )
            eingabe->setText( t = "0" );
         value = t.toInt( &ok, basis );
         if ( value > 65535 )
            ok = false;
         if ( !ok )
            eingabe->setText( altText );
         else
            emit valueChanged( value );
      }

   signals:
      void valueChanged( int );

   private:
      QSlider   *slider;
      QLineEdit *eingabe;
      int       basis;
      QString   altText;
};

#include "lcdwandel2.moc"

int main( int argc, char *argv[] )
{
   QApplication myapp( argc, argv );
   
   QWidget* mywidget = new QWidget();
   mywidget->setGeometry( 10, 10, 600, 320 );

   mywidget->setMinimumSize( 200, 300 );

   QColor blaugruen( 0, 255, 255 );
   LCDZahl* z[4];
   z[0] = new LCDZahl( "Dual", 16, QLCDNumber::BIN, 2, blaugruen,
                       QLCDNumber::Flat, mywidget );
    z[0]->setGeometry(0, 10, 150, 250);
   z[1] = new LCDZahl( "Oktal", 6, QLCDNumber::OCT, 8,
                       Qt::darkBlue,
                       QLCDNumber::Outline, mywidget );
    z[1]->setGeometry(150, 10, 150, 250);
   z[2] = new LCDZahl( "Dezimal", 5, QLCDNumber::DEC, 10,
                       Qt::darkGray,
                       QLCDNumber::Filled, mywidget );
    z[2]->setGeometry(300, 10, 150, 250);
   z[3] = new LCDZahl( "Hexadezimal", 4, QLCDNumber::HEX, 16,
                       Qt::darkYellow,
                       QLCDNumber::Filled, mywidget );
    z[3]->setGeometry(450, 10, 150, 250);

   for (int i=0; i<4; i++)
      for (int j=0; j < 4; j++)
         if (i != j)
            QObject::connect( z[i], SIGNAL(valueChanged(int)),
                              z[j], SLOT(setValue(int)) );

   QPushButton* quit = new QPushButton( "Quit", mywidget );
    quit->setGeometry( 0, 270, 600, 40 );
    quit->setFont( QFont( "Times", 18, QFont::Bold ) );
    quit->setPaletteBackgroundColor( Qt::yellow );
    QObject::connect( quit, SIGNAL(clicked()), qApp, SLOT(quit()) );

   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
