#include <qapplication.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qdrawutil.h>
#include <qtimer.h>
#include <stdlib.h>
#include <time.h>

#define MAXX  800
#define MAXY  600

const int  breite = 15,
           hoehe  = 15;

//........................................................ Klasse MalFenster
class MalFenster : public QWidget
{
   Q_OBJECT
public:
   MalFenster() {
      t = new QTimer( this );
      delay = 200;
      connect( t, SIGNAL(timeout()), this, SLOT(naechsteGeneration()) );
      t->start( delay, false );
      maxx = MAXX/breite,
      maxy = MAXY/hoehe;
      neueSimulation();
   }

protected:
   virtual void mousePressEvent( QMouseEvent *e ) {
      if ( e->button() == LeftButton ) {
         delay += 10;
         t->changeInterval( delay );
      } else if ( e->button() == RightButton ) {
         delay -= 10;
         if ( delay < 10 )
            delay = 10;
         t->changeInterval( delay );
      } else
         neueSimulation();
   }

   virtual void paintEvent ( QPaintEvent * ) {
      QPainter maler( this );
      maler.setPen( NoPen );
      QBrush leben( Qt::blue );
      QBrush tot( Qt::lightGray );
      for ( int x=0; x<maxx; x++ )
         for ( int y=0; y<maxy; y++ )
            if ( altGen[x][y] != neuGen[x][y] ) {
               maleZelle( &maler, neuGen[x][y] ? false : true,
                          neuGen[x][y] ? &leben : &tot, x, y );
               altGen[x][y] = neuGen[x][y];
            }
   }

private:
   void neueSimulation() {
         //................ Dichte der Population zufaellig
      int  dichte = rand() % 51+20; // 20..70%
      int  grenze = RAND_MAX/100*dichte;

         //................................. Initialisieren
      for ( int x=0; x<maxx; x++ )
         for ( int y=0; y<maxy; y++ ) {
            neuGen[x][y] = rand()<=grenze ? 1 : 0;
            altGen[x][y] = 2;
         }
      repaint( true );
   }

private slots:
   void naechsteGeneration() {
      int  x, y;
      for ( x=0; x<maxx; x++)
         for ( y=0; y<maxy; y++ ) {
            int l = (x-1) < 0     ? maxx-1 : x-1;      
            int r = (x+1) >= maxx ? 0      : x+1;      
            int o = (y-1) < 0     ? maxy-1 : y-1;      
            int u = (y+1) >= maxy ? 0      : y+1;      
            int sum = altGen[l][o] + altGen[l][y] + altGen[l][u] +
                      altGen[x][u] + altGen[r][u] + altGen[r][y] +
                      altGen[r][o] + altGen[x][o];
           
            if ( sum < 2 || sum > 3 )
               neuGen[x][y] = 0;
            else if ( sum == 3 )
               neuGen[x][y] = 1;
            else
               neuGen[x][y] = altGen[x][y];
         }
      repaint( false );
   }

private:
   void maleZelle( QPainter *p, bool sunken, QBrush *fill, int x, int y ) {
      qDrawShadePanel( p, x*breite, y*hoehe, breite, hoehe,
                       colorGroup(), sunken, 1, fill );
   }

   int  altGen [MAXX+2][MAXY+2],
        neuGen [MAXX+2][MAXY+2],
        maxx, maxy, 
        delay;
   QTimer *t;
};

#include "life.moc"

//..................................................................... main
int main( int argc, char* argv[] )
{
   QApplication myapp( argc, argv );

   srand( time(NULL) );
   MalFenster* mywidget = new MalFenster;
   mywidget->setGeometry( 0, 0, MAXX, MAXY );
   mywidget->setFixedSize( MAXX, MAXY );

   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
