#include <qapplication.h>
#include <qpushbutton.h>
#include <qslider.h>
#include <qlcdnumber.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qspinbox.h>

//................................................................. SpinListLCD
class SpinListLCD : public QWidget {
   Q_OBJECT
public:
   SpinListLCD( QWidget *parent, const char* text ) : QWidget( parent, text ) {
        //... Text-Label
      name = new QLabel( this );
      name->setText( text );
      name->setGeometry( 0, 10, 15, 30 );
        //...Schiebebalken
      schiebbalk = new QSlider( 0, 255, 5, 120, QSlider::Horizontal, this );
      schiebbalk->setTickmarks( QSlider::Below );
      schiebbalk->setTickInterval( 10 );
      schiebbalk->setGeometry( 20, 10, 200, 30 );
        //... LCD-Anzeige
      lcdzahl = new QLCDNumber( 3, this );
      lcdzahl->setSegmentStyle( QLCDNumber::Filled );
      lcdzahl->setPalette( QPalette( Qt::darkGray, Qt::darkGray ) );
      lcdzahl->setGeometry( 230, 10, 60, 30 );
      lcdzahl->display( 120 );
        //... Drei Spinboxen
      spinbox = new QSpinBox( -1, 255, 1, this );
      spinbox->setValue( 120 );  // Startwert
      spinbox->setSpecialValueText( "Default" );  // ist: 120
      spinbox->setWrapping( true );
      spinbox->setGeometry( 310, 10, 80, 30 );
        //... Verbinde Schiebebalken und Spinbox mit LCD-Anzeige
      QObject::connect( schiebbalk, SIGNAL( valueChanged( int ) ), 
                        this,       SLOT( aktualisiereSpinListLCD( int ) ) );
      QObject::connect( spinbox, SIGNAL( valueChanged( int ) ), 
                        this,    SLOT( aktualisiereSpinListLCD( int ) ) );
   }
signals:
   void farbeGeaendert( int farbwert );
private slots:
   void aktualisiereSpinListLCD( int wert ) {
      schiebbalk->setValue( wert==-1 ? 120 : wert );
      spinbox->setValue( wert );
      lcdzahl->display( wert==-1 ? 120 : wert );
      emit farbeGeaendert( wert==-1 ? 120 : wert );
   }
private:
   QLabel*     name;
   QSlider*    schiebbalk;
   QLCDNumber* lcdzahl;
   QSpinBox*   spinbox;
};
//................................................................... FarbLabel
class FarbLabel : public QLabel {
   Q_OBJECT
public:
   FarbLabel( QWidget *parent ) : QLabel( parent ) {
      setPaletteBackgroundColor( QColor( rot=120, gruen=120, blau=120 ) );
   }
public slots:
   void rotNeueFarbe( int neuefarbe ) {
      setPaletteBackgroundColor( QColor( rot=neuefarbe, gruen, blau ) );
   }
   void gruenNeueFarbe( int neuefarbe ) {
      setPaletteBackgroundColor( QColor( rot, gruen=neuefarbe, blau ) );
   }
   void blauNeueFarbe( int neuefarbe ) {
      setPaletteBackgroundColor( QColor( rot, gruen, blau=neuefarbe ) );
   }
private:
   int rot, gruen, blau;
};
#include "listspin.moc"
//........................................................................ main
int main( int argc, char* argv[] ) {
   QApplication myapp( argc, argv );
   QWidget* mywidget = new QWidget();
   mywidget->setGeometry( 100, 100, 540, 250 );
     //................ Erzeugen von drei SpinListLCD-Balken
   SpinListLCD* rotBalken = new SpinListLCD( mywidget, "R" );
   rotBalken->setGeometry( 10, 30, 390, 60 );
   SpinListLCD* gruenBalken = new SpinListLCD( mywidget, "G" );
   gruenBalken->setGeometry( 10, 100, 390, 60 );
   SpinListLCD* blauBalken = new SpinListLCD( mywidget, "B" );
   blauBalken->setGeometry( 10, 170, 390, 60 );
     //................ Erzeugen einer Farbanzeige
   FarbLabel* farbAnzeige = new FarbLabel( mywidget );
   farbAnzeige->setGeometry( 420, 30, 100, 200 );
     //................ Verbinde RGB-Balken mit Farbanzeige
   QObject::connect( rotBalken, SIGNAL( farbeGeaendert( int ) ), 
                     farbAnzeige, SLOT( rotNeueFarbe( int ) ) );
   QObject::connect( gruenBalken, SIGNAL( farbeGeaendert( int ) ), 
                     farbAnzeige, SLOT( gruenNeueFarbe( int ) ) );
   QObject::connect( blauBalken, SIGNAL( farbeGeaendert( int ) ), 
                     farbAnzeige, SLOT( blauNeueFarbe( int ) ) );
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
