#include <qapplication.h>
#include <qslider.h>
#include <qspinbox.h>
#include <qlabel.h>

//........................................................ Klasse SpinSlider
class SpinSlider : public QWidget
{
   Q_OBJECT   // notwendig, da eigene Slots enthalten sind

public:
   SpinSlider( bool horizontal, QWidget *parent, const char* name=0 )
                                                : QWidget( parent, name ) {
      //................ Erzeugen von drei Spinboxen
      spinbox = new QSpinBox(  0, 400, 1, this );
      spinbox->setValue( 100 );
      spinbox->setButtonSymbols( QSpinBox::PlusMinus );
      spinbox->setGeometry( 0, 0, 70, 30 );

      //................ Erzeugen eines Schiebebalken
      schiebbalk = new QSlider( 0, 400, 5, 100,
                       horizontal ? QSlider::Horizontal : QSlider::Vertical,
                       this );
      if ( horizontal )
         schiebbalk->setGeometry( 70, 0, 400, 30 );
      else
         schiebbalk->setGeometry( 70, 0, 30, 400 );

      //................ Verbinde Schiebebalken und LCD-Anzeige
      QObject::connect( schiebbalk, SIGNAL( valueChanged( int ) ), 
                        this, SLOT( aktualisiereSpinSlider( int ) ) );

      //................ Verbinde Spinbox und LCD-Anzeige
      QObject::connect( spinbox, SIGNAL( valueChanged( int ) ), 
                        this, SLOT( aktualisiereSpinSlider( int ) ) );
   }

signals:
   void groesseGeaendert( int wert );

private slots:
   void aktualisiereSpinSlider( int wert ) {
      schiebbalk->setValue( wert );
      spinbox->setValue( wert );
      emit groesseGeaendert( wert );
   }

private:
   QSlider   *schiebbalk;
   QSpinBox  *spinbox;
};

//......................................................... Klasse FarbLabel
class FarbLabel : public QLabel
{
   Q_OBJECT   // notwendig, da eigene Slots enthalten sind

public:
   FarbLabel( QWidget *parent ) : QLabel( parent ) {
      setPalette( QPalette( Qt::white, Qt::white ) );
      rechteck = new QLabel( this );
      rechteck->setPaletteBackgroundColor( Qt::blue );
      rechteck->setGeometry( 0, 0, breite=100, hoehe=100 );
      rechteck->show();
      show();
   }

public slots:
   void neueBreite( int b ) {
      rechteck->setGeometry( 0, 0, breite = b, hoehe );
   }
   void neueHoehe( int h ) {
      rechteck->setGeometry( 0, 0, breite, hoehe=h );
   }

private:
   int    breite, hoehe;
   QLabel *rechteck;
};

#include "lspin.moc"

//.................................................................... main
int main( int argc, char *argv[] )
{
   QApplication myapp( argc, argv );
   
   QWidget* mywidget = new QWidget;
   mywidget->setGeometry( 10, 10, 500, 500 );
  
   //................ Erzeugen von drei SpinSlider-Balken
   SpinSlider* waagrecht = new SpinSlider( true, mywidget );
   waagrecht->setGeometry( 30, 70, 470, 30 );
   SpinSlider* senkrecht = new SpinSlider( false, mywidget );
   senkrecht->setGeometry( 0, 100, 100, 400 );

   //................ Erzeugen einer Farbanzeige
   FarbLabel* rechteckAnzeige = new FarbLabel( mywidget );
   rechteckAnzeige->setGeometry( 100, 100, 400, 400 );

   //................ Verbinde RGB-Balken mit rechteckAnzeige
   QObject::connect( waagrecht,       SIGNAL( groesseGeaendert( int ) ), 
                     rechteckAnzeige, SLOT( neueBreite( int ) ) );
   QObject::connect( senkrecht,       SIGNAL( groesseGeaendert( int ) ), 
                     rechteckAnzeige, SLOT( neueHoehe( int ) ) );

   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
