#include <qapplication.h>
#include <qwidget.h>
#include <qmenubar.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <qmainwindow.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
//...................................... Texte fuer QWhatsThis .........
const char * dateiNewText = 
   "Klicke auf diesen Button, um ein neues Fenster zu erzeugen." 
   "Dies ist auch moeglich mit dem <b>Neu-Menueeintrag </b>"
   " vom Datei-Menue.";
const char * dateiOpenText = 
   "<img source=\"fileopen\"> \n"
   "Ein Klick auf diesen Button oeffnet eine "
   "<em>neue Datei </em>.<br><br>"
   "Dies ist auch moeglich mit dem <b>Oeffnen-Menueeintrag </b>"
   " vom Datei-Menue.";
const char * dateiSaveText = 
   "Klicke auf diesen Button, um die aktuelle Datei zu sichern." 
   "Dies ist auch moeglich mit dem <b>Sichern-Menueeintrag </b>"
   " vom Datei-Menue.";
const char * dateiPrintText =
   "Klicke auf diesen Button, um die aktuelle Datei zu drucken." 
   "Dies ist auch moeglich mit dem <b>Drucken-Menueeintrag </b>"
   " vom Datei-Menue.";
//........................................ HauptFenster-Klasse .........
class HauptFenster : public QMainWindow {
    Q_OBJECT
public:
    HauptFenster();
signals:
    void    anzeige( const QString& );
private slots:
    void status(const QString & text) { statusBar()->message( text, 3000 ); }
    void neu()    { emit anzeige( "Datei/Neu" );        }
    void open()   { emit anzeige( "Datei/Oeffnen" );    }
    void save()   { emit anzeige( "Datei/Speichern" );  }
    void print()  { emit anzeige( "Datei/Drucken" );    }
    void close()  { emit anzeige( "Datei/Schliessen" ); }
    void normal() { emit anzeige( "Schrift/Normal" );   }
    void fett()   { emit anzeige( "Schrift/Fett" );     }
    void kursiv() { emit anzeige( "Schrift/Kursiv" );   }
    void about()  { QMessageBox::about( this, "QMainWindow-Beispiel",
                                  "Dieses Beispiel demonstriert die\n"
                                  "Eigenschaften von QMainWindow");
    }
    void aboutQt()  { QMessageBox::aboutQt( this, "QMainWindow-Beispiel" ); }
private:
    QMenuBar     *menu;
    QToolBar     *dateiTools, *schriftTools;
    QLabel       *label, *normalStatus, *permanentStatus;
    QPixmap      newP, openP, saveP, printP, normalP, fettP, kursivP;
    void erzeugeToolBars( void );
};
#include "mainwindow.moc"
//................................... HauptFenster-Konstruktor .........
HauptFenster::HauptFenster() : QMainWindow( 0, "Beispiel zu QMainWindow" ),
              newP("new.xpm"),   openP("open.xpm"),
              saveP("save.xpm"), printP("print.xpm"),
              normalP("normal.xpm"),  fettP("fett.xpm"), kursivP("kursiv.xpm") {
    int id;
    erzeugeToolBars();
      //... Anlegen der Mens ........
    QPopupMenu *datei = new QPopupMenu( this );
    datei->insertItem( newP, "&Neu", this, SLOT(neu()), CTRL+Key_N );
    id = datei->insertItem( openP, "Oe&ffnen",  this, SLOT(open()), CTRL+Key_O);
    datei->setWhatsThis( id, dateiOpenText );
    id = datei->insertItem( saveP, "&Speichern", this, SLOT(save()), CTRL+Key_S);
    datei->setWhatsThis( id, dateiSaveText );
    datei->insertItem( "S&chliessen", this, SLOT(close()), CTRL+Key_W);
    datei->insertSeparator();
    id = datei->insertItem( printP, "&Drucken", this, SLOT(print()), CTRL+Key_P);
    datei->setWhatsThis( id, dateiPrintText );
    datei->insertSeparator();
    datei->insertItem( "&Beenden", qApp, SLOT(quit()), CTRL+Key_Q);
    QPopupMenu *hilfe = new QPopupMenu( this );
    hilfe->insertItem( "&About", this, SLOT(about()), CTRL+Key_H );
    hilfe->insertItem( "About &Qt", this, SLOT(aboutQt()) );
    hilfe->insertSeparator();
    hilfe->insertItem( "What's &This", this, SLOT(whatsThis()), SHIFT+Key_F1);
    menu = new QMenuBar( this );
    menu->insertItem( "&Datei", datei );
    menu->insertSeparator();
    menu->insertItem( "&Hilfe", hilfe );
    label = new QLabel( this );
    label->setFrameStyle( QFrame::Box | QFrame::Raised );
    label->setLineWidth( 5 );
    label->setAlignment( AlignCenter );
    label->setPaletteBackgroundColor( Qt::white );
      //... Konfigurieren der Statusbar .......
    normalStatus = new QLabel( "Normaler Status", statusBar() );
    normalStatus->setPaletteBackgroundColor( QColor( 0, 255, 255 ) );
    statusBar()->addWidget( normalStatus, 0, false );
    permanentStatus = new QLabel( "Beispiel  zu  QMainWindow", statusBar() );
    permanentStatus->setPaletteBackgroundColor( QColor( 255, 255, 0 ) );
    permanentStatus->setAlignment( AlignCenter );
    statusBar()->addWidget( permanentStatus, 0, true );
    statusBar()->message( "Bin bereit", 4000 );
    setCentralWidget( label );
    connect( this,  SIGNAL(anzeige(const QString&)),
             label, SLOT(setText(const QString&)) );
}
//............................................ erzeugeToolBars .........
void HauptFenster::erzeugeToolBars( void ) {
      //... Anlegen der dateiTools-Werkzeugleiste ........
    dateiTools = new QToolBar( this );
    QToolButton *dateiNew = new QToolButton(newP, "Neue Datei", QString::null,
                                            this, SLOT(neu()), dateiTools );
    QToolButton *dateiOpen = new QToolButton(openP,"Oeffne Datei", QString::null,
                                             this, SLOT(open()), dateiTools );
    QToolButton *dateiSave = new QToolButton(saveP, "Speichere Datei",
                                 QString::null, this, SLOT(save()), dateiTools );
    QToolButton *dateiPrint = new QToolButton( printP, "Drucke Datei",
                                QString::null, this, SLOT(print()), dateiTools );
    addDockWindow( dateiTools, "Datei-Operationen", Top, false );
    QToolTipGroup * dt = new QToolTipGroup( dateiTools );
    QToolTip::add( dateiNew, "Neue Datei", dt, "Legt ein neues Fenster an");
    QToolTip::add( dateiOpen, "Oeffne Datei", dt, "Oeffnet eine neue Datei");
    QToolTip::add( dateiSave, "Speichere Datei", dt, "Sichert aktuelle Datei");
    QToolTip::add( dateiPrint, "Drucke Datei", dt, "Druckt die aktuelle Datei");
    connect( dt, SIGNAL( showTip( const QString & ) ),
             this, SLOT( status( const QString & ) ) );
    (void)QWhatsThis::whatsThisButton( dateiTools );
    QWhatsThis::add( dateiNew, dateiNewText );
    QWhatsThis::add( dateiOpen, dateiOpenText );
    QWhatsThis::add( dateiSave, dateiSaveText );
    QWhatsThis::add( dateiPrint, dateiPrintText );
      //... Folg. Zeile ist notwendig, damit Pixmap
      //... fuer Datei-Oeffnen im Hilfstext angezeigt wird
    QMimeSourceFactory::defaultFactory()->setPixmap("fileopen", openP);
      //... Anlegen der schriftTools-Werkzeugleiste ........
    schriftTools = new QToolBar( this );
    QToolButton *normalSchrift = new QToolButton( normalP, "Normal",
                            QString::null, this, SLOT(normal()), schriftTools );
    schriftTools->addSeparator();
    QToolButton *fettSchrift = new QToolButton( fettP, "Fett", QString::null,
                                             this, SLOT(fett()), schriftTools );
    schriftTools->addSeparator();
    QToolButton *kursivSchrift = new QToolButton( kursivP, "Kursiv",
                            QString::null, this, SLOT(kursiv()), schriftTools );
    normalSchrift->setToggleButton( true );
    fettSchrift->setUsesTextLabel( true );
    fettSchrift->setToggleButton( true );
    kursivSchrift->setUsesTextLabel( true );
    kursivSchrift->setToggleButton( true );
    addDockWindow( schriftTools, "Schriftarten", Top, false );
    QToolTipGroup *st = new QToolTipGroup( schriftTools );
    QToolTip::add( normalSchrift, "Normal", st, "Normale Schrift");
    QToolTip::add( fettSchrift, "Fett", st, "Fette Schrift");
    QToolTip::add( kursivSchrift, "Kursiv", st, "Kursive Schrift");
    connect( st,   SIGNAL( showTip( const QString & ) ),
             this, SLOT( status( const QString & ) ) );
    dateiTools->show();
    schriftTools->show();
}
//....................................................... main .........
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    HauptFenster m;
    m.setGeometry( 10, 10, 400, 300 );
    a.setMainWidget( &m );
    m.show();
    return a.exec();
}
