#include <qapplication.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qpixmap.h>

//------------------------------------------ class MalFenster --------------
class MalFenster : public QWidget
{
public:
   MalFenster() {
       setBackgroundMode( NoBackground ); // beim Neumalen nicht loeschen
   }

protected:
      //... Diese virtuelle Methode wird immer aufgerufen, wenn Benutzer
      //... auf eine Maustaste ber diesem Fenster klickt
   virtual void mousePressEvent( QMouseEvent* event) {
      letztepos = event->pos(); // Speichere die Koordinaten vom Event
   }

      //... Diese virtuelle Methode wird immer aufgerufen, wenn Benutzer
      //... die Maus bei gedrueckter Taste bewegt
   virtual void mouseMoveEvent( QMouseEvent* event) {
      QPainter wMaler; // erzeugt QPainter-Object zum Zeichnen im Window
      QPainter pMaler; // erzeugt QPainter-Object zum Zeichnen im Puffer

      wMaler.begin( this );    // Start des Zeichnens im Window
      pMaler.begin( &puffer ); // Start des Zeichnens im Puffer
   
      wMaler.drawLine( letztepos, event->pos() ); // zeichnet Linie im Window
      pMaler.drawLine( letztepos, event->pos() ); // zeichnet Linie im Puffer

      wMaler.end();   // Beendigung des Zeichnens im Window
      pMaler.end();   // Beendigung des Zeichnens im Puffer

      letztepos = event->pos();  // merkt sich die akt. Mausposition
   }

      //... Diese virtuelle Methode wird immer aufgerufen, wenn ein
      //... neues Malen erforderlich ist, z.B. wenn das Bild durch
      //... ein anderes Fenster verdeckt wurde und nun wieder
      //... sichtbar sein soll.
   virtual void paintEvent( QPaintEvent* ) {
      bitBlt( this, 0, 0, &puffer ); // kopiert Puffer-Bild auf Bildschirm
   }

      //... Diese virtuelle Methode wird immer aufgerufen, wenn die
      //... Fenstergroesse sich aendert.
   virtual void resizeEvent( QResizeEvent* event ) {
      QPixmap save( puffer );         // Sichern des Bildes im Puffer `save'
      puffer.resize( event->size() ); // Puffergroesse auf neue
                                      // Bildgroesse setzen
      puffer.fill( white );  // Inhalt des Puffers mit weiss ueberschreiben
      bitBlt( &puffer, 0, 0, &save ); // Temporaeren Puffer `save' in den
                                      // Puffer `puffer' kopieren.
   }

private:
   QPoint      letztepos;
   QPixmap     puffer;
};

//------------------------------------------------------ main --------------
int main( int argc, char* argv[] )
{
   QApplication myapp( argc, argv );

   MalFenster* mywidget = new MalFenster();
   mywidget->setGeometry( 50, 50, 400, 400 );
 
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
