#include <qapplication.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qscrollview.h>

enum { WEISS, SCHWARZ, ROT, BLAU, GRUEN, GELB };

//------------------------------------------ class MalFlaeche --------------
class MalFlaeche : public QWidget
{
   Q_OBJECT  // notwendig, da MalFlaeche Slots enthaelt

public:
   MalFlaeche() {
       farbe = black;
       setBackgroundMode( NoBackground ); // beim Neumalen nicht loeschen
   }

   ~MalFlaeche() { } // noch kein Code fuer den Destruktor

public slots:
   void setzeFarbe( QColor neu_farbe ) { farbe = neu_farbe; }

protected:
      //... Diese virtuelle Methode wird immer aufgerufen, wenn Benutzer
      //... auf eine Maustaste ber diesem Fenster klickt
   virtual void mousePressEvent( QMouseEvent* event) {
      letztepos = event->pos(); // Speichere die Koordinaten vom Event
   }

      //... Diese virtuelle Methode wird immer aufgerufen, wenn Benutzer
      //... die Maus bei gedrueckter Taste bewegt
   virtual void mouseMoveEvent( QMouseEvent* event) {
      QPainter wMaler; // erzeugt QPainter-Object zum Zeichnen im Window
      QPainter pMaler; // erzeugt QPainter-Object zum Zeichnen im Puffer

      wMaler.begin( this );    // Start des Zeichnens im Window
      pMaler.begin( &puffer ); // Start des Zeichnens im Puffer
   
      if ( event->state() & RightButton ) {  // bei rechter Maustaste
         QPen stift;                         // dicker malen
         stift.setWidth( 15 );    // Stiftdicke auf 15 setzen
         stift.setColor( farbe ); // Stiftfarbe setzen
         wMaler.setPen( stift );  // Stift fuer Window setzen
         pMaler.setPen( stift );  // Stift fuer Puffer setzen
      } else {
         wMaler.setPen( farbe );  // Stiftfarbe fuer Window setzen
         pMaler.setPen( farbe );  // Stiftfarbe fuer Puffer setzen
      }

      wMaler.drawLine( letztepos, event->pos() ); // zeichnet Linie im Window
      pMaler.drawLine( letztepos, event->pos() ); // zeichnet Linie im Puffer

      wMaler.end();   // Beendigung des Zeichnens im Window
      pMaler.end();   // Beendigung des Zeichnens im Puffer

      letztepos = event->pos();  // merkt sich die akt. Mausposition
   }

      //... Diese virtuelle Methode wird immer aufgerufen, wenn ein
      //... neues Malen erforderlich ist, z.B. wenn das Bild durch
      //... ein anderes Fenster verdeckt wurde und nun wieder
      //... sichtbar sein soll.
   virtual void paintEvent( QPaintEvent* ) {
      bitBlt( this, 0, 0, &puffer ); // kopiert Puffer-Bild auf Bildschirm
   }

      //... Diese virtuelle Methode wird immer aufgerufen, wenn die
      //... Fenstergroesse sich aendert.
   virtual void resizeEvent( QResizeEvent* event ) {
      QPixmap save( puffer );         // Sichern des Bildes im Puffer `save'
      puffer.resize( event->size() ); // Puffergroesse auf neue
                                      // Bildgroesse setzen
      puffer.fill( white );  // Inhalt des Puffers mit weiss ueberschreiben
      bitBlt( &puffer, 0, 0, &save ); // Temporaeren Puffer `save' in den
                                      // Puffer `puffer' kopieren.
   }

      //... Diese virtuelle Methode wird immer bei einem 
      //... Maus-Doppelklick aufgerufen
   virtual void mouseDoubleClickEvent( QMouseEvent* ) {
      puffer.fill( white );  // Uebermale parallel mitgefuehrten Puffer weiss
      bitBlt( this, 0, 0, &puffer ); // und kopiere ihn ueber das Window
   }

      //... Diese virtuelle Methode wird immer aufgerufen, wenn
      //... eine Taste auf der Tastatur gedrueckt wird
   virtual void keyPressEvent( QKeyEvent* event ) {
      switch( event->key() ) {
         case Key_W:  farbe = white;  break;
         case Key_S:  farbe = black;  break;
         case Key_R:  farbe = red;    break;
         case Key_B:  farbe = blue;   break;
         case Key_G:  farbe = green;  break;
         case Key_L:  farbe = yellow; break;
      }
   }

private:
   QPoint      letztepos;
   QPixmap     puffer;
   QColor      farbe;
};

//------------------------------------------ class MalFenster --------------
class MalFenster : public QWidget
{
   Q_OBJECT  // notwendig, da MalFenster Slots enthaelt

public:
   MalFenster() {
       dateimenu = new QPopupMenu; //... erzeugt ein Datei-Menue
       dateimenu->insertItem( "&Beenden", qApp, SLOT( quit() ) ); 

       farbemenu = new QPopupMenu; //... erzeugt ein Farbe-Menue
       farbemenu->insertItem( "&Weiss",   WEISS );
       farbemenu->insertItem( "&Schwarz", SCHWARZ );
       farbemenu->insertItem( "&Rot",     ROT );
       farbemenu->insertItem( "&Blau",    BLAU );
       farbemenu->insertItem( "&Gruen",   GRUEN );
       farbemenu->insertItem( "Ge&lb",    GELB );
       QObject::connect( farbemenu, SIGNAL( activated( int ) ),
                         this, SLOT( farbeMenuSlot( int ) ) );

       hilfemenu = new QPopupMenu; //... erzeugt ein Hilfe-Menue
       hilfemenu->insertItem( "&About Malprog", this, SLOT( AboutSlot() ) );
       hilfemenu->insertItem( "About &Qt", this, SLOT( AboutQtSlot() ) );

       menubalken = new QMenuBar( this ); //... erzeugt einen Menue-Balken
       menubalken->insertItem( "&Datei", dateimenu );
       menubalken->insertItem( "&Farbe", farbemenu );
       menubalken->insertSeparator();
       menubalken->insertItem( "&Hilfe", hilfemenu );

       malflaeche = new MalFlaeche();
       malflaeche->setGeometry( 0, 0, 1000, 1000 );

       scrollview = new QScrollView( this );
       scrollview->setGeometry( 0, menubalken->height(), 
                                width(), height() - menubalken->height() );
       scrollview->addChild( malflaeche );

       QObject::connect( this, SIGNAL( farbeGeaendert( QColor ) ),
                         malflaeche, SLOT( setzeFarbe( QColor ) ) );
   }

   ~MalFenster() { } // noch kein Code fuer den Destruktor

private slots:
   void AboutSlot() {
      QMessageBox::information( this, "malprog4",
                                "4. Version des Malprogramms\n");
   }

   void AboutQtSlot()   { QMessageBox::aboutQt( this, "About Qt" ); }

   void farbeMenuSlot( int farbeId ) {
      switch( farbeId ) {
         case WEISS:   emit farbeGeaendert( white );  break;
         case SCHWARZ: emit farbeGeaendert( black );  break;
         case ROT:     emit farbeGeaendert( red );    break;
         case BLAU:    emit farbeGeaendert( blue );   break;
         case GRUEN:   emit farbeGeaendert( green );  break;
         case GELB:    emit farbeGeaendert( yellow ); break;
      }
   }

signals:
   void farbeGeaendert( QColor );

protected:
   virtual void resizeEvent( QResizeEvent* ) {
      scrollview->setGeometry( 0, menubalken->height(),
                               width(), height() - menubalken->height() );
   }

private:
   QMenuBar*    menubalken;
   QPopupMenu*  dateimenu;
   QPopupMenu*  farbemenu;
   QPopupMenu*  hilfemenu;
   QScrollView* scrollview;
   MalFlaeche*  malflaeche;
};

#include "malprog4.moc"

//------------------------------------------------------ main --------------
int main( int argc, char* argv[] )
{
   QApplication myapp( argc, argv );

   MalFenster* mywidget = new MalFenster();
   mywidget->setGeometry( 50, 50, 400, 400 );
 
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
