#include <qapplication.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qscrollview.h>
#include <qcursor.h>

enum { WEISS, SCHWARZ, ROT, BLAU, GRUEN, GELB };

class MalFenster;

//------------------------------------------ class MalFlaeche --------------
class MalFlaeche : public QWidget
{
   Q_OBJECT  // notwendig, da MalFlaeche Slots enthaelt

public:
   MalFlaeche( MalFenster *); 
   ~MalFlaeche() { delete popupmenu; } // loescht Popup-Menue
   QColor getFarbe()  { return farbe;  }
   int    getBreite() { return breite; }

public slots:
   void setzeFarbeBreite( QColor neue_farbe, int neue_breite ) {
      farbe  = neue_farbe;
      breite = neue_breite;
   }

   void sendeFarbeGeaendertSignal( int menupunkt ) {
      switch ( menupunkt ) {
         case WEISS:   emit stiftGeaendert( white, breite );  break;
         case SCHWARZ: emit stiftGeaendert( black, breite );  break;
         case ROT:     emit stiftGeaendert( red, breite );    break;
         case BLAU:    emit stiftGeaendert( blue , breite);   break;
         case GRUEN:   emit stiftGeaendert( green, breite );  break;
         case GELB:    emit stiftGeaendert( yellow, breite ); break;
      }
   }
   void sendeBreiteGeaendertSignal( int breite ) {
      emit stiftGeaendert( farbe, breite );
   }
   void sendeBreiteTasteGeaendertSignal( int delta ) {
      if (breite+delta >= 1 && breite+delta <= 20)
         emit stiftGeaendert( farbe, breite+delta );
   }

protected:
   virtual void mousePressEvent( QMouseEvent* event) {
      if ( event->button() == RightButton )
         popupmenu->exec( QCursor::pos() );
      else
         letztepos = event->pos(); // Speichere die Koordinaten vom Event
   }

   virtual void mouseMoveEvent( QMouseEvent* );

   virtual void paintEvent( QPaintEvent* ) {
      bitBlt( this, 0, 0, &puffer ); // kopiert Puffer-Bild auf Bildschirm
   }

   virtual void mouseDoubleClickEvent( QMouseEvent* ) {
      LoescheFlaecheSlot();
   }

   virtual void resizeEvent( QResizeEvent* event ) {
      QPixmap save( puffer );
      puffer.resize( event->size() );
      puffer.fill( white );
      bitBlt( &puffer, 0, 0, &save );
   }

private slots:
   void LoescheFlaecheSlot() {
      puffer.fill( white );  // Uebermale parallel mitgefuehrten Puffer weiss
      bitBlt( this, 0, 0, &puffer ); // und kopiere ihn ueber das Window
   }

signals:
   void stiftGeaendert( QColor, int );

private:
   QPoint      letztepos;
   QColor      farbe;
   int         breite;
   QPixmap     puffer;
   QPopupMenu* popupmenu;
};

//------------------------------------------ class MalFenster --------------
class MalFenster : public QWidget
{
   Q_OBJECT  // notwendig, da MalFenster Slots enthaelt

public:
   MalFenster();
   ~MalFenster() { }

public slots:
   void AendereMenu( QColor farbe, int breite ) {
      QPixmap pixmap( 40, breite);
      pixmap.fill( farbe );
      menubalken->changeItem( pixmap, breitemenu_id );
      for (int i=1; i<=20; i++) {
         pixmap.resize( 40, i);
         pixmap.fill( farbe );
         breitemenu->changeItem( pixmap, breitemenusub_id[i] );
      } 
      menubalken->repaint();
   }

   virtual void keyPressEvent( QKeyEvent* event ) {
      switch (event->key()) {
         case Key_Up   :  scrollview->scrollBy(  0, -10);  break;
         case Key_Down :  scrollview->scrollBy(  0,  10);  break;
         case Key_Left :  scrollview->scrollBy(-10,   0);  break;
         case Key_Right:  scrollview->scrollBy( 10,   0);  break;
         case Key_PageUp: scrollview->scrollBy( 0,
             -QMAX(scrollview->contentsHeight()/10, 10) ); break;
         case Key_PageDown: scrollview->scrollBy( 0,
             QMAX(scrollview->contentsHeight()/10, 10) );  break;

         case Key_W:  emit farbeTasteGedrueckt(WEISS);    break;
         case Key_S:  emit farbeTasteGedrueckt(SCHWARZ);  break;
         case Key_R:  emit farbeTasteGedrueckt(ROT);      break;
         case Key_B:  emit farbeTasteGedrueckt(BLAU);     break;
         case Key_G:  emit farbeTasteGedrueckt(GRUEN);    break;
         case Key_L:  emit farbeTasteGedrueckt(GELB);     break;

         case Key_Less   : emit breiteTasteGedrueckt(-1);  break;
         case Key_Greater: emit breiteTasteGedrueckt(1);   break;
      }
   }

private slots:
   void AboutSlot() {
      QMessageBox::information( this, "malprog",
                                "5. Version des Malprogramms\n");
   }

   void AboutQtSlot()   { QMessageBox::aboutQt( this, "About Qt" ); }

signals:
   void farbeTasteGedrueckt( int );
   void breiteTasteGedrueckt( int );

protected:
   virtual void resizeEvent( QResizeEvent* ) {
      scrollview->setGeometry( 0, menubalken->height(),
                               width(), height() - menubalken->height() );
   }

private:
   QMenuBar*    menubalken;
   QPopupMenu*  dateimenu;
   QPopupMenu*  farbemenu;
   QPopupMenu*  breitemenu;
   int          breitemenu_id;
   int          breitemenusub_id[30];
   QPopupMenu*  hilfemenu;
   QScrollView* scrollview;
   MalFlaeche*  malflaeche;
};

#include "malprog5.moc"

//------------------------------- Konstruktor fuer MalFlaeche --------------
MalFlaeche::MalFlaeche(MalFenster *m)
{
   farbe = black;   // voreingestellte Farbe ist schwarz
   breite = 4;      // voreingestellte Pinselbreite ist 4
   setBackgroundMode( NoBackground );

   popupmenu = new QPopupMenu; // erzeugt ein Popup-Menu
   QPixmap pixmap( 40, 20);
   pixmap.fill( white );  popupmenu->insertItem( pixmap, WEISS);
   pixmap.fill( black );  popupmenu->insertItem( pixmap, SCHWARZ );
   pixmap.fill( red );    popupmenu->insertItem( pixmap, ROT );
   pixmap.fill( blue );   popupmenu->insertItem( pixmap, BLAU );
   pixmap.fill( green );  popupmenu->insertItem( pixmap, GRUEN );
   pixmap.fill( yellow ); popupmenu->insertItem( pixmap, GELB );
   popupmenu->insertItem( "&Clear", this, SLOT( LoescheFlaecheSlot() ) );

   QObject::connect( popupmenu, SIGNAL( activated( int ) ),
                     this, SLOT( sendeFarbeGeaendertSignal( int ) ) );
   QObject::connect( this, SIGNAL( stiftGeaendert( QColor, int ) ),
                     this, SLOT( setzeFarbeBreite( QColor, int ) ) );
   QObject::connect( this, SIGNAL( stiftGeaendert( QColor, int ) ),
                     m, SLOT( AendereMenu( QColor, int ) ) );
}

//---------------------------- mouseMoveEvent fuer MalFlaeche --------------
void MalFlaeche::mouseMoveEvent( QMouseEvent* event )
{
   QPainter wMaler; // erzeugt  QPainter-Object zum Zeichnen im Window
   QPainter pMaler; // erzeugt  QPainter-Object zum Zeichnen im Puffer

   wMaler.begin( this );     // Start des Zeichnens im Window
   pMaler.begin( &puffer ); // Start des Zeichnens im Puffer
   
   if ( (event->state() & LeftButton) && (event->state() & ShiftButton) ) {
      wMaler.setPen( QPen(farbe, 10) ); // Neue Zeichenstiftfarbe festlegen
      pMaler.setPen( QPen(farbe, 10) );
   } else {
      wMaler.setPen( QPen(farbe, breite) ); // Neue Zeichenstiftfarbe festlegen
      pMaler.setPen( QPen(farbe, breite) );
   } 
   wMaler.drawLine( letztepos, event->pos() ); // zeichnet Linie im Window
   pMaler.drawLine( letztepos, event->pos() ); // zeichnet Linie im Puffer

   wMaler.end();   // Beendigung des Zeichnens im Window
   pMaler.end();   // Beendigung des Zeichnens im Puffer

   letztepos = event->pos();  // merkt sich die akt. Mausposition
}

//------------------------------- Konstruktor fuer MalFenster --------------
MalFenster::MalFenster()
{
   malflaeche = new MalFlaeche(this);
   malflaeche->setGeometry( 0, 0, 1000, 1000 );

   dateimenu = new QPopupMenu; //... erzeugt ein Datei-Menue
   dateimenu->insertItem( "&Beenden", qApp, SLOT( quit() ) ); 

   farbemenu = new QPopupMenu; //... erzeugt ein Farbe-Menue
   farbemenu->insertItem( "&Weiss",   WEISS );
   farbemenu->insertItem( "&Schwarz", SCHWARZ );
   farbemenu->insertItem( "&Rot",     ROT );
   farbemenu->insertItem( "&Blau",    BLAU );
   farbemenu->insertItem( "&Gruen",   GRUEN );
   farbemenu->insertItem( "Ge&lb",    GELB );
   QObject::connect( farbemenu, SIGNAL( activated( int ) ),
                     malflaeche, SLOT( sendeFarbeGeaendertSignal( int ) ) );

   breitemenu = new QPopupMenu; //... erzeugt ein Breite-Menue
   for (int i=1; i<=20; i++) {
      QPixmap pixmap( 40, i);
      pixmap.fill( malflaeche->getFarbe() );
      breitemenusub_id[i] = breitemenu->insertItem( pixmap, i );
   } 
   QObject::connect( breitemenu, SIGNAL( activated( int ) ),
                     malflaeche, SLOT( sendeBreiteGeaendertSignal( int ) ) );

   hilfemenu = new QPopupMenu; //... erzeugt ein Hilfe-Menue
   hilfemenu->insertItem( "&About Malprog", this, SLOT( AboutSlot() ) );
   hilfemenu->insertItem( "About &Qt", this, SLOT( AboutQtSlot() ) );

   menubalken = new QMenuBar( this ); //... erzeugt einen Menue-Balken
   menubalken->insertItem( "&Datei", dateimenu );
   menubalken->insertItem( "&Farbe", farbemenu );
    QPixmap pixmap( 40, malflaeche->getBreite() );
    pixmap.fill( malflaeche->getFarbe() );
    breitemenu_id = menubalken->insertItem( pixmap, breitemenu );
   menubalken->insertSeparator();
   menubalken->insertItem( "&Hilfe", hilfemenu );
   
   scrollview = new QScrollView( this );
   scrollview->setGeometry( 0, menubalken->height(), 
                            width(), height() - menubalken->height() );

   scrollview->addChild( malflaeche );

   QObject::connect( malflaeche, SIGNAL( stiftGeaendert( QColor, int ) ),
                     malflaeche, SLOT( setzeFarbeBreite( QColor, int ) ) );
   QObject::connect( malflaeche, SIGNAL( stiftGeaendert( QColor, int ) ),
                     this, SLOT( AendereMenu( QColor, int ) ) );
   QObject::connect( this, SIGNAL( farbeTasteGedrueckt( int ) ),
                     malflaeche, SLOT( sendeFarbeGeaendertSignal( int ) ) );
   QObject::connect( this, SIGNAL( breiteTasteGedrueckt( int ) ),
                     malflaeche, SLOT( sendeBreiteTasteGeaendertSignal( int ) ) );
}

//------------------------------------------------------ main --------------
int main( int argc, char* argv[] )
{
   QApplication myapp( argc, argv );

   MalFenster* mywidget = new MalFenster();
   mywidget->setGeometry( 50, 50, 400, 400 );
 
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
