#include <qapplication.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qgrid.h>
#include <qhbox.h>
#include <qvbox.h>
#include <math.h>

#define POS_ZAHL  5

QColor farben[] = { Qt::white,  Qt::blue,  Qt::red,   Qt::cyan, 
                    Qt::yellow, Qt::green, Qt::black, Qt::magenta };

const int farbZahl = sizeof( farben ) / sizeof( farben[0] );

//....................................................... Klasse MeinButton
class MeinButton : public QPushButton {
   Q_OBJECT
public:
   MeinButton( QWidget *parent=0, const char *name=0 )
                                           : QPushButton( parent, name ) {
      farbe = Qt::lightGray;
      setPaletteBackgroundColor( farbe );
      QObject::connect( this, SIGNAL( clicked() ),
                        this, SLOT( changeColor() ) );
   }
   QColor getColor() { return farbe; }

private slots:
   void changeColor( void ) {
      if ( farbe == Qt::lightGray )  farbe = Qt::white;
      else if ( farbe == Qt::white ) farbe = Qt::black;
      else if ( farbe == Qt::black ) farbe = Qt::lightGray;
      setPaletteBackgroundColor( farbe );
   }

private:
   QColor farbe;
};

//....................................................... Klasse Mastermind
class Mastermind : public QVBox {
   Q_OBJECT
public:
   Mastermind( QWidget *parent=0, const char *name=0 )
                                              : QVBox( parent, name ) {
      int  i, j, p;
      nothing = QColor( 1, 1, 1 );

      QPushButton *b = new QPushButton( "Ok", this );
      QObject::connect( b,    SIGNAL( clicked() ),
                        this, SLOT( streichen() ) );

         //........................... Horizontales Layout (QHBox)
      for ( i=0; i<15; i++) {
         QHBox *reihe = new QHBox( this );
         reihe->setSpacing(10);
 
         QHBox *stones = new QHBox( reihe );   
         stones->setSpacing(2);
         stones->setMargin(2);
         for (j=0; j<POS_ZAHL; j++) {
            stein[i*10+j] = new QLabel( "  ", stones ); 
            stein[i*10+j]->setFixedSize( 20, 20 );
            stein[i*10+j]->show();
         }
 
         QHBox *sticks = new QHBox( reihe );
         sticks->setSpacing(2);
         sticks->setMargin(2);
         for (j=0; j<POS_ZAHL; j++) {
            stoepsel[i*10+j] = new MeinButton( sticks ); 
            stoepsel[i*10+j]->setFixedSize( 20, 20);
         }
      }
 
         //... Erstellen aller moeglichen Kombinationen
      runde = 0;
      kombiZahl  = nochMoeglich = (int)pow(farbZahl, POS_ZAHL);
      kombination = new QColor* [kombiZahl];
 
      for ( i=0; i<kombiZahl; i++) {
         int z = i;
         kombination[i] = new QColor [POS_ZAHL];
         for ( p=0; p<POS_ZAHL; z /= (int)farbZahl, p++)
            kombination[i][p] = farben[z % farbZahl];
      }
      naechsteKombination();
   }

private slots:
       //............. Streichen aller nicht in Frage kommender Kombinationen
   void  streichen( ) {
      int     i, p, p2, pr, fr;

      positionRichtig = farbeRichtig = 0;
      for ( p=0; p<POS_ZAHL; p++) {
         QColor farbe = stoepsel[runde*10+p]->getColor();
         if (      farbe == Qt::white ) farbeRichtig++;
         else if ( farbe == Qt::black ) positionRichtig++;
      }

      for (i=0; i<kombiZahl; i++) {
         pr = fr = 0;
                    //.. Pruefen nicht noetig, wenn Kombination schon gestrichen
         if (kombination[i] == NULL)
            continue;
                    //.. Sichern, um Originale nicht zu zerstoeren
         for (p=0; p<POS_ZAHL; p++) {
            aktTemp[p]   = aktuelleKombi[p];
            kombiTemp[p] = kombination[i][p];
         }
                    //.. Positionen ueberpruefen
         for (p=0; p<POS_ZAHL; p++)
            if (kombiTemp[p] == aktTemp[p]) {
               pr++;
               aktTemp[p] = kombiTemp[p] = nothing; // bereits ausgewertet
            }
                    //.. Farben ueberpruefen
         for (p=0; p<POS_ZAHL; p++)
            for (p2=0; p2<POS_ZAHL; p2++)
               if (kombiTemp[p] != nothing && kombiTemp[p] == aktTemp[p2]) {
                  fr++;
                  aktTemp[p2] = kombiTemp[p] = nothing; // bereits ausgewertet
                  break;
               }
                    //.. Wenn nicht in Frage kommend, diese Kombi. streichen
         if (pr != positionRichtig || fr != farbeRichtig) {
            delete kombination[i];
            kombination[i] = NULL;
            nochMoeglich--;
         }
      }
      rest = nochMoeglich;
      runde++;
      naechsteKombination();
      if ( positionRichtig == 5 || rest <= 1 )
         auswert();
   }

   void  auswert( void ) {
      if ( positionRichtig == 5 || rest == 1 )
          QMessageBox::information( 0, "Fertig",
               "Ok, ich habe die Kombination gefunden", QMessageBox::Ok );
      else
          QMessageBox::information( 0, "Fehler",
               "Irgendwo hast Du mir falsche Infos gegeben",
                 QMessageBox::Ok );
      qApp->quit();
   }

private:
       //......................... Holen der naechsten moeglichen Kombination
   void naechsteKombination( void ) {
      for (int i=0; i<kombiZahl; i++)
         if (kombination[i] != NULL) {
            for (int p=0; p<POS_ZAHL; p++)
               aktuelleKombi[p] = kombination[i][p];
            break;
         }
      for (int p=0; p<POS_ZAHL; p++) {
         QColor farbe( aktuelleKombi[p] );
         stein[runde*10+p]->setPaletteBackgroundColor( farbe );
      }
   }

       //.................................................. private-Variablen
   QLabel      *stein[150];
   MeinButton  *stoepsel[150];
   QColor      **kombination,
               aktuelleKombi[POS_ZAHL],
               aktTemp[POS_ZAHL], kombiTemp[POS_ZAHL];
   int         runde, kombiZahl, nochMoeglich, rest;
   int         positionRichtig, farbeRichtig;
   QColor      nothing;
};

#include "mastermind.moc"

//.................................................................... main
int main( int argc, char *argv[] ) {
  QApplication a( argc, argv );

  Mastermind* mastermind = new Mastermind();
  mastermind->show();
  a.setMainWidget( mastermind );

  return a.exec();
}
