#include <qapplication.h>
#include <qwidget.h>
#include <qmessagebox.h>
#include <stdlib.h>
#include <time.h>
#include "bildtabelle.h"

const char *picNames[] = {
      "Ameise.xpm",    "Butterfly.xpm", "monalisa.xpm", "Deutschl.xpm",
      "Grossbri.xpm",  "Krebs.xpm",     "Pinguin.xpm",  "Suse.xpm",
      "lausbub.xpm",   "marble.xpm",    "ruins.xpm",    "calvin2.xpm",
      "Wizard.xpm",    "HandReach.xpm", "HomeOpen.xpm", "HandOpen.xpm",
      "HandPunch.xpm", "Eggs.xpm",      "3dpaint.xpm",  "DeskGlobe.xpm" };
const int picNo = sizeof( picNames ) / sizeof( *picNames );
const int cardNo = picNo * 2;
const int bord     = 5;
const int frameWid = 2;
const int cellWid  = 100 + 2*bord + 2*frameWid;
const int cellHigh = 100 + 2*bord + 2*frameWid;
//.......................................................... Klasse Memory
class Memory : public Bildtabelle {
    Q_OBJECT
public:
      //..................................... Memory (Konstruktor)
    Memory( int zeilen, int spalten, QWidget* p=0, const char* name=0 )
                             :  Bildtabelle( zeilen, spalten, p, name ) {
       ::srand( time(NULL) );
       zeilZahl = zeilen;
       spaltZahl = spalten;
       setCellSize( cellWid, cellHigh, bord, frameWid );
       if ( !back.load( "back.bmp" ) ) {
          QMessageBox::information( 0, "Ladefehler",
                     "Kann Bild-Datei `back.bmp' nicht laden" ,
                     QMessageBox::Ok );
          qApp->quit();
       }
       for ( int i=0; i<picNo; i++ )
          if ( !bild[i].load( picNames[i] ) ) {
             QString  text;
             text.sprintf( "Kann Bild-Datei `%s' nicht laden", picNames[i] );
             bild[i].resize( cellWid, cellHigh );
             bild[i].fill( QColor( ::rand()&255,::rand()&255,::rand()&255 ) );
             QMessageBox::information( 0, "Ladefehler", text,
                        QMessageBox::Ok );
             qApp->quit();
          }
       QObject::connect( this, SIGNAL( pressPos( int, int ) ),
                         this, SLOT( pressHandle( int, int ) ) );
       QObject::connect( this, SIGNAL( doubleClickPos( int, int ) ),
                         this, SLOT( doubleClickHandle( int, int ) ) );
       newGame();
    }
      //.................................................. newGame
    void newGame( void ) {
       int  i, c;
       oldIndex1 = oldIndex2 = -1;
       clickNo = 0;
       resetAllMarks();
       for ( i=0; i<picNo; i++ )
          picDrawn[ i ] = 0;
       for ( i=0; i<cardNo; i++ ) {
          do { } while ( picDrawn[ c = ::rand()%picNo ] >= 2 );
          picDrawn[ c ]++;
          boardCard[i] = c;
          setCell( i/spaltZahl, i%spaltZahl, back );
          boardCardOpen[ i ] = false;
       }
    }
private slots:
      //.............................................. pressHandle
    void pressHandle( int z, int s ) {
       int index = z*spaltZahl + s;
       if ( isCellMarkSet( z, s ) || boardCardOpen[ index ] )
           return;
       if ( ++clickNo >= 3 ) {
          clickNo = 1;
          if ( boardCard[ oldIndex1 ] != boardCard[ oldIndex2 ] ) {
             setCell( oldIndex1/spaltZahl, oldIndex1%spaltZahl, back );
             setCell( oldIndex2/spaltZahl, oldIndex2%spaltZahl, back );
             boardCardOpen[ oldIndex1 ] = boardCardOpen[ oldIndex2 ] = false;
          }
       }
       if ( clickNo == 1 )
          oldIndex1 = index;
       else if ( clickNo == 2 ) {
          oldIndex2 = index;
          if ( boardCard[ oldIndex1 ] == boardCard[ oldIndex2 ] ) {
              setCellMark( oldIndex1/spaltZahl, oldIndex1%spaltZahl, true );
              setCellMark( oldIndex2/spaltZahl, oldIndex2%spaltZahl, true );
          }
       }
       setCell( index/spaltZahl, index%spaltZahl, bild[ boardCard[index] ] );
       boardCardOpen[ index ] = true;
    }
      //........................................ doubleClickHandle
    void doubleClickHandle( int, int ) {
       newGame();
    }
private:
    int      zeilZahl, spaltZahl;
    QPixmap  back;
    QPixmap  bild[ picNo ];
    int      picDrawn[ picNo ];
    int      boardCard[ cardNo ];
    bool     boardCardOpen[ cardNo ];
    int      clickNo;
    int      oldIndex1, oldIndex2;
};
#include "memory.moc"
//................................................................... main
int main( int argc, char *argv[] ) {
   QApplication a(argc,argv);
   int  lines = cardNo/8,
        cols = 8;
   if ( cardNo%8 != 0 )
      lines++;
   Memory *m = new Memory( lines, cols );
   a.setMainWidget( m );
   m->resize( 8*cellWid+5, 5*cellHigh+5 );
   m->show();
   return a.exec();
}
