#include <qapplication.h>
#include <qwidget.h>
#include <qmenubar.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <stdlib.h>
class MenuBeispiel : public QWidget {
    Q_OBJECT
public:
    MenuBeispiel( QWidget *parent=0, const char *name=0 );
public slots:
    void oeffnen()    { emit anzeige( "Datei/Oeffnen" );           }
    void neu()        { emit anzeige( "Datei/Neu" );               }
    void speichern()  { emit anzeige( "Datei/Speichern" );         }
    void schliessen() { emit anzeige( "Datei/Schliessen" );        }
    void rueckgaeng() { emit anzeige( "Bearbeiten/Rueckgaengig" ); }
    void wiederhol()  { emit anzeige( "Bearbeiten/Wiederholen" );  }

    void normal()     {
       menu->setItemChecked( fettID, istFett = false );
       menu->setItemChecked( unterID, istUnter = false );
       emit anzeige( "Optionen/Normal" );
    }
    void fett() {
        menu->setItemChecked( fettID, istFett = !istFett );
        emit anzeige( "Optionen/Fett" );
    }
    void unterstrichen() {
        menu->setItemChecked( unterID, istUnter = !istUnter);
        emit anzeige( "Optionen/Unterstrichen" );
    }
    void about() {
        QMessageBox::about( this, "Qt Menue-Beispiel",
                                  "Dieses Beispiel demonstriert die\n"
                                  "einfache Verwendung von Menues in Qt");
    }
    void aboutQt()  { QMessageBox::aboutQt( this, "Menue-Beispiel" ); }
    void drucker()  { emit anzeige( "Datei/Drucken/an Drucker" );     }
    void datei()    { emit anzeige( "Datei/Drucken/in Datei" );       }
    void fax()      { emit anzeige( "Datei/Drucken/an Fax" );         }
    void druckEin() { emit anzeige( "Datei/Drucken/Einrichten" );     }
    void neuMenu() {
       bearbeiten->setItemEnabled( rueckID, rand()%2 );
       bearbeiten->setItemEnabled( wiederID, rand()%2 );
    }
protected:
    void resizeEvent( QResizeEvent * ) {
       label->setGeometry( 25, 50, width()-50, height()-75 );
    }
signals:
    void    anzeige( const QString& );
private:
    QMenuBar   *menu;
    QPopupMenu *bearbeiten;
    QLabel     *label;
    bool       istFett, istUnter;
    int        fettID, unterID, rueckID, wiederID;
};
#include "menues.moc"

MenuBeispiel::MenuBeispiel( QWidget *parent, const char *name )
                                          : QWidget( parent, name ) {
    QPixmap oeffnenP( "oeffnen.xpm" );
    QPixmap neuP( "neu.xpm" );
    QPixmap speichernP( "speichern.xpm" );

    QPopupMenu *druck = new QPopupMenu( this );
     druck->insertItem( "an D&rucker", this, SLOT(drucker()) );
     druck->insertItem( "in &Datei", this, SLOT(datei()) );
     druck->insertItem( "an Fa&x", this, SLOT(fax()) );
     druck->insertSeparator();
     druck->insertItem( "&Einrichten", this, SLOT(druckEin()) );
    QPopupMenu *datei = new QPopupMenu( this );
     datei->insertItem( oeffnenP, "Oe&ffnen",  this,
                        SLOT(oeffnen()), CTRL+Key_O );
     datei->insertItem( neuP, "&Neu", this, SLOT(neu()), CTRL+Key_N );
     datei->insertItem( speichernP, "&Speichern", this,
                        SLOT(speichern()), CTRL+Key_S );
     datei->insertItem( "S&chliessen", this, SLOT(schliessen()), CTRL+Key_W );
     datei->insertSeparator();
     datei->insertItem( "&Drucken", druck, CTRL+Key_P );
     datei->insertSeparator();
     datei->insertItem( "&Beenden",  qApp, SLOT(quit()), CTRL+Key_Q );
    bearbeiten = new QPopupMenu( this );
     rueckID = bearbeiten->insertItem( "&Rueckgaengig",
                                       this, SLOT(rueckgaeng()), CTRL+Key_Z  );
     wiederID = bearbeiten->insertItem( "&Wiederholen", this, SLOT(wiederhol()));
    connect( bearbeiten, SIGNAL( aboutToShow() ), this, SLOT( neuMenu() ) );

    QPopupMenu* optionen = new QPopupMenu( this );
     optionen->insertItem( "&Normal Font", this, SLOT(normal()) );
     optionen->insertSeparator();
     fettID = optionen->insertItem( "&Fett", this, SLOT(fett()) );
     unterID = optionen->insertItem( "&Unterstrichen",
                                     this, SLOT(unterstrichen()) );
     istFett = istUnter = false;
     optionen->setCheckable( TRUE );
    QPopupMenu *hilfe = new QPopupMenu( this );
     hilfe->insertItem( "&About", this, SLOT(about()), CTRL+Key_H );
     hilfe->insertItem( "About &Qt", this, SLOT(aboutQt()) );
    menu = new QMenuBar( this );
     menu->insertItem( "&Datei", datei );
     menu->insertItem( "&Bearbeiten", bearbeiten );
     menu->insertItem( "&Optionen", optionen );
     menu->insertSeparator();
     menu->insertItem( "&Hilfe", hilfe );
    label = new QLabel( this );
     label->setGeometry( 25, 50, width()-50, height()-75 );
     label->setFrameStyle( QFrame::Box | QFrame::Raised );
     label->setLineWidth( 5 );
     label->setAlignment( AlignCenter );
     label->setPaletteBackgroundColor( Qt::white );
    connect( this,  SIGNAL(anzeige(const QString&)),
             label, SLOT(setText(const QString&)) );
    setMinimumSize( 300, 150 );
}
int main( int argc, char ** argv ) {
    QApplication a( argc, argv );
    MenuBeispiel m;
    a.setMainWidget( &m );
    m.show();
    return a.exec();
}
