#include <qapplication.h>
#include <qwidget.h>
#include <qmessagebox.h>
#include <qmenubar.h>
#include <qlabel.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <stdio.h>

class MenuBeispiel : public QWidget
{
    Q_OBJECT
public:
    MenuBeispiel( QWidget *parent=0, const char *name=0 );

public slots:
    void neuGroesse( int i ) {
        if ( groesse < 10 )
           klein->setItemChecked( 1000+groesse, false );
        else if ( groesse < 30 )
           mittel->setItemChecked( 1000+groesse, false );
        else if ( groesse < 80 )
           gross->setItemChecked( 1000+groesse, false );

        groesse = i%1000;

        if ( groesse < 10 )
           klein->setItemChecked( 1000+groesse, true );
        else if ( groesse < 30 )
           mittel->setItemChecked( 1000+groesse, true );
        else if ( groesse < 80 )
           gross->setItemChecked( 1000+groesse, true );

        label->setFont( QFont( fontName, groesse ) );
        label->setText( "Text" );
        label->resize( label->sizeHint() );
    }

    void neuFont( int i ) {
        for ( int j = 0; j<3; j++ )
           font->setItemChecked( 2000+j, false );
        font->setItemChecked( i, true );
        switch (i%2000 ) {
           case 0: fontName = "Courier";    break;
           case 1: fontName = "Helevetica"; break;
           case 2: fontName = "Times";      break;
        }
        label->setFont( QFont( fontName, groesse ) );
        label->setText( "Text" );
        label->resize( label->sizeHint() );
    }

    void about() {
        QMessageBox::about( this, "Qt Men-bung",
                                  "Dieses Beispiel demonstriert die\n"
                                  "einfache Verwendung von Mens in Qt");
    }

    void aboutQt()  { QMessageBox::aboutQt( this, "Men-bung" ); }

protected:
    void mouseReleaseEvent( QMouseEvent *e ) {
       if ( e->button() == RightButton )
          if ( e->state() == (RightButton | AltButton) )
             klein->popup( mapToGlobal( e->pos() ) );
          else if ( e->state() == (RightButton | ControlButton) )
             mittel->popup( mapToGlobal( e->pos() ) );
          else if ( e->state() == (RightButton | ControlButton | AltButton) )
             gross->popup( mapToGlobal( e->pos() ) );
          else
             font->popup( mapToGlobal( e->pos() ) );
    }
private:
    QPopupMenu *klein, *mittel, *gross;
    QPopupMenu *font;
    int        groesse;
    QString    fontName;
    QLabel     *label;
};

#include "menues2.moc"

MenuBeispiel::MenuBeispiel( QWidget *parent, const char *name )
                   : QWidget( parent, name )
{
    int i;

    klein = new QPopupMenu( this );
    for ( i=0; i<10; i++) {
       QString t;
       t.sprintf("%d", i );
       klein->insertItem( t, 1000+i );
       klein->setAccel( ALT + Key_0+i, 1000+i );
    }
    QObject::connect( klein, SIGNAL( activated( int ) ),
                      this,  SLOT( neuGroesse( int ) ) );

    mittel = new QPopupMenu( this );
    for ( i=10; i<30; i+=2) {
       QString t;
       t.sprintf("%d", i );
       mittel->insertItem( t, 1000+i );
       mittel->setAccel( CTRL+Key_0+(i-10)/2, 1000+i );
    }
    groesse = 20;
    mittel->setItemChecked( 1000+groesse, true );  
    QObject::connect( mittel, SIGNAL( activated( int ) ),
                      this,   SLOT( neuGroesse( int ) ) );

    gross = new QPopupMenu( this );
    for ( i=30; i<80; i+=5) {
       QString t;
       t.sprintf("%d", i );
       gross->insertItem( t, 1000+i );
       gross->setAccel( ALT+CTRL+Key_0+(i-30)/5, 1000+i );
    }
    QObject::connect( gross, SIGNAL( activated( int ) ),
                      this,  SLOT( neuGroesse( int ) ) );

    font = new QPopupMenu( this );
    font->insertItem( "&Courier", 2000 );
    font->setAccel( CTRL+Key_C, 2000 );
    font->insertItem( "&Helvetica", 2001 );
    font->setAccel( CTRL+Key_H, 2001 );
    font->insertItem( "&Times", 2002 );
    font->setAccel( CTRL+Key_T, 2002 );
    fontName = "Courier" ;
    font->setItemChecked( 2000, true );  
    QObject::connect( font, SIGNAL( activated( int ) ),
                      this, SLOT( neuFont( int ) ) );

    QPopupMenu *hilfe = new QPopupMenu( this );
    hilfe->insertItem( "&About", this, SLOT(about()), CTRL+Key_A );
    hilfe->insertItem( "About &Qt", this, SLOT(aboutQt()) );

    QMenuBar *menu = new QMenuBar( this );
    menu->insertItem( "&Klein", klein );
    menu->insertItem( "&Mittel", mittel );
    menu->insertItem( "&Gross", gross );
    menu->insertItem( "&Font", font );
    menu->insertSeparator();
    menu->insertItem( "&Hilfe", hilfe );

    label = new QLabel( this );
    label->setFrameStyle( QFrame::Box | QFrame::Raised );
    label->setLineWidth( 5 );
    label->setAlignment( AlignCenter );
    label->setPaletteBackgroundColor( Qt::white );
    label->setFont( QFont( fontName, groesse ) );
    label->setText( "Text" );
    label->setGeometry( 25, 50, 100, 50 );
    label->show();

    setMinimumSize( 300, 150 );
}

int main( int argc, char *argv[] )
{
    QApplication a( argc, argv );
    MenuBeispiel m;

    a.setMainWidget( &m );
    m.show();
    return a.exec();
}
