#include <qapplication.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qdatetime.h>
#include <qpixmap.h>
#include <stdlib.h>

//...................................................... Klasse MalFenster
class MalFenster : public QWidget
{
public:
   MalFenster() {
      setPaletteBackgroundColor( Qt::white );
      linkeTaste = rechteTaste = dragging = false;
      lLetztePos = QPoint( 0, 0 );
   }

protected:
   virtual void mousePressEvent( QMouseEvent* event) {
      if ( event->button() == RightButton ) { //... Rechte Maustaste
         rechteTaste = true;
         repaint( false );
         rechteTaste = false;
      } else if ( event->button() == LeftButton ) { //.. Linke Maustaste
         ersteMal = linkeTaste = true;
         startPos = event->pos();
         timer.start();
      }
   }

   virtual void mouseMoveEvent( QMouseEvent* event) {
      if ( !linkeTaste ) //.... nur bei linker Maustaste ist diese
         return;         //.... Methode von Interesse

      if ( !dragging )
            //.... Dragging (Ziehen) einschalten, wenn
            //.... mehr als 300 Millisek. seit letzten Aufruf vergangen
            //.... oder neue Position mind. 20 Pixel entfernt ist
         if ( timer.elapsed() >= 300 ||
              QABS( startPos.x() - event->pos().x() ) >= 20 ||
              QABS( startPos.y() - event->pos().y() ) >= 20 ) {
            rStartPos = startPos;
            dragging = true;
         }
      if ( dragging ) {
         rNeuePos = event->pos();
         repaint( false );
      }
   }

   virtual void mouseReleaseEvent( QMouseEvent* event) {
      if ( linkeTaste && !dragging ) { //... Linie zeichnen
         lNeuePos = event->pos();
         repaint( false );
         lLetztePos = lNeuePos;
      }
      linkeTaste = dragging = false;
   }

   virtual void mouseDoubleClickEvent( QMouseEvent* event ) {
      if ( event->button() == MidButton ) { //... bei mittlerer Maustaste
         bild.fill( backgroundColor() );    //... Bild loeschen
         repaint();
      }
   }

   virtual void paintEvent( QPaintEvent *event ) {
      QPainter p( &bild ); 

      if ( linkeTaste ) {
         if ( dragging ) {
            if ( !ersteMal )
               p.eraseRect( QRect( rStartPos, rLetztePos ) );
            p.drawRect( QRect( rStartPos, rNeuePos ) );
            rLetztePos = rNeuePos;
         } else
            p.drawLine( lLetztePos, lNeuePos );
         ersteMal = false;

      } else if ( rechteTaste )
         p.fillRect( QRect( rStartPos, rNeuePos ),
             QBrush( QColor( ::rand()&255, ::rand()&255, ::rand()&255 ) ) );

      bitBlt( this, event->rect().topLeft(), &bild );
   }

   virtual void resizeEvent( QResizeEvent *event ) {
      QPixmap bild2( bild );
      bild.resize( event->size() );
      bild.fill( backgroundColor() );
      bitBlt( &bild, 0, 0, &bild2 );
   }
private:
   QTime       timer;
   QPoint      startPos, rStartPos, rLetztePos, rNeuePos,
               lLetztePos, lNeuePos;
   bool        linkeTaste, rechteTaste, dragging, ersteMal;
   QPixmap     bild;
};

//.................................................................. main 
int main( int argc, char* argv[] )
{
   QApplication myapp( argc, argv );

   MalFenster* mywidget = new MalFenster();
   mywidget->setGeometry( 50, 50, 400, 400 );
 
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
