#include <qapplication.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qprogressbar.h>
#include <qpainter.h>
#include <qmessagebox.h>
#include <qmovie.h>

//...................................................... Klasse FilmSpiel
class FilmSpiel : public QWidget {
    Q_OBJECT
public:
     //....................... FilmSpiel (Konstruktor)
    FilmSpiel( const QString& dateiname ) : QWidget(0, 0) {
       counted = false;

       film = QMovie( dateiname );
       film.setBackgroundColor( backgroundColor() );
       film.connectStatus( this, SLOT(filmStatus(int) ) );

       QWidget *r = new QWidget;
       r->setCaption( dateiname );
       QGridLayout *qvbox = new QGridLayout( r, 4, 3, 5 );

       QPushButton *b1 = new QPushButton( "Play/Stop", r );
       QObject::connect( b1,   SIGNAL( clicked() ), 
                         this, SLOT( playStop() ) );
       qvbox->addWidget( b1, 0, 0 );

       QPushButton *b2 = new QPushButton( "Rewind", r );
       QObject::connect( b2,   SIGNAL( clicked() ), 
                         this, SLOT( rewind() ) );
       qvbox->addWidget( b2, 0, 1 );

       QPushButton *b3 = new QPushButton( "Step", r );
       QObject::connect( b3,   SIGNAL( clicked() ), 
                         this, SLOT( step() ) );
       qvbox->addWidget( b3, 0, 2 );

       QLabel *f = new QLabel( r, "" );
       f->setFrameStyle( QFrame::StyledPanel|QFrame::Sunken );
       f->setMovie( film );
       f->setPaletteBackgroundColor( Qt::white );
       qvbox->addWidget( f, 1, 1 );

       progressBar = new QProgressBar ( r );
       qvbox->addMultiCellWidget( progressBar, 2, 2, 0, 2 );

       label = new QLabel ( r );
       label->setAlignment( Qt::AlignCenter );
       qvbox->addMultiCellWidget( label, 3, 3, 0, 2 );

       r->show();
    }

private slots:
     //.................................... filmStatus
    void filmStatus( int status ) {
        if ( status < 0 ) {
            QString msg;
            msg.sprintf("Kann Film nicht abspielen");
            QMessageBox::message("Error", msg);
        } else if ( status == QMovie::EndOfLoop && !counted ) {
            numberFrames = film.frameNumber();
            progressBar->setTotalSteps( numberFrames ) ;
            counted = true;
        } else if ( !counted ) {
            QString t;
            t.sprintf( "Frame %d", film.frameNumber()+1 );
            label->setText( t );
        } else if ( counted ) {
            progressBar->setProgress( film.frameNumber()+1 ) ;
            QString t;
            t.sprintf( "Frame %d of %d", film.frameNumber()+1,
                                         numberFrames );
            label->setText( t );
        }
    }

     //...................................... playStop
    void playStop() {
       if ( film.running() )
          film.pause();
       else
          film.unpause();
    }

     //........................................ rewind
    void rewind() {
       film.restart();
       film.pause();
    }

     //.......................................... step
    void step() {
       film.step();
    }

private:
    QMovie       film;
    bool         counted;
    int          numberFrames;
    QProgressBar *progressBar;
    QLabel       *label;
};

//....................................................... Klasse FilmWahl
class FilmWahl : public QFileDialog {
    Q_OBJECT
public:
     //........................ FilmWahl (Konstruktor)
    FilmWahl(const char *dir) : QFileDialog(dir, "*.gif") {
        setMode( ExistingFile );
        connect(this, SIGNAL( fileSelected(const QString&) ),
                this, SLOT( starteFilm(const QString&) ));
    }

private slots:
     //.......................................... done
    void done( int r ) {
       if ( r != Accepted )
          qApp->quit(); //... Ende bei `Cancel'
       setResult( r );
    }

    void starteFilm( const QString& dateiname ) {
       if ( dateiname )
          new FilmSpiel(dateiname);
    }
};

#include "movieframes.moc"

//................................................................... main
int main( int argc, char *argv[] ) {
   QApplication a(argc, argv);

   FilmWahl* fw = new FilmWahl(".");

   a.setMainWidget( fw );
   fw->show();
   fw->repaint();

   return a.exec();
}

