#include <qapplication.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qmessagebox.h>
#include <qmovie.h>
//.................................................... Klasse FilmFlaeche
class FilmFlaeche : public QFrame {
 Q_OBJECT
    QMovie  film;
    QString dateiname;
public:
     //..................... FilmFlaeche (Konstruktor)
    FilmFlaeche(const char* fname, QMovie m, QWidget* p=0,
                const char* name=0, WFlags f=0) : QFrame(p, name, f) {
        setCaption( dateiname = fname );
        film = m;
        setFrameStyle(QFrame::StyledPanel|QFrame::Sunken);
        film.setBackgroundColor(backgroundColor());
        setBackgroundMode(NoBackground);
        film.connectUpdate(this, SLOT(filmUpdated(const QRect&)));
        film.connectResize(this, SLOT(filmResized(const QSize&)));
        film.connectStatus(this, SLOT(filmStatus(int)));
    }
protected:
     //.................................. drawContents
    void drawContents( QPainter* p ) {
        QPixmap pf = film.framePixmap();
        QRect   r  = contentsRect();
          //...Aus Zeitgruenden: Nur Groesse veraendern, wenn noetig
        if ( r.size() != pf.size() ) {
            QWMatrix m;
            m.scale((double)r.width()/pf.width(),
                    (double)r.height()/pf.height());
            pf = pf.xForm(m);
        }
          //...Neues (evtl. vergroessertes/verkleinertes) Bild.
          //...filmUpdated() ruft repaint() nur mit der veraenderten Flaeche
          //...auf, so dass zur Neudarstellung nur ein minimaler Aufwand
          //...erforderlich ist.
        p->drawPixmap(r.x(), r.y(), pf);
        const char* message = 0;
        if (film.paused())         { message = "PAUSED";  }
        else if (film.finished())  { message = "THE END"; }
        else if (film.steps() > 0) { message = "FF >>";   }
        if (message) {
            p->setFont(QFont("Helvetica", 24));
              //...Suchen einer optimalen Schriftgroesse
            QFontMetrics fm = p->fontMetrics();
            if ( fm.width(message) > r.width()-10 )
               p->setFont(QFont("Helvetica", 18));
            fm = p->fontMetrics();
            if ( fm.width(message) > r.width()-10 )
                p->setFont(QFont("Helvetica", 14));
            fm = p->fontMetrics();
            if ( fm.width(message) > r.width()-10 )
                p->setFont(QFont("Helvetica", 12));
            fm = p->fontMetrics();
            if ( fm.width(message) > r.width()-10 )
                p->setFont(QFont("Helvetica", 10));
              //..."Shadow"-Effekt.
            p->setPen(black);
            p->drawText(1, 1, width()-1, height()-1, AlignCenter, message);
            p->setPen(white);
            p->drawText(0, 0, width()-1, height()-1, AlignCenter, message);
        }
    }
     //............................. mouseReleaseEvent
    void mouseReleaseEvent(QMouseEvent* event) {
        if (event->state() & ShiftButton)
            film.restart();
        else if (!film.paused())
            film.pause();
        else {
            if (event->button() & LeftButton)
                film.step((event->state() & ControlButton) ? 10 : 1);
            else if (event->button() & (MidButton|RightButton))
                film.unpause();
        }
        repaint();
    }
private slots:
     //................................... filmUpdated
    void filmUpdated( const QRect& flaeche ) {
        if (!isVisible())
            show();
        QRect r = contentsRect();
        if ( r.size() != film.framePixmap().size() )
           repaint( r ); //... ganze Bild neu malen
        else
            //...nur geaenderte Flaeche neu malen
           repaint( flaeche.x()+r.x(), flaeche.y()+r.x(),
                    flaeche.width(), flaeche.height() );
    }
     //................................... filmResized
    void filmResized( const QSize& size ) {
          //...Filmflaeche hat sich geaendert
        int fw = frameWidth();
        resize( size.width() + fw*2, size.height() + fw*2 );
        qApp->setMainWidget(this);
        qApp->setMainWidget(0);
    }
     //.................................... filmStatus
    void filmStatus( int status ) {
        if (status < 0) {
            QString msg;
            msg.sprintf("Kann Film \"%s\" nicht abspielen",
                        (const char*)dateiname);
            QMessageBox::message("Error", msg);
        } else if (status == QMovie::Paused || status == QMovie::EndOfMovie)
            repaint();
    }
};
//....................................................... Klasse FilmWahl
class FilmWahl : public QFileDialog {
    Q_OBJECT
public:
     //........................ FilmWahl (Konstruktor)
    FilmWahl(const char *dir) : QFileDialog(dir, "*") {
        setMode( ExistingFile );
        connect(this, SIGNAL( fileSelected(const QString&) ),
                this, SLOT( starteFilm(const QString&) ));
    }
public slots:
     //.................................... starteFilm
    void starteFilm( const QString& dateiname ) {
        if ( dateiname )
            (void)new FilmFlaeche(dateiname, QMovie(dateiname),
                                  0, 0, WDestructiveClose);
    }
     //.......................................... done
    void done( int r ) {
        if (r != Accepted)
           qApp->quit(); //... Ende bei `Cancel'
        setResult( r );
    }
};
//................................................................... main
int main( int argc, char *argv[] ) {
   QApplication a(argc, argv);

   if (argc > 1) { //.... Abspielen aller Filme, deren Dateiname
                   //.... auf der Kommandozeile angegeben ist
       for (int i=1; i<argc; i++)
           (void)new FilmFlaeche(argv[i], QMovie(argv[i]), 0, 0,
                                 Qt::WDestructiveClose);
       QObject::connect(qApp, SIGNAL( lastWindowClosed() ),
                        qApp, SLOT(quit()));
   } else {
       FilmWahl* fd = new FilmWahl(".");
       fd->show();
         //...Help-Infos fuer den Benutzer
       QLabel* help = new QLabel(
           "Waehle deine Filme aus!\n\n"
           "- Shift-Klick zum Neustart.\n\n"
           "- Klick zum Anhalten, dann\n"
           "    > Links-Klick zum schrittweisen Abspielen,\n"
           "    > Strg-Links-Klick zum Abspielen in 10er Schritten,\n"
           "    > Rechts-Klick, um normales Abspielen fortzusetzen.\n", 0);
       help->setCaption("movies - Help");
       help->setIndent(10);
       help->adjustSize();
       help->show();
   }
   return a.exec();
}
#include "movies.moc"
