#include <qapplication.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qslider.h>
#include <qlcdnumber.h>
#include <qpainter.h>
#include <qmessagebox.h>
#include <qmovie.h>
//...................................................... Klasse FilmSpiel
class FilmSpiel : public QWidget {
    Q_OBJECT
public:
     //....................... FilmSpiel (Konstruktor)
    FilmSpiel( const QString& dateiname ) : QWidget(0, 0) {
       film = QMovie( dateiname );
       film.setBackgroundColor( backgroundColor() );
       film.connectStatus( this, SLOT(filmStatus(int) ) );
       QWidget *r = new QWidget;
       r->setCaption( dateiname );
       QGridLayout *qvbox = new QGridLayout( r, 2, 2, 20 );
       QSlider *s = new QSlider( 0, 500, 10, 100,
                                 QSlider::Horizontal, r );
       s->setTickmarks( QSlider::Below );
       s->setTickInterval( 10 );
       s->setMinimumWidth( 200 );
       s->setPalette( QPalette( Qt:: yellow.light( 120 ) ) );
       qvbox->addWidget( s, 0, 0 );
       tempo = new QLCDNumber( 3, r );
       tempo->display( 100 );
       tempo->setPalette( QPalette( Qt:: red.light( 180 ) ) );
       qvbox->addWidget( tempo, 1, 0 );
       QLabel *f = new QLabel( r, "" );
       f->setFrameStyle( QFrame::StyledPanel|QFrame::Sunken );
       f->setMovie( film );
       f->setBackgroundColor( Qt::white );
       qvbox->addMultiCellWidget( f, 0, 1, 1, 1 );
       r->show();
       QObject::connect( s, SIGNAL( valueChanged( int ) ),
                         this, SLOT( speedSlot( int ) ) );
    }
private slots:
     //.................................... filmStatus
    void filmStatus( int status ) {
        if ( status < 0 ) {
            QString msg;
            msg.sprintf("Kann Film nicht abspielen");
            QMessageBox::message("Error", msg);
        }
    }
     //..................................... speedSlot
    void speedSlot( int wert ) {
       tempo->display( wert );
       film.setSpeed( wert );
    }
private:
    QMovie      film;
    QLCDNumber *tempo;    
};
//....................................................... Klasse FilmWahl
class FilmWahl : public QFileDialog {
    Q_OBJECT
public:
     //........................ FilmWahl (Konstruktor)
    FilmWahl(const char *dir) : QFileDialog(dir, "*.gif") {
        setMode( ExistingFile );
        connect(this, SIGNAL( fileSelected(const QString&) ),
                this, SLOT( starteFilm(const QString&) ));
    }
private slots:
     //.......................................... done
    void done( int r ) {
       if ( r != Accepted )
          qApp->quit(); //... Ende bei `Cancel'
       setResult( r );
    }
    void starteFilm( const QString& dateiname ) {
       if ( dateiname )
          new FilmSpiel(dateiname);
    }
};
//................................................................... main
int main( int argc, char *argv[] ) {
   QApplication a(argc, argv);
   FilmWahl* fw = new FilmWahl(".");
   a.setMainWidget( fw );
   fw->show();
   fw->repaint();
   return a.exec();
}

#include "movies2.moc"
