#include <qapplication.h>
#include <qhbox.h>
#include <qthread.h>
#include <qmutex.h>
#include <qlabel.h>
#include <qdatetime.h>
#include <stdlib.h>
#include <time.h>

QMutex mutex;
int    offset1, offset2;
//...................................................... Klasse WaitThread
class WaitThread: public QThread
{
public:
    WaitThread( QThread *thr1, QThread *thr2, QLabel *lbl )
                         : thread1(thr1), thread2(thr2), label(lbl) {}

    void run() {
       QTime time;
       time.start();
       thread1->wait();
       thread2->wait();

       qApp->lock();
       QLabel *l = new QLabel( label );
       l->setPaletteBackgroundColor( Qt::white );
       l->setText( "Ready" );
       l->setAlignment( Qt::AlignCenter );
       l->setGeometry( time.elapsed()/50+10, 35, 50, label->height()-45 );
       l->show();
       qApp->unlock();
    }

private:
    QThread *thread1, *thread2;
    QLabel  *label;
};

//....................................................... Klasse GUIThread
class GUIThread : public QThread
{
public:
    GUIThread( int id, bool sync, QLabel* l, QColor c )
                                            : label( l ), color( c ) {
       text.sprintf("%d", id );
       withMutex = sync;
       label->setFrameStyle( QFrame::Box | QFrame::Raised );
       label->setLineWidth( 5 );
       label->setAlignment( Qt::AlignHCenter | Qt::AlignTop );
       time.start();
    }

protected:
    void run() {
       int offset;
       for ( int i = 0; i < 4; i++ ) {
          if ( withMutex )
             mutex.lock();
          qApp->lock();
          int  sec = ::rand()%2500 + 501;
          QLabel *l = new QLabel( label );

          QString t;
          t.sprintf( "%s: Elapsed: %d; Duration: %d\n",
                     withMutex ? "mit Sync." : "ohne Sync",
                     time.elapsed(), sec );
          label->setText( t );
          l->setPaletteBackgroundColor( color );
          l->setText( text );
          l->setFrameStyle( QFrame::Box | QFrame::Sunken );
          l->setAlignment( Qt::AlignCenter );
          offset = ( withMutex ) ? offset1 : offset2;
          l->setGeometry( time.elapsed()/50+10, offset,
                          sec/50, label->height()-offset-10 );
          if ( withMutex )
             offset1 += 20;
          else
             offset2 += 20;
          l->show();
          qApp->unlock();
          msleep( sec );
          if (withMutex )
             mutex.unlock();
       }
    }

private:
    QLabel* label;
    QString text;
    QColor  color;
    bool    withMutex;
    QTime   time;
};

//................................................................... main
int main( int argc, char *argv[] ) {
    QApplication app( argc, argv );

    offset1 = offset2 = 50;

    ::srand( time( NULL ) );

    QHBox box( 0, 0, true );

    QLabel label1( &box );             //..... mit Synchronisation
    label1.setMinimumSize( 450, 300 );
    label1.show();

    GUIThread first1( 1, true, &label1, Qt::yellow );
    GUIThread second1( 2, true, &label1, Qt::green );
    WaitThread third1( &first1, &second1, &label1 );

    QLabel label2( &box );             //..... ohne Synchronisation
    label2.setMinimumSize( 450, 300 );

    GUIThread first2( 1, false, &label2, Qt::yellow );
    GUIThread second2( 2, false, &label2, Qt::green );
    WaitThread third2( &first2, &second2, &label2 );

    app.setMainWidget( &box );
    box.show();

    first1.start();
    second1.start();
    third1.start();

    first2.start();
    second2.start();
    third2.start();

    return app.exec();
}
