#include <qobject.h>
#include <qapplication.h>
#include <qwidget.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qsocket.h>
#include <qtextstream.h>
#include <qdns.h>
#include <qstring.h>
#include <qtimer.h>
#include <qmessagebox.h>
//................................................................ class Smtp
class Smtp : public QObject {
    Q_OBJECT
public:
    Smtp( const QString &from, const QString &to,
          const QString &subject, const QString &body ) {
       socket = new QSocket( this );
        connect ( socket, SIGNAL( readyRead() ), this, SLOT( readyRead() ) );
        connect ( socket, SIGNAL( connected() ), this, SLOT( connected() ) );
       mxLookup = new QDns( to.mid( to.find( '@' )+1 ), QDns::Mx );
        connect( mxLookup, SIGNAL(resultsReady()), this, SLOT(dnsLookup()) );
       message = QString::fromLatin1( "From: " ) + from +
                 QString::fromLatin1( "\nTo: " ) + to +
                 QString::fromLatin1( "\nSubject: " ) + subject +
                 QString::fromLatin1( "\n\n" ) + body + "\n";
       replace( message, QString::fromLatin1("\n"), QString::fromLatin1("\r\n"));
       replace( message, QString::fromLatin1( "\r\n.\r\n" ),
                         QString::fromLatin1( "\r\n..\r\n" ) );
       this->from = from;
       rcpt  = to;
       state = Init;
    }
    ~Smtp() { delete t; delete socket; }
signals:
    void finished();
    void status( const QString & );
private slots:
    void connected() {
       emit status( tr( "Connected to %1" ).arg( socket->peerName() ) );
    }
    void readyRead() {
       if ( !socket->canReadLine() ) // SMTP ist zeilenorientiert
          return;
       QString responseLine;
       do {
          responseLine = socket->readLine();
          response += responseLine;
       } while( socket->canReadLine() && responseLine[3] != ' ' );
       responseLine.truncate( 3 );
       if ( state == Init && responseLine[0] == '2' ) {
          *t << "HELLO there\r\n";  // Banner ok, es kann weitergehen
          state = Mail;
       } else if ( state == Mail && responseLine[0] == '2' ) {
          *t << "MAIL FROM: <" << from << ">\r\n";
          state = Rcpt;
       } else if ( state == Rcpt && responseLine[0] == '2' ) {
          *t << "RCPT TO: <" << rcpt << ">\r\n";
          state = Data;
       } else if ( state == Data && responseLine[0] == '2' ) {
          *t << "DATA\r\n";
          state = Body;
       } else if ( state == Body && responseLine[0] == '3' ) {
          *t << message << ".\r\n";
          state = Quit;
       } else if ( state == Quit && responseLine[0] == '2' ) {
          *t << "QUIT\r\n";
          state = Close;
          emit status( tr( "Message sent" ) );
       } else if ( state == Close ) {
          // wird ignoriert
       } else {
          QMessageBox::warning( qApp->activeWindow(), tr( "Qt Mail-Beispiel" ),
               tr( "Unerwartete Antwort vom SMTP-Server:\n\n" ) + response );
          state = Close;
       }
       response = "";
       if ( state == Close )
          QTimer::singleShot( 0, this, SLOT(deleteMe()) );
    }
    void deleteMe() { delete this; }
    void dnsLookup() {
       QValueList<QDns::MailServer> s = mxLookup->mailServers();
       if ( s.isEmpty() && mxLookup->isWorking() )
          return;
       emit status( tr( "Connecting to %1" ).arg( s.first().name ) );
       socket->connectToHost( s.first().name, 25 );
       t = new QTextStream( socket );
    }
private:
    enum State { Init, Mail, Rcpt, Data, Body, Quit, Close };
    int         state;
    QString     message, from, rcpt, response;
    QSocket     *socket;
    QTextStream *t;
    QDns        *mxLookup;
    void replace( QString& str, const QString& before, const QString& after ) {
       if ( before.length() == 0 )
           return;
       int maxsize = str.length();
       if ( before.length() <= after.length() )
          maxsize = ( str.length() / before.length() ) * after.length();
       QChar *buf = new QChar[maxsize + 1]; // + 1 fr Fall 'maxsize==0'
       const QChar *strBuf = str.unicode();
       int  prev = 0, cur = 0, i = 0;
       bool changed = false;
       while ( (uint) (cur = str.find(before, prev)) < str.length() ) {
           if ( cur > prev ) {
               memcpy( buf + i, strBuf + prev, sizeof(QChar) * (cur - prev) );
               i += cur - prev;
           }
           if ( after.length() > 0 ) {
               memcpy( buf + i, after.unicode(), sizeof(QChar) * after.length() );
               i += after.length();
           }
           prev = cur + before.length();
           changed = true;
       }
       if ( changed ) {
           memcpy( buf + i, strBuf + prev, sizeof(QChar) * (str.length() - prev) );
           i += str.length() - prev;
           str = QString( buf, i );
       }
       delete[] buf;
    }
};
//............................................................ class Composer
class Composer : public QWidget {
    Q_OBJECT
public:
    Composer( QWidget *parent=0 ) : QWidget( parent ) {
       QGridLayout * l = new QGridLayout( this, 1, 1, 6 );
       l->addWidget( new QLabel( tr( "From:" ), this ), 0, 0 );
         from = new QLineEdit( this );
         l->addWidget( from, 0, 1 );
       l->addWidget( new QLabel( tr( "To:" ), this ), 1, 0 );
         to = new QLineEdit( this );
         l->addWidget( to, 1, 1 );
       l->addWidget( new QLabel( tr( "Subject:" ), this ), 2, 0 );
         subject = new QLineEdit( this );
         l->addWidget( subject, 2, 1 );
       message = new QTextEdit( this );
         l->addMultiCellWidget( message, 3, 3, 0, 1 );
       send = new QPushButton( tr( "&Send" ), this );
         l->addWidget( send, 4, 0 );
         connect( send, SIGNAL( clicked() ), this, SLOT( sendMessage() ) );
       sendStatus = new QLabel( this );
         l->addWidget( sendStatus, 4, 1 );
    }
private slots:
    void enableSend() { send->setEnabled( true ); }
    void sendMessage() {
       send->setEnabled( false );
       sendStatus->setText( tr( "Looking up mail servers" ) );
       Smtp *smtp = new Smtp( from->text(), to->text(),
                              subject->text(), message->text() );
       connect( smtp, SIGNAL(destroyed()),
                this, SLOT(enableSend()) );
       connect( smtp,       SIGNAL(status(const QString &)),
                sendStatus, SLOT(setText(const QString &)) );
    }
private:
    QLineEdit   *from, *to, *subject;
    QTextEdit   *message;
    QLabel      *sendStatus;
    QPushButton *send;
};
#include "mymail.moc"
//..................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    Composer c;
    a.setMainWidget( &c );
    c.resize( 400, 500 );
    c.show();
    return a.exec();
}
