#include <qpainter.h>
#include <qmessagebox.h>
#include <qnp.h>
#include <stdlib.h>

//............................................................ BillardWidget
class BillardWidget : public QNPWidget {
    Q_OBJECT
public:
    BillardWidget( ) {
       x = y = 100;
       dx = dy = 1;
       first = true;
       const char *rad = instance()->arg("RADIUS");
       if (rad == 0 || strlen(rad) == 0)
          radius = 30;
       else
          radius = atoi(rad);
       const char *tempo = instance()->arg("SPEED");
       if (tempo == 0 || strlen(tempo) == 0)
          speed = 2;
       else
          speed = atoi(tempo);
       const char *col = instance()->arg("COLOR");
       if (col == 0 || strlen(col) == 0 || !strcmp( col, "MONO") )
          colorChange = false;
       else if ( !strcmp( col, "CHANGE") )
          colorChange = true;
       p = new QPainter( this );
       startTimer(speed);
    }
    ~BillardWidget( ) { }

    void paintEvent(QPaintEvent* ) {
       int w = width(),
           h = height();
       if ( first ) {
          p->setBrush( QColor( ::rand()%255, ::rand()%255, ::rand()%255 ) );
          p->drawRect( 0, 0, w, h );
          first = false;
          farbe = QColor( ::rand()%255, ::rand()%255, ::rand()%255 );
       }
       if (x >= w-radius || x <= radius)
          dx = -dx;
       if (y >= h-radius || y <= radius)
          dy = -dy;
       if ( colorChange )
          p->setPen( farbe );
       else
          p->setPen( Qt::black );
       p->setBrush( farbe );
       p->drawEllipse( x-radius, y-radius, 2*radius, 2*radius );
       x += dx;
       y += dy;
       if ( colorChange )
          farbe = QColor( ::rand()%255, ::rand()%255, ::rand()%255 );
    }

    void mousePressEvent( QMouseEvent *event ) {
       if ( event->button() == LeftButton ) {
          int w = width(),
              h = height();
          first = true;
          x = event->x();
          y = event->y();
          if ( x >= w-radius) x = w-radius-1;
          if ( x <= radius)   x = radius+1;
          if ( y >= h-radius) y = h-radius-1;
          if ( y <= radius)   y = radius+1;
          repaint( true );
       } else if ( event->button() == RightButton ) {
          colorChange = !colorChange;
          repaint( false );
       }
     }
 
    void timerEvent( QTimerEvent * ) {
       repaint( false );
    }

private:
    QPainter *p;
    bool     first;
    int      x, y, dx, dy;
    int      radius, speed;
    bool     colorChange;
    QColor   farbe;
};

//.......................................................... BillardInstance
class BillardInstance : public QNPInstance {
    Q_OBJECT
public:
    BillardInstance() { }

    QNPWidget* newWindow() {
        return new BillardWidget;
    }
};

//............................................................ BillardPlugin
class BillardPlugin : public QNPlugin {
public:
    BillardPlugin() { }
    ~BillardPlugin() { }

    QNPInstance* newInstance() { return new BillardInstance; }

    const char* getMIMEDescription() const {
       return "Nsbillard/mybillard:bld:Billard";
    }

    const char * getPluginNameString() const {
       return "Qt-based Cbillard Plugin";
    }

    const char * getPluginDescriptionString() const {
       return "A Qt-based plug-in that plays Billard";
    }
};

//....................................................... QNPlugin::create()
QNPlugin* QNPlugin::create() {
    return new BillardPlugin;
}

#include "nsbillard.moc"
