#include <qnp.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qslider.h>
#include <stdio.h>

//.............................................................. ViewWidget
class ViewWidget : public QWidget {
   Q_OBJECT
public:
   ViewWidget( QWidget* par=0, const char* nam=0 ) : QWidget( par, nam ) {
      scaling = 0;
      p = new QPainter( this );
   }

   void paintEvent( QPaintEvent* ) {
      if ( scaling == 0  )
         p->drawPixmap( 0, 0, pm );
      else {
         double percent = (scaling==-1) ? 0.95 : 1.05;
         p->scale( percent, percent );
         p->drawPixmap( 0, 0, pm );
      }
   }

   void mousePressEvent( QMouseEvent* event ) {
      if ( event->button() == LeftButton ) {
         scaling = -1;
         repaint( true );
      } else if ( event->button() == RightButton ) {
         scaling = 1;
         repaint( true );
      }
   }

   void showImage(const QImage& image) {
      pm.convertFromImage( image, QPixmap::Color );
      repaint( false );
   }


private:
   QPixmap  pm;
   int      scaling;
   QPainter *p;
};

//............................................................ ScaleGIFView
class ScaleGIFView : public QNPWidget 
{
public:
   ScaleGIFView() {
      view = new ViewWidget( this );
      view->show();
   }

   void resizeEvent( QResizeEvent* event ) {
      view->setGeometry( 0, 0, event->size().width(), 
                               event->size().height() );
   }
  
   void showImage( const QImage& image ) { view->showImage( image ); }

private:
   ViewWidget* view;
};

//.......................................................... ScaleGIFLoader
class ScaleGIFLoader : public QNPInstance 
{
public:
   ScaleGIFLoader() : view(0) { }
  
   QNPWidget* newWindow() {
      view = new ScaleGIFView;
      imageToGIF();
      return view;
   }

   void imageToGIF() {
      if (!view || image.isNull()) return;
       
      view->showImage( image );
      image.reset();
   }
  
   bool newStreamCreated( QNPStream*, StreamMode& smode ) {
      smode = AsFileOnly;
      return true;
   }
  
   void streamAsFile( QNPStream*, const char* fileName ) {
      image = QImage( fileName );
      if ( image.isNull() )
      debug( "Could not convert file: %s\n", fileName );
      imageToGIF();
   }

private:
   ScaleGIFView* view;
   QImage         image;
};

//............................................................. ScalePlugin
class ScalePlugin : public QNPlugin 
{
public:
   ScalePlugin() { }

   QNPInstance* newInstance() { return new ScaleGIFLoader; }

   const char* getMIMEDescription() const {
      return "image/gif:gif:GIF Image;"
             "image/x-xpixmap:xpm:XPM Image";
   }

   const char * getPluginNameString() const {
      return "GIF Qt-based Image Plugin";
   }

   const char * getPluginDescriptionString() const {
      return "Let's you view and scale a GIF image by mouse click";
   }
};

//...................................................... QNPlugin::create()
QNPlugin* QNPlugin::create() {
    return new ScalePlugin;
}

#include "nsgifscale.moc"
