#include  <stdio.h>

#define  bool  unsigned char

bool  schaltjahr(int j);
int   tagesnr(int t, int m, int j);
void  tagimjahr(int n, int j);
void  feiertage(int ostern, int j);
int   computus(int j);
int   gauss(int j);
int   obeirne(int j);
int   beide_kalender(int j);

/*------ main --------------------------------------------------------*/
int  main(void)
{
   int   jahr;

   printf("Ostersonntag in einem bestimmten Jahr\n"
          "=====================================\n\n");

   printf("Gib ein Jahr ein: ");
   scanf("%d", &jahr);

   if (jahr>=1582) {
      printf("-------------- Computus ecclesiasticus: ---- ");
      feiertage(computus(jahr), jahr);
   }

   if (jahr>=1582 && jahr<=2499) {
      printf("-------------- Gauss-Algorithmus:  ---- ");
      feiertage(gauss(jahr), jahr);
   }
   
   if (jahr>=1900 && jahr<=2099) {
      printf("-------------- O'Beirne-Algorithmus: ---- ");
      feiertage(obeirne(jahr), jahr);
   }

   printf("-------------- Algorithmus fuer beide Kalender: ---- ");
   feiertage(beide_kalender(jahr), jahr);
   return(0);
}

/*------- schaltjahr -------------------------------------------------*/
bool  schaltjahr(int j)
{
   return( (j%4==0 && j%100!=0) || (j%400==0) );
}

/*------- tagesnr ----------------------------------------------------*/
int  tagesnr(int t, int m, int j)
{
    int  a=j%4, b=j%100, c=j%400, d, e;

    d = (m+10)/13;
    e = t + 3055L * (m+2)/100 - 2*d - 91;
    return( e + (1 - (a+3)/4 + (b+99)/100 - (c+399)/400 ) * d );
}

/*------- tagimjahr --------------------------------------------------*/
void  tagimjahr(int n, int j)
{
   int  a = schaltjahr(j), b, c, t, m,
        h = n>59;

   b = (h>0) ? 2-a : 0;
   c = n+b+91;
   m = 100L*c/3055;
   t = c - 3055L*m/100;
   m = m-2;
   if (m==2 && t==30)
      t--;

   printf("%2d.%2d.%2d\n", t, m, j);
}

/*------- feiertage --------------------------------------------------*/
void  feiertage(int ostern, int j)
{
    printf("   Ostern: ");      tagimjahr(ostern, j);
    printf("      Aschermittwoch:      ");  tagimjahr(ostern-46, j);
    printf("      Christi Himmelfahrt: ");  tagimjahr(ostern+39, j);
    printf("      Pfingstsonntag:      ");  tagimjahr(ostern+49, j);
    printf("      Fronleichnam:        ");  tagimjahr(ostern+60, j);
}

/*------ computus ----------------------------------------------------*/
int  computus(int j)
{
   int  g = j%19+1,
        c = j/100+1,
        x, z, d, e, n;

   x = 3*c/4 -12;
   z = (8*c+5)/25 - 5; 
   d = 5*j/4 - x - 10;
   e = (11*g + 20 + z - x) % 30;
   if ( (e==25 && g>11) || e==24)
      e++;
   n = 44-e;
   if (n<21)  
     n += 30;
   n = n+7 - (d+n)%7;
   return( (n>31) ? tagesnr(n-31, 4, j) : tagesnr(n, 3, j) );
}

/*------ gauss -------------------------------------------------------*/
int  gauss(int j)
{
    int  a = j%19, b = j%4, c = j%7, d, e, m, n;

    if      (j>=1582 && j<=1699)  m=22, n=2;
    else if (j>=1700 && j<=1799)  m=23, n=3;
    else if (j>=1800 && j<=1899)  m=23, n=4;
    else if (j>=1900 && j<=2099)  m=24, n=5;
    else if (j>=2100 && j<=2199)  m=24, n=6;
    else if (j>=2200 && j<=2299)  m=25, n=0;
    else if (j>=2300 && j<=2399)  m=26, n=1;
    else if (j>=2400 && j<=2499)  m=55, n=1;

    d = (19*a+m)%30;
    e = (2*b+4*c+6*d+n)%7;
    if (22+d+e <= 31)
       return(tagesnr(22+d+e, 3, j));
    else if (d+e-9 == 26)
       return(tagesnr(19, 4, j));
    else if (d+e-9==25 && d==28 && a>10)
       return(tagesnr(18, 4, j));
    else
       return(tagesnr(d+e-9, 4, j));
}

/*------ obeirne -----------------------------------------------------*/
int  obeirne(int j)
{
    int  n=j-1900, a, b, m, q, w, p;

    a = n%19;
    b = (7*a+1)/19;  
    m = (11*a+4-b) % 29;
    q = n/4;
    w = (n+q+31-m) % 7;
    p = 25-m-w; 
    return ( (p>0) ? tagesnr(p, 4, j) : tagesnr(p+31, 3, j) );
}

/*------ beide_kalender ----------------------------------------------*/
int  beide_kalender(int jahr)
{
   int  a, b, c, d, e, f, g, h, i, j, k, l, m, n, p;
   if (jahr>1582) {
      a = jahr%19;
      b = jahr/100;
      c = jahr%100;
      d = b/4;
      e = b%4;
      f = (b+8)/25;
      g = (b-f+1)/3;
      h = (19*a+b-d-g+15)%30;
      i = c/4;
      j = c%4;
      k = j%100;
      l = (32+2*e+2*i-h-k)%7;
      m = (a+11*h+22*l)/451;
      n = (h+l-7*m+114)/31;
      p = (h+l-7*m+114)%31;
   } else {
      a = jahr%4;
      b = jahr%7;
      c = jahr%19;
      d = (19*c+15)%30;
      e = (2*a+4*b-d+34)%7;
      n = (d+e+114)/31;
      p = (d+e+114)%31;
   }
   return(tagesnr(p+1, n, jahr));
}
