#include <qapplication.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qslider.h>
#include <qlcdnumber.h>
#include <qhbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qcolordialog.h>

const char *penStylesName[] = { "NoPen",   "SolidLine",   "DashLine",
                                "DotLine", "DashDotLine", "DashDotDotLine" };
 
const char *brushStylesName[] = {
       "NoBrush",       "SolidPattern", 
       "Dense1Pattern", "Dense2Pattern", "Dense3Pattern", "Dense4Pattern",
       "Dense5Pattern", "Dense6Pattern", "Dense7Pattern",
       "HorPattern",    "VerPattern",    "CrossPattern", 
       "BDiagPattern",   "FDiagPattern", "DiagCrossPattern"              };                                                                                                                        
const int penStylesNr   = sizeof(penStylesName)/sizeof( *penStylesName );
const int brushStylesNr = sizeof(brushStylesName)/sizeof( *brushStylesName );

//........................................................... Klasse PenFill
class PenFill : public QWidget
{
   Q_OBJECT
public:
   PenFill( QWidget *p = 0 ) : QWidget( p ) {
      int  i;
      QSlider *widthSlider = new QSlider( 0, 100, 1, penWidth = 1,
                                         QSlider::Horizontal, this );
      widthSlider->setGeometry( 500, 2, 200, 25 );
      QObject::connect( widthSlider, SIGNAL( valueChanged( int ) ),
                        this,        SLOT( newPenWidth( int ) ) );

      setFont( QFont( "Times", 10 ) );
      width = new QLCDNumber( 3, this );
      width->display( penWidth );
      width->setGeometry( 720, 2, 50, 25 );

      QHBox *box1 = new QHBox( this );
      QButtonGroup *grp1 = new QButtonGroup( penStylesNr, Vertical,
                                             "PenStyle", box1 );
      grp1->setExclusive( true );
      for ( i=0; i<penStylesNr; i++) 
         penStyles[i] = new QRadioButton( penStylesName[i], grp1 );
      penStyles[penStyle=1]->setChecked( true );
      QObject::connect( grp1, SIGNAL( clicked( int ) ),
                        this, SLOT( newPenStyle( int ) ) );
      box1->setGeometry( 500, 30, 130, 180 );

      QHBox *box2 = new QHBox( this );
      QButtonGroup *grp2 = new QButtonGroup( brushStylesNr, Vertical,
                                             "BrushStyle", box2);
      grp2->setExclusive( true );
      for ( i=0; i<brushStylesNr; i++) 
         brushStyles[i] = new QRadioButton( brushStylesName[i], grp2 );
      brushStyles[brushStyle=1]->setChecked( true );
      QObject::connect( grp2, SIGNAL( clicked( int ) ),
                        this, SLOT( newBrushStyle( int ) ) );
      box2->setGeometry( 650, 30, 150, 400 );

      b1 = new QPushButton( "Neue Pen-Farbe", this );
      b1->setGeometry( 500, 450, 150, 30 );
      penColor = Qt::black;
      b1->setPalette( QPalette( penColor, Qt::black ) );
      QObject::connect( b1,   SIGNAL( clicked() ),
                        this, SLOT( newPenColor() ) );

      b2 = new QPushButton( "Neue Brush-Farbe", this );
      b2->setGeometry( 650, 450, 150, 30 );
      brushColor = Qt::black;
      b2->setPalette( QPalette( brushColor, Qt::black ) );
      QObject::connect( b2,   SIGNAL( clicked() ),
                        this, SLOT( newBrushColor() ) );
   }

protected:
   virtual void mousePressEvent( QMouseEvent * ) {
      repaint( true );
   }

   virtual void paintEvent( QPaintEvent* ) {
      QPainter p( this );

      p.setBackgroundColor( Qt::white );
      p.setPen( QPen( penColor, penWidth, (PenStyle)penStyle ) );
      p.setBrush( QBrush( brushColor, (BrushStyle)brushStyle ) );
      p.drawRect( 70, 70, 340, 340 );
   }

private slots:
   void newPenWidth( int value ) {
      width->display( penWidth = value );
      repaint( true );
   }
   void newPenStyle( int value ) {
      penStyle = value;
      repaint( true );
   }
   void newBrushStyle( int value ) {
      brushStyle = value;
      repaint( true );
   }
   void newPenColor( void ) {
      QColor newColor = QColorDialog::getColor( penColor, 0 );
      if ( newColor.isValid() ) {
         b1->setPalette( QPalette( penColor=newColor, Qt::black ) );
         repaint( true );
      }
   }
   void newBrushColor( void ) {
      QColor newColor = QColorDialog::getColor( brushColor, 0 );
      if ( newColor.isValid() ) {
         b2->setPalette( QPalette( brushColor=newColor, Qt::black ) );
         repaint( true );
      }
   }

private:
   int           penWidth;
   QLCDNumber   *width;
   int           penStyle;
   QRadioButton *penStyles[penStylesNr];
   int           brushStyle;
   QRadioButton *brushStyles[brushStylesNr];
   QColor        penColor,
                 brushColor;
   QPushButton  *b1,
                *b2;
};

#include "penfill.moc"

//..................................................................... main
int main( int argc, char *argv[] )
{
   QApplication myapp( argc, argv );

   PenFill* mywidget = new PenFill();
   mywidget->setGeometry( 0, 0, 800, 500 );

   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
