#include <qapplication.h>
#include <qhbox.h>
#include <qthread.h>
#include <qmutex.h>
#include <qpainter.h>
#include <qpushbutton.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>

QSemaphore exclusive(4);
QMutex     gabel[5];

typedef enum { thinking, hungry, eating } State;
int     gabelPhilo[5];

const double pi = 4 *atan(1);

//....................................................... Klasse Philosoph
class Philosoph: public QThread
{
public:
    Philosoph( bool correct, int id, QWidget *r ) : receiver( r ) {
       noDeadlock = correct;
       nr = id;
       state = thinking;
    }

    void run() {
       while (1) {
          msleep( (noDeadlock) ? (::rand()%10500 + 501) : (5));
          state = hungry;
          if ( noDeadlock )
             exclusive++;

          gabel[nr].lock();  
          gabelPhilo[ nr ] = nr;
          QCustomEvent *event1 = new QCustomEvent( 12345 );
          QApplication::postEvent( receiver, event1 );

          gabel[(nr+1) % 5].lock();  
          gabelPhilo[(nr+1) % 5] = nr;
          QCustomEvent *event2 = new QCustomEvent( 12345 );
          QApplication::postEvent( receiver, event2 );

          state = eating;
          QCustomEvent *event3 = new QCustomEvent( 12345 );
          QApplication::postEvent( receiver, event3 );
          if ( noDeadlock )
             msleep( ::rand()%10500+ 501 );

          state = thinking;
          gabel[nr].unlock();  
          gabel[(nr+1) % 5].unlock();  
          QCustomEvent *event4 = new QCustomEvent( 12345 );
          QApplication::postEvent( receiver, event4 );

          if ( noDeadlock )
             exclusive--;
       }
    }

    State getState( void ) { return state; }

private:
    bool      noDeadlock;
    int       nr;
    QWidget   *receiver;
    State     state;
};

//......................................................... Klasse Display
class PhiloDisplay: public QWidget
{
public:
    PhiloDisplay( bool correct, QWidget *p=0, const char *name=0 )
                                          : QWidget( p, name ) {
       int  i;
       setBackgroundMode( NoBackground );
       setPaletteBackgroundColor( Qt::lightGray );
       for ( i=0; i<5; i++ ) {
          philos[i] = new Philosoph( correct, i, this );
          philoColor[i].setRgb(
                  ::rand()&255, ::rand()&255, ::rand()&255 );
       }
       for ( i=0; i<5; i++ )
          philos[i]->start(); 
    }

protected:
    virtual void customEvent( QCustomEvent *ev ) {
       if ( ev->type() == 12345 )
          repaint(false);
    }

    virtual void paintEvent( QPaintEvent * ) {
       int x, y, i;

       QPainter p( this );

       for ( i=0; i<5; i++ ) {
          float w = i*2*pi/5;
          x = (int) (sin(w) * 200);
          y = (int) (-cos(w) * 200);
          p.setPen( QPen( philoColor[i], 10 ) );
          if      ( philos[i]->getState() == thinking ) p.setBrush( Qt::white );
          else if ( philos[i]->getState() == hungry )   p.setBrush( Qt::red );
          else if ( philos[i]->getState() == eating )   p.setBrush( Qt::green );
          p.drawEllipse( 200+x, 220+y, 100, 100 );    

          float w2 = w - 2*pi/10;
          if ( gabel[i].locked() )
             p.setPen( QPen( philoColor[ gabelPhilo[ i ] ], 20 ) );
          else
             p.setPen( QPen( Qt::white, 20 ) );
          p.drawLine( int(250 + sin(w2)*150), int(250 + -cos(w2)*150),
                      int(250 + sin(w2)*250), int(250 + -cos(w2)*250) );
       }
    }
private:
    Philosoph *philos[5];
    QColor     philoColor[5];
};

//................................................................... main
int main( int argc, char *argv[] ) {
    QApplication app( argc, argv );

    ::srand( time( NULL ) );

    PhiloDisplay *d;
    if ( argc > 1 && !strcmp( argv[1], "-f") )
       d = new PhiloDisplay( false );
    else
       d = new PhiloDisplay( true );
    d->resize( 500, 520 );

    app.setMainWidget( d );
    d->show();
    return app.exec();
}
