#include <qapplication.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qtooltip.h>
#include "bildtabelle.h"

#define cellWidth  100
#define cellHeight 100

//..................................................... Klasse PicOverview
class PicOverview : public Bildtabelle
{
    Q_OBJECT
public:
    PicOverview( int argc, char *argv[], int zeilen, int spalten,
                 QWidget* p=0, const char* name=0 )
                             :  Bildtabelle( zeilen, spalten, p, name ) {
       int  i, w=800, h=600;

       zeilZahl = zeilen;
       spaltZahl = spalten;
       setCellSize( cellWidth, cellHeight, 5, 2 );
       resetAllMarks();
       setCellMark( aktZeil = 0, aktSpalt = 0, true );

       QPixmap bild;
       fileName = new QString [ argc-1 ];
       choicePossible = new bool [ argc-1 ];

       for ( i=0; i<zeilen*spalten; i++)
          choicePossible[ i ] = false;

       for ( i=1; i<argc; i++) {
          int  z = (i-1) / spalten,
               s = (i-1) % spalten;
          fileName[i-1].sprintf( "%s", argv[i] );
          if ( !bild.load( argv[i] ) ) {
             QString  text;
             text.sprintf( "Kann Bild-Datei `%s' nicht laden", argv[i] );
             QMessageBox::information( 0, "Ladefehler", text,
                        QMessageBox::Ok );
             text.sprintf( "%s (no picture)", argv[i] );
             QToolTip::add( this,
                  QRect( s*cellWidth, z*cellHeight, cellWidth, cellHeight ),
                  text );
          } else {
             choicePossible[ i-1 ] = true;
             setCell( z, s, fileName[ i-1 ] );
             QToolTip::add( this,
                  QRect( s*cellWidth, z*cellHeight, cellWidth, cellHeight ),
                  fileName[ i-1 ] );
          }
       }
       QObject::connect( this, SIGNAL( doubleClickPos( int, int ) ),
                         this, SLOT( doubleClickHandle( int, int ) ) );
       QObject::connect( this, SIGNAL( pressPos( int, int ) ),
                         this, SLOT( mousePressHandle( int, int ) ) );
       QObject::connect( this, SIGNAL( keyPressed( int ) ),
                         this, SLOT( keyPressedHandle( int ) ) );
       if ( spalten < 8 ) w = spalten*cellWidth;
       if ( zeilen < 6 )  h = zeilen*cellHeight;

       resize( w+5, h+5 );
    }

private slots:
    void doubleClickHandle( int z, int s ) {
       if ( choicePossible[ z*spaltZahl + s ] ) {
          setCellMark( aktZeil, aktSpalt, false );
          setCellMark( aktZeil = z, aktSpalt = s, true );
          showPicture( aktZeil, aktSpalt );
       }
    }

    void mousePressHandle( int z, int s ) {
       if ( choicePossible[ z*spaltZahl + s ] ) {
          setCellMark( aktZeil, aktSpalt, false );
          setCellMark( aktZeil = z, aktSpalt = s, true );
       }
    }

    void keyPressedHandle( int key ) {
       int  dz=0, ds=0, newZ, newS;

       switch ( key ) {
          case Key_Up:     if ( aktZeil - 1 >= 0 )         dz = -1; break;
          case Key_Down:   if ( aktZeil + 1 < zeilZahl )   dz =  1; break;
          case Key_Left:   if ( aktSpalt - 1 >= 0 )        ds = -1; break;
          case Key_Right:  if ( aktSpalt + 1 < spaltZahl ) ds =  1; break;
          case Key_Return: showPicture( aktZeil, aktSpalt);         break;
       }
       if ( dz != 0 || ds != 0 ) {
          newZ = aktZeil + dz;
          newS = aktSpalt + ds;
          if ( choicePossible[ newZ*spaltZahl + newS ] ) {
             setCellMark( aktZeil, aktSpalt, false );
             setCellMark( aktZeil = newZ, aktSpalt = newS, true );
          }
       }
    }

private:
    void showPicture( int z, int s ) {
       int      i = z*spaltZahl + s;
       if ( choicePossible[ z*spaltZahl + s ] ) {
          QLabel  *picWidget = new QLabel( 0 );
          QPixmap  bild( fileName[i] );
          picWidget->setPixmap( bild );
          picWidget->resize( bild.size() );
          picWidget->show();
       }
    }

    QString  *fileName;
    bool     *choicePossible;
    int      zeilZahl, spaltZahl, aktZeil, aktSpalt;
};

#include "picoverview.moc"

//................................................................... main
int main( int argc, char *argv[] ) {
   QApplication a(argc,argv);

   int  lines = (argc-1)/8,
        cols = 8;
   if ( (argc-1)%8 != 0 )
      lines++;
   PicOverview *m = new PicOverview( argc, argv, lines, cols );

   a.setMainWidget( m );
   m->show();
   return a.exec();
}
