#include <qapplication.h>
#include <qwidget.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qmessagebox.h>
#include <qlayout.h>
#include <stdlib.h>
#include <time.h>
#include "bildtabelle.h"

#define highestCard  52

//........................................................... Klasse Poker
//------------------------------------------------------------------------
class Poker : public Bildtabelle
{
    Q_OBJECT

public:
      //...................................... Poker (Konstruktor)
    Poker( int zeilen, int spalten, QWidget* p=0, const char* name=0 )
                             :  Bildtabelle( zeilen, spalten, p, name ) {
          //... 0 = As, 1 = King, 2 = Queen, ..., 11 = three, 12 = two
       for ( int w=0; w< highestCard / 4; w++)
           cardHelp[w*4+1]=cardHelp[w*4+2]=cardHelp[w*4+3]=cardHelp[w*4+4]=w;
       newGame();
   }
      //.................................................. newGame
   void newGame( void ) {
       QPixmap  bild;
       if ( !bild.load( "back.bmp" ) ) {
          QMessageBox::information( 0, "Ladefehler",
                     "Kann Bild-Datei `back.bmp' nicht laden" ,
                     QMessageBox::Ok );
          qApp->quit();
       }
       setCellSize( bild.width()+16, bild.height()+16, 6, 2 );
       for ( int i = 0; i < 5; i++ )
          setCell( 0, i, bildName[i] = "back.bmp" );
       resize( 450, 120 );
   }
      //................................................ firstDraw
   void firstDraw( void ) {
       int      i, c;
       QString  msg;
       QPixmap  bild;

       resetAllMarks();
       for ( i = 0; i < highestCard; i++ )
          cardsDrawn[i] = false;
       for ( i = 0; i < 5; i++ ) {
          do {
          } while ( cardsDrawn[ c = ::rand()%highestCard ] );
          cardsDrawn[ c ] = true;
          bildName[i].sprintf("%d.bmp", c+1);

          if ( !bild.load( bildName[i] ) ) {
             msg.sprintf( "Kann Bild-Datei %s nicht laden",
                          (const char *)bildName[i] );
             QMessageBox::information( 0, "Ladefehler", msg, QMessageBox::Ok );
             qApp->quit();
          } else {
             setCell( 0, i, bildName[i] );
             cards[i] = c+1;
          }
       }
       QObject::connect( this, SIGNAL( pressPos( int, int ) ),
                         this, SLOT( pressHandle( int, int ) ) );
    }
      //............................................... secondDraw
   void secondDraw( void ) {
       int      i, c;
       QString  msg;
       QPixmap  bild;

       for ( i = 0; i < 5; i++ )
          if ( !isCellMarkSet( 0, i ) ) {
             do {
             } while ( cardsDrawn[ c = ::rand()%highestCard ] );
             cardsDrawn[ c ] = true;
             bildName[i].sprintf("%d.bmp", c+1);

             if ( !bild.load( bildName[i] ) ) {
                msg.sprintf( "Kann Bild-Datei %s nicht laden",
                             (const char *)bildName[i] );
                QMessageBox::information( 0, "Ladefehler", msg, QMessageBox::Ok );
                qApp->quit();
             } else {
                cards[i] = c+1;
                setCell( 0, i, bildName[i] );
             }
         }
       resetAllMarks();
       QObject::disconnect( SIGNAL( pressPos( int, int ) ),
                            this, SLOT( pressHandle( int, int ) ) );
    }
      //................................................. testHand
    int testHand() {  //... Veraenderter Algorithmus aus kpoker
       int   matching=0, w, w2, isRoyal;
       bool  oneValidPair = false;
 
       if ( ( isRoyal = testStraight() ) ) {
          if ( testFlush() ) {
             if ( isRoyal == 2 )
                return 10; //... Royal Flush
             else
                return 9;  //... Straight Flush
          } else
             return 7;
       }
       if ( testFlush() )
           return 8;
 
      for ( w=0; w<5; w++) //... sucht Paare, Drillinge, Vierlinge
         for ( w2=w+1; w2<5; w2++) {
            if ( cardHelp[cards[w]] == cardHelp[cards[w2]] ) {
               if ( cards[w] <= 16 )
                  oneValidPair = true;
               matching++;
               setCellMark( 0, w, true );
               setCellMark( 0, w2, true );
            }
         }
      if ( matching == 1 && !oneValidPair ) {
         resetAllMarks();
         return 0;
      }
      return matching;
   }

private slots:
    void pressHandle( int z, int s ) {
       setCellMark( z, s, !isCellMarkSet( z, s ) );
    }

private:
      //............................................ findCardTypes
    bool findCardTypes(int cardT[5], int card) {
       for ( int w=0; w<5; w++)
          if (cardT[w] == card)
             return true;
       return false;
    }
      //................................................ testFlush
    bool testFlush() {
       int w, mycolor=cards[0] % 4;
       for ( w=1; w<5; w++ )
          if (mycolor != cards[w] %4)
             return false;
       for ( w=0; w<5; w++ )
          setCellMark( 0, w, true );
       return true;
    }
      //............................................. testStraight
    int testStraight() {
       int cardTypes[5], w,
           lowest=1000; // erste untersuchte Karte wird so zunaechst lowest
 
       for (w=0; w<5; w++) {
          cardTypes[w]=cardHelp[cards[w]];
          if ( cardTypes[w] < lowest )
              lowest=cardTypes[w];
       }
 
       //.. SpecialFall: As-2-3-4-5
       if ( !(findCardTypes(cardTypes, 0) && findCardTypes(cardTypes, 12) &&
              findCardTypes(cardTypes, 11) && findCardTypes(cardTypes, 10) &&
              findCardTypes(cardTypes, 9) ) )
           for (w=0; w<5; w++)
              if (!findCardTypes(cardTypes,lowest+w))
                  return 0;
       for (w=0; w<5; w++)
          setCellMark( 0, w, true );
 
       if (lowest == 0)
          return 2; // koennte royal flush sein
       return 1;
    }

    QString bildName[5];
    bool    cardsDrawn[highestCard];
    int     cardHelp[highestCard+1];
    int     cards[5];
};

//........................................................ Klasse MyWidget
//------------------------------------------------------------------------
class MyWidget : public QWidget {
    Q_OBJECT
public:
    MyWidget( QWidget* p=0, const char* name=0 ) :  QWidget( p, name ) {
       ::srand( time( NULL ) );
       status = 0;
       konto = 100;

       QGridLayout *grid = new QGridLayout( this, 5, 1, 10 );

       poker = new Poker( 1, 5, this );
       grid->addWidget( poker, 0, 0 );   

       l1 = new QLabel( this );
       l1->setMaximumHeight( 30 );
       l1->setPalette( Qt::white );
       l1->setText( "" );
       grid->addWidget( l1, 1, 0 );   

       l2 = new QLabel( this );
       l2->setMaximumHeight( 30 );
       l2->setText( "" );
       l2->setPalette( Qt::lightGray );
       grid->addWidget( l2, 2, 0 );   

       b = new QPushButton( "Draw !", this );
       grid->addWidget( b, 3, 0 );   
       QObject::connect( b, SIGNAL( clicked() ),
                         this, SLOT( statusChanged() ) );

       q = new QPushButton( "Quit", this );
       grid->addWidget( q, 4, 0 );   
       QObject::connect( q, SIGNAL( clicked() ), qApp, SLOT( quit() ) );
    }

private slots:
    void statusChanged( void ) {
       switch ( status ) {
          case 0:
             poker->firstDraw();
             l1->setText( "Klicken sie die zu behaltenden Karten an" );
             break;
          case 1:
             poker->secondDraw();
             QString  text;
             int      gewinn;

             konto -= 5;
             switch ( poker->testHand() ) {
                case 1: text = "Ein Paar";       gewinn = 5;    break;
                case 2: text = "Zwei Paare";     gewinn = 10;   break;
                case 3: text = "Drilling";       gewinn = 15;   break;
                case 4: text = "Full House";     gewinn = 40;   break;
                case 6: text = "4 einer Art";    gewinn = 125;  break;
                case 7: text = "Straight";       gewinn = 20;   break;
                case 8: text = "Flush";          gewinn = 25;   break;
                case 9: text = "Straight Flush"; gewinn = 250;  break;
               case 10: text = "Royal Flush";    gewinn = 2000; break;
               default: text = "Nichts";         gewinn = 0;    break;
             }                                                                 
             l1->setText( text );
             konto += gewinn;
             text.sprintf("%d Dollar %s; Kontostand: %d",
                          abs(gewinn), "gewonnen", konto);
             l2->setText( text );
             l2->setPalette( (gewinn <= 0) ? Qt::black : Qt::green );
             break;
       }
       status = ++status % 2;
    }
      
private:
    QLabel      *l1, *l2;
    QPushButton *b, *q;
    Poker       *poker;
    int         status, konto;
};

#include "poker.moc"

//................................................................... main
//------------------------------------------------------------------------
int main( int argc, char *argv[] ) {
   QApplication a(argc,argv);

   MyWidget *w = new MyWidget;

   a.setMainWidget( w );
   w->resize( 460, 300 );
   w->show();
   return a.exec();
}
