#include <qapplication.h>
#include <qmainwindow.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qtextedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qwhatsthis.h>
#include <qsimplerichtext.h> 

#include "filesave.xpm"
#include "fileopen.xpm"
#include "fileprint.xpm"

const char * fileOpenText = "<img source=\"fileopen\"> "
    "Klicke diesen Button, um eine <em>neue Datei</em> zu ffnen. <br><br>"
    "Dies ist auch mit dem <b>Oeffnen-Eintrag</b> des Datei-Mens mglich.";

const char * fileSaveText = "Button zum Speichern der Datei.<br><br>"
    "Dies ist auch mit dem <b>Speichern-Eintrag</b> des Datei-Mens mglich.";

const char * filePrintText = "Button zum Drucken der Datei.<br><br>"
    "Dies ist auch mit dem <b>Drucken-Eintrag</b> des Datei-Mens mglich.";

//......................................................... Klasse MeinEditor
class MeinEditor: public QMainWindow
{
    Q_OBJECT
public:
    MeinEditor();
    ~MeinEditor() { delete printer; }
    
protected:
    void closeEvent( QCloseEvent* ce ) {      //........... closeEvent
       if ( !e->isModified() ) {
          ce->accept();
          return;
       }
       switch( QMessageBox::information( this, "Beispiel zum Drucken von Text",
                  "Datei wurde gendert, aber noch nicht gespeichert",
                  "Sichern", "Abbrechen", "Ohne Sichern beenden", 0, 1 ) ) {
          case 0 : save(); ce->accept(); break;
          case 1 :
          default: ce->ignore();         break;
          case 2 : ce->accept();         break;
       }
    }

private slots:
    void neuDatei() {                                //............. neuDatei
       MeinEditor *ed = new MeinEditor;
       ed->show();
    }
    void load() {                                    //................ load
        QString dn = QFileDialog::getOpenFileName( 
                               QString::null, QString::null, this);
        if ( !dn.isEmpty() )
           load( dateiname = dn );
        else
           statusBar()->message( "Laden der Datei abgebrochen", 2000 );
    }
    void saveAs() {                                  //............... saveAs
        QString dn = QFileDialog::getSaveFileName(
                               QString::null, QString::null, this);
        if ( !dn.isEmpty() ) {
           dateiname = dn;
           save();
        } else
           statusBar()->message( "Speichern der Datei abgebrochen", 2000 );
    }
    void about() {                                  //................ about
        QMessageBox::about( this, "Beispiel zum Drucken von Text",
             "Dieses Programm zeigt, wie Text ausgedruckt werden kann.");
    }
    void aboutQt() {                               //................ aboutQt
        QMessageBox::aboutQt( this, "Beispiel zum Drucken von Text" );
    }

    void load( const char *dateiName );
    void save();
    void print();

private:
    QPrinter   *printer;
    QTextEdit  *e;
    QToolBar   *dateiTools;
    QString    dateiname;
};

#include "printtext.moc"

//.................................................. MeinEditor (Konstruktor)
MeinEditor::MeinEditor() : QMainWindow( 0, "Beispiel zum Drucken von Text",
                                        WDestructiveClose ) {
    int id;

    printer = new QPrinter;
    QPixmap openIcon, saveIcon, printIcon;

    dateiTools = new QToolBar( this );

    openIcon = QPixmap( fileopen );
    QToolButton *fileOpen = new QToolButton( openIcon, "Oeffne Datei",
                    QString::null, this, SLOT(load()), dateiTools );

    saveIcon = QPixmap( filesave );
    QToolButton *fileSave = new QToolButton( saveIcon, "Sichere Datei",
                    QString::null, this, SLOT(save()), dateiTools );

    printIcon = QPixmap( fileprint );
    QToolButton *filePrint = new QToolButton( printIcon, "Drucke Datei",
                    QString::null, this, SLOT(print()), dateiTools );

    (void)QWhatsThis::whatsThisButton( dateiTools );

    QWhatsThis::add( fileOpen, fileOpenText );
    QMimeSourceFactory::defaultFactory()->setPixmap( "fileopen", openIcon );
    QWhatsThis::add( fileSave, fileSaveText );
    QWhatsThis::add( filePrint, filePrintText );

    QPopupMenu *datei = new QPopupMenu( this );
    menuBar()->insertItem( "&Datei", datei );

    datei->insertItem( "&Neu", this, SLOT(neuDatei()), CTRL+Key_N );

    id = datei->insertItem( openIcon, "&Oeffnen",
                            this, SLOT(load()), CTRL+Key_O );
    datei->setWhatsThis( id, fileOpenText );

    id = datei->insertItem( saveIcon, "&Speichern",
                            this, SLOT(save()), CTRL+Key_S );
    datei->setWhatsThis( id, fileSaveText );
    id = datei->insertItem( "Speichern &unter ...", this, SLOT(saveAs()) );
    datei->setWhatsThis( id, fileSaveText );
    datei->insertSeparator();
    id = datei->insertItem( printIcon, "&Drucken",
                           this, SLOT(print()), CTRL+Key_P );
    datei->setWhatsThis( id, filePrintText );
    datei->insertSeparator();
    datei->insertItem( "S&chliessen", this, SLOT(close()), CTRL+Key_W );
    datei->insertItem( "&Beenden", qApp, SLOT(closeAllWindows()), CTRL+Key_Q );

    QPopupMenu *help = new QPopupMenu( this );
    menuBar()->insertSeparator();
    menuBar()->insertItem( "&Hilfe", help );

    help->insertItem( "&About", this, SLOT(about()), Key_F1);
    help->insertItem( "About&Qt", this, SLOT(aboutQt()));
    help->insertSeparator();
    help->insertItem( "What's &This", this, SLOT(whatsThis()), SHIFT+Key_F1);

    e = new QTextEdit( this, "editor" );
    e->setFocus();
    setCentralWidget( e );
    statusBar()->message( "Ready", 2000 );
    resize( 450, 600 );
}

//..................................................................... load
void MeinEditor::load( const char *dateiName ) {
    QFile f( dateiName );
    if ( !f.open( IO_ReadOnly ) )
       return;
    e->clear();

    QTextStream t(&f);
    e->setText( t.read() );
    f.close();
    e->setModified( false );
    setCaption( dateiName );
    QString s;
    s.sprintf( "Lade gerade Datei %s", dateiName );
    statusBar()->message( s, 2000 );
}

//..................................................................... save
void MeinEditor::save() {
    if ( dateiname.isEmpty() ) {
       saveAs();
       return;
    }
    QFile f( dateiname );
    if ( !f.open( IO_WriteOnly ) ) {
       statusBar()->message(
         QString("Kann nicht in Datei %1 schreiben").arg(dateiname), 2000 );
       return;
    }
    QTextStream t( &f );
    t << e->text();
    e->setModified( false );
    f.close();
    setCaption( dateiname );
    statusBar()->message(
            QString( "Datei %1 gespeichert" ).arg( dateiname ), 2000 );
}
//.................................................................... print
void MeinEditor::print() {
    printer->setMinMax( 1, 10000 );
    printer->setFullPage( true );
    if ( printer->setup(this) ) {  // Oeffnen eines QPrinter-Dialogs
       int von = printer->fromPage(),
           bis = printer->toPage();
       QPainter p;
       if ( !p.begin( printer ) ) {
          statusBar()->message( "Drucken abgebrochen...", 2000 );
          return;
       }
       QPaintDeviceMetrics metrics( p.device() );
       int dpiy = metrics.logicalDpiY();
       int margin = (int) ( (2/2.54)*dpiy ); // 2 cm Rand
       QRect body( margin, margin, metrics.width() - 2*margin,
                                   metrics.height() - 2*margin );
       QSimpleRichText richText( QStyleSheet::convertFromPlainText(e->text()),
                                 QFont( "Courier", 10 ),
                                 e->context(),
                                 e->styleSheet(),
                                 e->mimeSourceFactory(),
                                 body.height() );
       richText.setWidth( &p, body.width() );
       QRect view( body );
       QString str;
       int seite = 1;
       while ( true ) {
          if (seite >= von && seite <= bis )
             richText.draw( &p, body.left(), body.top(), view, colorGroup() );
          view.moveBy( 0, body.height() );
          p.translate( 0 , -body.height() );
          if (seite >= von && seite <= bis ) {
             p.drawText( view.right()
                          - p.fontMetrics().width( QString::number(seite) ),
                         view.bottom() + p.fontMetrics().ascent()+5,
                         QString::number(seite) );
          }
          if ( view.top()  >= richText.height() || seite+1 > bis )
             break;
          if (seite >= von && seite <= bis ) {
             str.sprintf( "Drucke Seite %d ...", seite );
             statusBar()->message( str );
             printer->newPage();
          }
          seite++;
       }
       statusBar()->message( "Drucken beendet...", 2000 );
   } else
       statusBar()->message( "Drucken abgebrochen...", 2000 );
}
//..................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    MeinEditor * mw = new MeinEditor();
    mw->show();
    a.connect( &a, SIGNAL(lastWindowClosed()), &a, SLOT(quit()) );
    return a.exec();
}
