#include <qapplication.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qprogressbar.h>
#include <qbuttongroup.h>
#include <qvbox.h>
#include <qtimer.h>
//................................................................. ProgressBar
class ProgressBar : public QVBox {
    Q_OBJECT
public:
    ProgressBar( QWidget *parent = 0, const char *name = 0 );
protected:
    QRadioButton  *langsam, *normal, *schnell;
    QPushButton   *start, *pause, *ruecksetz;
    QProgressBar  *progress;
    QTimer timer;
protected slots:
    void slotStart();
    void slotRuecksetz();
    void slotTimeout();
};
#include "progressbar.moc"
//..................................................... ProgressBar-Konstruktor
ProgressBar::ProgressBar( QWidget *parent, const char *name )
               : QVBox( parent, name ), timer()               {
    setMargin( 10 );
      // Kreiere eine exklusive Buttongruppe mit zwei Spalten
    QButtonGroup *bg = new QButtonGroup( 2, Horizontal, this );
    bg->setRadioButtonExclusive( TRUE );
      // Drei Radiobuttons zum Festlegen der Geschwindigkeit und
      // zwei Pushbutton zum Start/Pause/Fortsetzen and Ruecksetzen
      // des progressbars
    langsam   = new QRadioButton( "&Langsam", bg );
    start     = new QPushButton( "S&tart", bg );
    normal    = new QRadioButton( "&Normal", bg );
    ruecksetz = new QPushButton( "&Ruecksetzen", bg );
    schnell   = new QRadioButton( "&Schnell", bg );
    start->setPaletteBackgroundColor( QColor( 0, 255, 255 ) );
    ruecksetz->setPaletteBackgroundColor( Qt::white );

    progress = new QProgressBar( 100, this );

    connect( start, SIGNAL( clicked() ), this, SLOT( slotStart() ) );
    connect( ruecksetz, SIGNAL( clicked() ), this, SLOT( slotRuecksetz() ) );
    connect( &timer, SIGNAL( timeout() ), this, SLOT( slotTimeout() ) );

    normal->setChecked( true );
}
//...................................................... ProgressBar::slotStart
// wird aufgerufen, wenn Benutzer auf Start/Pause/Fortsetzen klickt
void ProgressBar::slotStart() {
    if ( progress->progress() == -1 ) { // progressbar steht am Anfang
        if ( langsam->isChecked() )
            progress->setTotalSteps( 10000 );
        else if ( normal->isChecked() )
            progress->setTotalSteps( 1000 );
        else
            progress->setTotalSteps( 150 );
        langsam->setEnabled( false );
        normal->setEnabled( false );
        schnell->setEnabled( false );
    }
    if ( !timer.isActive() ) {  // progressbar laeuft gerade nicht
        timer.start( 1 ); // starte Zeitschaltuhr (und Progress) mit
                          // einem Intervall von 1 Millisekunde

        start->setText( "&Pause" ); // Text 'Pause' fuer 
                                    // Start/Pause/Fortsetzen-Button
    } else {  // progressbar laeuft gerade
        timer.stop(); // halte Zeitschaltuhr (und Progress) an
        start->setText( "&Fortsetzen" ); // Text 'Fortsetzen' fuer 
                                         // Start/Pause/Fortsetzen-Button
    }
}
//.................................................. ProgressBar::slotRuecksetz
// wird aufgerufen, wenn Benutzer auf ruecksetz-Button klickt
void ProgressBar::slotRuecksetz() {
    timer.stop(); // halte Zeitschaltuhr (und Progress) an
    start->setText( "S&tart" ); // Text 'Start' fuer 
                                // Start/Pause/Fortsetzen-Button
    start->setEnabled( true );
    langsam->setEnabled( true );
    normal->setEnabled( true );
    schnell->setEnabled( true );
    progress->reset(); // setze progressbar zurueck;
}
//.................................................... ProgressBar::slotTimeout
// wird jede ms aufgerufen, wenn Schaltuhr laeuft (also progressbar aktiv ist)
void ProgressBar::slotTimeout() {
    int p = progress->progress();
    if ( p == progress->totalSteps() )  { // progress beendet
       start->setText( "S&tart" ); // Text 'Start' fuer 
                                   // Start/Pause/Fortsetzen-Button
       start->setEnabled( false );
    } else
       progress->setProgress( ++p ); // erhoehe den progress
}
//....................................................................... main
int main(int argc, char *argv[]) {
    QApplication a(argc,argv);
    ProgressBar progressbar;
    progressbar.resize(300,200);
    progressbar.setCaption("Beispiel fuer eine ProgressBar");
    a.setMainWidget(&progressbar);
    progressbar.show();
    return a.exec();
}
