#include <qprogressdialog.h>
#include <qapplication.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qpainter.h>
#include <stdlib.h>
#include <math.h>
//.................................................................. LinienMaler
class LinienMaler : public QWidget {
  Q_OBJECT
    enum { start_zahl_id = 1000, ende_zahl_id = 1006 };

    int maleLinienZahl(int id) { return( (int)pow(10, (id-start_zahl_id)+1) ); }

    QString textLinienZahl(int id) {
       return QString("%1 Linien").arg( maleLinienZahl(id) );
    }
public:
    LinienMaler() {
       menubar = new QMenuBar( this, "menu" );
       QPopupMenu* datei = new QPopupMenu();
       menubar->insertItem( "&Datei", datei );
       for (int i=start_zahl_id; i<=ende_zahl_id; i++)
          datei->insertItem( textLinienZahl(i), i );
       connect( menubar, SIGNAL(activated(int)),
                this,    SLOT(welcheZahl(int)) );
       datei->insertSeparator();
       datei->insertItem( "Quit", qApp,  SLOT(quit()) );
       optionen = new QPopupMenu();
       menubar->insertItem( "&Optionen", optionen );
       td_id = optionen->insertItem( "Timer driven", this, SLOT(timerDriven()) );
       ld_id = optionen->insertItem( "Loop driven", this, SLOT(loopDriven()) );
       optionen->insertSeparator();
       md_id = optionen->insertItem( "Keine Mindestdauer", this,
                                     SLOT(schalteEinAusMinimalDauer()) );
       optionen->setCheckable( TRUE );
       loopDriven();
       setFixedSize( 400, 300 );
       setPaletteBackgroundColor( Qt::black );
    }
public slots:
    void welcheZahl(int id) {
       if (id >= start_zahl_id && id <= ende_zahl_id)
          male( maleLinienZahl(id) );
    }
    void stopMalen() { aufhoeren = TRUE; }
    void timerDriven() {
       zeit_gesteuert = TRUE;
       optionen->setItemChecked( td_id, TRUE );
       optionen->setItemChecked( ld_id, FALSE );
    }
    void loopDriven() {
       zeit_gesteuert = FALSE;
       optionen->setItemChecked( ld_id, TRUE );
       optionen->setItemChecked( td_id, FALSE );
    }
    void schalteEinAusMinimalDauer() {
       optionen->setItemChecked( md_id, !optionen->isItemChecked( md_id ) );
    }
private:
    void timerEvent( QTimerEvent* ) {
       QPainter p(this);
       QString  str;
       int      i;
       p.moveTo( old_x, old_y );
       p.setPen( QColor(rand()%255, rand()%255, rand()%255) );
       p.lineTo( old_x = rand()%width(), old_y = rand()%height() ); 
       pb->setProgress( i = pb->totalSteps() - linien-- );
       if (i%10000==0) {
          str.sprintf("Zeichne Linien (Zeit gesteuert)\n"
                      "%d (%d) Linien gezeichnet\n", i, pb->totalSteps()); 
          pb->setLabelText(str);
       }
       if (!linien || aufhoeren) {
          pb->setProgress( pb->totalSteps() );
          QPainter p(this);
          p.fillRect(0, 0, width(), height(), backgroundColor());
          aktiviereLinienMenues(TRUE);
          killTimers();
          delete pb;
          pb = 0;
       }
    }
    QProgressDialog* newProgressDialog( const char* label,
                                        int steps, bool modal ) {
       QProgressDialog *d = new QProgressDialog(label, "Cancel", steps, this,
                                                "progress", modal);
       if ( optionen->isItemChecked( md_id ) )
          d->setMinimumDuration(0);
       return d;
    }
    void aktiviereLinienMenues(bool ja_nein) {
       for (int i=start_zahl_id; i<=ende_zahl_id; i++) {
           menubar->setItemEnabled(i, ja_nein);
       }
    }
    void male(int n) {
       if ( zeit_gesteuert ) {
          linien = n;
          pb = newProgressDialog("Zeichne Linien.\nZeit gesteuert.", n, FALSE);
          pb->setCaption("Bitte warten...");
          connect(pb, SIGNAL(canceled()), this, SLOT(stopMalen()));
          aktiviereLinienMenues(FALSE);
          startTimer(0);
          aufhoeren = FALSE;
       } else {
          QProgressDialog* lpb = newProgressDialog(
                     "Zeichne Linien.\nCode gesteuert.", n, TRUE);
          lpb->setCaption("Bitte warten...");
          QPainter p(this);
          QString  str;
          for (int i=0; i<n; i++) {
             lpb->setProgress(i);
             if (i%10000==0) {
                str.sprintf("Zeichne Linien (Code gesteuert)\n"
                            "%d (%d) Linien gezeichnet\n", i, n); 
                lpb->setLabelText(str);
             }
             if ( lpb->wasCanceled() )
                 break;
             p.moveTo( old_x, old_y );
             p.setPen( QColor(rand()%255, rand()%255, rand()%255) );
             p.lineTo( old_x = rand()%width(), old_y = rand()%height() ); 
          }
          lpb->setProgress(n);
          p.fillRect(0, 0, width(), height(), backgroundColor());
       }
    }
    QMenuBar         *menubar;
    QProgressDialog  *pb;
    QPopupMenu       *optionen;
    int              old_x, old_y, td_id, ld_id, md_id, linien;
    bool             zeit_gesteuert, aufhoeren;
};
#include "progressdialog.moc"
//......................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    LinienMaler* lmaler = new LinienMaler;
    a.setMainWidget(lmaler);
    lmaler->show();
    return a.exec();
}
