#include <qapplication.h>
#include <qgrid.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qtextedit.h>
#include <qpushbutton.h>
#include <qptrqueue.h>
#include <qtimer.h>
#include <stdlib.h>
#include <time.h>

#define  MAX_QUEUE  10

const char *vorname[] = { "Hans", "Eva",  "Toni", "Mike", "Anna",
                          "Mark", "Gert", "Lola", "Otto", "Lisa" };
const int vZahl = sizeof(vorname) / sizeof(vorname[0]);

//....................................................... Klasse Simulation
class Simulation : public QGrid
{
    Q_OBJECT
public:
    Simulation( QWidget *parent=0, const char *name=0 )
                             : QGrid( 2, parent, name ) {
       setMargin( 10 );
       setSpacing( 5 );
       setupSimulation();
    }

private slots:
    void ankunftTimerSlot() {
       ankunftTimer->changeInterval( 1000 + ::rand() % 2000 );
       simuliereAnkunft();
    }

    void  simuliereBedienung( void );

private:
    QLabel             *empfang, *platz[MAX_QUEUE], *service;
    QTimer             *ankunftTimer;
    QTextEdit          *logFenster;
    QPtrQueue<QString>  warteSchlange;

    void  setupSimulation( void );
    void  simuliereAnkunft( void );
};

//......................................................... setupSimulation
void Simulation::setupSimulation( void )
{
    ::srand( time(NULL) );
    warteSchlange.setAutoDelete( true );
    ankunftTimer = new QTimer( this );

    QVBox *vbox = new QVBox( this );
    empfang = new QLabel( vbox );
    empfang->setFrameStyle( QFrame::Box | QFrame::Raised );
    empfang->setLineWidth( 5 );
    empfang->setPaletteBackgroundColor( Qt::red.light( 180 ) );
    empfang->setAlignment( AlignCenter );
    empfang->setText( "" );
    for (int i=MAX_QUEUE-1; i>=0; i--) {
       platz[i] = new QLabel( vbox );
       platz[i]->setFrameStyle( QFrame::Box | QFrame::Raised );
       platz[i]->setPaletteBackgroundColor( Qt::white );
       platz[i]->setMinimumWidth( 200 );
       platz[i]->setAlignment( AlignCenter );
       platz[i]->setText( "" );
    }
    service = new QLabel( vbox );
    service->setFrameStyle( QFrame::Box | QFrame::Raised );
    service->setLineWidth( 5 );
    service->setPaletteBackgroundColor( Qt::green.light( 180 ) );
    service->setAlignment( AlignCenter );
    service->setText( "" );
 
    vbox = new QVBox( this );
    new QLabel( "Protokollfenster", vbox );
    logFenster = new QTextEdit( vbox );
   
    QPushButton *b1 = new QPushButton( "Nchste(r) bitte", vbox ); 
    b1->setPaletteBackgroundColor( Qt::green.light( 100 ) );
    QObject::connect( b1, SIGNAL( clicked() ), SLOT( simuliereBedienung() ) );
    ankunftTimer->start( ::rand()%2000, true );
    connect( ankunftTimer, SIGNAL( timeout() ),
             this        , SLOT( ankunftTimerSlot() ) );
}

//........................................................ simuliereAnkunft
void Simulation::simuliereAnkunft( void )
{
    QString name = vorname[rand() % vZahl];
    name += " ";
    name += 'A' + rand() % 26;
    name += "."; 
    QString log = name;

    if ( warteSchlange.count() < MAX_QUEUE ) {
       warteSchlange.enqueue( new QString(name) );
       platz[warteSchlange.count()-1]->setText( name );
       log += " kommt an\n";
       logFenster->insert( log ); 
       if ( warteSchlange.count() == 1 && service->text() == "" )
          simuliereBedienung();
    } else {
       empfang->setText( name );
       log += " wird abgewiesen (Warteschlange voll)\n";
       logFenster->insert( log ); 
    }
    logFenster->setCursorPosition( logFenster->lines(), 0 );
}

//...................................................... simuliereBedienung
void Simulation::simuliereBedienung( void )
{
    if ( !warteSchlange.isEmpty() ) {
       QString log = *warteSchlange.head();
       log += " wird bedient\n";
       logFenster->insert( log ); 
       service->setText( *warteSchlange.head() );
       for (int i=0; i<MAX_QUEUE-1; i++) 
          platz[i]->setText( platz[i+1]->text() );
       warteSchlange.dequeue();
       platz[warteSchlange.count()]->setText( "" );
    } else {
       service->setText( "" );
       logFenster->insert( "Pause (Warteschlange leer)\n" ); 
    }
    logFenster->setCursorPosition( logFenster->lines(), 0 );
}

#include "ptrqueue.moc"

//.................................................................... main
int main( int argc, char *argv[] )
{
    QApplication a( argc, argv );

    QWidget w;
    Simulation queueSim( &w );
    w.setCaption( "Warteschlangen-Simulation" );
    queueSim.resize( 550, 550 );

    a.setMainWidget( &w );
    w.setGeometry( 0, 0, 550, 550 );
    w.show();

    return a.exec();
}
