#include <qapplication.h>
#include <qwidget.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <stdlib.h>
#include <time.h>
#include "bildtabelle.h"

const char *picNames[] = { "kids1.xpm", "niklas.xpm",
                           "niklclow.xpm", "lausbub.xpm" };
const int picNo = sizeof( picNames ) / sizeof( picNames[0] );
const int zeilen = 7;
const int spalten = 7;
const int teilNo = zeilen * spalten;
const int bord     = 2;
const int frameWid = 1;
const int cellWid  = 600/spalten + 2*bord + 2*frameWid;
const int cellHigh = 600/zeilen + 2*bord + 2*frameWid;
//.......................................................... Klasse Puzzle
class Puzzle : public Bildtabelle {
    Q_OBJECT
public:
      //..................................... Memory (Konstruktor)
    Puzzle( QWidget* p=0 ) :  Bildtabelle( zeilen, spalten, p ) {
       ::srand( time(NULL) );
       setCellSize( cellWid, cellHigh, bord, frameWid );
       QObject::connect( this, SIGNAL( pressPos( int, int ) ),
                         this, SLOT( pressHandle( int, int ) ) );
       QObject::connect( this, SIGNAL( releasePos( int, int ) ),
                         this, SLOT( releaseHandle( int, int ) ) );
       QObject::connect( this, SIGNAL( doubleClickPos( int, int ) ),
                         this, SLOT( doubleClickHandle( int, int ) ) );
       newGame();
    }
      //.................................................. newGame
    void newGame( void ) {
       int  i, j, x, y, n;
       resetAllMarks();
       pressZ = pressS = -1;
       if ( !bild.load( picNames[n = ::rand() % picNo] ) ) {
          QString  text;
          text.sprintf( "Kann Bild-Datei `%s' nicht laden", picNames[n] );
          QMessageBox::information( 0, "Ladefehler", text, QMessageBox::Ok );
          qApp->quit();
       }
       int sizeX = bild.width() / spalten,
           sizeY = bild.height() / zeilen;
       for ( i=0; i<zeilen; i++)
          for ( j=0; j<spalten; j++) {
             picTeile[i][j].resize( sizeX, sizeY );
             QPainter p( &(picTeile[i][j]) );
             p.drawPixmap( 0, 0, bild, j*sizeX, i*sizeY, sizeX, sizeY );
             picTeileID[i][j] = i*1000 + j;
             puzzleTeileID[i][j] = -1;
          }
       for ( i=0; i<zeilen; i++)
          for ( j=0; j<spalten; j++) {
             do {
             } while ( puzzleTeileID[ x = ::rand()%zeilen ]
                                    [ y = ::rand()%spalten ] >= 0 );
             puzzleTeileID[x][y] = picTeileID[i][j];              
          }
       for ( i=0; i<zeilen; i++)
          for ( j=0; j<spalten; j++)
              setCell( i, j, picTeile[ puzzleTeileID[i][j] / 1000 ]
                                     [ puzzleTeileID[i][j] % 1000 ] );
       for ( i=0; i<zeilen; i++)
          for ( j=0; j<spalten; j++)
              if ( puzzleTeileID[i][j] == picTeileID[i][j] )
                 setCellMark( i, j, true );
    }
private slots:
      //.............................................. pressHandle
    void pressHandle( int z, int s ) {
       if ( isCellMarkSet( z, s ) ) {
          pressZ = pressS = -1;
          return;
       }
       pressZ = z;
       pressS = s;
    }
      //............................................ releaseHandle
    void releaseHandle( int z, int s ) {
       if ( isCellMarkSet( z, s ) || pressZ < 0 || pressS < 0 ) {
          pressZ = pressS = -1;
          return;
       }
       int  swap = puzzleTeileID[z][s];
       puzzleTeileID[z][s] = puzzleTeileID[pressZ][pressS];
       puzzleTeileID[pressZ][pressS] = swap;
       setCell( z, s, picTeile[ puzzleTeileID[z][s] / 1000 ]
                              [ puzzleTeileID[z][s] % 1000 ] );
       setCell( pressZ, pressS,
                picTeile[ puzzleTeileID[pressZ][pressS] / 1000 ]
                        [ puzzleTeileID[pressZ][pressS] % 1000 ] );
       if ( puzzleTeileID[z][s] == picTeileID[z][s] )
          setCellMark( z, s, true );
       if ( puzzleTeileID[pressZ][pressS] == picTeileID[pressZ][pressS] )
          setCellMark( pressZ, pressS, true );
    }
      //........................................ doubleClickHandle
    void doubleClickHandle( int, int ) {
       newGame();
    }
private:
    QPixmap  bild;
    QPixmap  picTeile[ zeilen ][ spalten ];
    int      picTeileID[ zeilen ][ spalten ],
             puzzleTeileID[ zeilen ][ spalten ],
             pressZ, pressS;
};
#include "puzzle.moc"
//................................................................... main
int main( int argc, char *argv[] ) {
   QApplication a(argc,argv);
   Puzzle *m = new Puzzle;
   a.setMainWidget( m );
   m->resize( spalten*cellWid+5, zeilen*cellHigh+5 );
   m->show();
   return a.exec();
}
