#include <qapplication.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qvalidator.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

//........................................... Makro zum Aendern der Richtung
#define   LINKS(delta)   phi += ((delta)*PI)/180

//...................................................... Groesse des Widgets
#define MAXX  800
#define MAXY  600

//........................................................ Klasse Pythagoras
class Pythagoras : public QWidget
{
   Q_OBJECT

public:
   Pythagoras( QWidget *p=0 ) : QWidget( p ), PI( 4*atan(1) ) {
       setCaption( "Baum des Pythagoras" );
       setPaletteBackgroundColor( Qt::white );

       relation = maxLength = 0;

       QLabel *l = new QLabel( " Katheden-Relation", this );
       l->setGeometry( 0, 0, 120, 20 );
       l->setFont( QFont( "Times", 12 ) );
       l->setPaletteBackgroundColor( Qt::lightGray );
       relationKath = new QLineEdit( this );
       relationKath->setValidator(
                      new QDoubleValidator( 0.0, 1000.0, 4, relationKath ) );
       relationKath->setGeometry( 120, 0, 30, 20 );
       QObject::connect( relationKath, SIGNAL( textChanged( const QString & ) ),
                         this,     SLOT( readRelation( const QString & ) ) );
       l = new QLabel( " Linien-Lnge", this );
       l->setFont( QFont( "Times", 12 ) );
       l->setPaletteBackgroundColor( Qt::lightGray );
       l->setGeometry( 0, 20, 120, 20 );
       maxLengthLine = new QLineEdit( this );
       maxLengthLine->setValidator(
                      new QDoubleValidator( 0.0, 1000.0, 4, maxLengthLine ) );
       maxLengthLine->setGeometry( 120, 20, 30, 20 );
       QObject::connect( maxLengthLine,
                         SIGNAL( textChanged( const QString & ) ),
                         this, SLOT( readMaxLength( const QString & ) ) );
   }

protected:
   virtual void mousePressEvent( QMouseEvent * ) {
      if ( relation != 0 && maxLength != 0 ) {
         w = atan(relation) * 180 / PI;
         xmin=1e6, xmax=-1e6, ymin=1e6, ymax=-1e6;
         phi = (90*PI)/180;
         x = y = altX = altY = 0;

         for ( int i=0; i<100; i++ )
            color[i] = QColor( ::rand()&255, ::rand()&255, ::rand()&255 );

         quadrat( 0, 100.0, 1, 0 );

         phi = (90*PI)/180;
         x = y = altX = altY = 0;
         repaint( true );
      }
   }

   virtual void paintEvent( QPaintEvent* ) {
      if ( relation != 0 && maxLength != 0 ) {
         QPainter p( this );

         p.setWindow( int(xmin), int(ymin), int(xmax-xmin+1), int(ymax-ymin+1) );
         quadrat( 0, 100.0, 2, &p );
         LINKS(270);
         calculatePaint( 0, 2, 100, &p );
      }
   }

private slots:
   void readRelation( const QString & text ) {
      relation = text.toDouble();
   }
   void readMaxLength( const QString & text ) {
      maxLength = text.toDouble();
   }
 
private:
   QLineEdit *relationKath,
             *maxLengthLine;
   double    relation, maxLength,
             x, y, altX, altY, phi, PI, w,
             xmin, xmax, ymin, ymax;
   QColor    color[100];

   void quadrat( int level, double c, int what, QPainter *p ) {
      double  a, b;
         
      if ( c <= maxLength ) {
         calculatePaint( level, what, c, p ); LINKS(270);
         calculatePaint( level, what, c, p ); LINKS(270);
         calculatePaint( level, what, c, p );
      } else {
         b = c / sqrt (1 + relation*relation );
         a = relation*b;
         calculatePaint( level, what, c, p ); LINKS(90-w);
         quadrat( level+1, a, what, p ); LINKS(270);
         calculatePaint( level, what, a, p ); LINKS(90+w);
         calculatePaint( level, what, c, p ); LINKS(180-w);
         calculatePaint( level, what, b, p ); LINKS(270);
         quadrat( level+1, b, what, p ); LINKS(w);
         calculatePaint( level, what, c, p );
      }
   }

   void calculatePaint( int level, int what, double l, QPainter *p ) {
      x = altX + l*cos(-phi);
      y = altY + l*sin(-phi);

      if ( what == 1 ) {
         if (x > xmax)      xmax = x;
         else if (x < xmin) xmin = x;

         if (y > ymax)      ymax = y;
         else if (y < ymin) ymin = y;

      } else if ( what == 2) {
         p->setPen( color[level] );
         p->drawLine( int(altX), int(altY), int(x), int(y) );
      }
      altX = x;
      altY = y;
   }
};

#include "pythagoras.moc"

//..................................................................... main
int main( int argc, char *argv[] )
{
   srand( time( NULL ) );
   QApplication myapp( argc, argv );

   Pythagoras *mywidget = new Pythagoras;
   mywidget->setGeometry( 50, 50, MAXX, MAXY );

   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
