#ifndef RAHMENLAYOUT_H
#define RAHMENLAYOUT_H

#include <qlayout.h>
#include <qptrlist.h>
//................................................ Klasse RahmenWidgetItem
class RahmenWidgetItem : public QWidgetItem {
  public:
    RahmenWidgetItem( QWidget *w ) : QWidgetItem( w ) {}
    void setGeometry( const QRect &r ) { widget()->setGeometry( r ); }
};
//.................................................... Klasse RahmenLayout
class RahmenLayout : public QLayout {
  public:
      //.............................. enum- und struct-Deklarationen
    enum Position { Links, Oben, Unten, Rechts, Mitte };
    enum SizeType { Minimum, SizeHint };

    struct RahmenLayoutStruct {
        RahmenLayoutStruct( QLayoutItem *i, Position p ) {
            item = i;
            pos = p;
        }
        QLayoutItem *item;
        Position pos;
    };
    RahmenLayout( QWidget *p, int margin = 0, int space = -1 )
             : QLayout( p, margin, space ), cached( 0, 0 ), mcached( 0, 0 ),
               sizeDirty( true ), msizeDirty( true ) {}
    RahmenLayout( QLayout* p, int space = -1 )
             : QLayout( p, space  ), cached( 0, 0 ), mcached( 0, 0 ),
               sizeDirty( true ), msizeDirty( true ) {}
    RahmenLayout( int space = -1 ) : QLayout( space ),
                    cached( 0, 0 ), mcached( 0, 0 ),
                    sizeDirty( true ), msizeDirty( true ) {}
    ~RahmenLayout() { deleteAllItems(); }

    void addItem( QLayoutItem *item ) { add( item, Links ); }
    void addWidget( QWidget *widget, Position pos ) {
       add( new RahmenWidgetItem( widget ), pos );
    }
    void add( QLayoutItem *item, Position pos ) {
       list.append( new RahmenLayoutStruct( item, pos ) );
       sizeDirty = msizeDirty = true;
       berechneGroesse( SizeHint );
       berechneGroesse( Minimum );
    }
    QSize sizeHint() const    { return cached;  }
    QSize minimumSize() const { return mcached; }
    QSizePolicy::ExpandData expanding() const {
       return QSizePolicy::BothDirections;
    }
    QLayoutIterator iterator();
  protected:
    void setGeometry( const QRect &rect ) {
       QLayout::setGeometry( rect );
       doLayout( rect );
    }
  private:
    void doLayout( const QRect &rect, bool testonly = false );
    void berechneGroesse( SizeType st );

    QPtrList<RahmenLayoutStruct> list;
    QSize  cached, mcached;
    bool   sizeDirty, msizeDirty;
};
//............................................ Klasse RahmenLayoutIterator
class RahmenLayoutIterator : public QGLayoutIterator {
  public:
    RahmenLayoutIterator( const QPtrList<RahmenLayout::RahmenLayoutStruct> *l )
        : pos( 0 ), list( (QPtrList<RahmenLayout::RahmenLayoutStruct>*)l ) {}

    uint count() const { return list->count(); }
    QLayoutItem *current() { return pos < (int)count() ? list->at(pos)->item : 0;}
    RahmenLayout::RahmenLayoutStruct *currentStruct() {
       return pos < (int)count() ? list->at( pos ) : 0;
    }
    void toFirst() { pos = 0; }
    QLayoutItem *next() { pos++; return current(); }

    QLayoutItem *takeCurrent() {
       RahmenLayout::RahmenLayoutStruct *b
               = pos < int( list->count() ) ? list->take(  pos  ) : 0;
       QLayoutItem *item =  b ? b->item : 0;
       delete b;
       return item;
    }
    RahmenLayoutIterator &operator++() { next(); return *this; }
private:
    int pos;
    QPtrList<RahmenLayout::RahmenLayoutStruct> *list;
};
#endif
