#include <qapplication.h>
#include <qwidget.h>
#include <qpushbutton.h>
#include <qthread.h>
#include <qlayout.h>
#include <qmessagebox.h>
#include <qpainter.h>
#include <stdlib.h>
#include <time.h>

#if defined(QT_NO_THREAD)
#  error Thread-Support nicht eingeschaltet
#endif   
#define READER_ZAHL 50
#define WRITER_ZAHL 10

QSemaphore readersSem(READER_ZAHL), writerSem(1);
int  readWait, writeWait, readServ, writeServ, readReady, writeReady;
//..................................................... Klasse ReadersThread
class ReadersThread : public QThread {
 public:
    ReadersThread( int d, int t, QWidget *o ) : receiver(o), time(t), delay(d) {}
    void run() {
       msleep( delay );
       qApp->postEvent( receiver, new QCustomEvent(20000) ); // Read-Anforderung
       while ( writerSem.available() == 0 || writeWait > 0 )
          msleep( 10 );
       readersSem++;
       qApp->postEvent( receiver, new QCustomEvent(20001) ); // Read
       msleep( time );
       readersSem--;
       qApp->postEvent( receiver, new QCustomEvent(20002) ); // Read-Ende
    }
 private:
    QWidget *receiver;
    int     time, delay;
};
//...................................................... Klasse WriterThread
class WriterThread: public QThread {
 public:
    WriterThread( int d, int t, QWidget *o ) : receiver(o), time(t), delay(d) {}
    void run() {
       msleep( delay );
       qApp->postEvent( receiver, new QCustomEvent(30000) ); // Write-Anforderung
       while ( readersSem.available() < readersSem.total() )
          msleep( 10 );
       writerSem++;
       qApp->postEvent( receiver, new QCustomEvent(30001) ); // Write
       msleep( time );
       writerSem--;
       qApp->postEvent( receiver, new QCustomEvent(30002) ); // Write-Ende
    }
 private:
    QWidget *receiver;
    int     time, delay;
};
//..................................................... Klasse ReadersWriter
class ReadersWriter : public QWidget {
    Q_OBJECT
public:
    ReadersWriter() : QWidget() {
       int i;
         //........ Anlegen von READER_ZAHL Read- und WRITER_ZAHL Write-Threads
       for ( i=0; i<READER_ZAHL; i++ )
          readers[i] = new ReadersThread( rand()%15000+1000, rand()%2000, this );
       for ( i=0; i<WRITER_ZAHL; i++ )
          writer[i] = new WriterThread( rand()%18000+1000, rand()%2000, this );
       button = new QPushButton("Start Simulation!", this);
        connect(button, SIGNAL(clicked()), SLOT(startSimulation()));
       anzeige = new QWidget( this );
       QVBoxLayout *vbox = new QVBoxLayout( this, 5 );
       vbox->addWidget( button );
       vbox->addWidget( anzeige );
    }
protected:
    virtual void customEvent( QCustomEvent *event ) {
       switch ( event->type() ) {
          case 20000: readWait++;                break;
          case 20001: readWait--; readServ++;    break;
          case 20002: readServ--; readReady++;   break;
          case 30000: writeWait++;               break;
          case 30001: writeWait--; writeServ++;  break;
          case 30002: writeServ--; writeReady++; break;
          default: qWarning("Unbekannter Event-Typ: %d", event->type()); break;
       }
       repaint();
    }
    virtual void paintEvent( QPaintEvent * ) {
       int  i;
       QPainter p( anzeige );
       QString t;
       t.sprintf( "Readers: %d; Writer: %d", readServ, writeServ );
       if ( readServ==0 && writeServ==0 )  p.setBrush( Qt::white );
       else if ( readServ != 0 )           p.setBrush( Qt::green );
       else if ( writeServ != 0 )          p.setBrush( Qt::blue.light(150) );
       p.drawRect( 100, 200, 300, 100 );    
       p.drawText( 200, 270, t );
       p.setBrush( Qt::green );     //.... Zeichnen wartender/fertiger Reads
       for ( i=0; i<readWait; i++ )
          p.drawRect( 110, 200-(i+1)*5, 80, 5 );    
       for ( i=0; i<readReady; i++ )
          p.drawRect( 110, 300+i*5, 80, 5 );    
       p.setBrush( Qt::lightGray ); //.... Zeichnen Read-Queue-Plaetze
       for ( i=readWait; i<READER_ZAHL; i++ )
          p.drawRect( 110, 200-(i+1)*5, 80, 5 );    
       for ( i=readReady; i<READER_ZAHL; i++ )
          p.drawRect( 110, 300+i*5, 80, 5 );    
       p.setBrush( Qt::blue.light(150) ); // Zeichnen wartender/fertiger Writes
       for ( i=0; i<writeWait; i++ )
          p.drawRect( 310, 200-(i+1)*5, 80, 5 );    
       for ( i=0; i<writeReady; i++ )
          p.drawRect( 310, 300+i*5, 80, 5 );    
       p.setBrush( Qt::lightGray );   //.... Zeichnen Write-Queue-Plaetze
       for ( i=writeWait; i<WRITER_ZAHL; i++ )
          p.drawRect( 310, 200-(i+1)*5, 80, 5 );    
       for ( i=writeReady; i<WRITER_ZAHL; i++ )
          p.drawRect( 310, 300+i*5, 80, 5 );    
    }
public slots:
    void startSimulation() {
       int  i;
       for ( i=0; i<READER_ZAHL; i++)
         if ( readers[i]->running() || (i < WRITER_ZAHL && writer[i]->running() ) ) {
            QMessageBox::information( this, "Sorry", "Simulation luft noch" );
            return;
         }
       readWait = writeWait = readServ = writeServ = readReady = writeReady = 0;
       repaint( true );
       for ( i=0; i<READER_ZAHL; i++ )
          readers[i]->start(); 
       for ( i=0; i<WRITER_ZAHL; i++ )
          writer[i]->start(); 
    }
private:
    QWidget      *anzeige;
    QPushButton  *button;
    ReadersThread *readers[READER_ZAHL];
    WriterThread  *writer[WRITER_ZAHL];
};
#include "readerwriter.moc"
//..................................................................... main
int main( int argc, char *argv[] ) {
    ::srand( time( NULL ) );
    QApplication app(argc, argv);
    ReadersWriter rw;
    app.setMainWidget( &rw );
    rw.resize( 500, 600 );
    rw.show();
    return app.exec();
}
