#include <qapplication.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qregexp.h>
#include <qlabel.h>
#include <qvbox.h>
#include <stdio.h>

//..................................................... Klasse IPValidator
class IPValidator : public QValidator
{
public:
   IPValidator() : QValidator( 0 ) { };

protected:
   virtual State validate( QString &text, int & ) const {
      QRegExp  ipAdresse( "[0-9][0-9]?[0-9]?\\.[0-9][0-9]?[0-9]?\\."
                          "[0-9][0-9]?[0-9]?\\.[0-9][0-9]?[0-9]?" );
      QRegExp  zahl( "[0-9]+" );

      int   len = text.length(),
            i, pkt = 0, index = 0, zLen, z;

      if( len == 0 )            //... Leerer String ist intermediate
         return Intermediate;

      if ( !text[0].isDigit() ) //... Erste Zeichen muss Ziffer sein
         return Invalid;

      for ( i=0; i<len; i++ )   //... Nur Punkte (max. 3) + Ziffern erlaubt
         if ( ( text[i] == '.' && ++pkt > 3 ) ||
            ( !text[i].isDigit() && text[i] != '.' ) )
            return Invalid;

      while  ( ( index = zahl.match( text, index, &zLen ) ) >= 0 ) {
                            //... Pruefen der einzelnen Zahlenbloecke, ob 
                            //... nicht laenger als 3 Ziffern und <= 255
         if ( zLen > 3 )
            return Invalid;
         z = text.mid( index, zLen ).toInt();
         if ( z > 255 )
            return Invalid;
         index += zLen;
                        //... 1 Punkt, aber keine 2 Punkte muessen folgen
         if ( (index < len && text[index] != '.') ||
              (index+1 < len && text[index+1] == '.') )
            return Invalid;
         ++index;
      }
                            //... Pruefen, ob schon vollstaendig
      if ( ipAdresse.search( text, 0 ) < 0 )
         return Intermediate;
      else
         return Acceptable;
   }
};

//............................................................ Klasse Code
class Code : public QVBox
{
    Q_OBJECT
public:
    Code( QWidget *parent=0, const char *name=0 ) : QVBox( parent, name ) {
       setMargin( 10 );

       QVBox *box = new QVBox( this ); // Widget fuer das Layout
       box->setMargin( 5 );
       box->setSpacing( 20 );
       (void)new QLabel( "Geben Sie bitte eine IP-Adresse ein: ", box );

       eingabe = new QLineEdit( box );
       IPValidator *codVal = new IPValidator();
       eingabe->setValidator( codVal );

       logMeldung = new QLabel( "Noch nicht mit Eingabe begonnen", box );
       logMeldung->setPalette( QPalette( Qt::blue.light(180),
                                         Qt::blue.light(180) ) );
  
       QObject::connect( eingabe, SIGNAL( textChanged( const QString & ) ),
                         this, SLOT( slotTextChanged( const QString & ) ) );

       QObject::connect( eingabe, SIGNAL( returnPressed() ),
                         this, SLOT( slotReturnPressed() ) );
   }

private slots:
   void slotTextChanged( const QString &text ) {
       QString  t;
       t.sprintf("Noch nicht fertig; bisher: %s", (const char *)text );
       logMeldung->setText( t ); 
       logMeldung->setPalette( QPalette( Qt::yellow, Qt::yellow ) );
   }

   void slotReturnPressed() {
       QString  endeText;
       endeText.sprintf("Fertig und akzeptiert: %s",
                        (const char *)eingabe->text() ); 
       logMeldung->setText( endeText ); 
       logMeldung->setPalette( QPalette( Qt::green, Qt::green ) );
   }

private:
   QLineEdit *eingabe;
   QLabel    *logMeldung;
};

#include "regexp3.moc"

//................................................................... main
int main( int argc, char *argv[] )
{
    QApplication a( argc, argv );

    Code cod;
    cod.resize( 400, cod.sizeHint().height() );
    cod.setCaption( "Eingabe einer richtigen IP-Adresse" );

    a.setMainWidget( &cod );
    cod.show();
    return a.exec();
}
