#include <qapplication.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qregexp.h>
#include <qlabel.h>
#include <qvbox.h>
#include <stdio.h>

//....................................................... Klasse CAdresse
class CAdresse : public QVBox
{
    Q_OBJECT
public:
    CAdresse( QWidget *parent=0, const char *name=0 )
                                              : QVBox( parent, name ) {
       setMargin( 10 );

       QVBox *box = new QVBox( this ); // Widget fuer das Layout
       box->setMargin( 5 );
       box->setSpacing( 20 );
       (void)new QLabel( "PLZ Stadt", box );

       input = new QLineEdit( box );
       QRegExp  plzStadt( "[0-9]{5} +[A-Za-z][A-Za-z-]*" );
       adrValidator = new QRegExpValidator( plzStadt, this);
       input->setValidator( adrValidator );

       logMeldung = new QLabel( "Noch nicht mit Eingabe begonnen", box );
       logMeldung->setFont( QFont( "Times", 16 ) );
       logMeldung->setPaletteBackgroundColor( Qt::blue.light( 180 ) );
  
       QObject::connect( input, SIGNAL( textChanged( const QString & ) ),
                         this,  SLOT( slotTextChanged( const QString & ) ) );

       QObject::connect( input, SIGNAL( returnPressed() ),
                         this,  SLOT( slotReturnPressed() ) );
   }

private slots:

   void slotTextChanged( const QString &text ) {
       QString  t = text;
       int      pos;
       QValidator::State zustand = adrValidator->validate( t, pos );
       if ( zustand == QValidator::Acceptable ) {
          logMeldung->setPaletteBackgroundColor( Qt::green.light( 180 ) );
          t.sprintf("gltig, aber noch nicht besttigt: %s",
                    (const char *)text);
       } else if ( zustand == QValidator::Intermediate ) {
          logMeldung->setPaletteBackgroundColor( Qt::yellow.light( 160 ) );
          t.sprintf("gltig, aber noch nicht fertig; bisher: %s",
                    (const char *)text );
       }
       logMeldung->setText( t );
   }
   void slotReturnPressed() {
       QString  endeText;
       endeText.sprintf("Fertig und akzeptiert: %s",
                        (const char *)input->text() ); 
       logMeldung->setText( endeText ); 
       logMeldung->setPaletteBackgroundColor( Qt::white );
   }

private:
   QRegExpValidator *adrValidator;
   QLineEdit        *input;
   QLabel           *logMeldung;
};

#include "regexpvalidator2.moc"

//................................................................... main
int main( int argc, char *argv[] )
{
    QApplication a( argc, argv );

    CAdresse adr;
    adr.resize( 400, adr.sizeHint().height() );
    adr.setCaption( "Eingabe einer Stadt mit Postleitzahl" );

    a.setMainWidget( &adr );
    adr.show();
    return a.exec();
}
