#include <qapplication.h>
#include <qthread.h>
#include <qpainter.h>
#include <stdlib.h>
#include <time.h>
#include <stdio.h>

#define NoRunner  8

//.......................................................... Klasse Runner
class Runner : public QThread
{
public:
    Runner( int id, QWidget *r ) : receiver( r ) { nr = id; };

protected:
    void run() {
       int distance = 0;
       while ( distance < 400 ) {
          msleep( ::rand() % 20+1 );
          distance++;

          QCustomEvent *event = new QCustomEvent( 20000+nr );
          QString *t = new QString();
          t->sprintf("%d", distance );
          event->setData( t );
          QApplication::postEvent( receiver, event );
       }
    }

private:
    int      nr;
    QWidget *receiver;
};

//................................................... Klasse RunnerDisplay
class RunnerDisplay: public QWidget
{
public:
    RunnerDisplay( QWidget *p=0, const char *name=0 ) : QWidget( p, name ) {
       int i;
       order = 0;
       for ( i=0; i<NoRunner; i++ ) {
          oldDistance[i] = newDistance[i] = 0;
          runner[i] = new Runner( i, this );
          runnerColor[i].setRgb(
                  ::rand()&255, ::rand()&255, ::rand()&255 );
       }
       for ( i=0; i<NoRunner; i++ )
          runner[i]->start(); 
    }

protected:
    virtual void customEvent( QCustomEvent *ev ) {
       int  whichRunner = ev->type() % 20000;

       if ( whichRunner >= 0 && whichRunner < NoRunner ) {
          QString *d = (QString *)ev->data();
          newDistance[ whichRunner ] = d->toInt();
          repaint( false );
       }
    }

    virtual void paintEvent( QPaintEvent * ) {
       for ( int i=0; i<NoRunner; i++ )
          if ( newDistance[i] != oldDistance[i] ) {
             QPainter p;
             p.begin( this );
             if ( newDistance[i] < 400 ) {
                p.setPen( QPen( runnerColor[i], 5 ) );
                p.drawRect( newDistance[i]+1, 20*i+10, 10, 10 );
             } else {
                p.setPen( Qt::black );
                QString text;
                text.sprintf("%d", ++order);
                p.drawText( 450, 20*i+20, text);
             }
             p.end();
             oldDistance[i] = newDistance[i];
          }
    }
private:
    Runner *runner[NoRunner];
    QColor runnerColor[NoRunner];
    int    newDistance[NoRunner], oldDistance[NoRunner], order;
};

//................................................................... main
int main( int argc, char *argv[] ) {
    QApplication app( argc, argv );

    ::srand( time( NULL ) );

    RunnerDisplay *stadium = new RunnerDisplay;
    stadium->setFixedSize( 500, NoRunner*20 + 20 );

    app.setMainWidget( stadium );
    stadium->show();
    return app.exec();
}
