#include <qapplication.h>
#include <qwidget.h>
#include <qthread.h>
#include <qtextedit.h>
#include <qtimer.h>
#include <stdlib.h>
#include <time.h>

#if defined(QT_NO_THREAD)
#  error Thread support not enabled.
#endif   

#define  NoSeats         8
#define  NoGuestGroups  10 
#define  MaxGroupSize    5

QSemaphore  spaceSem(NoSeats);
QMutex      mutex;

//........................................................... Klasse Arrival
class Arrival : public QThread
{
public:
    Arrival( int id, int p, QWidget *o ) : receiver(o) {
       nr = id;
       persZahl = p;
    }

    void run() {
       QString *t = new QString();
       t->sprintf("%d", persZahl );

       srand( persZahl );
       sleep( rand()%(nr*2+1)+1 );

       mutex.lock();                                 
       QCustomEvent *arrived = new QCustomEvent( 20000+nr ); // Speisewunsch
       arrived->setData( t );
       QApplication::postEvent( receiver, arrived );
       mutex.unlock();

       spaceSem += persZahl;

       mutex.lock();                                        // Zutritt
       QCustomEvent *zutritt = new QCustomEvent( 30000+nr );
       QApplication::postEvent( receiver, zutritt );
       mutex.unlock();

       msleep( ::rand()%5000+5000 ); // Essen

       mutex.lock();                 // Verlassen des Restaurants
       spaceSem -= persZahl;   
       QCustomEvent *ready = new QCustomEvent( 40000+nr );
       QApplication::postEvent( receiver, ready );
       mutex.unlock();
    }

private:
    int     nr;
    QWidget *receiver;
    int     persZahl;
};

//........................................................ Klasse Simulation
class Simulation : public QTextEdit
{
    Q_OBJECT
    enum State { idle, waiting, eating, done };
public:
    Simulation() : QTextEdit() {
       int i;
       for ( i=0; i<NoGuestGroups; i++ ) { //.... Anlegen Besucher-Threads
          guestGroup[i] = new Arrival( i, rand()%MaxGroupSize+1, this );
          groupData[i].state = idle;
          groupData[i].persZahl = 0;
       }
       freeSeats = NoSeats;

       setFont( QFont( "Courier", 10 ) );
       QString t;
       t.sprintf("%10s | %-20s | %-20s | %s\n",
                 "Event-Type", "Waiting", "Eating", "Free Places" );
       insert( t );
       insert("----------------------------------------------"
              "-----------------------------\n");
       QTimer *timer = new QTimer( this );
       connect( timer, SIGNAL( timeout() ), SLOT( handlePostedEvents() ) );
       timer->start( 30, false );

       for ( i=0; i<NoGuestGroups; i++ )
          guestGroup[i]->start(); 
    }

private slots:
    void handlePostedEvents() {
       qApp->sendPostedEvents();
       qApp->processEvents();
    }

protected:
    virtual void customEvent( QCustomEvent *event ) {
       int  i;
       mutex.lock();

       QString what, t;
       int type = event->type();
       if ( type < 30000 ) {
          int nr = type%20000;
          groupData[ nr ].state = waiting;
          QString *d = (QString *)event->data();
          groupData[ nr ].persZahl = d->toInt();
          what.sprintf("%10s |", "Arrival" );
       } else if ( type < 40000 ) {
          int  nr = type % 30000;
          groupData[ nr ].state = eating;
          freeSeats -= groupData[nr].persZahl;
          what.sprintf("%10s |", "Access" );
       } else {
          int  nr = type % 40000;
          groupData[ nr ].state = done;
          freeSeats += groupData[nr].persZahl;
          what.sprintf("%10s |", "Finished" );
       }

       int  z=0;
       insert( what );
       for ( i=0; i<NoGuestGroups; i++ )
          if ( groupData[i].state == waiting ) {
             t.sprintf("%2d,", groupData[i].persZahl );
             insert( t );
             z++;
          }
       for ( i=0; i<7-z; i++)
          insert("   ");
       insert(" | ");

       z = 0;
       for ( i=0; i<NoGuestGroups; i++ )
          if ( groupData[i].state == eating ) {
             t.sprintf("%d, ", groupData[i].persZahl );
             insert( t );
             z++;
          }
       for ( i=0; i<7-z; i++)
          insert("   ");
       insert("| ");

       t.sprintf("%2d\n", freeSeats );
       insert( t );

       mutex.unlock();
    }

private:
    int     freeSeats;
    Arrival *guestGroup[NoGuestGroups];
    struct {
        enum State  state;
        int         persZahl;
    } groupData[NoGuestGroups];
};

#include "restaurant.moc"

//..................................................................... main
int main( int argc, char *argv[] ) {
    ::srand( time( NULL ) );
    QApplication app(argc, argv);

    Simulation sim;
    app.setMainWidget( &sim );

    sim.resize( 600, 600 );
    sim.show();
    return app.exec();
}
