#include  <qwidget.h>
#include  <qscrbar.h>
#include  <qobjcoll.h>
#include  "scrollbar.h"

//............................................... Scroller (Konstruktor)
Scroller::Scroller(QWidget *parent, const char *name)
               : QWidget(parent, name), hbar(Qt::Horizontal, this),
                 vbar(Qt::Vertical, this), anzeigefenster(this) {
    connect(&hbar, SIGNAL(valueChanged(int)), this, SLOT(hslide(int)));
    connect(&vbar, SIGNAL(valueChanged(int)), this, SLOT(vslide(int)));
    moveFlag = true;
}
//.......................................................... sichtWidget
QWidget* Scroller::sichtWidget() const {
      // `children` liefert eine Liste aller Subwidgets
    const QObjectList *l = anzeigefenster.children();
    if (l) {
        QObjectListIterator iter(*l);
        QObject *ch = iter.current(); // liefert aktuelle Subwidget
        if (ch && ch->isWidgetType())
            return (QWidget*)ch;
    }
    return 0;
}
//..................................................... resizeScrollBars
void Scroller::resizeScrollBars() {
    int  w = width(), h = height(),
         top, bottom;
    if (sichtWidget()) {
       int  anzeigeW, anzeigeH;
       bool neuH = w < sichtWidget()->width(),
            neuV = h < sichtWidget()->height();
       if (neuH && h-scrollBarWidth() < sichtWidget()->height())
           neuV = true;
       if (neuV && w-scrollBarWidth() < sichtWidget()->width())
           neuH = true;
       if (neuH) {
           hbar.show();
           anzeigeH = h-scrollBarWidth();
       } else {
           hslide(0);
           hbar.hide();
           anzeigeH = h;
       }
       if (neuV) {
           vbar.show();
           anzeigeW = w-scrollBarWidth();
       } else {
           vslide(0);
           vbar.hide();
           anzeigeW = w;
       }
       if (neuV) {
          vbar.setRange(0, sichtWidget()->height()-anzeigeH);
          vbar.setSteps(5, 50);
       }
       if (neuH) {
          hbar.setRange(0, sichtWidget()->width()-anzeigeW);
          hbar.setSteps(5, 50);
       }
       if (neuH) {
          int right = ((neuV && emptyCorner()) || alwaysEmptyCorner())
                       ? w-scrollBarWidth() : w;
          if (scrollBarOnTop()) {
             hbar.setGeometry( 0, h-scrollBarWidth(), right, scrollBarWidth() );
             top = scrollBarWidth();
             bottom = h;
          } else {
             hbar.setGeometry( 0, h-scrollBarWidth(), right, scrollBarWidth() );
             top = 0;
             bottom = h-scrollBarWidth();
          }
       } else {
          top = 0;
          bottom = h;
       }
       if (neuV) {
          if (scrollBarOnLeft()) {
             vbar.setGeometry(0, top, scrollBarWidth(), bottom );
             anzeigefenster.setGeometry(scrollBarWidth(), top,
                                        w-scrollBarWidth(), bottom );
          } else {
             vbar.setGeometry( w-scrollBarWidth(), top,
                               scrollBarWidth(), bottom );
             anzeigefenster.setGeometry( 0, top, w-scrollBarWidth(), bottom );
          }
       } else
           anzeigefenster.setGeometry( 0, top, w, bottom );
   } else {
       hbar.hide();
       vbar.hide();
       anzeigefenster.setGeometry(0, 0, w, h);
   }
}
//........................................................ ensureVisible
//    bewegt die Sichtflaeche so, dass (x,y) sichtbar wird und
//    zumindest der geforderte Pixelrand eingehalten wird (wenn moegl.)
//    ansonsten wird Sichtflaeche zentriert
void Scroller::ensureVisible(int x, int y, int xmargin, int ymargin) {
    int aw = anzeigeFenster()->width(),
        ah = anzeigeFenster()->height();
    QWidget* child = sichtWidget();
    if (!child)
       return;
    int cx = child->x(),     cy = child->y(),
        cw = child->width(), ch = child->height();
    if (aw < xmargin*2) xmargin=aw/2;
    if (ah < ymargin*2) ymargin=ah/2;
    if (cw <= aw) { xmargin=0; cx=0; }
    if (ch <= ah) { ymargin=0; cy=0; }
    if (x < -cx+xmargin)
       cx = -x+aw-xmargin;
    else if (x >= -cx+aw-xmargin)
       cx = -x+xmargin;
    if (y < -cy+ymargin)
       cy = -y+ah-ymargin;
    else if (y >= -cy+ah-ymargin)
       cy = -y+ymargin;
    if (cx >= 0)                  cx=aw/2-cw/2;
    else if (cx < aw-cw && cw>aw) cx=aw-cw;
    if (cy >= 0)                  cy=ah/2-ch/2;
    else if (cy < ah-ch && ch>ah) cy=ah-ch;
    moveFlag = false;
    child->move(cx,cy);
    vbar.setValue(-cy);
    hbar.setValue(-cx);
    resizeScrollBars();
    moveFlag = true;
}
