#ifndef SCROLLBAR_H
#define SCROLLBAR_H

#include  <qwidget.h>
#include  <qscrollbar.h>

class Scroller : public QWidget {
   Q_OBJECT
public:
   Scroller(QWidget *parent=0, const char *name=0 );

   QWidget* anzeigeFenster() { return &anzeigefenster; }

   QScrollBar& horizontalScrollBar() { return hbar; }
   QScrollBar& verticalScrollBar()   { return vbar; }
   
   void show() {  //.... Reimplementierung, um sicherzustellen, dass Laufbalken
      resizeScrollBars(); // bei der Anzeige die richtige Groesse haben
      QWidget::show();
   }
   void ensureVisible(int x, int y, int xmargin=50, int ymargin=50);

   void centerOn(int x, int y) { ensureVisible( x, y, 32000, 32000 ); }

   /* Raender als Bruchteil der sichtbaren Flaeche
       0.0 = (x,y) direkt an der Ecke ohne Raender
       0.5 = (x,y) in der Mitte von 50% der sichtbaren Flaeche
       1.0 = CenterOn(x,y).  */
   void centralize(int x, int y, float xmargin=0.5, float ymargin=0.5) {
      int aw = anzeigeFenster()->width(),
          ah = anzeigeFenster()->height();
      ensureVisible( x, y, int(xmargin/2.0*aw+0.5), int(ymargin/2.0*ah+0.5) );
   }
   virtual int scrollBarWidth() const   { return 16;    } // Voreinst.: 16
   virtual bool scrollBarOnLeft() const { return false; } // Voreinst.: rechts
   virtual bool scrollBarOnTop() const  { return false; } // Voreinst.: unten
     // Leere Ecke (true), wenn beide Laufbalken sichtbar,
     // sonst wird diese vom horizontalen Laufbalken genutzt
   virtual bool emptyCorner() const     { return true;  }
     // Immer eine leere Ecke zwischen den Laufbalken (true),
     // sonst, wenn ein Laufbalken sichtbar, nutzt er diese, und
     // wenn beide sichtbar, legt emtpyCorner() den Effekt fest
   virtual bool alwaysEmptyCorner() const { return false; }
   virtual void keyPressEvent( QKeyEvent* event ) {
      switch ( event->key() ) {
         case Key_Up      :  vbar.setValue( vbar.value()-5 );  break;
         case Key_Down    :  vbar.setValue( vbar.value()+5 );  break;
         case Key_PageUp  :  vbar.setValue( vbar.value()-50 ); break;
         case Key_PageDown:  vbar.setValue( vbar.value()+50 ); break;
         case Key_Left    :  hbar.setValue( hbar.value()-5 );  break;
         case Key_Right   :  hbar.setValue( hbar.value()+5 );  break;
      }
   }
public slots:
   //... aufzurufen, wenn sich die Eigenschaften dynamisch aendern
   void resizeScrollBars();

   virtual void resizeEvent( QResizeEvent* event ) {
      QWidget::resizeEvent(event);
      resizeScrollBars();
   }
   QWidget* sichtWidget() const; //... Kind des anzeigeFensters
private:
   QScrollBar  hbar, vbar;
   QWidget     anzeigefenster;
   bool        moveFlag; // Flag, das es der Methode `ensureVisible`
                         // ermoeglicht, beide Laufbalken zu aktualisieren;
                         // zweimaliges Aktualisieren bewirkt Flimmern
private slots:
   // void vorwaertsZeileSlot() { vbar.setValue(vbar.value()+10); }
   void hslide(int pos) {
      if (moveFlag && sichtWidget())
         sichtWidget()->move( -pos, sichtWidget()->y() );
   }
   void vslide(int pos) {
      if (moveFlag && sichtWidget())
         sichtWidget()->move( sichtWidget()->x(), -pos);
   }
};
#endif
