#include <qscrollview.h>
#include <qapplication.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qpushbutton.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qtextedit.h>
#include <qsizegrip.h>
#include <stdlib.h>

static const int mw_id  = 0x8000;
static const int max_mw = 40;
static const QString bild[] = { "pingu.xpm", "suse.xpm", "crab.xpm", "bf.xpm" };
static const int bildZahl = sizeof(bild) / sizeof(*bild);
//............................................ Klasse SchrumpfFenster ..........
class SchrumpfFenster : public QFrame {
    Q_OBJECT
public:
    SchrumpfFenster(QWidget* parent) : QFrame(parent) {    //.... Konstruktor
       int h=35, b=0;
       setPaletteBackgroundColor( QColor( 0, 255, 255 ) );
       for (int y=0; y<2000-h; y+=h+10) {
          if (y == 0) {
             QButton *q = new QPushButton("Beenden", this);
             connect(q, SIGNAL(clicked()), qApp, SLOT(quit()));
          } else {
             QString str;
             if ( ++b > 1 ) {
                QColor f( ::rand()&255, ::rand()&255, ::rand()&255 );
                str.sprintf("Button %d", b-1);
                QPushButton *button = new QPushButton(str, this);
                button->move(y/2,y);
                button->setPaletteBackgroundColor( f );
             } else {
                str.sprintf("Ich schrumpfe (Schau mal auf den Laufbalken)" );
                QLabel *label = new QLabel(str, this);
                label->setPaletteBackgroundColor( Qt::yellow );
                label->setGeometry(y/2,y, 400, 30);
             }
          }
       }
       setMinimumSize(50, 50);
       resize(1000, 2000);
       startTimer(250);
    }
    void timerEvent(QTimerEvent*)          { resize( width()-1, height()-2); }
    void mouseReleaseEvent(QMouseEvent* e) { emit clicked(e->x(), e->y());   }
signals:
    void clicked(int,int);
};
//.............................................. Klasse RiesenFenster ..........
class RiesenFenster : public QScrollView {
  public:
    RiesenFenster(QWidget *p) : QScrollView(p, "matrix"), bg("marble.xpm") {
       bg.load("marble.xpm");
       resizeContents(400000, 300000);
       srand(1); // um gleiche Anordnung zu erreichen
       dragging = 0;
    }
    void viewportMousePressEvent(QMouseEvent *e) {
       int x, y;
       viewportToContents( e->x(),  e->y(), x, y );
       dragging = new QTextEdit(viewport());
       addChild(dragging, x, y);
       dragging->show();
    }
    void viewportMouseReleaseEvent(QMouseEvent*) { dragging = 0; }
    void viewportMouseMoveEvent(QMouseEvent* e) {
       if ( dragging ) {
          int mx, my, cx = childX(dragging), cy = childY(dragging);
          viewportToContents( e->x(), e->y(), mx, my );
          int w = mx - cx + 1, h = my - cy + 1;
          QString msg;
          msg.sprintf("Linke obere Ecke: %d,%d; %d breit, %d hoch", cx, cy, w, h);
          dragging->setText(msg);
          dragging->resize(w, h);
       }
    }
  protected:
    void drawContents(QPainter* p, int cx, int cy, int cw, int ch) {
       p->fillRect(cx, cy, cw, ch, QColor(240,222,208)); //..... Der Hintergrund
       if ( !bg.isNull() ) {
           int bottom_zeile  = (cy+ch+bg.height()-1)/bg.height(),
               rechte_spalte = (cx+cw+bg.width() -1)/bg.width();
           for (int r=cy/bg.height(); r<=bottom_zeile; r++)
              for (int c=cx/bg.width(); c<=rechte_spalte; c++)
                 p->drawPixmap(c*bg.width(), r*bg.height(), bg);
       }
       QFontMetrics fm    = p->fontMetrics(); //..... Zeilen- und Spaltennummern
       int zeilen_hoehe   = fm.lineSpacing(),
           top_zeile      = cy/zeilen_hoehe,
           bottom_zeile   = (cy+ch+zeilen_hoehe-1)/zeilen_hoehe,
           spalten_breite = fm.width("00000,000000 ")+3,
           linke_spalte   = cx/spalten_breite,
           rechte_spalte  = (cx+cw+spalten_breite-1)/spalten_breite;
       QString str;
       p->setPen(blue);
       for (int r=top_zeile; r<=bottom_zeile; r++)
          for (int c=linke_spalte; c<=rechte_spalte; c++) {
             str.sprintf("%d,%d",c,r);
             p->drawText(c*spalten_breite+3, r*zeilen_hoehe+fm.ascent(), str);
          }
       if (linke_spalte<30 && top_zeile<20) { //.... Grosse Schrift (links oben)
          p->setFont(QFont("Helvetica", 100));
          p->setPen(red);
          p->drawText(100, 300, 
               "Hier in linker oberen Ecke sind nette kleine Tiere versteckt");
       }
       p->setPen( QPen(red, 50) );  //..... Bereichsmarkierung fuer kleine Tiere
       p->drawLine(5050, 0, 5050, 5050);
       p->drawLine(0, 5050, 5075, 5050);
    }
  private:
    QPixmap   bg;
    QTextEdit *dragging;
};
//.......................................... Klasse ScrollViewFenster ..........
class ScrollViewFenster : public QWidget {
    Q_OBJECT
public:                                                    //.... Konstruktor
    ScrollViewFenster(int methode, QString titel,
                      QWidget *par=0, const char *nam=0) : QWidget(par, nam) {
       QMenuBar   *menubar = new QMenuBar(this);
       QPopupMenu *datei   = new QPopupMenu();
       this->setCaption( titel );
       menubar->insertItem( "&Datei", datei );
       datei->insertItem( "Beenden", qApp, SLOT(quit()) );
       vp_optionen = new QPopupMenu();
       menubar->insertItem( "&ScrollView", vp_optionen );
       connect( vp_optionen, SIGNAL(activated(int)),
                this,        SLOT(doVPMenuItem(int)) );
       vauto_id = vp_optionen->insertItem( "Vertikal Auto" );
       vaoff_id = vp_optionen->insertItem( "Vertikal immer Aus" );
       vaon_id  = vp_optionen->insertItem( "Vertikal immer An" );
       vp_optionen->insertSeparator();
       hauto_id = vp_optionen->insertItem( "Horizontal Auto" );
       haoff_id = vp_optionen->insertItem( "Horizontal immer Aus" );
       haon_id  = vp_optionen->insertItem( "Horizontal immer An" );
       vp_optionen->insertSeparator();
       corn_id = vp_optionen->insertItem( "eckenWidget" );
       if (methode == 1) {
          vp = new QScrollView(this);
          SchrumpfFenster *bs = new SchrumpfFenster(0);
          vp->addChild(bs);
          QObject::connect(bs, SIGNAL(clicked(int,int)),
                           vp, SLOT(center(int,int)));
       } else {
          int i;
          vp = new RiesenFenster(this);
          vp->enableClipper( methode == 3 );
          QPixmap b[bildZahl];
          for (i=0; i<bildZahl; i++)
             b[i].load(bild[i]);
          for (i=0; i<500; i++) {
             QLabel *l = new QLabel(vp->viewport());
             l->setPixmap( b[::rand()%bildZahl] );
             l->adjustSize();
             vp->addChild(l, ::rand()%5000, ::rand()%5000);
          }
       }
       r_optionen = new QPopupMenu();
       menubar->insertItem( "&Rand", r_optionen );
       connect(r_optionen, SIGNAL(activated(int)), this, SLOT(doFMenuItem(int)));
       for (int mw = 0; mw <= max_mw; mw+=4) {
          QString str;
          str.sprintf("%d Pixel", mw);
          r_optionen->insertItem( str, mw_id | mw );
       }
       setVPMenuItems(); setFMenuItems();
       QVBoxLayout *vbox = new QVBoxLayout(this);
        vbox->setMenuBar(menubar);
        vbox->addWidget(vp);
        vbox->activate();
       ecke = new QSizeGrip(this); ecke->hide();
    }
private slots:
    void doVPMenuItem(int id) {
       if      (id==vauto_id) vp->setVScrollBarMode(QScrollView::Auto);
       else if (id==vaoff_id) vp->setVScrollBarMode(QScrollView::AlwaysOff);
       else if (id==vaon_id)  vp->setVScrollBarMode(QScrollView::AlwaysOn);
       else if (id==hauto_id) vp->setHScrollBarMode(QScrollView::Auto);
       else if (id==haoff_id) vp->setHScrollBarMode(QScrollView::AlwaysOff);
       else if (id==haon_id)  vp->setHScrollBarMode(QScrollView::AlwaysOn);
       else if (id==corn_id) vp->setCornerWidget(!vp->cornerWidget() ? ecke : 0);
       else    return;
       setVPMenuItems();
    }
    void setVPMenuItems() {
       QScrollView::ScrollBarMode vm = vp->vScrollBarMode();
        vp_optionen->setItemChecked( vauto_id, vm == QScrollView::Auto );
        vp_optionen->setItemChecked( vaoff_id, vm == QScrollView::AlwaysOff );
        vp_optionen->setItemChecked( vaon_id, vm == QScrollView::AlwaysOn );
       QScrollView::ScrollBarMode hm = vp->hScrollBarMode();
        vp_optionen->setItemChecked( hauto_id, hm == QScrollView::Auto );
        vp_optionen->setItemChecked( haoff_id, hm == QScrollView::AlwaysOff );
        vp_optionen->setItemChecked( haon_id, hm == QScrollView::AlwaysOn );
       vp_optionen->setItemChecked( corn_id, vp->cornerWidget() );
    }
    void doFMenuItem(int id) {
       vp->setMargin( id & ~mw_id );
       vp->update();
       setFMenuItems();
    }
    void setFMenuItems() {
       for (int mw=0; mw<=max_mw; mw+=4)
           r_optionen->setItemChecked( mw_id|mw, vp->margin() == mw );
    }
private:
    QScrollView *vp;
    QPopupMenu  *vp_optionen, *r_optionen;
    QSizeGrip   *ecke;
    int vauto_id, vaoff_id, vaon_id, hauto_id, haoff_id, haon_id, corn_id;
};
#include "scrollview.moc"
//.............................................................. main ..........
int main( int argc, char **argv ) {
    QApplication a( argc, argv );
    ScrollViewFenster sf1( 1, "Methode 1", 0 );
    ScrollViewFenster sf2( 2, "Methode 2", 0 );
    ScrollViewFenster sf3( 3, "Methode 3", 0 );
    sf1.setGeometry(0, 0, 600, 600);
    sf1.show(); sf2.show(); sf3.show();
    QObject::connect(qApp, SIGNAL(lastWindowClosed()), qApp, SLOT(quit()));
    return a.exec();
}
