#include <qapplication.h>
#include <qscrollview.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <stdlib.h>

//..................................................... Klasse SubScrollView
class SubScrollView : public QScrollView
{
    Q_OBJECT
public:
    SubScrollView( QPixmap *bild, QWidget* parent = 0 ) : QScrollView( parent ) {
        QLabel *l = new QLabel( viewport() );
        l->setPixmap( *bild );
        l->resize( bild->size() );
        addChild( l );
        if ( l->width() > 200 || l->height() > 200 ) {
           corner = new QLabel( this );
           corner->setPaletteBackgroundColor( Qt::white );
           setCornerWidget( corner );
           anzeige = new QLabel( corner );
           anzeige->setPaletteBackgroundColor( Qt::blue );

           xFactor = (double)cornerWidget()->width() / contentsWidth();
           yFactor = (double)cornerWidget()->height() / contentsHeight();

           QObject::connect( this , SIGNAL( contentsMoving( int, int ) ),
                             this,  SLOT( showPosInCorner( int, int ) ) );
           showPosInCorner( 0, 0 );
        }
    }

public slots:
    void showPosInCorner( int, int ) {
       anzeige->setGeometry( int(contentsX()*xFactor+0.5),
                             int(contentsY()*yFactor+0.5),
                             int(visibleWidth()*xFactor+0.5),
                             int(visibleHeight()*yFactor+0.5) );
    }

private:
    QLabel  *corner, *anzeige;
    double  xFactor, yFactor;
};

//...................................................... Klasse MyScrollView
class MyScrollView : public QScrollView
{
    Q_OBJECT
public:
    MyScrollView( QWidget* parent = 0 ) : QScrollView( parent ) {
        bild[0].load( "kids1.xpm" );
        bild[1].load( "lausbub.xpm" );
        bild[2].load( "niklclow.xpm" );
        bild[3].load( "penguin.xpm" );

        QWidget *w = new QWidget( this->viewport() );
        w->resize( 800, 1000 );

        corner = new QLabel( this );
        corner->setPaletteBackgroundColor( Qt::white );
        setCornerWidget( corner );
        anzeige = new QLabel( corner );
        anzeige->setPaletteBackgroundColor( Qt::blue );

        for ( int i=0; i<20; i++ ) {
           SubScrollView *sv = new SubScrollView( &bild[::rand()%4], w );
           sv->setGeometry( i%4*200, i/4*200, 200, 200 );
        }
        addChild( w );
        resize( 600, 600 );

        xFactor = (double)cornerWidget()->width() / contentsWidth();
        yFactor = (double)cornerWidget()->height() / contentsHeight();

        QObject::connect( this , SIGNAL( contentsMoving( int, int ) ),
                          this,  SLOT( showPosInCorner( int, int ) ) );
        showPosInCorner( 0, 0 );
    }

public slots:
    void showPosInCorner( int, int ) {
       anzeige->setGeometry( int(contentsX()*xFactor+0.5),
                             int(contentsY()*yFactor+0.5),
                             int(visibleWidth()*xFactor+0.5),
                             int(visibleHeight()*yFactor+0.5) );
    }

private:
    QPixmap bild[4];
    QLabel  *corner, *anzeige;
    double  xFactor, yFactor;
};

#include "scrollview2.moc"

//..................................................................... main
int main( int argc, char *argv[] )
{
    QApplication a( argc, argv );

    MyScrollView *myWidget =  new MyScrollView;

    a.setMainWidget( myWidget );
    myWidget->show();
    myWidget->showPosInCorner( 0, 0 );
    
    return a.exec();
}
