#include <qwidget.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qcolordialog.h>
#include <qfiledialog.h>
#include <qfontdialog.h>
#include <qsettings.h>
const QString WINDOWS_REGISTRY = "/Trolltech/settingbeispiel";
const QString APP_KEY = "/Setting/";
//................................................................. MyLabel
class MyLabel  : public QWidget {
    Q_OBJECT
public:
    MyLabel () : QWidget( 0, 0) {
        QSettings    config;
        config.insertSearchPath( QSettings::Windows, WINDOWS_REGISTRY );
          //.... Lesen der eingestellten Farbe
        rot   = config.readNumEntry( APP_KEY + "color/rgb/r", 120 ),
        gruen = config.readNumEntry( APP_KEY + "color/rgb/g", 220 ),
        blau  = config.readNumEntry( APP_KEY + "color/rgb/b", 60 );
          //.... Lesen des eingestellten Fonts
        font = QFont( "Helvetica", 12 );
        font.fromString( config.readEntry( APP_KEY + "font", font.toString() ) );
          //.... Lesen des eingestellten Textes
        text = config.readEntry( APP_KEY + "zuletzt/text", "Noch kein Text" );
          //.... Pushbuttons
        QPushButton *farbeButton = new QPushButton( "Neue Farbe", this );
         connect( farbeButton, SIGNAL( clicked() ), SLOT( neueFarbe() ) );
         farbeButton->setGeometry( 0, 0, 100, 30 );
        QPushButton *fontButton = new QPushButton( "Neuer Font", this );
         connect( fontButton, SIGNAL( clicked() ), SLOT( neuerFont() ) );
         fontButton->setGeometry( 100, 0, 120, 30 );
        QPushButton *dateiButton = new QPushButton( "Neue Datei", this );
         connect( dateiButton, SIGNAL( clicked() ), SLOT( neueDatei() ) );
         dateiButton->setGeometry( 220, 0, 130, 30 );
        QPushButton *sicherButton = new QPushButton( "Einstellungen sichern", this );
         connect( sicherButton, SIGNAL(clicked()), this, SLOT( saveConfig() ) );
         sicherButton->setGeometry( 350, 0, 200, 30 );
        QPushButton *quitButton = new QPushButton( "Beenden", this );
         connect( quitButton, SIGNAL(clicked()), qApp, SLOT( quit() ) );
         quitButton->setGeometry( 550, 0, 100, 30 );
        label = new QLabel( this );
         label->setPaletteBackgroundColor( QColor( rot, gruen, blau ) );
         label->setAlignment( Qt::AlignCenter );
         label->setFont( font );
         label->setGeometry( 0, 30, 650, 370 );
        showText();
    }
private slots:
    void  neueFarbe() {
       QColor neuFarbe = QColorDialog::getColor( QColor(rot, gruen, blau), 0 );
       if ( neuFarbe.isValid() ) {
          rot   = neuFarbe.red();
          gruen = neuFarbe.green();
          blau  = neuFarbe.blue();
          showText();
       }
    }
    void  neuerFont() {
       bool ok = true;
       font = QFontDialog::getFont( &ok, font, this );
       if ( ok ) {
          label->setFont( font );
          showText();
       }
    }
    void  neueDatei() {
       QStringList list( QFileDialog::getOpenFileNames( "*", 0, this) );
       if ( !list.isEmpty() ) {
          text = "\n";
          for ( QStringList::Iterator it = list.begin();
                it != list.end();
                ++it ) {
             text += *it;
             text += "\n";
          }
          showText();
       }
    }
    void saveConfig() {
       QSettings    config;
       config.writeEntry( APP_KEY + "color/rgb/r", rot );
       config.writeEntry( APP_KEY + "color/rgb/g", gruen );
       config.writeEntry( APP_KEY + "color/rgb/b", blau );
       config.writeEntry( APP_KEY + "zuletzt/text", text );
       config.writeEntry( APP_KEY + "font", font.toString() );
    }
private:
    void showText() {
       label->setPaletteBackgroundColor( QColor( rot, gruen, blau ) );
       label->setText( text );
       label->show();
    }
    int          rot, gruen, blau;
    QFont        font;
    QString      text;
    QLabel      *label;
};
#include "setting.moc"
//.................................................................... main
int main( int argc, char *argv[] ) {
  QApplication a( argc, argv );
  MyLabel  *w = new MyLabel ;
  w->show();
  w->resize( 650, 400 );
  a.setMainWidget( w );
  return a.exec();
}
