#include <qapplication.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qfiledialog.h>
#include <qpainter.h>
//........................................................ Klasse SnapShot
class SnapShot : public QWidget {
 public:
    SnapShot( QWidget *par=0, const char *nam=0 ) : QWidget( par, nam) {
       resize(350,350);
       grabx = graby = -1;
       grabbing = false;
       setMouseTracking( true );
       grabAround( QPoint(grabx=qApp->desktop()->width()/2,
                          graby=qApp->desktop()->height()/2) );
    }
 protected:
    void paintEvent( QPaintEvent * ) {
       if ( !p.isNull() ) {
          QPainter paint( this );
          paint.drawPixmap( 0, 0, p, 0,0, width(), height() );
       }
    }
    void mousePressEvent( QMouseEvent *e ) {
       if ( e->button() == RightButton) {
          if ( !p.isNull() ) {
             QString fileName = QFileDialog::getSaveFileName();
             if ( !fileName.isEmpty() )
                p.save( fileName, "BMP" );
          }
       } else {
          if ( !grabbing ) {
             grabbing = true;
             grabMouse( crossCursor ); // Cursor als Kreuz anzeigen
             grabx = graby = -1;
             oldRx = oldRy = -1;
          } else {
             grabx = mapToGlobal(e->pos()).x();
             graby = mapToGlobal(e->pos()).y();
          }
       }
    }
    void mouseMoveEvent( QMouseEvent * e ) {
        if ( grabbing && grabx > 0 && graby > 0 )
           grabAround(e->pos());
    }
    void mouseReleaseEvent( QMouseEvent * e ) {
        if ( grabbing && grabx >= 0 && graby >= 0 ) {
           grabbing = false;
           grabAround(e->pos());
           releaseMouse();
        }
    }
    void resizeEvent( QResizeEvent * ) { grab(); }
 private:
    void grabAround( QPoint pos ) {
       int rx = mapToGlobal(pos).x(),
           ry = mapToGlobal(pos).y(),
           w = QABS(rx-grabx),
           h = QABS(ry-graby);
       if ( w > 10 && h > 10  && 
            ( QABS(oldRx-rx) > 10 || QABS( oldRy-ry) > 10 ) ) {
          resize( w, h );
          grab();
          oldRx = rx;
          oldRy = ry;
       }
    }
    void grab() {
       if ( !isVisible() || grabx < 0 || graby < 0 )
          return;
       int w = width()-1, h = height()-1;

       if ( grabx + w > QApplication::desktop()->width() )
          grabx = QApplication::desktop()->width()-w;
       else if ( grabx < 0 )
          grabx = 0;
       if ( graby + h > QApplication::desktop()->height() ) 
          graby = QApplication::desktop()->height()-h;
       else if ( graby < 0 )
          graby = 0;
       p = QPixmap::grabWindow( QApplication::desktop()->winId(),
                                grabx, graby, w, h );
    }
    QPixmap    p;
    bool       grabbing;
    int        grabx, graby;
    int        oldRx, oldRy;
};
//.................................................................. main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    SnapShot m;
    a.setMainWidget( &m );
    m.show();
    return a.exec();
}
