#include <qapplication.h>
#include <qpushbutton.h>
#include <qpixmap.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qpainter.h>
#include <qtimer.h>

//........................................................ Klasse SnapShot
class SnapShot : public QWidget {
    Q_OBJECT
public:
    SnapShot( QWidget *parent=0, const char *name=0 ) :
         QWidget( parent, name,
                  WStyle_Customize | WStyle_NormalBorder| WX11BypassWM )
    {
       QFrame *frame = new QFrame( this );
       frame->resize( 410, 310 );
       frame->setFrameStyle( QFrame::Box | QFrame::Plain );
       frame->setLineWidth( 5 );

       showShot = new QLabel( this );
       showShot->setGeometry( 5, 5, 400, 250 );
       showShot->setPaletteBackgroundColor( Qt::white );

       selectButton = new QPushButton( "Mouse-Select", this );
       selectButton->setGeometry( 5, 255, 100, 50 );
       QObject::connect( selectButton, SIGNAL( clicked() ),
                         this,         SLOT( select() ) );
       showButton = new QPushButton( "Show Shot", this );
       showButton->setGeometry( 105, 255, 100, 50 );
       QObject::connect( showButton, SIGNAL( clicked() ),
                         this,       SLOT( showPic() ) );
       saveButton = new QPushButton( "Save", this );
       saveButton->setGeometry( 205, 255, 100, 50 );
       QObject::connect( saveButton, SIGNAL( clicked() ),
                         this,       SLOT( save() ) );
       quitButton = new QPushButton( "Quit", this );
       quitButton->setGeometry( 305, 255, 100, 50 );
       QObject::connect( quitButton, SIGNAL( clicked() ),
                         qApp,       SLOT( quit() ) );
       puffer.resize( showShot->size() );

       setFixedSize( 410, 310 );
       show();

       hide();
       grab( 1, QPoint(0,0) );
       grabx = graby = -1;
       grabbing = false;
       setMouseTracking( true );
    }

protected:
    virtual void mousePressEvent( QMouseEvent *e ) {
       if ( grabbing ) {
          grabx = mapToGlobal(e->pos()).x();
          graby = mapToGlobal(e->pos()).y();
       }
    }

    virtual void mouseReleaseEvent( QMouseEvent * e ) {
        if ( grabbing && grabx >= 0 && graby >= 0 ) {
           releaseMouse();
           grabbing = false;
           hide();
           grab( 2, e->pos() );
        }
    }

private slots:
    void save( void ) {
       QString fileName = QFileDialog::getSaveFileName();
       if ( !fileName.isEmpty() )
          p.save( fileName, "BMP" );
    }

    void select( void ) {
       grabbing = true;
       grabMouse( crossCursor ); // Cursor als Kreuz anzeigen
       grabx = graby = -1;
    }

    void showPic( void ) {
       QLabel *l = new QLabel( 0);
       l->setPixmap( p );
       l->show();
       l->resize( l->sizeHint() );
    }

private:
    void grab( int type, QPoint pos ) {
       if ( type == 1 ) {
          p = QPixmap::grabWindow( QApplication::desktop()->winId() );
       } else if ( type == 2 ) {
          int x = mapToGlobal(pos).x(),
              y = mapToGlobal(pos).y(),
              w = QABS( x - grabx ),
              h = QABS( y - graby );
          if ( x > grabx )
             x = grabx;
          if ( y > graby )
             y = graby;
          p = QPixmap::grabWindow( QApplication::desktop()->winId(),
                                   x, y, w, h );
       }
       QPainter paint;
       paint.begin( &puffer );
       paint.scale( (double)showShot->width()/p.width(),
                    (double)showShot->height()/p.height() );
       paint.drawPixmap( 0, 0, p );
       paint.end();
       showShot->setPixmap( puffer );
       show();
    }

    QPixmap    p,
               puffer;
    bool       grabbing;
    int        grabx, graby;

    QLabel      *showShot;
    QPushButton *selectButton,
                *showButton,
                *saveButton,
                *quitButton;
};

#include "snapshot2.moc"

//.................................................................. main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    SnapShot *m = new SnapShot;
    a.setMainWidget( m );
    return a.exec();
}
