#include <qapplication.h>
#include <qsplitter.h>
#include <qpainter.h>
#include <stdlib.h>
#include <time.h>

//...................................................... Klasse RechtEcke
class RechtEcke : public QWidget {
public:
    RechtEcke( QWidget* parent=0, const char* name=0, int f=0 ) :
                                        QWidget(parent, name, f) {
        p = new QPainter( this );
        p->setBrush( QColor( ::rand()%255, ::rand()%255, ::rand()%255 ) );
        startTimer(10); // alle 10 Millisek. Signal schicken (timerEvent
                        // aufrufen), um Bild zu aktualisieren
        dx = dy = 1;
        x = y = 0;
    }
    ~RechtEcke() { delete p; }

    void timerEvent( QTimerEvent * ) {
       repaint( false ); // false: altes Bild nicht loeschen
    }

    void paintEvent( QPaintEvent * ) {
        int w = width(), h = height();
        
        x += dx;
        y += dy;
        if ( dx > 0 ) {
           farbe = QColor( ::rand()&255, ::rand()&255, ::rand()&255 );
           p->setBrush( NoBrush );
        } else
           p->setBrush( QBrush( farbe ) );
        p->setPen( farbe );
        p->drawRect( QRect( QPoint( x, y ), QPoint( w-x, h-y ) ) );
        if ( x <= 0 || y <= 0 || x >= w-x || y >= h-y ) {
           dx = -dx;
           dy = -dy;
        }
    }

private:
    QPainter *p;
    QColor   farbe;
    int      x, y, dx, dy;
};

//.................................................................. main
int main( int argc, char *argv[] )
{
    QApplication a( argc, argv );

    ::srand(time(NULL));

     //... Hauptsplitter, in dem vertikal drei Subsplitter angeordnet werden
    QSplitter *s1 = new QSplitter( QSplitter::Vertical, 0 , "main" );
    s1->setGeometry( 0, 0, 600, 400 );

     //... Oberer Subsplitter mit zwei horizontal angeordneten Subsplittern
    QSplitter *s2 = new QSplitter( QSplitter::Horizontal, s1, "medium" );
     RechtEcke *b1 = new RechtEcke( s2 );
      b1->setPaletteBackgroundColor( Qt::blue.light(180) );
      b1->setMinimumSize( 150, 150 );
     RechtEcke *b2 = new RechtEcke( s2 );
      b2->setPaletteBackgroundColor( Qt::green.light(180) );
      s2->setResizeMode( b2, QSplitter::KeepSize );
      s2->moveToFirst( b2 );

     //... Unterer Subsplitter mit drei horizontal angeordneten Subsplittern
    QSplitter *s3 = new QSplitter( QSplitter::Horizontal,  s1, "bottom" );
     RechtEcke *b3 = new RechtEcke( s3 );
      b3->setPaletteBackgroundColor( Qt::red.light(180) );
     RechtEcke *b4 = new RechtEcke( s3 );
      b4->setPaletteBackgroundColor( Qt::white );
     RechtEcke *b5 = new RechtEcke( s3 );
      b5->setMaximumHeight( 250 );
      b5->setMinimumSize( 80, 100 );
      b5->setPaletteBackgroundColor( Qt::yellow.dark(120) );

     //... Subwidgets der Splitter s1, s2 und s3
     //... erfahren sofort alle Resize-Events
    s1->setOpaqueResize( true );
//    s2->setOpaqueResize( true );  // s2 nicht
    s3->setOpaqueResize( true );

    a.setMainWidget( s1 );
    s1->show();
    return a.exec();
}
