#include <qapplication.h>
#include <qsplitter.h>
#include <qpainter.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>

//....................................................... Klasse RotKreis
class RotKreis : public QWidget {
public:
    RotKreis( QWidget* parent=0, const char* name=0, int f=0 ) :
                                        QWidget(parent, name, f) {
        p = new QPainter( this );
        p->setBrush( QColor( ::rand()%255, ::rand()%255, ::rand()%255 ) );
        startTimer(20); // alle 20 Millisek. Signal schicken (timerEvent
                        // aufrufen), um Bild zu aktualisieren
        s = 0;
    }
    ~RotKreis() { delete p; }

    void timerEvent( QTimerEvent * ) {
       repaint( false ); // false: altes Bild nicht loeschen
    }

    void paintEvent( QPaintEvent * ) { //... 3 Kreise um Mittelpunkt rotieren
        const double  PI = 4*atan(1);
        int xmitte = width()/2,
            ymitte = height()/2;
        double r,
               x1, y1, x2, y2, x3, y3;

        if ( s >= 10*PI )
           s = 0;
        if ( s == 0 ) {
            radius   = 50.0/(::rand()%60)+1;
            r_factor = 50.0/(::rand()%60)+1;
            x_factor = 50.0/(::rand()%60)+1;
            y_factor = 50.0/(::rand()%60)+1;
            farbe1 = QColor( ::rand()&255, ::rand()&255, ::rand()&255 );
            farbe2 = QColor( ::rand()&255, ::rand()&255, ::rand()&255 );
            farbe3 = QColor( ::rand()&255, ::rand()&255, ::rand()&255 );
        }
        r = r_factor*s;
           //........ Mittelpunkte der Kreise berechnen
        x1 = r*cos(s)*x_factor + xmitte;
        y1 = r*sin(s)*y_factor + ymitte;
        x2 = r*cos(s+2*PI/3)*x_factor + xmitte;
        y2 = r*sin(s+2*PI/3)*y_factor + ymitte;
        x3 = r*cos(s+4*PI/3)*x_factor + xmitte;
        y3 = r*sin(s+4*PI/3)*y_factor + ymitte;
            //........ Kreise malen
        double  rad = radius*s;
        p->setPen( farbe1 );
        p->setBrush( QBrush( farbe1 ) );
        p->drawLine( int(x1), int(y1), int(x2), int(y2) );
        p->drawEllipse( int(x1-rad/2), int(y1-rad/2), int(rad), int(rad) );
        p->setPen( farbe2 );
        p->setBrush( QBrush( farbe2 ) );
        p->drawLine( int(x2), int(y2), int(x3), int(y3));
        p->drawEllipse( int(x2-rad/2), int(y2-rad/2), int(rad), int(rad) );
        p->setPen( farbe3 );
        p->setBrush( QBrush( farbe3 ) );
        p->drawLine( int(x3), int(y3), int(x1), int(y1) );
        p->drawEllipse( int(x3-rad/2), int(y3-rad/2), int(rad), int(rad) );
        s += 0.05;
    }

private:
    QPainter *p;
    double   s, radius, r_factor, x_factor, y_factor;
    QColor   farbe1, farbe2, farbe3;
};

//.................................................................. main
int main( int argc, char *argv[] )
{
    QApplication a( argc, argv );

    ::srand(time(NULL));

     //... Hauptsplitter, in dem vertikal drei Subsplitter angeordnet werden
    QSplitter *s1 = new QSplitter( QSplitter::Vertical, 0 , "main" );
    s1->setGeometry( 0, 0, 600, 400 );

     //... Oberer Subsplitter mit zwei horizontal angeordneten Subsplittern
    QSplitter *s2 = new QSplitter( QSplitter::Horizontal, s1, "medium" );
     RotKreis *b1 = new RotKreis( s2 );
      b1->setBackgroundColor( Qt::blue.light(180) );
      b1->setMinimumSize( 150, 150 );
     RotKreis *b2 = new RotKreis( s2 );
      b2->setBackgroundColor( Qt::green.light(180) );
      s2->setResizeMode( b2, QSplitter::KeepSize );
      s2->moveToFirst( b2 );

     //... Unterer Subsplitter mit drei horizontal angeordneten Subsplittern
    QSplitter *s3 = new QSplitter( QSplitter::Horizontal,  s1, "bottom" );
     RotKreis *b3 = new RotKreis( s3 );
      b3->setBackgroundColor( Qt::red.light(180) );
     RotKreis *b4 = new RotKreis( s3 );
      b4->setBackgroundColor( Qt::white );
     RotKreis *b5 = new RotKreis( s3 );
      b5->setMaximumHeight( 250 );
      b5->setMinimumSize( 80, 100 );
      b5->setBackgroundColor( Qt::yellow.dark(120) );

     //... Subwidgets der Splitter s1, s2 und s3
     //... erfahren sofort alle Resize-Events
    s1->setOpaqueResize( true );
//    s2->setOpaqueResize( true );  // s2 nicht
    s3->setOpaqueResize( true );

    a.setMainWidget( s1 );
    s1->show();
    return a.exec();
}
