#include <qapplication.h>
#include <qtabdialog.h>
#include <qtextedit.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qlabel.h>
#include <qslider.h>
#include <qlayout.h>
#include <qstring.h>
#include <qlcdnumber.h>

//........................................................ Klasse PersDaten
class PersDaten: public QLabel
{
    Q_OBJECT

public:
    PersDaten( QWidget * parent = 0, const char * name = 0 );

private slots:
    void groesseAnzeige( int wert ) { groesse->display( wert ); }

    void setup() {
       ed1->setText( "Hans Mustermann" );
       ed2->setText( "Musterstr. 4711\n99999 Musterstadt" );
       groesse->display( 150 );
       schieb->setValue( 150 );
    }

    void apply() {
       QString s;
       s.sprintf( "Name:\n%s\n\nAdresse:\n%s\n\n"
                  "Geschlecht: %s\n\nGroesse: %d cm\n",
                  (const char *)(ed1->text()), (const char *)(ed2->text()),
                  b1->isChecked() ? "maennlich"
                                  : (b2->isChecked() ? "weiblich" : "keines"),
                  schieb->value() );
       setText( s );
    }

private:
    QTextEdit *ed1,  *ed2; // fuer 1. Seite

    QButtonGroup *bg;    // fuer 2. Seite
    QRadioButton *b1, *b2;
    QSlider      *schieb;
    QLCDNumber   *groesse;
};

//................................................. PersDaten (Konstruktor)
PersDaten::PersDaten( QWidget * parent, const char * name )
                                              : QLabel( parent, name )
{
    QTabDialog * tab = new QTabDialog;
    tab->setCaption( "Persnliche Daten" );

        //.......................................... 1. Seite
    QWidget *w = new QWidget( tab, "1. Seite" );

    QGridLayout *g = new QGridLayout( w, 2, 2, 5 );

      //..... Zwei QTextEdit-Widgets in 1. Spalte
    ed1 = new QTextEdit( w );
    g->addWidget( ed1, 0, 1 );
     ed1->setText( "" );
     ed1->setMinimumSize( QSize( 100, 10 ) );
     ed2 = new QTextEdit( w );
    g->addWidget( ed2, 1, 1 );
     ed2->setText( "" );
     ed2->setMinimumSize( QSize( 100, 10 ) );
    g->setColStretch( 1, 1 ); // QTextEdit-Widgets sind dehnbar

      //..... Zwei Labels in 0. Spalte
    QLabel *l = new QLabel( w );
    g->addWidget( l, 0, 0 );
     l->setText( "&Name" );
     l->setBuddy( ed1 );
     l->setMinimumSize( l->sizeHint() );
     l = new QLabel( w );
    g->addWidget( l, 1, 0 );
     l->setText( "&Adresse" );
     l->setBuddy( ed2 );
     l->setMinimumSize( l->sizeHint() );
    g->setColStretch( 0, 0 ); // Labels sind nicht dehnbar

     //..... Seite in TabDialog einfuegen
    tab->addTab( w, "&Name und Adresse" );

        //.......................................... 2. Seite
    w = new QWidget( tab, "2. Seite" );
    QBoxLayout *b = new QBoxLayout( w, QBoxLayout::LeftToRight, 5 );

      // zwei vertikale Boxen in einer horizontalen Box
    QBoxLayout * radioBoxes = new QBoxLayout( QBoxLayout::Down );
    b->addLayout( radioBoxes );

      // linke vertikale Box fllen
    b1 = new QRadioButton( w );
     b1->setText( "Mnnlich" );
     b1->setMinimumSize( b1->sizeHint() );
    radioBoxes->addWidget( b1, AlignLeft|AlignTop );
    b2 = new QRadioButton( w );
     b2->setText( "Weiblich" );
     b2->setMinimumSize( b2->sizeHint() );
    radioBoxes->addWidget( b2, AlignLeft|AlignTop );

    radioBoxes->addStretch( 1 ); // verhindert Stretching fuer Radiobuttons
    bg = new QButtonGroup();
     bg->insert( b1 );
     bg->insert( b2 );

    b->addStretch( 1 ); // Zwischenraum zw. Radiobuttons und Schiebebalken

      // Schiebebalken
    schieb = new QSlider( 50, 250, 1, 150, QSlider::Vertical, w );
     schieb->setMinimumSize( schieb->sizeHint() );
     schieb->setMaximumSize( schieb->minimumSize().width(), 500 );
    b->addWidget( schieb, AlignLeft|AlignTop|AlignBottom );

      // Oben: Beschriftung fuer Schiebebalken; Unten: aktueller Wert
    QBoxLayout *labels = new QBoxLayout( QBoxLayout::Down );
    b->addLayout( labels );
    l = new QLabel( "Groesse (in cm)", w );
     l->setFixedSize( l->sizeHint() );
     labels->addWidget( l, AlignTop|AlignLeft );
    labels->addStretch( 1 );
    groesse = new QLCDNumber( 3, w );
     groesse->setFixedSize( 150, 100 ); // groesse->sizeHint() );
     connect( schieb, SIGNAL( valueChanged( int ) ),
              this,   SLOT( groesseAnzeige( int ) ) );
    labels->addWidget( groesse, AlignBottom|AlignLeft );
    tab->addTab( w, "&Sonstiges" );

      //..... Uebernehmen- und Abbrechen-Button
    tab->setApplyButton( "&Uebernehmen" );
    tab->setCancelButton( "&Abbrechen" );

    connect( tab, SIGNAL( applyButtonPressed()), SLOT( apply() ) );
    connect( tab, SIGNAL( cancelButtonPressed() ), qApp, SLOT( quit() ) );
    connect( tab, SIGNAL( aboutToShow() ),  SLOT( setup() ) );

    tab->resize( 200, 200 );

    setIndent( 20 );
    setText( "Hier werden immer die\naktuell gueltigen Daten angezeigt\n" );

    resize( 300, 300 );
    show();
    tab->show();
}

#include "tabdialog2.moc"

//.................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    PersDaten mw;
    a.setMainWidget( &mw );
    mw.setCaption( "Persnliche Daten" );
    mw.show();
    return a.exec();
}
