#include <qapplication.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qkeycode.h>
#include <qgridview.h>
#include "tableview.h"

const int zahlZeilen  = 31; 
const int zahlSpalten = 31;
//................................................ Tableview (Konstruktor)
Tableview::Tableview(int zeilZahl, int spaltZahl, QWidget *p, const char *nam)
                  : QGridView(p,nam) {
    aktZeile = aktSpalte = 0;         // Aktuell ausgewaehlte Zelle
    setNumCols( spaltZahl );         // Festlegen der Spalten- und
    setNumRows( zeilZahl );          //  Zeilenzahl in der Tabelle
    setCellWidth( 100 );   // Festlegen der Breite und
    setCellHeight( 30 );   // Hoehe der Zellen (in Pixel)
    resize( 600, 400 );
    inhalt      = new QString[zeilZahl * spaltZahl]; // Speicher fuer Inhalt
    zellenFarbe = new QColor[zeilZahl * spaltZahl];  // Speicher fuer Farben
    eingabewert = 0;
    eingabeGemacht = false;
}
//.............................................................. paintCell
void Tableview::paintCell( QPainter* p, int zeile, int spalte ) {
    int w = cellWidth(), h = cellHeight();
    if ( zeile == aktZeile && spalte == aktSpalte ) { 
       p->setBrush( Qt::yellow );
       p->setPen( DotLine );
       p->drawRect( 0, 0, w-1, h-1 );
       p->setPen( SolidLine );
    } else
       p->fillRect( 0, 0, w, h, zellenFarbe[ indexOf( zeile, spalte ) ] );
    p->drawLine( w-1, 0, w-1, h-1 ); // Rahmen um Zelle malen
    p->drawLine( 0, h-1, w-1, h-1 );
      //... Inhalt (Text) der Zelle ausgeben
    p->drawText( 0, 0, w, h, AlignCenter, inhalt[indexOf(zeile,spalte)] );
}
//........................................................ mousePressEvent
void Tableview::mousePressEvent( QMouseEvent* e ) {
   int  altZeile = aktZeile, altSpalte = aktSpalte;
     //... Position ermitteln, an der Mauszeiger geklickt wurde
   QPoint clickedPos = viewportToContents( e->pos() );
   aktZeile = rowAt( clickedPos.y() );
   aktSpalte = columnAt( clickedPos.x() );
     //... Wenn aktuelle Zelle sich geaendert hat
   if ( aktZeile != altZeile || aktSpalte != altSpalte ) {
       pruefEingabe( altZeile, altSpalte );  // alten Rahmen loeschen
       updateCell( aktZeile, aktSpalte );  // Rahmen fuer neue Zelle
   }
   eingabewert = 0;
}
//.......................................................... keyPressEvent
void Tableview::keyPressEvent( QKeyEvent* e ) {
    int      altZeile = aktZeile, altSpalte = aktSpalte;
    QString  s;
    if ( eingabewert == 0 )
       alt = inhalt[ indexOf( aktZeile, aktSpalte ) ];
    switch( e->key() ) {
       case Key_Left:
           pruefEingabe( aktZeile, aktSpalte );
           if ( aktSpalte > 0 )
              ensureCellVisible(aktZeile, --aktSpalte);
           break;
       case Key_Right:
           pruefEingabe( aktZeile, aktSpalte );
           if ( aktSpalte < numCols()-1 )
              ensureCellVisible(aktZeile, ++aktSpalte);
           break;
       case Key_Up:
           pruefEingabe( aktZeile, aktSpalte );
           if ( aktZeile > 0 )
              ensureCellVisible(--aktZeile, aktSpalte);
           break;
       case Key_Down:
           pruefEingabe( aktZeile, aktSpalte );
           if ( aktZeile < numRows()-1 )
              ensureCellVisible(++aktZeile, aktSpalte);
           break;
       default:
           if ( e->key() >= Key_0 && e->key() <= Key_9) {
              eingabeGemacht = true;
              eingabewert = eingabewert * 10 + e->key()-Key_0;
              s.sprintf("%d", eingabewert);
              setzeZellenInhalt( aktZeile, aktSpalte, s );
           } else if ( e->key() == Key_Return ) {
              pruefEingabe( aktZeile, aktSpalte );
           } else {
              e->ignore();  //.... Alle anderen Tasten ignorieren
              return;   
           }
    }
     //... Wenn aktuelle Zelle sich geaendert hat
   if ( aktZeile != altZeile || aktSpalte != altSpalte ) {
       updateCell( altZeile, altSpalte );  // alten Rahmen loeschen
       updateCell( aktZeile, aktSpalte );  // Rahmen fuer neue Zelle
   }
}
//................................................................... main
int main( int argc, char *argv[] ) {
   QApplication a(argc,argv);   		
   Tableview t( zahlZeilen, zahlSpalten );
   QString s;
   for( int i = 0; i < zahlZeilen; i++ ) {
      for( int j = 0; j < zahlSpalten; j++ ) {
         s.sprintf("%d x %d", i, j);
         t.setzeZellenInhalt( i, j, s ); //... Zelleninhalt (Malaufgabe)
         t.setzeZellenFarbe( i, j, Qt::white ); //... Zellenfarbe
      }
   }
   a.setMainWidget( &t );
   t.show();
   return a.exec();
}
