#include <qapplication.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qlabel.h>
#include <qvbox.h>
#include <ctype.h>
//................................................ Klasse EingabeValidator
class EingabeValidator : public QValidator {
 public:
   EingabeValidator() : QValidator( 0 ) { };
 protected:
   virtual State validate( QString &text, int &pos ) const {
      int   textLaenge = text.length();
      if( textLaenge == 0 ) //... Leerer String wird akzeptiert
         return Intermediate;
      if ( ( pos <= 3 && !isalpha( text.at(pos-1).latin1() ) )            ||
           ( pos > 3 && pos <= 8 && !isdigit( text.at(pos-1).latin1() ) ) ||
             pos > 8 )
         return Invalid;
      else if ( pos <= 8 && textLaenge < 8 )
         return Intermediate;
      fixup( text ); //.... Expliziter Aufruf von fixup()
      return Acceptable;
   }
    //... fixup wird automat. aufgerufen, wenn validate() Intermediate
    //... zurueckgegeben hatte und anschliessend Return durch
    //... den Benutzer gedrueckt wird.
   virtual void fixup( QString& text ) const {
      for ( unsigned i= 0; i<3 && i < text.length(); i++ )
         text.at(i) = toupper( text.at(i).latin1() );
   }
};
//............................................................ Klasse Code
class Code : public QVBox {
    Q_OBJECT
public:
    Code( QWidget *parent=0, const char *name=0 ) : QVBox( parent, name ) {
       setMargin( 10 );
       QVBox *box = new QVBox( this ); // Widget fuer das Layout
       box->setMargin( 5 );
       box->setSpacing( 20 );
       (void)new QLabel(
           "Geben Sie drei Buchstaben gefolgt von 5 Ziffern ein: ", box );
       eingabe = new QLineEdit( box );
       EingabeValidator *codVal = new EingabeValidator();
       eingabe->setValidator( codVal );
       logMeldung = new QLabel( "Noch nicht mit Eingabe begonnen", box );
       logMeldung->setPaletteBackgroundColor( Qt::blue.light( 180 ) );
       QObject::connect( eingabe, SIGNAL( textChanged( const QString & ) ),
                         this,    SLOT( slotTextChanged( const QString & ) ) );
       QObject::connect( eingabe, SIGNAL( returnPressed() ),
                         this,    SLOT( slotReturnPressed() ) );
   }
private slots:
   void slotTextChanged( const QString &text ) {
       QString  t;
       t.sprintf("Noch nicht fertig; bisher: %s", (const char *)text );
       logMeldung->setText( t ); 
       logMeldung->setPaletteBackgroundColor( Qt::red.light( 160 ) );
   }
   void slotReturnPressed() {
       QString  endeText;
       endeText.sprintf("Fertig und akzeptiert: %s",
                        (const char *)eingabe->text() ); 
       logMeldung->setText( endeText ); 
       logMeldung->setPaletteBackgroundColor( Qt::green.light(160) );
   }
private:
   QLineEdit *eingabe;
   QLabel    *logMeldung;
};
#include "validator.moc"
//................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );
    Code cod;
    cod.resize( 400, cod.sizeHint().height() );
    cod.setCaption( "QValidator-Beispiel" );
    a.setMainWidget( &cod );
    cod.show();
    return a.exec();
}
