#include <qapplication.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qpainter.h>
#include <qtimer.h>
#include <stdlib.h>
#include <time.h>

#define MAXX  800
#define MAXY  600

#define UNDEFINIERT Qt::blue
#define KEIN_BAUM   Qt::black
#define STUMPF      Qt::gray
#define ABGEBRANNT  Qt::yellow
#define BRENNT      Qt::red
#define BAUM        Qt::green

const int  speed = 50,
           breite = 3,
           hoehe = 3;

//........................................................ Klasse MalFenster
class MalFenster : public QWidget
{
   Q_OBJECT
public:
   MalFenster() {
      l = new QLabel( this );
      l->setGeometry( 0, MAXY, MAXX, 40 );
      l->setPalette( QPalette( Qt::white, Qt::white ) );
      QTimer *t = new QTimer( this );
      connect( t, SIGNAL(timeout()), this, SLOT(naechsteSimulation()) );
      t->start( speed, false );
      maxx = MAXX/breite,
      maxy = MAXY/hoehe;
      neueSimulation();
   }

protected:
   virtual void mousePressEvent( QMouseEvent * ) {
      neueSimulation();
   }
   virtual void paintEvent ( QPaintEvent * ) {
      QPainter maler( this );
      maler.setPen( NoPen );
      for ( int x=0; x<maxx; x++ )
         for ( int y=0; y<maxy; y++ )
            if ( altWald[x][y] != neuWald[x][y] ) {
               maler.setBrush( QBrush( neuWald[x][y] ) );
               maleBaum( maler, x, y );
               altWald[x][y] = neuWald[x][y];
            }
   }

private:
   void neueSimulation() {
         //.................... Dichte des Waldes zufaellig
      QString  s;
      int      dichte = rand() % 21 + 50; // 50..70%
      int      grenze = RAND_MAX/100*dichte;
      s.sprintf(" Walddichte: %d %%\n", dichte );
      l->setText( s );
      l->show();

         //................................. Initialisieren
      for ( int x=0; x<maxx; x++ )
         for ( int y=0; y<maxy; y++ ) {
            neuWald[x][y] = ( rand()<=grenze ) ? BAUM : KEIN_BAUM;
            altWald[x][y] = UNDEFINIERT;
         }
         //................ Erste Baumreihe in Brand stecken
      for ( int y=0; y<maxy; y++ )
         if ( neuWald[0][y]==BAUM )
            neuWald[0][y] = BRENNT;
      repaint( true );
   }

private slots:
   void naechsteSimulation() {
      int  x, y;
      for ( x=0; x<maxx; x++)
         for ( y=0; y<maxy; y++ )
            if ( altWald[x][y] == BRENNT ) {
               if ( altWald[x-1][y] == BAUM )
                  neuWald[x-1][y] = BRENNT;
               if ( altWald[x+1][y] == BAUM )
                  neuWald[x+1][y] = BRENNT;
               if ( altWald[x][y-1] == BAUM )
                  neuWald[x][y-1] = BRENNT;
               if ( altWald[x][y+1] == BAUM )
                  neuWald[x][y+1]=BRENNT;
            }

      for ( x=0; x<maxx; x++ )
         for ( y=0; y<maxy; y++ )
            if ( altWald[x][y] == BRENNT )
               neuWald[x][y] = ABGEBRANNT;
            else if ( altWald[x][y] == ABGEBRANNT)
               neuWald[x][y] = STUMPF;
            else if ( altWald[x][y] == STUMPF)
               neuWald[x][y] = KEIN_BAUM;
      repaint( false );
   }

private:
   void maleBaum( QPainter &p, int x, int y ) {
      p.drawRect( x*breite, y*hoehe, breite, hoehe );
   }

   QLabel *l;
   QColor altWald [MAXX][MAXY],
          neuWald [MAXX][MAXY];
   int    maxx, maxy;
};

#include "waldbrand.moc"

//..................................................................... main
int main( int argc, char* argv[] )
{
   QApplication myapp( argc, argv );

   srand( time(NULL) );
   MalFenster* mywidget = new MalFenster;
   mywidget->setGeometry( 0, 0, MAXX, MAXY+40 );

   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
