#include <qapplication.h>
#include <qlabel.h>
#include <qwidgetstack.h>
#include <qmessagebox.h>
#include <qpixmap.h>
#include <stdlib.h>
#include <time.h>

const char *picNames[] = {
      "Ameise.xpm",    "Butterfly.xpm", "monalisa.xpm", "Deutschl.xpm",
      "Grossbri.xpm",  "Krebs.xpm",     "Pinguin.xpm",  "Suse.xpm",
      "Wizard.xpm",    "HandReach.xpm", "Eggs.xpm",     "3dpaint.xpm" };

const int picNo = sizeof( picNames ) / sizeof( picNames[0] );
const int cardNo = picNo * 2;

//.......................................................... Klasse MyLabel
class MyLabel : public QLabel
{
   Q_OBJECT
public:
   MyLabel( int z, int s, QPixmap bild, QWidget *parent=0,
                          const char *name = 0 ) : QLabel(parent, name ) {
      setAlignment( Qt::AlignCenter );
      setPixmap( bild );
      this->z = z;
      this->s = s;
   }

signals:
   void clicked( int, int );

protected:
   virtual void mousePressEvent( QMouseEvent * ) {
      emit clicked( z, s );
   };

private:
   int  z, s;
};

//......................................................... Klasse MyWidget
class MyWidget : public QWidget {
   Q_OBJECT
public:
   MyWidget( QWidget *parent=0, const char *name = 0, int f=0 )
                                 : QWidget(parent, name, f) {
       int  i, j, c, id;
       for ( i=0; i<picNo; i++ )
          if ( !bild[i].load( picNames[i] ) ) {
             QString  text;
             text.sprintf( "Kann Bild-Datei `%s' nicht laden", picNames[i] );
             bild[i].fill( QColor( ::rand()&255,::rand()&255,::rand()&255 ) );
             QMessageBox::information( 0, "Ladefehler", text,
                        QMessageBox::Ok );
             qApp->quit();
          }

       wstack = new QWidgetStack( 0 );
       wstack->setGeometry( 700, 100, 100, 100);

       offen = 0;
       for ( i=0; i<cardNo; i++ )
          boardCard[ i ] = -1;

       for ( i=0; i<picNo; i++ )
          for ( j=0; j<2; j++ ) {
             do {
             } while ( boardCard[ c = ::rand()%cardNo ] >= 0 );
             boardCard[ c ] = id = i*2 + j;
             label[id] = new QLabel( wstack );
             label[id]->setAlignment( Qt::AlignCenter );
             label[id]->setPixmap( bild[i] );
             label[id]->setGeometry( c%6*100, c/6*100, 100, 100 ); 
             wstack->addWidget( label[id], id );
          }
       if ( !back.load( "back.bmp" ) ) {
          QMessageBox::information( 0, "Ladefehler",
                     "Kann Bild-Datei `back.bmp' nicht laden" ,
                     QMessageBox::Ok );
          qApp->quit();
       } else {
          for ( i=0; i<cardNo; i++ ) {
             bLabel[i] = new MyLabel( i/6, i%6, back, this );
             bLabel[i]->setFrameStyle( QFrame::Box | QFrame::Sunken );
             bLabel[i]->setLineWidth( 2 );
             bLabel[i]->setGeometry( i%6*100, i/6*100, 100, 100 ); 
             QObject::connect( bLabel[i], SIGNAL( clicked( int, int ) ),
                               this,      SLOT( geklickt( int, int ) ) );
          }
       }
   }

public slots:
   void geklickt( int z, int s ) {
      QWidget *visible = wstack->visibleWidget(),
              *clicked = wstack->widget( boardCard[ z*6 + s] );

      if ( clicked == 0 )
         return;

      if ( offen == 2 ) {
         bLabel[ altZ[0]*6+altS[0] ]->setPixmap( back );
         bLabel[ altZ[1]*6+altS[1] ]->setPixmap( back );
         offen = 0;
      }
      wstack->raiseWidget( clicked );
      bLabel[ z*6+s ]->setPixmap( *((QLabel *)clicked)->pixmap() );
      altZ[offen] = z;
      altS[offen] = s;
      offen++;
 
      if ( offen == 2 && wstack->id(visible)/2 == wstack->id(clicked)/2 ) {
         bLabel[ altZ[0]*6+altS[0] ]->setPaletteBackgroundColor( Qt::green );
         bLabel[ altZ[1]*6+altS[1] ]->setPaletteBackgroundColor( Qt::green );
         wstack->removeWidget( visible );
         wstack->removeWidget( clicked );
         offen = 0;
      }
   }

private:
   QPixmap       back;
   QPixmap       bild[ picNo ];
   int           boardCard[ cardNo ];
   QLabel       *label[ cardNo ];
   QLabel       *helpLabel1, *helpLabel2;
   MyLabel      *bLabel[ cardNo ];
   QWidgetStack *wstack;
   int           altZ[2], altS[2], offen;
};

#include "widgetstack2.moc"

//.................................................................... main
int main( int argc, char *argv[] )
{
   QApplication myapp( argc, argv );

   ::srand( time( NULL ) );
   MyWidget *mywidget = new MyWidget();
   mywidget->setGeometry( 50, 50, 600, 400 );
    
   myapp.setMainWidget( mywidget );
   mywidget->show();
   return myapp.exec();
}
