#include <qapplication.h>
#include <qwidget.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qvalidator.h>
#include <qwizard.h>
//............................................................ Klasse Wizard
class Wizard : public QWizard {
    Q_OBJECT
public:
    Wizard( QWidget *p = 0, const char *nam = 0 ) : QWizard( p, nam, true ) {
       setupPage1();
       setupPage2();
       setupPage3();
       cancelButton()->setText( "Abbrechen" );
       backButton()->setText( "Zurck" );
       nextButton()->setText( "Weiter" );
       finishButton()->setText( "Fertigstellen" );
       pin->setFocus();
    }
    void showPage(QWidget* page);
protected:
    void setupPage1();
    void setupPage2();
    void setupPage3();
    QHBox      *page1, *page2, *page3;
    QLineEdit  *pin, *vorName, *nachName, *adresse, *gebDat;
    QLabel     *lPin, *lVorName, *lNachName, *lAdresse, *lGebDat;
protected slots:
    void pinChanged( const QString &text ) {
       nextButton()->setEnabled( text.length() == 4 );
    }
    void dataChanged( const QString & ) {
       nextButton()->setEnabled( !vorName->text().isEmpty()  &&
                                 !nachName->text().isEmpty() &&
                                 !adresse->text().isEmpty()     );
    }
};
//............................................................... setupPage1
void Wizard::setupPage1() {
    page1 = new QHBox( this );
    page1->setSpacing(8);
    QLabel *info = new QLabel( page1 );
     info->setPaletteBackgroundColor( Qt::blue.light( 180 ) );
     info->setText( "  Gib Deinen PIN-Code ein  \n\n"
                    "  Der PIN-Code besteht aus 4 Ziffern  \n" );
    info->setMaximumWidth( info->sizeHint().width() );
    QVBox *page = new QVBox( page1 );
    QHBox *z1 = new QHBox( page );
    (void)new QLabel( "PIN-Code:", z1 );
    pin = new QLineEdit( z1 );
     pin->setMinimumWidth( pin->sizeHint().width() );
     pin->setMaxLength( 4 );
     pin->setValidator( new QIntValidator( 0, 9999, pin ) );
    connect( pin,  SIGNAL( textChanged( const QString & ) ),
             this, SLOT( pinChanged( const QString & ) ) );
    addPage( page1, "Eingabe des Pin-Codes" );
    setNextEnabled( page1, false );
    setHelpEnabled( page1, false );
}
//............................................................... setupPage2
void Wizard::setupPage2() {
    page2 = new QHBox( this );
    page2->setSpacing(8);
    QLabel *info = new QLabel( page2 );
     info->setPaletteBackgroundColor( Qt::blue.light( 180 ) );
     info->setText( "\n  Bitte hier die  \n"
                    "  persnlichen Daten eingeben.  \n\n"
                    "  Vor-, Nachname und Adresse  \n"
                    "  mssen eingegeben werden. \n" );
    QVBox *page = new QVBox( page2 );
    QHBox *z1 = new QHBox( page );
     QHBox *z2 = new QHBox( page );
     QHBox *z3 = new QHBox( page );
     QHBox *z4 = new QHBox( page );
    QLabel *l1 = new QLabel( " Vorname: ",  z1 );
     QLabel *l2 = new QLabel( " Nachname: ", z2 );
     QLabel *l3 = new QLabel( " Adresse: ",  z3 );
     QLabel *l4 = new QLabel( " Geburtsdatum: ", z4 );
    l1->setMinimumWidth( l4->sizeHint().width() );
     l2->setMinimumWidth( l4->sizeHint().width() );
     l3->setMinimumWidth( l4->sizeHint().width() );
     l4->setMinimumWidth( l4->sizeHint().width() );
    vorName  = new QLineEdit( z1 );
    nachName = new QLineEdit( z2 );
    adresse  = new QLineEdit( z3 );
    gebDat   = new QLineEdit( z4 );
    connect( vorName, SIGNAL( textChanged( const QString & ) ),
             this,    SLOT( dataChanged( const QString & ) ) );
    connect( nachName, SIGNAL( textChanged( const QString & ) ),
             this,     SLOT( dataChanged( const QString & ) ) );
    connect( adresse, SIGNAL( textChanged( const QString & ) ),
             this,    SLOT( dataChanged( const QString & ) ) );
    addPage( page2, "Persnliche Angaben" );
    setHelpEnabled( page2, false );
}
//............................................................... setupPage3
void Wizard::setupPage3() {
    page3 = new QHBox( this );
    page3->setSpacing(8);
    QLabel *info = new QLabel( page3 );
     info->setPaletteBackgroundColor( Qt::blue.light( 180 ) );
     info->setText( "\n  Ihre eingegebenen Daten  \n"
                    "  sehen Sie rechts.  \n\n"
                    "  Klicken Sie auf `Fertigstellen',  \n"
                    "  wenn diese richtig sind.  \n\n"
                    "  Zum Korrigieren mssen Sie  \n"
                    "  auf `Zurck' klicken." );
     info->setMaximumWidth( info->sizeHint().width() );
    QVBox *page = new QVBox( page3 );
     QHBox *z1 = new QHBox( page );
     QHBox *z2 = new QHBox( page );
     QHBox *z3 = new QHBox( page );
     QHBox *z4 = new QHBox( page );
     QHBox *z5 = new QHBox( page );
    QLabel *l1 = new QLabel( " PIN-Code: ", z1 );
     QLabel *l2 = new QLabel( " Vorname: ", z2 );
     QLabel *l3 = new QLabel( " Nachname: ", z3 );
     QLabel *l4 = new QLabel( " Adresse: ", z4 );
     QLabel *l5 = new QLabel( " Geburtsdatum: ", z5 );
    l1->setMinimumWidth( l5->sizeHint().width() );
     l2->setMinimumWidth( l5->sizeHint().width() );
     l3->setMinimumWidth( l5->sizeHint().width() );
     l4->setMinimumWidth( l5->sizeHint().width() );
     l5->setMinimumWidth( l5->sizeHint().width() );
    lPin      = new QLabel( z1 );
    lVorName  = new QLabel( z2 );
    lNachName = new QLabel( z3 );
    lAdresse  = new QLabel( z4 );
    lGebDat   = new QLabel( z5 );
    addPage( page3, "Kontrollseite" );
    setFinish( page3, true );
    setHelpEnabled( page3, false );
}
//................................................................. showPage
void Wizard::showPage( QWidget* page ) {
    if ( page == page3 ) {
        lPin->setText( pin->text() );
        lVorName->setText( vorName->text() );
        lNachName->setText( nachName->text() );
        lAdresse->setText( adresse->text() );
        lGebDat->setText( gebDat->text() );
    }
    QWizard::showPage(page);
    if ( page == page1 ) {
        pinChanged( pin->text() );
        pin->setFocus();
    } else if ( page == page2 ) {
        dataChanged( vorName->text() );
        vorName->setFocus();
    } else if ( page == page3 ) {
        finishButton()->setEnabled( true );
        finishButton()->setFocus();
    }
}
#include "wizard.moc"
//..................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a(argc,argv);
    Wizard wizard;
    wizard.setCaption("Wizard-Beispiel");
    return wizard.exec();
}
