#include <qapplication.h>
#include <qwidget.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qvalidator.h>
#include <qwizard.h>
#include <stdlib.h>
#include <stdio.h>

const char *value[6] = { "100", "200", "400", "500", "800", "1000" };;

//............................................................ Klasse Wizard
class Wizard : public QWizard
{
    Q_OBJECT

public:
    Wizard( QWidget *parent = 0, const char *name = 0 )
                           : QWizard( parent, name, true ) {
       setupPage1();
       setupPage2();
       setupPage3();
       setupPage4();
       setupPage5();
       setupPage6();

       cancelButton()->setText( "Abbrechen" );
       nextButton()->setText( "Ok" );

       backButton()->hide();
       helpButton()->hide();
       finishButton()->hide();

       pin->setFocus();
    }

    void showPage(QWidget* page);

protected:
    void setupPage1();
    void setupPage2();
    void setupPage3();
    void setupPage4();
    void setupPage5();
    void setupPage6();

    QHBox        *page1, *page2, *page3, *page4, *page5, *page6;
    QRadioButton *anzeige, *abheben, *hundert[6];
    int          choicePage2, choicePage4;
    QLineEdit    *pin;
    QLabel       *infoPage3, *infoPage5;

protected slots:
    void pinChanged( const QString &text ) {
       bool enable  = ( text == "1234" );
       nextButton()->setEnabled( enable );
    }

    void choicePage2Changed( int id ) {
       bool enable = anzeige->isChecked() || abheben->isChecked();
       nextButton()->setEnabled( enable );
       choicePage2 = id;
    }

    void choicePage4Changed( int id ) {
       bool enable = false;
       for ( int i=0; i<6; i++)
           if ( hundert[i]->isChecked() )
              enable = true;
       nextButton()->setEnabled( enable );
       choicePage4 = id;
    }

    virtual void next( ) {
       QWidget *p = currentPage();
       if ( p == page2 )
          showPage( choicePage2 == 3 ? page3 : page4 );
       else if ( p == page4 )
          showPage( page5 );
       else
          showPage( page2 );
    }
};

//............................................................... setupPage1
void Wizard::setupPage1()
{
    page1 = new QHBox( this );

    QVBox *page = new QVBox( page1 );
    QHBox *z1 = new QHBox( page );

    (void)new QLabel( "Geheimzahl:", z1 );
    pin = new QLineEdit( z1 );
    pin->setEchoMode( QLineEdit::Password );
    pin->setMinimumWidth( pin->sizeHint().width() );
    pin->setMaxLength( 4 );
    pin->setValidator( new QIntValidator( 0000, 9999, pin ) );

    connect( pin,  SIGNAL( textChanged( const QString & ) ),
             this, SLOT( pinChanged( const QString & ) ) );

    addPage( page1, "Gib Deine Geheimzahl ein" );

    setNextEnabled( page1, false );
}

//............................................................... setupPage2
void Wizard::setupPage2()
{
    page2 = new QHBox( this );

    QButtonGroup *buttonBox1 =
           new QButtonGroup( 2, Horizontal, "", page2);
    buttonBox1->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );

    anzeige = new QRadioButton( "Kontostand anzeigen", buttonBox1 );
    buttonBox1->insert( anzeige, 3 );

    abheben = new QRadioButton( "Geld abheben", buttonBox1 );
    buttonBox1->insert( abheben, 4 );

    connect( buttonBox1, SIGNAL( clicked( int ) ),
             this,       SLOT( choicePage2Changed( int ) ) );

    addPage( page2, "Whlen Sie bitte aus" );
    setBackEnabled( page2, false );
}

//............................................................... setupPage3
void Wizard::setupPage3()
{
    page3 = new QHBox( this );

    infoPage3 = new QLabel( page3 );
    infoPage3->setPaletteBackgroundColor( Qt::yellow );

    addPage( page3, "Kontostand" );
    setNextEnabled( page3, true );
}

//............................................................... setupPage4
void Wizard::setupPage4()
{
    page4 = new QHBox( this );

    QButtonGroup *buttonBox1 =
           new QButtonGroup( 2, Horizontal, "", page4);
    buttonBox1->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );

    for ( int i=0; i<6; i++) {
       hundert[i] = new QRadioButton( value[i], buttonBox1 );
       buttonBox1->insert( hundert[i], i );
    }
    connect( buttonBox1, SIGNAL( clicked( int ) ),
             this,       SLOT( choicePage4Changed( int ) ) );

    addPage( page4, "Whlen Sie bitte Ihren Betrag aus" );
}

//............................................................... setupPage5
void Wizard::setupPage5()
{
    page5 = new QHBox( this );

    infoPage5 = new QLabel( page5 );
    infoPage5->setPaletteBackgroundColor( Qt::yellow );

    addPage( page5, "Auszahlung" );
    setNextEnabled( page5, true );
}

//............................................................... setupPage6
void Wizard::setupPage6()
{
    page6 = new QHBox( this );
    addPage( page6, "" );
    setNextEnabled( page6, true );
}

//................................................................. showPage
void Wizard::showPage( QWidget* page )
{
    QString t;

    QWizard::showPage(page);

    if ( page == page1 ) {
        pinChanged( pin->text() );
        pin->setFocus();
    } else if ( page == page2 ) {
        anzeige->setChecked( false );
        abheben->setChecked( false );
        nextButton()->setEnabled( false );
    } else if ( page == page3 ) {
        t.sprintf( " Ihr aktueller Kontostand ist %c%d,%02d Euro\n",
                   ::rand()%2 ? '+' : '-', ::rand()%10000, ::rand()%100 );
        infoPage3->setText( t );
        nextButton()->setEnabled( true );
        nextButton()->setFocus();
    } else if ( page == page4 ) {
        for ( int i=0; i<6; i++)
           hundert[i]->setChecked( false );
        nextButton()->setEnabled( false );
    } else if ( page == page5 ) {
        t.sprintf( "Bitte entnehmen Sie Ihre %s Euro\n",
                  value[ choicePage4 ] );
        infoPage5->setText( t );
        nextButton()->setEnabled( true );
        nextButton()->setFocus();
    }
}

#include "wizard2.moc"

//..................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a(argc,argv);

    Wizard wizard;
    wizard.setCaption("Wizard-Beispiel");
    return wizard.exec();
}
