#include <qapplication.h>
#include <qpushbutton.h>
#include <qtextbrowser.h>
#include <qmessagebox.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qfile.h>
#include <qxml.h>
#include <qstringlist.h>

//.............................................................. MeinRichText
class MeinRichText : public QVBox {
    Q_OBJECT
public:
    MeinRichText( const QStringList &s, QWidget *p=0, const char *n=0 )
                                                         : QVBox( p, n ) {
       texte = s;
       setMargin( 5 );

       QBrush paper;
       paper.setPixmap( QPixmap( "marble.xpm" ) );

       anzeige = new QTextBrowser( this );
       anzeige->setPaper( paper );
       anzeige->setFont( QFont( "Times", 12 ) );

       anzeige->setText( texte[0] );
       anzeige->setMinimumSize( 450, 250 );

       QHBox *buttons = new QHBox( this );
       buttons->setMargin( 5 );

       bBeenden = new QPushButton( "&Beenden", this );
       QWidget *w = new QWidget( this );
       w->setMaximumHeight( bBeenden->sizeHint().height() );

       bZurueck = new QPushButton( "<< &Zurueck", buttons );
       bWeiter  = new QPushButton( "&Weiter >>", buttons );

       buttons->setMaximumHeight( bWeiter->sizeHint().height() + 10 );
       bZurueck->setEnabled( false );

       connect( bBeenden, SIGNAL( clicked() ), qApp, SLOT( quit() ) );
       connect( bZurueck, SIGNAL( clicked() ), this, SLOT( zurueck() ) );
       connect( bWeiter, SIGNAL( clicked() ), this, SLOT( weiter() ) );

       nr = 0;
   }

protected:
    QTextBrowser *anzeige;
    QPushButton  *bBeenden, *bWeiter, *bZurueck;
    QStringList  texte;
    int nr;

protected slots:
    void zurueck() {
       if ( nr <= 0 )
          return;
       nr--;
       anzeige->setText( texte[nr] );
       if ( nr == 0 )
          bZurueck->setEnabled( false );
       bWeiter->setEnabled( true );
    }

    void weiter() {
       if ( nr >= (int)texte.count()-1 )
          return;
       nr++;
       anzeige->setText( texte[nr] );
       if ( nr >= (int)texte.count()-1 )
          bWeiter->setEnabled( false );
       bZurueck->setEnabled( true );
    }
};

//............................................................. QuoteHandler
class QuoteHandler : public QXmlDefaultHandler
{
public:
    QuoteHandler() { }
    ~QuoteHandler() { }

    QStringList quotes()    { return quoteList; }
    QString errorProtocol() { return errorProt; }

    bool startDocument() { 
       quoteList.clear();
       errorProt = "";
       state = StateInit;
       return true;
    }

    bool startElement( const QString&, const QString&,
                       const QString& qName, const QXmlAttributes& atts ) {
       if ( qName == "quotations" && state == StateInit ) {
          state = StateDocument;
       } else if ( qName == "quote" && state == StateDocument ) {
          state = StateQuote;
          QString tmp;
          quoteList.append( tmp );
          author = atts.value( "author" );
          reference = atts.value( "reference" );
          mytitle = atts.value( "mytitle" );
          quoteList.last() += "<b>";
          quoteList.last() += QString::number( quoteList.count() ) + ". ";
          quoteList.last() += (mytitle.isEmpty()) ? QString("") : mytitle;
          quoteList.last() += "</b><br><hr><br><br><big>";
       } else if ( qName == "line" && state == StateQuote ) {
          state = StateLine;
       } else if ( qName == "heading" && state == StateQuote ) {
          state = StateHeading;
          quoteList.last() += "<b>";
       } else if ( qName == "p" && state == StateQuote ) {
          state = StateP;
          quoteList.last() += "<p>";
       } else
          return false;  // Fehler
       return true;
    }

    bool endElement( const QString&, const QString&, const QString& ) {
       switch ( state ) {
          case StateQuote:
             state = StateDocument;
             quoteList.last() += "</big><br><br><center><i>";
             if ( author.isEmpty() )
                quoteList.last() += QString("");
             else
                quoteList.last() += "-- " + author;
             if ( !reference.isEmpty() )
                quoteList.last() += ", \"" + reference + "\"";
             quoteList.last() += "</i></center><i>";
             break;
          case StateLine:
             state = StateQuote; quoteList.last() += "<br>";     break;
          case StateHeading:
             state = StateQuote; quoteList.last() += "</b><br>"; break;
          case StateP:
             state = StateQuote; quoteList.last() += "</p>";     break;
          default: break;
       }
       return true;
    }

    bool characters( const QString& ch ) {
       QString ch_simplified = ch.simplifyWhiteSpace();
       if ( ch_simplified.isEmpty() )
          return true;

       switch ( state ) {
          case StateQuote:
          case StateLine:
          case StateHeading:
          case StateP: quoteList.last() += ch_simplified; break;
          default: return false;
       }
       return true;
    }

    QString errorString() {
       return "the document is not in the quote file format";
    }

    bool fatalError( const QXmlParseException& exception ) {
       errorProt += QString( "fatal parsing error: %1 in line %2, column %3\n")
                           .arg( exception.message() )
                           .arg( exception.lineNumber() )
                           .arg( exception.columnNumber() );
       return QXmlDefaultHandler::fatalError( exception );
    }

private:
    QStringList quoteList;
    QString     errorProt, author, reference, mytitle;

    enum State { StateInit, StateDocument, StateQuote,
                 StateLine, StateHeading,  StateP };
    State state;
};

#include "xmlrichtext.moc"

//...................................................................... main
int main( int argc, char *argv[] ) {
    QApplication a( argc, argv );

    QuoteHandler handler;
    QFile file( "sayings.xml" );
    QXmlInputSource source( file );
    QXmlSimpleReader reader;
    reader.setContentHandler( &handler );
    reader.setErrorHandler( &handler );
    bool ok = reader.parse( source );
    file.close();
    if ( !ok ) {
       QMessageBox::critical( 0, a.tr( "Parse Error" ),
                              a.tr( handler.errorProtocol() ) );
       return -1;
    }

    MeinRichText richtext( handler.quotes() );
    richtext.resize( 450, 350 );
    richtext.setCaption( "Kluge Sprueche und Anekdoten" );
    a.setMainWidget( &richtext );
    richtext.show();
    return a.exec();
}
